<?php

session_start();
define('INCLUDE_ADMIN_FUNCTIONS', true);
require '../visitorheader.inc.php';
require CMS_ROOT. 'cmsincludes/admin.inc.php';
useGZIP();

//Quickly save an admin's Storekeeper preferences
if (isset($_POST['prefs']) && checkPriv()) {
	saveAdminStorekeeperPrefs($_POST['prefs']);
	exit;
}

//Add the admin id and type up as constants
define('ADMIN_ID', (int) session('admin_userid'));
define('ADMIN_TYPE', "'". mysql_real_escape_string(session('admin_authtype')). "'");
define('GETTING_ITEM_NAME', (int) issetArrayKey($_REQUEST, '_getting_item_name'));


/*
	This file is used to drive the AJAX interface for Storekeeper and Admin Boxes
	It's main features are:
		Generating a complete "Map" of everything in Storekeeper
		"Focusing" on a specific Panel in Storekeeper, and sending detailed information on that
	Two new features are currently in development for our new Admin Box system:
		"Focusing" on a specific Admin Box, and sending detailed information on that
		Validating and Saving Admin Boxes
	
	It has a couple of other, rarely used features:
		An XML generation mode for the Languages Panel, to enable a Language Store
		A CSV export system for the users Panel
*/



if (get('_ab')) {
	$type = 'admin_boxes';
} else {
	$type = 'storekeeper';
}

//Read all of the xml files, and merge them together into something the JavaScript engine on the client can read


function likeEscape($sql) {
	if ($sql == '*') {
		return '_';
	} else {
		return str_replace('*', '%', str_replace('%', '\\%', str_replace('_', '\\_', mysql_real_escape_string($sql))));
	}
}

//Include a Plugin
function tribiqAJAXIncludePlugin(&$plugins, $className) {
	
	if ($dbChanged) {
		connectLocalDB();
	}
	
		if ($plugins[$className]) {
			return;
		}
		
		if (inc($className) && ($plugins[$className] = activatePluginClass($className))) {
			$plugins[$className]->setGETVariables($_GET);
			$plugins[$className]->setPOSTVariables($_POST);
		
		} elseif ($className == $tags['class_name']) {
			echo getPhrase('Could not activate the [[class_name]] Plugin.', array('class_name' => $tags['class_name']));
			exit;
		}
	
	if ($dbChanged) {
		connectGlobalDB();
	}
}


//Strip out any tags/sections that require a priv that the current admin does not have
//Also count each tags' children
function tribiqAJAXParseXML3(&$tags, &$removedColumns, $type, $parentKey = false, $parentParentKey = false) {
	
	$count = 0;
	$noPrivs = array();
	
	$countItems = false;
	$orderItems = false;
	if ($type == 'storekeeper') {
		$countItems =
		$orderItems = $parentKey === false
					|| $parentKey == 'items'
					|| $parentKey == 'columns'
					|| $parentKey == 'waypoints'
					|| $parentKey == 'item_buttons'
					|| $parentKey == 'collection_buttons'
					|| $parentKey == 'second_level_items';
	
	} elseif ($type == 'admin_boxes') {
		$orderItems = $parentKey === false
					|| $parentKey == 'tabs'
					|| $parentKey == 'fields';
	}
	
	if (is_array($tags)) {
		//Strip out any tags/sections that require a priv that the current admin does not have
		foreach ($tags as $key => &$value) {
			if ((string) $key == 'priv') {
				if (!checkPriv((string) $value)) {
					return false;
				}
			} elseif ((string) $key == 'superadmins_only') {
				if (engToBoolean($value) && session('admin_authtype') != 'super') {
					return false;
				}
			} elseif (!tribiqAJAXParseXML3($value, $removedColumns, $type, (string) $key, $parentKey)) {
				$noPrivs[] = $key;
			} elseif (is_array($value) && ($countItems || $orderItems)) {
				++$count;
				
				if ($parentKey != 'items' && !isset($value['ord']) && $orderItems) {
					$value['ord'] = $count;
				}
			}
		}
		
		foreach($noPrivs as $key) {
			unset($tags[$key]);
		}
		unset($tags['priv']);
		
		//Add counts to items, buttons and top/second level items
		if ($countItems && !isset($tags['count']) && $countItems) {
			$tags['count'] = $count;
		}
		
		//Strip the class_name tag out except where needed
		if ($type == 'storekeeper') {
			if ($parentKey != 'ajax'
			 && $parentParentKey != 'columns'
			 && $parentKey != 'combine_items'
			 && $parentParentKey != 'refiners'
			 && $parentKey != 'floating_box'
			 && $parentKey != 'pick_items'
			 && $parentKey != 'reorder'
			 && $parentKey != 'upload') {
				unset($tags['class_name']);
			}
		
		} else {
			if ($parentParentKey != 'tabs'
			 && $parentParentKey != 'fields') {
				unset($tags['class_name']);
			}
		}
		
		//Don't send any SQL to the client
		if ($type == 'storekeeper') {
			if ($parentKey === false || $parentKey == 'panel') {
				if ($tags['db_items']) {
					$tags['db_items'] = array(
						'client_side' => engToBooleanArray($tags, 'db_items', 'client_side'),
						'encode_id_column' => engToBooleanArray($tags, 'db_items', 'encode_id_column'));
				}
				
				if (is_array($tags['columns'])) {
					foreach ($tags['columns'] as &$col) {
						if (is_array($col)) {
							if (issetArrayKey($col, 'db_column')) {
								$col['db_column'] = true;
							}
							if (issetArrayKey($col, 'search_column')) {
								$col['search_column'] = true;
							}
							if (isset($col['table_join'])) {
								unset($col['table_join']);
							}
						}
					}
				}
			
			} elseif (($parentParentKey === false || $parentParentKey == 'panel') && $parentKey == 'columns') {
				$removedColumns = array();
				foreach ($tags as $key => &$value) {
					if (is_array($value) && engToBoolean(arrayKey($value, 'server_side_only'))) {
						$removedColumns[] = $key;
					}
				}
			}
		}
	}
	
	return true;
}

function prefixTableJoin($tableJoin) {
	$tableJoin = preg_replace('/\s\s+/', ' ', trim($tableJoin));
	$words = preg_split('/\W+/', strtoupper($tableJoin), 4);
	
	if (arrayKey($words, 1) != 'JOIN'
	 && arrayKey($words, 2) != 'JOIN'
	 && arrayKey($words, 1) != 'STRAIGHT_JOIN'
	 && arrayKey($words, 2) != 'STRAIGHT_JOIN') {
		$tableJoin = 'LEFT JOIN '. $tableJoin;
	}
	
	return $tableJoin;
}

function tribiqAJAXShortenPath($path, $type) {
	if ($type == 'storekeeper') {
		$path =
			str_replace('/panel/', '/',
			str_replace('/second_level_items/', '/',
			str_replace('/collection_buttons/', '/',
			str_replace('/item_buttons/', '/',
			str_replace('/waypoints/', '/',
			str_replace('/items/', '/',
				$path
			))))));
	}
	
	return $path;
}

$requestedPath = false;
if (get('path')) {
	$requestedPath = tribiqAJAXShortenPath(preg_replace('/[^\w\/]/', '', get('path')), $type);
}

$filters = array();
if ($type == 'storekeeper' && $_GET['_filters']) {
	$filters = json_decode($_GET['_filters'], true);
}

//List all running Plugins and their dependancies
$plugins = array();
//Ensure that the core plugin is included first
$plugins['tribiq_cms_core'] = array();
foreach (getRunningPlugins() as $plugin) {
	$plugins[$plugin['class_name']] = array('name' => $plugin['name'], 'depends' => getPluginDependencies($plugin['plugin_name']), 'included' => false);
}

//Include every Plugin's XML files in dependency order
$tags = array();
$limit = 9999;
do {
	$includesDone = false;
	
	foreach ($plugins as $className => &$plugin) {
		//No need to include a Plugin twice
		if ($plugin['included']) {
			continue;
		}
		
		//Skip activating this Plugin if not all of its dependencies are running
		foreach ($plugin['depends'] as $depends) {
			if (!$plugins[$depends['dependency_plugin_class_name']]['included']) {
				continue 2;
			}
		}
		
		//Include any xml files in the directory
		if (is_dir($dir = CMS_ROOT. 'plugins/'. $plugin['name']. '/'. $type. '/')) {
			foreach (scandir($dir) as $file) {
				if (substr($file, 0, 1) != '.' && substr($file, -4) == '.xml') {
					$xml = simplexml_load_file($dir. $file);
					tribiqAJAXParseXML($tags, $xml, $type, $className, $requestedPath);
					unset($xml);
				}
			}
		}
		
		$plugin['included'] = true;
		$includesDone = true;
	}
//Loop while includes are still being done
} while ($includesDone && --$limit);


//Readjust the start to get rid of the outer tag
$tags = $tags[$type];

//If we had a requested path, drill straight down to that level
if ($requestedPath) {
	foreach(explode('/', $requestedPath) as $path) {
		if (is_array($tags[$path])) {
			$tags = $tags[$path];
		
		} elseif ($type == 'storekeeper' && is_array($tags['panel']) && is_array($tags['panel'][$path])) {
			$tags = $tags['panel'][$path];
		
		} elseif ($type == 'storekeeper' && is_array($tags['second_level_items']) && is_array($tags['second_level_items'][$path])) {
			$tags = $tags['second_level_items'][$path];
		
		} elseif ($type == 'storekeeper' && is_array($tags['collection_buttons']) && is_array($tags['collection_buttons'][$path])) {
			$tags = $tags['collection_buttons'][$path];
		
		} elseif ($type == 'storekeeper' && is_array($tags['item_buttons']) && is_array($tags['item_buttons'][$path])) {
			$tags = $tags['item_buttons'][$path];
		
		} elseif ($type == 'storekeeper' && is_array($tags['waypoints']) && is_array($tags['waypoints'][$path])) {
			$tags = $tags['waypoints'][$path];
		
		} elseif ($type == 'storekeeper' && is_array($tags['items']) && is_array($tags['items'][$path])) {
			$tags = $tags['items'][$path];
		
		} elseif ($type == 'storekeeper') {
			if (is_array($tags['panel'])) {
				$tags = $tags['panel'];
				
				if (is_array($tags['second_level_items']) && is_array($tags['second_level_items'][$path])) {
					$tags = $tags['second_level_items'][$path];
				
				} elseif (is_array($tags['collection_buttons']) && is_array($tags['collection_buttons'][$path])) {
					$tags = $tags['collection_buttons'][$path];
				
				} elseif (is_array($tags['item_buttons']) && is_array($tags['item_buttons'][$path])) {
					$tags = $tags['item_buttons'][$path];
				
				} elseif (is_array($tags['waypoints']) && is_array($tags['waypoints'][$path])) {
					$tags = $tags['waypoints'][$path];
				
				} elseif (is_array($tags['items']) && is_array($tags['items'][$path])) {
					$tags = $tags['items'][$path];
				
				} else {
					echo getPhrase('[[path]] was not found.', array('path' => $requestedPath));
					exit;
				}
			} else {
				echo getPhrase('[[path]] was not found.', array('path' => $requestedPath));
				exit;
			}
		} else {
			echo getPhrase('[[path]] was not found.', array('path' => $requestedPath));
			exit;
		}
	}
	
	if ($type == 'storekeeper' && is_array($tags['panel'])) {
		$tags = $tags['panel'];
	}

} else {
	//There's no "map" for admin Admin Boxes; they must have a path
	if ($type == 'admin_boxes') {
		exit;
	}
}

//Tidy up a bit
tribiqAJAXParseXML2($tags);

//Check that an admin is logged in, and kick them if not
	//Note that there's an option to bipass this and allow visitors in!
if (!checkPriv() && !(get('_xml') && engToBooleanArray($tags, 'allow_unauthenticated_xml_access'))) {
	echo '<!--Logged_Out-->', getPhrase('You have been logged out.');
	exit;
}

//If this is a request for a specific path, run that Plugin and let it manage its output in PHP
unset($plugins);
$plugins = array();
if ($requestedPath && $tags['class_name']) {
	tribiqAJAXIncludePlugin($plugins, $tags['class_name']);
	
	if ($type == 'storekeeper') {
		
		//Add definitions for any refiners supplied in the request
		$refinersPresent = array('refinerId' => 'REFINER_ID');
		foreach ($_REQUEST as $key => $value) {
			if (substr($key, 0, 9) == 'refiner__') {
				$refinersPresent[$key] = strtoupper($key);
			}
		}
		
		foreach ($refinersPresent as $req => $def) {
			if (isset($_GET[$req])) {
				$refiners = '';
				foreach (explode(',', get($req)) as $i => $refiner) {
					$refiners .= $i? ',' : '';
					$refiner = decodeItemIdForStorekeeper($refiner);
					
					if (is_numeric($refiner)) {
						$refiners .= (int) $refiner;
					} else {
						$refiners .= "'". mysql_real_escape_string($refiner). "'";
					}
					
					if (!$i) {
						define($def, $refiners);
						define($def. '__SET', 1);
						define($def. '__WORD_CHARS_ONLY', preg_replace('/[^\w\d_]/', '', $refiner));
					}
				}
				define($def. 'S', '('. $refiners. ')');
			} else {
				define($def, '0');
				define($def. 'S', '(0)');
			}
		}
		
		//Start to populate the Storekeeper Panel:
		//Firstly, see if other Plugins have added columns/refiners. If so, they'll need their own placeholder methods executing as well
		
		//Have any buttons been added that need formatting from their own Plugin?
		if (get('refinerName') && issetArrayKey($tags, 'refiners', get('refinerName'))) {
			tribiqAJAXIncludePlugin($plugins, $className = $tags['refiners'][get('refinerName')]['class_name']);
		}
		
		//Note down if any buttons need any code from a different Plugin
		foreach (array('collection_buttons', 'item_buttons') as $buttonType) {
			if (is_array($tags[$buttonType])) {
				foreach ($tags[$buttonType] as &$button) {
					if (is_array($button)) {
						if (engToBoolean($button['php'])) {
							tribiqAJAXIncludePlugin($plugins, $button['class_name']);
						}
					}
				}
			}
		}
		
		//Have any columns been added that need formatting from their own Plugin?
		if (!get('_queued')) {
			if (is_array($tags['columns'])) {
				foreach ($tags['columns'] as $colName => &$col) {
					if (engToBoolean($col['php'])) {
						tribiqAJAXIncludePlugin($plugins, $col['class_name']);
					}
				}
			}
		}
		
		//Run the Plugins' lining method to add/modify/remove columns/refiners/other tags
		foreach ($plugins as $className => &$plugin) {
			$plugin->lineStorekeeper($requestedPath, $tags, request('refinerName'), request('refinerId'), (bool) get('_select_mode'), (bool) get('_xml'));
		}
		
		
		//Secondly, run any SQL queries that are needed to populate the items.
		//Refiners and pagination are also applied at this point
		if (!issetArrayKey($tags, 'items') && issetArrayKey($tags, 'db_items', 'table') && issetArrayKey($tags, 'db_items', 'id_column') && is_array($tags['columns'])) {
			
			if (!get('_queued')) {
				
				$dbChanged = false;
				if (engToBoolean($tags['db_items']['global'])) {
					connectGlobalDB();
					$dbChanged = true;
				}
				
					if (!is_array($tags['items'])) {
						$tags['items'] = array();
					}
					
					if (get('_csv')) {
						//Create a file in the temp directory to start writing a CSV file to.
						$filename = tempnam(sys_get_temp_dir(), 'tmpfiletodownload');
						$f = fopen($filename, 'wb');
						
						$csvCols = array();
						if (!is_array($sortedColumns = json_decode(get('_sortedColumns'), true))
						 || !is_array($shownColumnsInCSV = json_decode(get('_shownColumnsInCSV'), true))) {
							exit;
						}
						
						$sortedColumns = array_flip($sortedColumns);
					}
					
					
					//Apply a refiners, if this panel has any and one has been selected
					$limit = "";
					$orderBy = "";
					$whereStatement = "";
					$extraTables = array();
					$sortExtraTables = array();
					
					if (get('refinerName') && issetArrayKey($tags, 'refiners', get('refinerName'))) {
						
						//allow_unauthenticated_xml_access must be repeated on a refiner if they are both used
						if (!checkPriv() && !(get('_xml') && engToBooleanArray($tags, 'refiners', get('refinerName'), 'allow_unauthenticated_xml_access'))) {
							echo '<!--Logged_Out-->', getPhrase('You have been logged out.');
							exit;
						}
						
						if (issetArrayKey($tags, 'refiners', get('refinerName'), 'sql')) {
							$whereStatement .= "
								AND ". addConstantsToString($tags['refiners'][get('refinerName')]['sql']);
						}
						
						//Add any table-joins for refiners
						if (issetArrayKey($tags, 'refiners', get('refinerName'), 'table_join')) {
							$extraTables[prefixTableJoin($tags['refiners'][get('refinerName')]['table_join'])] = true;
							$sortExtraTables[prefixTableJoin($tags['refiners'][get('refinerName')]['table_join'])] = true;
						}
					
					} elseif (engToBooleanArray($tags, 'refiner_required')) {
						echo 'A refiner was required, but none was set.';
						exit;
					}
					
					
					//Loop through each database-column defined in the XML schema
					$columns = "";
					$sortColumn = false;
					
					foreach ($tags['columns'] as $colName => &$col) {
						if (is_array($col) && $col['db_column']) {
							//Add it to the SELECT
							$columns .= ",
								". addConstantsToString($col['db_column']);
							
							//Add it to the sort if we're sorting by it
							if ($colName == get('_sort_col') && !engToBooleanArray($col, 'disallow_sorting')) {
								$sortColumn = addConstantsToString($col['db_column']);
							}
							
							//If it's from a different table, join to that table
							if (issetArrayKey($col, 'table_join')) {
								$extraTables[prefixTableJoin($col['table_join'])] = true;
								
								if ($colName == get('_sort_col')
								 || (engToBoolean(arrayKey($col, 'searchable') && (isset($_GET['_search']) || issetArrayKey($filters, $colName, 'searchcol_'))))
								 || (arrayKey($col, 'format') == 'enum' && issetArrayKey($filters, $colName, 'enum_'))
								 || (arrayKey($col, 'format') == 'yes_or_no' && issetArrayKey($filters, $colName, 'yes_or_no_'))
								 || ((arrayKey($col, 'format') == 'date' || arrayKey($col, 'format') == 'datetime')
								  && (issetArrayKey($filters, $colName, 'date_after_col_') || issetArrayKey($filters, $colName, 'date_before_col_')))) {
									$sortExtraTables[prefixTableJoin($col['table_join'])] = true;
								}
							}
							
							//Add it to the list of columns for a CSV export
							if (get('_csv') && issetArrayKey($shownColumnsInCSV, $colName)) {
								$csvCols[$colName] = (int) arrayKey($sortedColumns, $colName);
							}
						}
					}
					
					if (get('_csv')) {
						//Sort the array by ordinal
						asort($csvCols);
						//Then flip the array so the column names are the values
						$csvCols = array_flip($csvCols);
						
						//Print the column headers for a CSV export
						fputcsv($f, $csvCols);
					}
					
					
					//Apply a search from the search box
					if (isset($_GET['_search'])) {
						$whereStatement .= "
						  AND (";
						
						$first = true;
						foreach ($tags['columns'] as $colName => &$col) {
							if (is_array($col) && $col['db_column'] && engToBoolean(arrayKey($col, 'searchable'))) {
								//Group functions can't be used in a query
								if (!preg_match('/COUNT\s*\(/i', $col['db_column'])) {
									
									if ($first) {
										$first = false;
									} else {
										$whereStatement .= " OR";
									}
									
									$whereStatement .= "
										". addConstantsToString(ifNull(arrayKey($col, 'search_column'), $col['db_column'])). " LIKE '%". likeEscape(get('_search')). "%'";
								}
							}
						}
						
						if ($first) {
							$whereStatement .= "TRUE";
						}
						
						$whereStatement .= "
						  )";
					}
					
					
					//Apply filters
					foreach ($tags['columns'] as $colName => &$col) {
						if (is_array($col) && issetArrayKey($col, 'db_column') && !engToBoolean(arrayKey($col, 'disallow_filtering'))) {
							//Group functions can't be used in a query
							if (!preg_match('/COUNT\s*\(/i', $col['db_column'])) {
								
								//Do a text search for filters on text fields
								if (issetArrayKey($filters, $colName, 'searchcol_') && engToBoolean(arrayKey($col, 'searchable'))) {
									$whereStatement .= "
									  AND ". addConstantsToString(ifNull(arrayKey($col, 'search_column'), $col['db_column'])).
										  (issetArrayKey($filters, $colName, 'not')? " NOT" : "").
										  " LIKE '%". likeEscape($filters[$colName]['searchcol_']). "%'";
								}
								
								//enum filters
								if (issetArrayKey($filters, $colName, 'enum_') && arrayKey($col, 'format') == 'enum') {
									$whereStatement .= "
									  AND ". addConstantsToString(ifNull(arrayKey($col, 'search_column'), $col['db_column'])).
										  (issetArrayKey($filters, $colName, 'not')? " != " : " = ").
										  "'". mysql_real_escape_string($filters[$colName]['enum_']). "'";
								}
								
								//Yes/No type filters on tinyint columns
								if (issetArrayKey($filters, $colName, 'yes_or_no_') && arrayKey($col, 'format') == 'yes_or_no') {
									$whereStatement .= "
									  AND ". addConstantsToString(ifNull(arrayKey($col, 'search_column'), $col['db_column'])).
										  (issetArrayKey($filters, $colName, 'not')? " = 0" : " != 0");
								}
								
								//Search for any dates on or after a certain date
								if (issetArrayKey($filters, $colName, 'date_after_col_')
								 && (arrayKey($col, 'format') == 'date' || arrayKey($col, 'format') == 'datetime')
								 && preg_replace('/\d{4}-\d{2}-\d{2}/', '', $filters[$colName]['date_after_col_']) == '') {
									$whereStatement .= "
									  AND ". addConstantsToString(ifNull(arrayKey($col, 'search_column'), $col['db_column'])).
										  " >= '". mysql_real_escape_string($filters[$colName]['date_after_col_']). "'";
								}
								
								//Search for any dates on or before a certain date
								if (issetArrayKey($filters, $colName, 'date_before_col_')
								 && (arrayKey($col, 'format') == 'date' || arrayKey($col, 'format') == 'datetime')
								 && preg_replace('/\d{4}-\d{2}-\d{2}/', '', $filters[$colName]['date_before_col_']) == '') {
									$whereStatement .= "
									  AND ". addConstantsToString(ifNull(arrayKey($col, 'search_column'), $col['db_column'])).
									  	" <= '". mysql_real_escape_string($filters[$colName]['date_before_col_']). "'";
								}
							}
						}
					}
					
					
					
					$idColumn = addConstantsToString($tags['db_items']['id_column']);
					
					if ($tags['db_items']['group_by']) {
						$groupBy = addConstantsToString($tags['db_items']['group_by']);
					} else {
						$groupBy = $idColumn;
					}
					
					//Load the standard where-statement, if this panel has one
					if ($tags['db_items']['where_statement']) {
						$whereStatement = addConstantsToString($tags['db_items']['where_statement']). $whereStatement;
					} else {
						$whereStatement = "WHERE 1=1". $whereStatement;
					}
					
					//Order by the sort column
					if ($sortColumn) {
						$orderBy = $sortColumn. " ". (get('_sort_desc')? "DESC" : "ASC"). ", ". $groupBy;
					} else {
						$orderBy = $groupBy;
					}
					
					//Apply pagination
					$start = 0;
					if (get('_limit')) {
						$limit = "
						LIMIT ". (int) ($start = (int) get('_start')). ", ". (int) get('_limit');
					}
					
					
					
					//Only look for a few specific items
						//(Somewhat of a bispoke feature at the moment, it's not in the schema)
					if ($groupBy === $idColumn && issetArrayKey($_GET, '_get_item_links')) {
						$whereStatement .= "
							AND ". $groupBy. " IN (";
						
						foreach(explode(',', get('_get_item_links')) as $i => $id) {
							$whereStatement .= ($i? ',' : ''). (is_numeric($id)? (int) $id : "'". mysql_real_escape_string($id). "'");
						}
						
						$whereStatement .= ")";
						
					//Get a count of all the rows, and get each id in the correct order
					} elseif (!get('_xml')) {
						$sql = "
							SELECT ". $idColumn. "
							FROM ". addConstantsToString($tags['db_items']['table']);
					
						foreach ($sortExtraTables as $join => $dummy) {
							$sql .= "
								". addConstantsToString($join);
						}
						
						$sql .= "
							". $whereStatement. "
							GROUP BY ". $groupBy. "
							ORDER BY ". $orderBy;
						
						$tags['__item_sort_order__'] = array();
						$result = my_mysql_query($sql);
						$count = mysql_num_rows($result);
						while ($row = mysql_fetch_row($result)) {
							$tags['__item_sort_order__'][] = $row[0];
						}
						
						
						if (get('_limit')) {
							//If we are using pagination, and have had specific item(s) requested,
							//only show the page that item is on, or the first page that we come to with items on
							if (isset($_GET['_item'])) {
								$pos = false;
								foreach (explode(',', $_GET['_item']) as $item) {
									$itemPos = array_search($item, $tags['__item_sort_order__']);
									
									if ($itemPos !== false && ($pos === false || $itemPos < $pos)) {
										$pos = $itemPos;
									}
								}
								
								if ($pos !== false) {
									$limit = "
										LIMIT ". (int) ($start = $pos - ($pos % (int) get('_limit'))). ", ". (int) get('_limit');
								}
							}
							
							//Set which page this should be
							$tags['__page__'] = 1 + (int) ($start / (int) get('_limit'));
							
							//Don't sent the whole collection of ids if we are using server-side pagination
							$stop = $start + (int) get('_limit');
							
							//...but add an id either side to help with forward/back buttons in close-up view
							if ($start > 0) {
								--$start;
							}
							if ($stop < $count) {
								++$stop;
							}
							
							$new__item_sort_order__ = array();
							for ($i = $start; $i < $stop; ++$i) {
								if ($tags['__item_sort_order__'][$i] !== null) {
									$new__item_sort_order__[$i] = $tags['__item_sort_order__'][$i];
								} else {
									break;
								}
							}
							
							unset($tags['__item_sort_order__']);
							$tags['__item_sort_order__'] = $new__item_sort_order__;
							unset($new__item_sort_order__);
						}
					}
					
					
					//Get all of the rows on the current page
					$sql = "
						SELECT
							". $idColumn. 
							$columns. "
						FROM ". addConstantsToString($tags['db_items']['table']);
					
					foreach ($extraTables as $join => $dummy) {
						$sql .= "
							". addConstantsToString($join);
					}
					
					$sql .= "
						". $whereStatement. "
						GROUP BY ". $groupBy. "
						ORDER BY ". $orderBy.
						$limit;
	
					
					//Loop through the results adding them into the items array (or alternately into the CSV file for CSV exports)
					$result = my_mysql_query($sql);
					while ($row = mysql_fetch_row($result)) {
						if (get('_csv')) {
							$i = 0;
							$assoc = array();
							
							foreach ($tags['columns'] as $colName => &$col) {
								if (is_array($col) && $col['db_column']) {
									$assoc[$colName] = $row[++$i];
								}
							}
	
							//Run the Plugin's formatStorekeeperCSV() method to add formating, and some other required attributes
							//If other Plugins have added columns/refiners, run their fill method to add their own formatting
							foreach ($plugins as $className => &$plugin) {
								$plugin->formatStorekeeperCSV($requestedPath, $assoc, request('refinerName'), request('refinerId'));
							}
							
							$csvLine = array();
							foreach ($csvCols as $ord => $colName) {
								$csvLine[$ord] = arrayKey($assoc, $colName);
								
								if (arrayKey($tags, 'columns', $colName, 'format') == 'yes_or_no') {
									$csvLine[$ord] = engToBoolean($csvLine[$ord])? getPhrase('Yes') : getPhrase('No');
								
								} elseif (arrayKey($tags, 'columns', $colName, 'format') == 'true_or_false') {
									$csvLine[$ord] = engToBoolean($csvLine[$ord])? getPhrase('True') : getPhrase('False');
								}
							}
							unset($assoc);
	
							fputcsv($f, $csvLine);
							unset($csvLine);
						} else {
							$id = $row[$i = 0];
							
							if (engToBoolean($tags['db_items']['encode_id_column'])) {
								$id = encodeItemIdForStorekeeper($id);
							}
							
							$tags['items'][$id] = array();
							
							foreach ($tags['columns'] as $colName => &$col) {
								if (is_array($col) && $col['db_column']) {
									$tags['items'][$id][$colName] = $row[++$i];
								}
							}
						}
					}
					
					//If this is a CSV export, offer it for download without any formatting and then exit
					if (get('_csv')) {
						fclose($f);
						
						//...and finally offer it for download
						header('Content-Type: text/x-csv');
						header('Content-Disposition: attachment; filename="'. str_replace('/', '_', $requestedPath). '.csv"');
						header('Content-Length: '. filesize($filename)); 
						readfile($filename);
						
						//Remove the file from the temp directory
						@unlink($filename);
						
						exit;
					}
				
				connectLocalDB();
				$dbChanged = false;
			
			
			} else {
				//For panel requests that are part of a queue, don't attempt to fetch any items. But to try and load other information such as the title.
				$tags['items'] = array();
			}
		
		} else {
			if (get('refinerName') && issetArrayKey($tags, 'refiners', get('refinerName'))) {
				//allow_unauthenticated_xml_access must repeated it on a refiner 
				if (!checkPriv() && !(get('_xml') && engToBooleanArray($tags, 'refiners', get('refinerName'), 'allow_unauthenticated_xml_access'))) {
					echo '<!--Logged_Out-->', getPhrase('You have been logged out.');
					exit;
				}
			}
		}
		
		//Thirdly, run each Plugins' fill method to add formating, and some other required attributes
		foreach ($plugins as $className => &$plugin) {
			$plugin->fillStorekeeper($requestedPath, $tags, request('refinerName'), request('refinerId'), (bool) get('_select_mode'), (bool) get('_xml'));
		}
		
		//Set the current item count
		if (isset($count)) {
			$tags['items']['count'] = $count;
		}
		
	
	} elseif ($type == 'admin_boxes') {
		
		if (is_array($tags['tabs'])) {
			foreach ($tags['tabs'] as &$tab) {
				if (issetArrayKey($tab, 'class_name')) {
					tribiqAJAXIncludePlugin($plugins, $tab['class_name']);
				}
				
				if (is_array($tab['fields'])) {
					foreach ($tab['fields'] as &$field) {
						if (issetArrayKey($field, 'class_name')) {
							tribiqAJAXIncludePlugin($plugins, $field['class_name']);
						}
					}
				}
			}
		}
		
		//Remove anything the current admin has no access to
		$removedColumns = false;
		tribiqAJAXParseXML3($tags, $removedColumns, $type);
		$values = array();
		
		//Special logic for Validating and Saving
		if (post('_format') || post('_validate') || post('_save') || post('_download')) {
			//Get the current version of the admin box
			$box = json_decode($_POST['_box'], true);
			checkBoxDefinition($box, $tags);
			
			if (is_array($tags['tabs'])) {
				foreach ($tags['tabs'] as $tabName => &$tab) {
					$values[$tabName] = array();
					if (is_array($tab) && is_array($tab['fields'])) {
						foreach ($tab['fields'] as $fieldName => &$field) {
							$values[$tabName][$fieldName] = false;
						}
					}
				}
			}
			if (is_array($box['tabs'])) {
				foreach ($box['tabs'] as $tabName => &$tab) {
					if (is_array($tab) && is_array($tab['fields'])) {
						$tab['errors'] = array();
						
						if (!is_array($values[$tabName])) {
							$values[$tabName] = array();
						}
						
						foreach ($tab['fields'] as $fieldName => &$field) {
							if (!engToBooleanArray($tab, 'edit_mode', 'on')
							 || engToBooleanArray($field, 'read_only')
							 || (isset($field['value']) && !isset($field['_value']))) {
								$values[$tabName][$fieldName] = arrayKey($field, 'value');
							} else {
								$values[$tabName][$fieldName] = arrayKey($field, '_value');
							}
						}
					}
				}
			}
			
			
			$doSave = false;
			$doFormat = true;
			$commentMade = false;
			if (post('_validate') || post('_save') || post('_download')) {
				//Take the current state of the box as a JSON object, and validate it
				
				//Apply standard validation formats
				if (is_array($tags['tabs'])) {
					foreach ($box['tabs'] as $tabName => &$tab) {
						if (engToBooleanArray($tab, 'edit_mode', 'on')) {
							$tab['errors'] = array();
							if (isset($tags['tabs'][$tabName]['fields']) && is_array($tags['tabs'][$tabName]['fields'])) {
								foreach ($tags['tabs'][$tabName]['fields'] as $fieldName => &$field) {
									if (($msg = arrayKey($field, 'validation', 'required')) && !trim(arrayKey($values, $tabName, $fieldName))) {
										$tab['errors'][] = $msg;
									
									} elseif (($msg = arrayKey($field, 'validation', 'required_if_not_hidden')) && !engToBooleanArray($tab, 'fields', $fieldName, 'hidden') && !trim(arrayKey($values, $tabName, $fieldName))) {
										$tab['errors'][] = $msg;
									
									} elseif (($msg = arrayKey($field, 'validation', 'email')) && !validateEmailAddress(arrayKey($values, $tabName, $fieldName))) {
										$tab['errors'][] = $msg;
									
									} elseif (($msg = arrayKey($field, 'validation', 'no_spaces')) && preg_replace('/\S/', '', arrayKey($values, $tabName, $fieldName))) {
										$tab['errors'][] = $msg;
									
									} elseif (($msg = arrayKey($field, 'validation', 'numeric')) && !is_numeric(arrayKey($values, $tabName, $fieldName))) {
										$tab['errors'][] = $msg;
									
									} elseif (($msg = arrayKey($field, 'validation', 'screen_name')) && !validateScreenName(arrayKey($values, $tabName, $fieldName))) {
										$tab['errors'][] = $msg;
									}
								}
							}
						}
					}
				}
				
				
				//Apply the Plugins' specific validation
				foreach ($plugins as $className => &$plugin) {
					$plugin->validateAdminBox($requestedPath, $box, $tags, $values);
				}
				
				//If the Admin is trying to save, and the box was valid, fire the save methods
				if (post('_save') || post('_download')) {
					
					//Check if there are any errors
					if (is_array($box['tabs'])) {
						$doSave = true;
						foreach ($box['tabs'] as &$tab) {
							if (is_array($tab['errors'])) {
								foreach ($tab['errors'] as $error) {
									if ($error) {
										$doSave = false;
										break 2;
									}
								}
							}
						}
					}
					
					if ($doSave) {
						if (!post('_download')) {
							echo '<!--Valid-->';
							$commentMade = true;
						}
						
						//Check if a confirmation is needed
						if (engToBooleanArray($box, 'confirm', 'show') && !(post('_confirm') || post('_download'))) {
							echo '<!--Confirm-->';
							$commentMade = true;
							
						} else if (engToBooleanArray($box, 'confirm', 'download') && !post('_download')) {
							echo '<!--Download-->';
							exit;
							
						} else {
							$doFormat = false;
							foreach ($plugins as $className => &$plugin) {
								$plugin->saveAdminBox($requestedPath, $box, $tags, $values);
							}
							
							if (engToBooleanArray($box, 'confirm', 'download')) {
								//Bugfix for IE 6
								if (strpos($_SERVER['HTTP_USER_AGENT'], 'MSIE 6') !== false) {
									header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
									header('Pragma: public');
								}
								
								foreach ($plugins as $className => &$plugin) {
									$plugin->adminBoxDownload($requestedPath, $box, $tags, $values);
								}
								exit;
							
							} else {
								echo '<!--Saved-->';
								$commentMade = true;
							}
						}
					}
				}
			}
			
			//If we're going to wind up displaying the box again, format it again
			if ($doFormat) {
				foreach ($plugins as $className => &$plugin) {
					$plugin->formatAdminBox($requestedPath, $box, $tags, $values);
				}
			}
			
			if (!$commentMade) {
				header('Content-Type: text/javascript; charset=UTF-8');
			}
			
			echo json_encode($box);
			exit;
			
		} else {
			if (is_array($tags['key'])) {
				foreach ($tags['key'] as $key => &$value) {
					if (get($key)) {
						$value = get($key);
					}
				}
			}
			
			foreach ($plugins as $className => &$plugin) {
				$plugin->fillAdminBox($requestedPath, $tags);
			}
			
			if (is_array($tags['tabs'])) {
				foreach ($tags['tabs'] as $tabName => &$tab) {
					$values[$tabName] = array();
					if (is_array($tab) && is_array($tab['fields'])) {
						foreach ($tab['fields'] as $fieldName => &$field) {
							if (isset($field['pick_items'])) {
								//Check the value of a Storekeeper Select field
								if (!is_array($field['picked_items'])) {
									//If the picked_items tag is not set, attempt to convert the values tag to an array
									$field['picked_items'] = array();
									foreach (explode(',', arrayKey($field, 'value')) as $item) {
										if ($item && $item !== 0 && $item !== '0') {
											$field['picked_items'][$item] = true;
										}
									}
								}
								
								//Then ensure the values tag matches up by setting it to a comma-seperated list of the picked items
								$first = true;
								foreach ($field['picked_items'] as $pickedItem => $label) {
									if ($first) {
										$first = false;
										$values[$tabName][$fieldName] = $pickedItem;
									} else {
										$values[$tabName][$fieldName] .= ','. $pickedItem;
									}
								}
							
							} else {
								//Otherwise load the value normally
								$values[$tabName][$fieldName] = arrayKey($field, 'value');
							}
						}
					}
				}
			}
			
			foreach ($plugins as $className => &$plugin) {
				$plugin->formatAdminBox($requestedPath, $tags, $tags, $values);
			}
		}
	}

} elseif ($type == 'admin_boxes') {
	//Admin Boxes require a specific path
	exit;
}


//Tidy away some Storekeeper tags
if ($type == 'storekeeper') {
	//Remove anything the current admin has no access to, count each tags' children
	$removedColumns = false;
	tribiqAJAXParseXML3($tags, $removedColumns, $type);
	
	//Don't send any SQL to the client
	if (is_array($tags['items']) && is_array($removedColumns)) {
		foreach ($tags['items'] as &$item) {
			foreach ($tags['items'] as &$item) {
				foreach ($removedColumns as $unset) {
					unset($item[$unset]);
				}
			}
		}
	}
}


//Item links don't need most things in the panel
if (get('_get_item_links')) {
	$tags = array('items' => $tags['items'], 'item' => $tags['item']);
	foreach ($tags['items'] as &$item) {
		$item = array(
			'name' => $item['name'],
			'css_class' => $item['css_class'],
			'deep_link' => $item['deep_link'],
			'frontend_link' => $item['frontend_link']);
	}
}


//Display the output, either as JSON to send to the Storekeeper's JavaScript engine, or as a print_r for debuggers
if (get('_xml')) {
	$xml = new XMLWriter();
	header('Content-Type: text/xml; charset=UTF-8');
	
	$xml->openURI('php://output');
	$xml->startDocument('1.0', 'UTF-8');
	$xml->setIndent(4);
	$xml->startElement('storekeeper');
	
	function XMLWriterRecurse(&$xml, &$tags, $first = true) {
		//Remove a few things we don't need for an XML feed
		if ($first) {
			unset($tags['php']);
			unset($tags['view_mode']);
			unset($tags['db_items']);
			unset($tags['refiners']);
			unset($tags['label_tag']);
			unset($tags['columns']);
			unset($tags['item']);
			unset($tags['collection_buttons']);
			unset($tags['item_buttons']);
			unset($tags['waypoints']);
			unset($tags['link']);
			unset($tags['allow_unauthenticated_xml_access']);
		}
		
		unset($tags['back_link']);
		unset($tags['panel']);
		unset($tags['count']);
		unset($tags['ord']);
		
		
		foreach ($tags as $key => &$child) {
			
			if (is_numeric($key)) {
				$key = '_'. $key;
			}
			
			if (is_array($child)) {
				$xml->startElement($key);
				XMLWriterRecurse($xml, $child, false);
				$xml->endElement();
			} else {
				$xml->writeElement($key, $child);
			}
		}
	}
	
	XMLWriterRecurse($xml, $tags);
	$xml->endElement();
	$xml->endDocument();
	$xml-flush();
	
} elseif (get('_json')) {
	header('Content-Type: text/javascript; charset=UTF-8');
	echo json_encode($tags);

} else {
	echo '<pre>'; print_r($tags); echo '<pre>'; 
}
