<?php
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */


/*
	Functions to help make dynamic forms in thickboxes
*/






//Shortcut function for checking the $_GET/$_POST and the database for a value of a field, used in dynamicThickboxFormInner()
function getUserFieldValue($field, $valuesInCache, $valuesInDB) {
	if (isset($valuesInCache[$field])) {
		return $valuesInCache[$field];
	} else {
		return $valuesInDB[$field];
	}
}


//Proceude the Standard HTML header and footer for a thickbox tab
function thickboxForm(&$formHTMLHead, &$formHTMLFoot, $editMode, $showEditButton = false, $aboveTableHTML = false, $errorDisplayColspan = 1) {
	$formHTMLHead = thickboxFormHead($editMode, $aboveTableHTML, $errorDisplayColspan);
	$formHTMLFoot = thickboxFormFoot($editMode, $showEditButton);
}


function thickboxFormHead($editMode, $aboveTableHTML = false, $errorDisplayColspan = 1) {
	
	$formHTMLHead = '
		<form name="tab_form" id="tab_form" onsubmit="formHandler(); return false;">
			<input type="hidden" id="formMode" name="formMode" value="'. ($editMode? 'edit' : 'view'). '" />';
	
	if ($aboveTableHTML) {
		$formHTMLHead .= $aboveTableHTML;
	}
	
	$formHTMLHead .= '
			<table id="content_form" border="0" class="thickbox_table">';
	
	if ($editMode) {
		$formHTMLHead .= '
				<tr id="errorDisplayRow" style="display: none;">
					<td>&nbsp;</td>
					<td colspan="'. (int) $errorDisplayColspan. '"><div id="errorDisplay"></div></td>
				</tr>';
	}
	
	return $formHTMLHead;
}


function thickboxFormFoot($editMode, $showEditButton = false) {

	$formHTMLFoot = '
				<tr>
					<td>&nbsp;</td>
					<td>';

	if ($editMode && $showEditButton) {
		$formHTMLFoot .= '
						<input type="button" id="submit" name="submit" class="submit" value="'. getPhrase('_CANCEL'). '" onclick="cancelEdit(); formHandler();" />';
	} elseif($showEditButton) {
		$formHTMLFoot .= '
						<input type="submit" id="submit" name="submit" class="submit" value="'. getPhrase('_EDIT'). '" />';
	} else {
		$formHTMLFoot .= '
						<input type="hidden" id="submit" name="submit" value="'. getPhrase('_EDIT'). '" />';
	}
	
	$formHTMLFoot .= '
					</td>
			 	</tr>
			</table>
		</form>';
	
	return $formHTMLFoot;
}





//Wrapper for thickboxForm and dynamicThickboxFormInner
function dynamicThickboxForm($aboveTableHTML, 
			$editMode, $showEditButton, $fields, $requiredFields, $valuesInCache, $valuesInDB, $columnsNotToShow = array(), $usrFormLabels = array()) {
	
	thickboxForm($formHTMLHead, $formHTMLFoot, $editMode, $showEditButton, $aboveTableHTML);

	return 
		$formHTMLHead.
		dynamicThickboxFormInner($editMode, $fields, $requiredFields, $valuesInCache, $valuesInDB, $columnsNotToShow, $usrFormLabels).
		$formHTMLFoot;
}



function buildCategoriesList(&$checkboxes, $field, &$valuesInCache, &$valuesInDB, $disabled, &$formHTML, $firstCall = true, $level = '') {
	$recurseCount++;

	if ($firstCall) {
		$formHTML.= '
			<ul class="categories_list">';
	}

	if ($disabled) {
		$disabledString = ' disabled="disabled"';
	}
	
	$userFieldArray = explode(',',getUserFieldValue($field, $valuesInCache, $valuesInDB));
	
	$i = 0;
	foreach ($checkboxes as $box) {
		++$i;
		$formHTML .= '
			<li>
			  <div class="'. ($firstCall? 'top_level' : 'sub_level'). '">';
		
		if ($box['total_children']) {
			$formHTML .=  '
				<div class="totals">
					<a href="#" onclick="tribiqTab.showHideCheckboxArrayField(this, \''. $level. $i. '\'); return false;" id="open_contents_of_box_'. $level. $i. '" class="closed">
						<span id="count_for_box_'. $level. $i. '"></span>
						/ '. $box['total_children']. '
						&nbsp;&nbsp;&nbsp;
					</a>
				</div>';
		}
		
		$formHTML .= '
				<input type="checkbox" value="'. htmlspecialchars($box['id']). '" id="box_'. $level. $i. '"
					';
		
		if (in_array($box['id'], $userFieldArray) || ($box['children_selected'] && $box['total_children'] && $box['children_selected'] == $box['total_children'])) {
			$formHTML .= ' checked="checked"';
		} else {
			$formHTML .= '';
		}
		
		if ($box['children_selected'] && $box['total_children'] && $box['children_selected'] < $box['total_children']) {
			$formHTML .= ' class="semi_on"';
		}
		
		if (issetArrayKey($box, 'disabled')) {
			$formHTML .= ' disabled="disabled"';
		}
		
		$formHTML .= '	onclick="tribiqTab.setCheckboxArrayField(this, \''. $field. '\');"' . $disabled . '' . $disabledString . ' />
			<label for="box_'. $level. $i. '">' . htmlspecialchars($box['name']). '</label>';
		
		if (isset($box['item_count'])) {
			$formHTML .= ' (' . $box['item_count'] . ')';
		}
		
		$formHTML .=  '
			  </div>';
		
		if (is_array($box['children'])) {
			if (count($box['children'])) {
				if ($box['total_children']) {
					$formHTML .= '
						<ul id="contents_of_box_'. $level. $i. '">';
				} else {
					$formHTML .= '
						<ul>';
				}
				
				if ($recurseCount<=10) {
					buildCategoriesList($box['children'], $field, $valuesInCache, $valuesInDB, $disabled, $formHTML, false, $level. $i. '_');
				}
				
				$formHTML .= '
					</ul>';
			}
		}
		
		$formHTML .= '
			</li>';
	}
	
	if ($firstCall) {
		$formHTML .= '
			</ul>';
	}
}

	
//Given a collection of fields, this function automatically draws them for use in a thickbox
function dynamicThickboxFormInner($editMode, $fields, $requiredFields, $valuesInCache, $valuesInDB, $columnsNotToShow = array(), $usrFormLabels = array()) {
	
	$sameRow = false;
	$formHTML = '';

	//Loop through each field
	foreach($fields as $field => $details) {
	  if (!in_array($field, $columnsNotToShow)) {
		
		//Fix naming convention mismatch between the users/enet_user_field_labels tables
		$fieldNameInDB = $field;
		if ($details['Table'] == 'users' && substr($field, 0, 7) == 'consent') {
			$field = 'spare_'. $field;
		}
		
		//Check if this is a spare field?
		if (substr($field, 0, 6) == 'spare_') {
			//If so, check the custom labels to see if we are using it
			if ($usrFormLabels[$field. '_label']) {
				//If so, set the title appropriately
				$title = $usrFormLabels[$field. '_label'];
			} else {
				//If not, leave it out
				continue;
			}
		
		//Look for a custom label, if there is one
		} elseif ($details['Label']) {
			$title = $details['Label'];
			
			if (substr($title, 0, 1) == '_') {
				$title = getPhrase($title);
			}
			
		//If not, use getPhrase to determine the title
		} elseif (substr($field, -3) == '_id') {
			$title = getPhrase('_'. strtoupper(substr($field, 0, -3)). ':');
		} else {
			$title = getPhrase('_'. strtoupper($field). ':');
		}
		
		$mandatoryClass = '';
		if ($editMode && in_array($field, $requiredFields)) {
			if (getUserFieldValue($field, $valuesInCache, $valuesInDB)) {
				$mandatoryClass = 'class="mandatory_correct"';
			} else {
				$mandatoryClass = 'class="mandatory"';
			}
		}
		
		
		if (isset($usrFormLabels[$field. ' row above'])) {
			$formHTML .= $usrFormLabels[$field. ' row above'];
		}
		
		
		if (!$sameRow) {
			//Some thickboxes need attributes on the rows for fields
			if (isset($usrFormLabels[$field. ' attributes'])) {
				$formHTML .= '
				<tr '. $usrFormLabels[$field. ' attributes']. '>';
			
			//Hide the "ignore warnings" row by default.
			} elseif ($field == 'ignore_warnings' && !getUserFieldValue($field, $valuesInCache, $valuesInDB)) {
				$formHTML .= '
				<tr id="ignore_warnings_row" style="display: none;">';
			
			} else {
				$formHTML .= '
				<tr>';
			}
			
			$tdId = '';
			if ($details['Type'] == 'password') {
				$tdId = 'id="password_cell"';
			} elseif ($details['Type'] == 'password_reconfirm') {
				$tdId = 'id="password_reconfirm_cell"';
			}
			
			
			//Allow some things to be shown depending on a checkbox
			$requiresOpen = '';
			$requiresClose = '';
			if ($details['Requires']) {
				$requiresOpen = '<div class="requires_'. $details['Requires'];
				
				if ($details['Requires Value']) {
					$fieldShown = false;
					foreach (explode(',', $details['Requires Value']) as $requires) {
						if (getUserFieldValue($details['Requires'], $valuesInCache, $valuesInDB) == $requires) {
							$fieldShown = true;
						}
						$requiresOpen .= ' requires_'. $details['Requires']. '__'. $requires;
					}
					
					$requiresOpen .= '"';
					if (!$fieldShown) {
						$requiresOpen .= ' style="display: none;"';
					}
				} else {
					$requiresOpen .= '"';
					if (!getUserFieldValue($details['Requires'], $valuesInCache, $valuesInDB)) {
						$requiresOpen .= ' style="display: none;"';
					}
				}
				
				$requiresOpen .= '>';
				$requiresClose = '</div>';
			}
			
			
			//Have the options for labels without any values
			if ($details['Type'] == 'description') {
				$formHTML .= '
						<td colspan="2">'. $title. '</td></tr>';
				continue;
			}
			
			$formHTML .= '
					<td class="title">'. $requiresOpen. '
						<label for="'. $field. '">'. $title. '</label>
					'. $requiresClose. '</td><td '. $tdId. ' '. $mandatoryClass. '>'. $requiresOpen;
		}
		
		//Undo the fix so our field names still match the database
		$field = $fieldNameInDB;
		
		
		//Always display "read only" fields in display mode, even if edit mode is enabled
		$rememberEditMode = $editMode;
		if ($details['Read Only']) {
			$editMode = false;
		}
		
		
		//Work out which type of field this is, and display it appropriately:
		
		//Convert some specific content types into more general ones
		
		//Handle contenttypes as a select list
		if ($details['Type'] == 'content_html') {
			$details['Type'] = 'content';
			$details['Target_cType'] = 'html';
			$details['Force_cType'] = true;
			$details['HTML_Id_Only'] = true;
			
		} elseif ($details['Type'] == 'email_template' && inc('tribiq_email_template_manager')) {
			$details['Type'] = 'sk';
			$details['Nothing Selected Phrase'] = getPhrase('No Email Template Selected');
			$details['Path'] =
			$details['Target Path'] =
			$details['Min Path'] =
			$details['Max Path'] = 'tribiq__email_template_manager';
			$details['Disallow Identical Min Path'] = true;
			
		} elseif ($details['Type'] == 'contenttype') {
			$details['Type'] = 'select';
			$details['Null'] = 'NO';
			$details['Options'] = array();
			foreach (getContentTypes() as $cType) {
				$details['Options'][$cType['content_type_id']] = htmlspecialchars($cType['content_type_name_en']);
			}
			
		} elseif ($details['Type'] == 'contenttype_with_all') {
			$details['Type'] = 'select';
			$details['Null'] = 'NO';
			$details['Options'] = array();

			$details['Options']["all"] = htmlspecialchars("All Content Types");

			foreach (getContentTypes() as $cType) {
				$details['Options'][$cType['content_type_id']] = htmlspecialchars($cType['content_type_name_en']);
			}
			
		} elseif ($details['Type'] == 'menu_section') {
			$details['Type'] = 'select';
			$details['Null'] = 'NO';
			$details['Options'] = array();
			
			$sql = "
				SELECT section_name
				FROM ". DB_NAME_PREFIX. "menu_sections
				ORDER BY section_name";
			
			$result = my_mysql_query($sql);
			while ($row = mysql_fetch_assoc($result)) {
				$details['Options'][$row['section_name']] = htmlspecialchars($row['section_name']);
			}
			
		} elseif ($details['Type'] == 'pagination') {
			$details['Type'] = 'select';
			$details['Null'] = 'NO';
			$details['Options'] = array();

			foreach (getRunningPlugins() as $plugin) {
				if (file_exists(CMS_ROOT. 'plugins/'. $plugin['name']. '/db_updates/description.xml')
				 && getPluginPaginationTypesFromXMLDescription('plugins/'. $plugin['name']. '/db_updates', 'description.xml', $paginationTypes)) {
					foreach ($paginationTypes as $type => $label) {
						$details['Options'][$type] = $label;
					}
				}
			}
			
			asort($details['Options'], SORT_STRING);
			
		} elseif ($details['Type'] == 'vlp_class') {
			$details['Type'] = 'select';
			$details['Null'] = 'YES';
			$details['Null Phrase'] = getPhrase('Core Features');
			$details['Options'] = array();

			foreach (getPlugins($onlyGetRunningPlugins = false, $ignoreUninstalledPlugins = true) as $plugin) {
				if ($plugin['vlp_class'] && $plugin['vlp_class'] == $plugin['class_name']) {
					$details['Options'][$plugin['vlp_class']] = htmlspecialchars($plugin['display_name']);
				}
			}
			
			sqlArraySort($details['Options']);
			
		} elseif ($details['Type'] == 'category') {
			$details['Type'] = 'select';
			$details['Null'] = 'YES';
			$details['Null Phrase'] = getPhrase('All Categories &amp; uncategorized');
			$details['Options'] = array();
			if (is_array($categories = getCategories())) {
				foreach ($categories as $category) {
					$details['Options'][$category['id']] = htmlspecialchars($category['name']);
				}
			}
		}
		
		if (issetArrayKey($details, 'Pre Field HTML')) {
			$formHTML .= '<div style="float: left">'. $details['Pre Field HTML']. '</div>';
		}
		
			
		//Turn enum columns into Radio Groups
		if (substr($details['Type'], 0, 4) == 'enum') {
				
			if (!is_array($details['Options'])) {
				$details['Options'] = array();
				foreach(explode("','", substr($details['Type'], 6, -2)) as $option) {
					if (is_numeric($option)) {
						$label = $option;
					
					} elseif (!$option) {
						$label = getPhrase('_NOT_SET');
					
					} else {
						$label = getPhrase($option);
					}
					
					$details['Options'][$option] = $label;
				}
			}
			
			if ($editMode) {
				
				$pickedValue = getUserFieldValue($field, $valuesInCache, $valuesInDB);
				
				$i=0;
				foreach($details['Options'] as $optionValue => $label) {
					
					$formHTML .= '
					<div style="margin-bottom: 3px;">
						<input type="radio" name="'. $field. '" id="'. $field. $i. '" value="'. htmlspecialchars($optionValue). '"
							onclick="tribiqAFB.radioChange(this);"
							'. ($optionValue == $pickedValue? 'checked="checked"' :null). '
						/>
						<label for="'. $field. $i. '">
							'. htmlspecialchars($label). '
						</label>
					</div>';
					
					++$i;
				}

			} else {
				$option = getUserFieldValue($field, $valuesInCache, $valuesInDB);
				
				$formHTML .= htmlspecialchars($details['Options'][$option]);
			}
		
		
		//Take the same format as "enum", but allow a select list
		//Or alternately allow Options to be set with values and display values as an override
		} elseif (substr($details['Type'], 0, 6) == 'select') {
			
			if (!$details['Options']) {
				$details['Options'] = array();
				foreach(explode("','", substr($details['Type'], 8, -2)) as $option) {
					
					if (is_numeric($option)) {
						$label = $option;
					
					} elseif (!$option) {
						$label = getPhrase('_NOT_SET');
					
					} else {
						$label = getPhrase($option);
					}
					
					$details['Options'][$option] = $label;
				}
			}
			
			$pickedValue = getUserFieldValue($field, $valuesInCache, $valuesInDB);
			if ($editMode) {
				
				$formHTML .= '
					<select id="'. $field. '" name="'. $field. '" class="form_field" onchange="'. ifNull($details['onchange'], 'tribiqAFB.radioChange(this, true);'). '">';
				
				if ($details['Null'] == 'YES') {
					$formHTML .= '
					<option value="" '. ($pickedValue? null: 'selected="selected"'). '>-- '.
						($details['Null Phrase']? $details['Null Phrase'] : getPhrase('_PLEASE_SELECT')).
					' --</option>';
				}
					
				foreach($details['Options'] as $value => $option) {
					
					$formHTML .= '
						<option '. ($value == $pickedValue? 'selected="selected"' :null). ' value="'. htmlspecialchars($value). '">
							'. $option. '
						</option>';
				}

				$formHTML .= '
					</select>';
			} else {
				if (issetArrayKey($details['Options'], $pickedValue)) {
					$formHTML .= $details['Options'][getUserFieldValue($field, $valuesInCache, $valuesInDB)];
				} else {
					$formHTML .= $details['Null Phrase'];
				}
			}
		
		//Default Template id select list
		} elseif ($details['Type'] == 'template_id' || $field == 'default_template_id') {
			if ($editMode) {
				$formHTML .= '
					<select id="'. $field. '" name="'. $field. '" class="form_field">
						'. getTemplateOptions(getUserFieldValue($field, $valuesInCache, $valuesInDB), arrayKey($details, 'Template_cType'), arrayKey($details, 'Template_Hide_Null_Option')). '
					</select>';
			} else {

				if ($defaultTemplateNameAndStatus = getTemplateNameAndStatus(getUserFieldValue($field, $valuesInCache, $valuesInDB))) {
					$formHTML .= htmlspecialchars($defaultTemplateNameAndStatus);
				} else {
					$formHTML .= getPhrase('Not set');
				}
			}
		
		//Skin id select list
		//Relies on the "family_name" field also being available to work correctly
		} elseif ($field == 'skin_id') {
			if ($editMode) {
				$formHTML .= '
					<select id="'. $field. '" name="'. $field. '" class="form_field" onchange="tribiqAFB.pk.skinChanged = true; tribiqAFB.pk.selectedSkinId = this.value;">
						'. getSkinOptions(getUserFieldValue('family_name', $valuesInCache, $valuesInDB), getUserFieldValue($field, $valuesInCache, $valuesInDB), false, $details['Table']). '
					</select>';
			} else {
				$formHTML .= htmlspecialchars(getSkinOptions(getUserFieldValue('family_name', $valuesInCache, $valuesInDB), getUserFieldValue($field, $valuesInCache, $valuesInDB), true, $details['Table']));
			}
			
		} elseif ($details['Type'] == 'hierarchical_categories') {
			if (!$editMode) {
				$disabled = true;
			} else {
				$formHTML .= "<input type=\"hidden\" name=\"". $field. "\" id=\"". $field. "\" value=\"". htmlspecialchars(getUserFieldValue($field, $valuesInCache, $valuesInDB)). "\"/>\n";
				$disabled = false;
			}
			
			$checkboxes = tribiq_cms_core_probusiness::getHierarchicalCategoriesArray();
			buildCategoriesList($checkboxes, $field, $valuesInCache, $valuesInDB, $disabled, $formHTML);
		
		} elseif ($details['Table'] == 'users' && strpos($field, 'country_id') !== false) {
			if (in_array(DEFAULT_LANGUAGE, array('en', 'fr', 'de', 'pt', 'es'))) {
				$countryNameInLanguage = 'name_'.DEFAULT_LANGUAGE;
			} else {
				$countryNameInLanguage = 'name_en';
			}

			if ($editMode) {
				$sql = "
					SELECT
						iso2 AS `code`,
						". $countryNameInLanguage. " AS `name`
					FROM ". DB_NAME_PREFIX . "countries
					ORDER BY 2";
				$result = my_mysql_query($sql);
	
				$formHTML .= '
					<select id="'. $field. '" name="'. $field. '" class="form_field">';
				
				if ($pickedValue) {
					$formHTML .= '
						<option value="">-- '. getPhrase('_PLEASE_SELECT') .' --</option>';
				} else {
					$formHTML .= '
						<option selected="selected" value=""></option>';
				}	
				
				while ($row = mysql_fetch_array ($result)) {
					$formHTML .= '
						<option '. ($row['code'] == getUserFieldValue($field, $valuesInCache, $valuesInDB)? 'selected="selected"' :null). ' value="'. $row['code']. '">
							'. $row['name']. '
						</option>';
				}

				$formHTML .= '
					</select>';
			} else {
				$sql = "
					SELECT
						". $countryNameInLanguage. " AS `name`
					FROM ". DB_NAME_PREFIX . "countries
					WHERE iso2 = '". mysql_real_escape_string(getUserFieldValue($field, $valuesInCache, $valuesInDB)). "'";
				
				$result = my_mysql_query($sql);
				$row = mysql_fetch_assoc($result);
				
				$formHTML .= htmlspecialchars($row['name']);
			}
		
		//Admin and Superadmin: use_language_id
		} elseif ($field == 'use_language_id' && ($details['Table'] == 'admins' || $details['Table'] == 'superadmins')) {

			if ($editMode) {
				$formHTML .= '
					<select id="'. $field. '" name="'. $field. '" class="form_field">';
				
				foreach(getAdminLanguages() as $lang) {
					$formHTML .= '
						<option '. ($lang[0] == getUserFieldValue($field, $valuesInCache, $valuesInDB)? 'selected="selected"' :null). ' value="'. htmlspecialchars($lang[0]). '">
							'. htmlspecialchars($lang[1]). '
						</option>';
				}

				$formHTML .= '
					</select>';
			} else {
				$formHTML .= htmlspecialchars(getUserFieldValue($field, $valuesInCache, $valuesInDB));
			}
	
		//Make Address Fields into textareas
		} elseif ($details['Table'] == 'users' && strpos($field, 'address') !== false && strpos($field, 'email') === false) {
			if ($editMode) {
				$formHTML .= '
					<textarea id="'. $field. '" name="'. $field. '" class="form_field" maxlength="250">'. htmlspecialchars(getUserFieldValue($field, $valuesInCache, $valuesInDB)). '</textarea>';
			} else {
				$formHTML .= nl2br(htmlspecialchars(getUserFieldValue($field, $valuesInCache, $valuesInDB)));
			}
	
		//Allow generic textareas if requested
		} elseif ($details['Type'] == 'text' || $details['Type'] == 'textarea') {
			if ($editMode) {
				$formHTML .= '
					<textarea id="'. $field. '" name="'. $field. '" class="form_field" rows="10">'. htmlspecialchars(getUserFieldValue($field, $valuesInCache, $valuesInDB)). '</textarea>';
			} else {
				$formHTML .= nl2br(htmlspecialchars(getUserFieldValue($field, $valuesInCache, $valuesInDB)));
			}
	
		//tinyMCE
		} elseif ($details['Type'] == 'tinymce' || $details['Type'] == 'tinymce_with_images' || $details['Type'] == 'ckeditor' || $details['Type'] == 'fckeditor') {
			if ($editMode) {
				$formHTML .= '
					<textarea id="'. $field. '" name="'. $field. '" class="'. ($details['Type'] == 'tinymce_with_images'? 'tinymce_with_images' : 'tinymce'). '"
					  style="width: 500px; height: 250px; visibility: hidden;">'.
						htmlspecialchars(getUserFieldValue($field, $valuesInCache, $valuesInDB)).
					'</textarea>';
				
			} else {
				$formHTML .= '
					<textarea id="'. $field. '" name="'. $field. '" class="tinymce_readonly" style="width: 500px; visibility: hidden;">'.
						htmlspecialchars(getUserFieldValue($field, $valuesInCache, $valuesInDB)).
					'</textarea>';
			}
		
		//Make Password Fields into password input types
		//I'm varying the size against the size of the column in the database
		} elseif ($details['Type'] == 'password' || $details['Type'] == 'password_reconfirm') {
			if ($editMode) {
				$formHTML .= '
					<input type="password" id="'. $field. '" name="'. $field. '" class="form_field" value="'. htmlspecialchars(getUserFieldValue($field, $valuesInCache, $valuesInDB)). '"
						style="width: 250px;" maxlength="50" onClick="tribiqA.mandatoryPasswordFields(this);" onKeyUp="tribiqA.mandatoryPasswordFields(this);" />';
			} else {
				$formHTML .= '*****';
			}
		
		//Add a hidden field
		} elseif ($details['Type'] == 'hidden') {
			if ($editMode) {
				$formHTML .= '
					<input type="hidden" id="'. $field. '" name="'. $field. '" value="'. htmlspecialchars(getUserFieldValue($field, $valuesInCache, $valuesInDB)). '"/>';
			}
		
		//Turn any other varchar field into a text box.
		//I'm varying the size against the size of the column in the database
		} elseif ($details['Type'] == 'email' || substr($details['Type'], 0, 7) == 'varchar') {
			if ($editMode) {
				$width = '';
				$maxlength = substr($details['Type'], 8, -1);
				
				if ($maxlength == 1) {
					$width = 'style="width: 2em;"';
				} elseif (issetArrayKey($details, 'Pre Field HTML')) {
					$width = 'style="width: 20em;"';
				}
				
				$formHTML .= '
					<input type="text" id="'. $field. '" name="'. $field. '" class="form_field" maxlength="'. $maxlength. '" '. $width. '
						value="'. htmlspecialchars(getUserFieldValue($field, $valuesInCache, $valuesInDB)). '"'.
						($details['ReadOnly']? ' readonly="readonly"' : '').
						($details['onclick']? ' onclick="'. $details['onclick']. '"' : '').
						($details['onkeyup']? ' onkeyup="'. $details['onkeyup']. '"' : '').
					' />';
			
			} elseif ($details['Type'] == 'email' && (validateEmailAddress($email = getUserFieldValue($field, $valuesInCache, $valuesInDB)))) {
				$formHTML .=
					'<a href="mailto:'. htmlspecialchars($email). '">'. htmlspecialchars($email). '</a>';
			
			} else {
				$formHTML .= htmlspecialchars(getUserFieldValue($field, $valuesInCache, $valuesInDB));
			}
		
		//Create a numeric-only field.
		//I'm adding some client-side validation, to make sure the user types a number, but the receiving function still needs to validate it!
		} elseif (substr($details['Type'], 0, 3) == 'int') {
			$value = getUserFieldValue($field, $valuesInCache, $valuesInDB);
			
			if ($value !== '') {
				$value = (int) $value;
			}
			
			if ($editMode) {
				$formHTML .= '
					<input type="text" id="'. $field. '" name="'. $field. '" class="form_field"  value="'. $value. '"
						onKeyUp="var n = 1*this.value.replace(/[^0-9]/g, \'\'); n = n? n : 0; if (this.value !== n && this.value !== \'\') this.value = n;" />';
			} else {
				$formHTML .= $value;
			}
		
		//Call a display function dynamically by using the syntax "function nameOfMyFunction"
			//The function called needs to take the field name as its first input, a value as its second input,
			//and whether or not this is edit mode as its third input
		} elseif (substr($details['Type'], 0, 8) == 'function') {
			
			$functionName = explode(' ',$details['Type']);
			$functionName = $functionName[1];

			$formHTML .= $functionName($field, getUserFieldValue($field, $valuesInCache, $valuesInDB), $editMode);
		
		//Call a class' method dynamically by using the syntax "function nameOfMyFunction"
			//The method called needs to take the field name as its first input, a value as its second input,
			//and whether or not this is edit mode as its third input
		} elseif (substr($details['Type'], 0, 5) == 'class') {
			
			$classAndMethod = explode(' ', $details['Type']);
			
			$instanceOfClass = new $classAndMethod[1];
			$methodInClass = $classAndMethod[2];
			
			$formHTML .= $instanceOfClass->$methodInClass($field, getUserFieldValue($field, $valuesInCache, $valuesInDB), $editMode);
			
			unset($instanceOfClass);
		
		//Turn tinyints into checkboxes.
		//I'm varying the size against the size of the column in the database
		} elseif ($details['Type'] == 'tinyint(1)' || $details['Type'] == 'tinyint') {
			if ($editMode) {
				$formHTML .= '
					<input type="checkbox" id="'. $field. '" name="'. $field. '" value="1" '. (getUserFieldValue($field, $valuesInCache, $valuesInDB)? 'checked="checked"' :null).
						' onclick="'. ifNull($details['onclick'], 'tribiqAFB.checkboxChange(this);'). '" />';
			} else {
				$formHTML .= getUserFieldValue($field, $valuesInCache, $valuesInDB)? getPhrase('_YES') : getPhrase('_NO');
			}
		
		//Pick a content item from Storekeeper
		} elseif ($details['Type'] == 'content') {
			
			//Get the name of the content item
			$filled = false;
			$cIDAndCType = '';
			$contentTag = $nothingSelectedPhrase = getPhrase('No Item selected');
			if ($cIDAndCType = getUserFieldValue($field, $valuesInCache, $valuesInDB)) {
				$cIDAndCTypeArray = explode('_', $cIDAndCType);
				
				if ($details['HTML_Id_Only']) {
					$cIDAndCTypeArray[1] = 'html';
				}
				
				if ($cIDAndCTypeArray[0] && $cIDAndCTypeArray[1]) {
					$contentTag = getTag($cIDAndCTypeArray[0], $cIDAndCTypeArray[1]);
					$filled = true;
				}
			}
			
			if ($editMode) {
				$formHTML .= '
					<input type="text" id="name_for_'. $field. '" value="'. htmlspecialchars($contentTag). '" readonly="readonly" class="form_field" />
					<br /><br />
					<input type="button" class="launch_keeper" value="'. getPhrase('_SELECT_ITEM'). '" onClick="
						tribiqA.getFromSK(\''. $field. '\', \''. $details['Type']. '\', \''. jsOnClickEscape($details['Target_cType']). '\', \''. jsOnClickEscape($details['Target_LanguageId']). '\', '. (int) ($details['Force_cType']). ', '. (int) ($details['Force_LanguageId']). ', '. (int) ($details['HTML_Id_Only']). ');
					" />
					<input type="button" class="submit" value="'. getPhrase('Remove selection'). '" onClick="tribiqA.removeNameAndId(\''. $field. '\');" />
					<input type="hidden" id="'. $field. '" name="'. $field. '" value="'. htmlspecialchars($cIDAndCType). '" />
					<input type="hidden" id="nothing_selected_phrase_for_'. $field. '" value="'. $nothingSelectedPhrase. '" />';
			
			} elseif ($filled) {
				$language = getLanguage(getContentLang($cIDAndCTypeArray[0], $cIDAndCTypeArray[1]));
				
				$formHTML .= '
					<img src="'. httpOrhttps(). $_SERVER["HTTP_HOST"] . SUBDIRECTORY. 'admin/images/flags/'. $language['flag']. '.gif" width="16" height="11" 
						alt="'. $language['flag']. '" title="'. $language['flag']. '">
					<a target="_blank" href="'. httpOrhttps(). $_SERVER["HTTP_HOST"] . SUBDIRECTORY. indexDotPHP(). '?cID='. (int) $cIDAndCTypeArray[0]. '&cType='. htmlspecialchars($cIDAndCTypeArray[1]) .'">
						'. htmlspecialchars($contentTag). '
					</a>';
				
			} else {
				$formHTML .= htmlspecialchars($contentTag);
			}
		
		
		//Pick a user id from Storekeeper
		}  elseif ($details['Type'] == 'user') {
			
			$displayName = ifNull(
				getUsername(getUserFieldValue($field, $valuesInCache, $valuesInDB)),
				$nothingSelectedPhrase = getPhrase('No User Selected'));
			
			if ($editMode) {
				$formHTML .= '
					<input type="text" id="name_for_'. $field. '" value="'. htmlspecialchars($displayName). '" readonly="readonly" class="form_field" />
					<br /><br />
					<input type="button" class="launch_keeper" value="'. getPhrase('Select User...'). '" onClick="tribiqA.getFromSK(\''. $field. '\', \''. $details['Type']. '\');" />
					<input type="button" class="submit" value="'. getPhrase('Remove selection'). '" onClick="tribiqA.removeNameAndId(\''. $field. '\');" />
					<input type="hidden" id="'. $field. '" name="'. $field. '" value="'. (int) getUserFieldValue($field, $valuesInCache, $valuesInDB). '" />
					<input type="hidden" id="nothing_selected_phrase_for_'. $field. '" value="'. $nothingSelectedPhrase. '" />';
			} else {
				$formHTML .= htmlspecialchars($displayName);
			}
		
		
		//Pick a media id from Storekeeper
		} elseif ($details['Type'] == 'image' || $details['Type'] == 'movie' || $details['Type'] == 'media') {
			
			//Get the name of the file
			if ($details['Type'] == 'image') {
				$mediaFileName = $nothingSelectedPhrase = getPhrase('No Image Selected');
			
			} elseif ($details['Type'] == 'movie') {
				$mediaFileName = $nothingSelectedPhrase = getPhrase('No Animation Selected');
			
			} elseif ($details['Type'] == 'media') {
				$mediaFileName = $nothingSelectedPhrase = getPhrase('No Media Selected');
			}
			
			if ($mediaIdAndType = getUserFieldValue($field, $valuesInCache, $valuesInDB)) {
				$mediaIdAndTypeArray = explode('_', $mediaIdAndType);
				if (($mediaIdAndTypeArray[0] == 'image' || $mediaIdAndTypeArray[0] == 'movie') && $mediaIdAndTypeArray[1]) {
					$sql = "
						SELECT filename
						FROM ". DB_NAME_PREFIX. mysql_real_escape_string($mediaIdAndTypeArray[0]). "s
						WHERE checksum_inc_name = '". mysql_real_escape_string($mediaIdAndTypeArray[1]). "'";
					$result = my_mysql_query($sql);
					if ($row = mysql_fetch_assoc($result)) {
						$mediaFileName = $row['filename'];
					}
				}
			}
			
			if ($editMode) {
				$formHTML .= '
					<input type="text" id="name_for_'. $field. '" value="'. htmlspecialchars($mediaFileName). '" readonly="readonly" class="form_field" />
					<br /><br />';
				
				if ($details['Type'] != 'movie') {
					$formHTML .= '
					<input type="button" class="launch_keeper" value="'. getPhrase('_SELECT_IMAGE'). '" onClick="tribiqA.getFromSK(\''. $field. '\', \'image\');" />';
				}
				
				if ($details['Type'] != 'image') {
					$formHTML .= '
					<input type="button" class="launch_keeper" value="'. getPhrase('_SELECT_MOVIE'). '" onClick="tribiqA.getFromSK(\''. $field. '\', \'movie\');" />';
				}
				
				$formHTML .= '
					<input type="button" class="submit" value="'. getPhrase('Remove selection'). '" onClick="tribiqA.removeNameAndId(\''. $field. '\');" />
					<input type="hidden" id="'. $field. '" name="'. $field. '" value="'. htmlspecialchars($mediaIdAndType). '" />
					<input type="hidden" id="nothing_selected_phrase_for_'. $field. '" value="'. $nothingSelectedPhrase. '" />';
			} else {
				$formHTML .= htmlspecialchars($mediaFileName);
			}
		
		//Pick a group from Storekeeper
		} elseif ($details['Type'] == 'user_group') {
			
			$displayName = ifNull(
				getGroupPrivateName(getUserFieldValue($field, $valuesInCache, $valuesInDB)),
				$nothingSelectedPhrase = getPhrase('No Group Selected'));
			
			if ($editMode) {
				$formHTML .= '
					<input type="text" id="name_for_'. $field. '" value="'. htmlspecialchars($displayName). '" readonly="readonly" class="form_field" />
					<br /><br />
					<input type="button" class="launch_keeper" value="'. getPhrase('_SELECT_GROUP'). '" onClick="tribiqA.getFromSK(\''. $field. '\', \''. $details['Type']. '\');" />
					<input type="button" class="submit" value="'. getPhrase('Remove selection'). '" onClick="tribiqA.removeNameAndId(\''. $field. '\');" />
					<input type="hidden" id="'. $field. '" name="'. $field. '" value="'. (int) getUserFieldValue($field, $valuesInCache, $valuesInDB). '" />
					<input type="hidden" id="nothing_selected_phrase_for_'. $field. '" value="'. $nothingSelectedPhrase. '" />';
			} else {
				$formHTML .= htmlspecialchars($displayName);
			}
		
		
		//Pick something generic from Storekeeper
		} elseif ($details['Type'] == 'sk') {
			
			$nothingSelectedPhrase = ifNull($details['Nothing Selected Phrase'], getPhrase('Nothing Selected'));
			
			if ($editMode) {
				$formHTML .= '
					<input type="text" id="name_for_'. $field. '" value="'.
							(getUserFieldValue($field, $valuesInCache, $valuesInDB)? '' : htmlspecialchars($nothingSelectedPhrase)).
						'" readonly="readonly" class="form_field look_value_from_sk"/>
					<br /><br />
					<input type="button" class="launch_keeper" value="'. ifNull($details['Select Phrase'], getPhrase('Select')). '" onClick="
						tribiqA.getCustomFromSK(\''. $field. '\', \''. $details['Path']. '\', \''. $details['Target Path']. '\', \''. $details['Min Path']. '\', \''. $details['Max Path']. '\', '. (int) $details['Disallow Identical Min Path']. ');
					" />
					<input type="button" class="submit" value="'. getPhrase('Remove selection'). '" onClick="tribiqA.removeNameAndId(\''. $field. '\');" />
					<input type="hidden" id="'. $field. '" name="'. $field. '" value="'. htmlspecialchars(getUserFieldValue($field, $valuesInCache, $valuesInDB)). '" />
					<input type="hidden" id="nothing_selected_phrase_for_'. $field. '" value="'. $nothingSelectedPhrase. '" />
					<input type="hidden" id="target_path_for_'. $field. '" value="'. $details['Target Path']. '" />';
			
			} elseif (getUserFieldValue($field, $valuesInCache, $valuesInDB)) {
				$formHTML .= '
					<span id="name_for_'. $field. '" class="look_value_from_sk">'.
						(getUserFieldValue($field, $valuesInCache, $valuesInDB)? '' : htmlspecialchars($nothingSelectedPhrase)).
					'</span>
					<input type="hidden" id="target_path_for_'. $field. '" value="'. $details['Target Path']. '" />
					<input type="hidden" id="'. $field. '" value="'. htmlspecialchars(getUserFieldValue($field, $valuesInCache, $valuesInDB)). '" />';
			
			} else {
				$formHTML .= htmlspecialchars($nothingSelectedPhrase);
			}
		
		//A simple date field
		} elseif ($details['Type'] == 'date') {
			if ($editMode) {
				$dateArray = explode( '-', getUserFieldValue($field, $valuesInCache, $valuesInDB));
				$dateDay = (int) $dateArray[2];
				$dateMonth = $dateArray[1];
				$dateYear = $dateArray[0];
				
				$formHTML .= '
					<select id="'. $field. '_day" class="form_field" style="width: 50px;" onChange="changeDateField(\''. $field. '\');">
						<option value="00"></option>';
					
				for ($day = 1; $day <= 31; ++$day) {
					$formHTML .= '
						<option value="' . sprintf('%02d', $day) . '" '. ($dateDay == $day? 'selected="selected"' :null). '>' . $day . '</option>';
				}
				
				$formHTML .= '
					</select>
					<select id="'. $field. '_month" class="form_field" style="width: 200px;" onChange="changeDateField(\''. $field. '\');">
						<option value="00"></option>';
				
				foreach(
				  array(
					'01' => getPhrase('_JAN'),
					'02' => getPhrase('_FEB'),
					'03' => getPhrase('_MAR'),
					'04' => getPhrase('_APR'),
					'05' => getPhrase('_MAY'),
					'06' => getPhrase('_JUN'),
					'07' => getPhrase('_JUL'),
					'08' => getPhrase('_AUG'),
					'09' => getPhrase('_SEP'),
					'10' => getPhrase('_OCT'),
					'11' => getPhrase('_NOV'),
					'12' => getPhrase('_DEC'))
				  as $month => $monthName) {
					$formHTML .= '
						<option value="' . $month . '" '. ($dateMonth == $month? 'selected="selected"' :null). '>' . $monthName . '</option>';
				}
				
				$formHTML .= '
					</select>
					<select id="'. $field. '_year" class="form_field" style="width: 100px;" onChange="changeDateField(\''. $field. '\');">
						<option value="0000"></option>';
				
				$yearEnd = date( 'Y' );
				for ($year = date( 'Y' ) - 100; $year <= $yearEnd; ++$year) {
					$formHTML .= '
						<option value="'. $year. '" '. ($dateYear == $year? 'selected="selected"' :null). '>' . $year . '</option>';
				}
			
				$formHTML .= '
					</select>
					<input type="hidden" id="'. $field. '" name="'. $field. '" value="'. getUserFieldValue($field, $valuesInCache, $valuesInDB). '" />';
			} else {
				
				$formHTML .= formatDateNicely(getUserFieldValue($field, $valuesInCache, $valuesInDB), getSiteConfig('vis_date_format_long'), '');
			}
		
		//Raw HTML, just add it to the output as-is
		} elseif ($details['Type'] == 'html') {
			$formHTML .= getUserFieldValue($field, $valuesInCache, $valuesInDB);
		
		//Anything else? Just display it.
		} else {
			$formHTML .= nl2br(htmlspecialchars(getUserFieldValue($field, $valuesInCache, $valuesInDB)));
		}
		
		if ($mandatoryClass) {
			$formHTML .= '
					<div class="icon_field" />';
		}
		
		if (issetArrayKey($details, 'Post Field HTML')) {
			$formHTML .= $details['Post Field HTML'];
		}
		
		if ($details['Note Below']) {
			$formHTML .= '
					<br />
					<i>'. $details['Note Below']. '</i>';
		}
		
		if (!$sameRow = arrayKey($details, 'Same Row')) {
			$formHTML .= '
					'. $requiresClose. '</td>
				</tr>';
		}
		
		$editMode = $rememberEditMode;
	  }
	}
	
	return $formHTML;
}


function getContentStatusIcon($cID, $cType, $cVersion, $status = false) {
	
	if ($status === false) {
		$status = getContentStatus($cID, $cType, $cVersion);
	}
	
	if ($status=="private_draft" && $cVersion>1) {
		$iconClass = "content_draft_published";
	} elseif ($status=="published") {
		$iconClass = "content_published";
	} elseif ($status=="private_draft") {
		$iconClass = "content_draft";
	} elseif ($status=="hidden") {
		$iconClass = "content_hidden";
	} elseif ($status=="archived") {
		$iconClass = "content_trashed";
	} else {
		$iconClass = "";
	}
	
	return '<div class="'. $iconClass. '"></div>';
}







function lookForAdminFloatingBoxTabs(&$tabs, $pluginId = false, $tabId = false, $pluginClassName = false, $name = false) {
	
	if (!is_array($tabs)) {
		$tabs = array();
	}
	
	if ($pluginId) {
		$plugins = array($pluginId => getPluginDetails($pluginId));
	} else {
		$plugins = getRunningPlugins("class_name = '". mysql_real_escape_string($pluginClassName). "' DESC, name");
	}
		
	foreach($plugins as $plugin) {
		if (validatePluginFromXMLDescription($plugin['rel_path']. 'db_updates', 'description.xml', $plugin['plugin_name'], $plugin['plugin_id'], $desc, $suppressErrors = true)) {
			if ($desc->admin_floating_box_tabs) {
				if ($desc->admin_floating_box_tabs->tab) {
					$i = 0;
					foreach($desc->admin_floating_box_tabs->tab as $tab) {
						$id = $plugin['plugin_id']. '_'. ++$i;
						$tabs[$id] = array();
						$tabs[$id]['priv'] = (string) $tab->priv;
						$tabs[$id]['target_class_name'] = ifNull((string) $tab->target_class_name, (string) $desc->class_name);
						$tabs[$id]['thickbox_name'] = (string) $tab->floating_box_name;
						$tabs[$id]['suppress_if_class_exists'] = (string) $tab->suppress_if_class_exists;
						
						if ($tabs[$id]['priv'] && !checkPriv($tabs[$id]['priv'])) {
							unset($tabs[$id]);
							continue;
						}
						
						if ($tabId) {
							if ($id != $tabId) {
								unset($tabs[$id]);
								continue;
							}
						}
						
						if ($pluginClassName && $name) {
							if ($tabs[$id]['target_class_name'] != $pluginClassName || $tabs[$id]['thickbox_name'] != $name) {
								unset($tabs[$id]);
								continue;
							}
						}
						
						if ($tabs[$id]['suppress_if_class_exists'] && getPluginDetails($tabs[$id]['suppress_if_class_exists'], 'class')) {
							unset($tabs[$id]);
							continue;
						}
						
						$tabs[$id]['tab_ordinal'] = $i;
						$tabs[$id]['plugin_id'] = $plugin['plugin_id'];
						$tabs[$id]['plugin_name'] = $plugin['plugin_name'];
						$tabs[$id]['class_name'] = $plugin['class_name'];
						
						$tabs[$id]['tab_label'] = (string) $tab->tab_label;
						
						if ($tab->max_height) {
							$tabs[$id]['max_height'] = (int) $tab->max_height;
						}
						
						if (engToBoolean($tab->loaded_by_default)) {
							$tabs[$id]['loaded_by_default'] = true;
						}
						
						if ($path = (string) $tab->tab_css_path) {
							$tabs[$id]['tab_css_path'] = $plugin['rel_path']. $path;
						}
						
						if ($path = (string) $tab->tab_js_path) {
							$tabs[$id]['tab_js_path'] = $plugin['rel_path']. $path;
						}
					}
				}
			}
		}
	}
}



//Display the header for a Tribiq-style Admin Thickbox
function thickboxHeader(
			$title,
			$tabs,							//An array of tabs, in name => displayName format
			$name = false,					//A name for this thickbox, so that Plugins can reference it and add tabs to it.
												//Should be prefixed by the Plugin's Class name if this is the thickbox for a Plugin
												//You can have a non-standard thickbox, in which case this can be left as "false"
													//If this is set to false, AJAX loading of tabs will not be used! (Note in this case you must handle it in a custom method)
													//No further parameters need be entered in this case
			$primaryKeys = false,			//A primary key (or an array containing a multiple primary keys)
			$parentJSFile = false,			//JavaScript library for the entire thickbox
			$useJSONForDisplay = false,		//Will AJAX calls return using JSON when displaying? (Most don't, so this can usually be ignored)
			$firstTab = false,				//The first tab to display. Will default to the first tab in $tabs if set to false
			$fins = false,					//Feature for Pro, ignore for now
			$finDropdowns = false,			//Feature for Pro, ignore for now
			$pluginClassName = 'core',		//The name of the plugin class calling this thickbox. Core should be used for CMS Core thickboxes
			$primaryKeyValues = false,		//An array of values that make up the primary key, should you not wish them to be drawn automatically from the $_REQUEST
			$confirmFile = false
) {
	
	$debug = '';
	$firstPK = false;
	
	//If only one primary key is set, allow it to be passed as a string as well as an array
	if (!is_array($primaryKeys)) {
		if ($primaryKeys) {
			$primaryKeys = array($primaryKeys);
		} else {
			$primaryKeys = array();
		}
	}
	
	if (!is_array($primaryKeyValues)) {
		$primaryKeyValues = array();
		
		foreach ($primaryKeys as $i => $key) {
			if (!$firstPK) {
				$firstPK = $key;
			}
			
			$primaryKeyValues[$key] = $_REQUEST[$key];
		}
	}
	
	//Little work around to allow floating boxes to be opened without knowing the name of the primary key, but knowing the value
	if (!isset($_REQUEST[$firstPK]) && !isset($primaryKeyValues['id']) && isset($_REQUEST['id'])) {
		$primaryKeyValues[$firstPK] = $_REQUEST['id'];
	}
		
	//Load in information on Plugin Tabs
	if ($name) {
		lookForAdminFloatingBoxTabs($tabs, $pluginId = false, $tabId = false, $pluginClassName, $name);
	}
	
	//Logic for the new style admin floating boxes
	if (count($tabs)) {

		foreach($tabs as $tabId => &$tab) {
			//Check the tab descriptions are in array-format, and convert them if not
			if (!is_array($tab)) {
				$tab = array('tab_label' => $tab);
			}
			
			//Convert a few of the old thickbox definitions to the new style needed for floating boxes
			if (!$tab['plugin_id']) {
				if (!$tab['tab_label'] && $tab['text']) {
					$tab['tab_label'] = $tab['text'];
				}
				
				if (file_exists(CMS_ROOT. 'tb/js/tb_tab_prog_'. $tabId. '.js')) {
					$tab['tab_js_path'] = 'tb/js/tb_tab_prog_'. $tabId. '.js';
				}
			}
		}
		
		$details = array('fbName' => $name, 'title' => $title, 'pk' => $primaryKeyValues, 'tabs' => $tabs);
		
		if ($parentJSFile && file_exists(CMS_ROOT. 'tb/js/tb_parent_prog_'. $parentJSFile. '.js')) {
			$details['parent_js_path'] = 'tb/js/tb_parent_prog_'. $parentJSFile. '.js';
		}
		
		if ($confirmFile && file_exists(CMS_ROOT. 'tb/confirm/'. $confirmFile. '.php')) {
			$details['confirm_path'] = 'tb/confirm/'. $confirmFile. '.php';
		}
		
		if ($debug) {
			$details['debug'] = $debug;
		}
		
		echo json_encode($details);
		//add maxHeight...
	} else {
		echo json_encode(false);
	}
	
	exit;
}








//No longer used functions for the old thickboxes
function thickboxNonStandardFooter($buttons = '&nbsp;') {}
function thickboxFooter() {}



?>