<?php
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;

if (!$siteConfig && !$welcome) {
	$siteConfig = loadSiteConfig();
}

$gzf = issetArrayKey($siteConfig, 'compress_web_pages')? '?gz=1' : '?gz=0';
$gz = issetArrayKey($siteConfig, 'compress_web_pages')? '&amp;gz=1' : '&amp;gz=0';

//Work out what's on this page, which wrappers we need to include, and which Plugin/Themes need to be requested

//Send the id of each plugin on the page to a JavaScript wrapper to include their JavaScript
//Send the plugin name and theme directory of each
$pluginThemes = '';
if (is_array($slotContents) && count($slotContents)) {
	$comma = '';
	$comma2 = '';
	$JavaScriptOnPage = array();
	$themesOnPage = array();
	
	foreach($slotContents as &$instance) {
		
		if (issetArrayKey($instance, 'class')) {
			if (!issetArrayKey($JavaScriptOnPage, $instance['class_name'])) {
				$JavaScriptOnPage[$instance['class_name']] = true;
				cms_core::$pluginJS .= $comma. $instance['plugin_id'];
				$comma = ',';
			}
			
			//Check if a theme has been set
			if ($instance['theme']) {
				//Check if this theme has a stylesheet, and include it if so
				if (($path = $instance['class']->cmsPluginApiLinkGetThemeDir()) && (!issetArrayKey($themesOnPage, $path))) {
					$themesOnPage[$path] = true;
					
					$instance['class']->cmsPluginApiLinkGetThemeParams($frameworkPluginName, $frameworkPluginClassName, $themePluginName);
					
					$pluginThemes .= $comma2. $themePluginName. ','. $instance['theme'];
					$comma2 = ',';
				}
			}
		}
	}
}


//Add the site's favicon
if ($welcome) {
	$faviconUrl = 'favicon.ico';
} elseif ($checksum = getRow('favicons', 'checksum', array('`usage`' => 'favicon'))) {
	$faviconUrl = 'cmsincludes/favicon.php?usage=favicon&checksum='. $checksum;
} elseif (file_exists(CMS_ROOT. 'favicon_'. SITENAME. '.ico')) {
	$faviconUrl = 'favicon_'. SITENAME. '.ico';
} else {
	$faviconUrl = 'favicon.ico';
}

echo '
<link rel="shortcut icon" href="', htmlspecialchars($faviconUrl), '"/>';

//Add the site's mobile icon
if (!$welcome && ($checksum = getRow('favicons', 'checksum', array('`usage`' => 'mobile')))) {
	$faviconUrl = 'cmsincludes/favicon.php?usage=mobile&checksum='. $checksum;
}

echo '
<link rel="apple-touch-icon-precomposed" href="', htmlspecialchars($faviconUrl), '"/>';



//Add CSS needed for the CMS
echo '
<link rel="stylesheet" type="text/css" media="screen" href="', $prefix, 'cmsstyles/inc.css.php?v=', urlencode(CSS_JS_VERSION), $gz, '"/>';

//Add CSS needed for the CMS in Admin mode
if ($welcome || checkPriv()) {
	echo '
<link rel="stylesheet" type="text/css" media="screen" href="', $prefix, 'cmsstyles/inc-admin.css.php?v=', urlencode(CSS_JS_VERSION), $gz, '"/>
<link rel="stylesheet" type="text/css" media="print" href="', $prefix, 'cmsstyles/print.min.css"/>';
}

//Add the CSS for a skin, if there is a skin
if ($skinId) {
	echo '
<link rel="stylesheet" type="text/css" media="screen" href="', $prefix, 'cmsstyles/skin.css.php?v=', urlencode(TRIBIQ_CMS_VERSION), '&amp;id=', (int) $skinId, $gz, '"/>
<link rel="stylesheet" type="text/css" media="print" href="', $prefix, 'cmsstyles/skin.css.php?v=', urlencode(TRIBIQ_CMS_VERSION), '&amp;id=', (int) $skinId, '&amp;print=1', $gz, '"/>';
}

//Add CSS needed for Plugins
if ($pluginThemes) {
	echo '
<link rel="stylesheet" type="text/css" media="screen" href="', $prefix, 'cmsstyles/plugin.css.php?v=', urlencode(CSS_JS_VERSION), '&amp;t=', $pluginThemes, $gz, '"/>';
}

//Are there Plugins on this page..?
if (is_array($slotContents) && !empty($slotContents)) {
	//Include the Head for any plugin instances on the page, if they have one
	foreach($slotContents as &$instance) {
		if (issetArrayKey($instance, 'class')) {
			$instance['class']->addToPageHead();
		}
	}
}

echo '
<script type="text/javascript">
	var RecaptchaOptions = {
		lang: "', addslashes(htmlspecialchars(substr($_SESSION['user_lang'], 0, 2))), '",
		theme: "', addslashes(htmlspecialchars(arrayKey($siteConfig, 'google_recaptcha_theme'))), '"};
</script>';

?>