<?php
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;

if (!$this->frameworkLoaded) {
	$this->cmsPluginApiLinkLoadFramework();
}

//Check the theme and theme section exist
if (!$this->framework) {
	echo 'This plugin requires a framework, but no framework was set. ';
	return false;
} elseif (!$this->frameworkData) {
	echo 'The &quot;', htmlspecialchars($this->framework), '&quot; framework was not loaded successfully. ';
	return false;
} elseif (!is_array($this->frameworkData[$section])) {
	echo 'Section &quot;', htmlspecialchars($section), '&quot; was not found in the &quot;', htmlspecialchars($this->framework), '&quot; framework. ';
	return false;
}

//Check if we have an array or an array of arrays
$arrayOfArrays = false;
foreach($mergeFields as $mergeFieldsRow) {
	$arrayOfArrays = is_array($mergeFieldsRow);
	break;
}

//Allow an array of arrays to be passed as mergeFields. If this happens, then we'll
//display the template once for each mergeField.
if (!$arrayOfArrays) {
	//Otherwise just display the template once by turning the array into an
	//array of a single array
	$mergeFields = array($mergeFields);
}


foreach($mergeFields as &$mergeFieldsRow) {
	
	//Start off display things by default, unless we should only display the "second half" of the section
	$displayThings = $half != 2;
	$conditionMet = true;
	
	//Loop through everything in the section
	foreach ($this->frameworkData[$section] as &$sectionContents) {
		foreach ($sectionContents as $type => &$thing) {
			
			if ($type == CMS_FRAMEWORK_SECTION) {
				//If we run into a section, call framework() to display it, but only if it is in a list of
				//sections that we should display
				if ($subSectionDepthLimit && $halfwayPoint !== $thing && issetArrayKey($allowedSubSections, $thing)) {
					if ($displayThings && $conditionMet) {
						if (is_array($allowedSubSections[$thing])) {
							$this->cmsPluginApiLinkFramework(
									$thing, $allowedSubSections[$thing],
									$allowedSubSections, $subSectionDepthLimit-1,
									false, false);
						} else {
								$this->cmsPluginApiLinkFramework(
									$thing, $mergeFieldsRow,
									$allowedSubSections, $subSectionDepthLimit-1,
									false, false);
						}
					}
				
				//If we run into a section we're not allowed to display, don't display it.
				//However follow some special logic...
				} else {
					//Check to see if this is the halfway point
					if ($halfwayPoint === true || $halfwayPoint == $thing) {
						//If half is set to one, then as soon as we get to a section that we should stop on,
						//do not display *anything* after
						if ($half == 1) {
							$displayThings = false;
						
						//And also include the opposite logic, and do not *start* displaying things until we
						//get to the stop point
						} elseif ($half == 2) {
							$displayThings = true;
						}
					}
				}
				
				continue;
			
			//Conditions: show and hide things depending on the value of a field
			} elseif ($type == CMS_FRAMEWORK_CONDITION) {
				if (!issetArrayKey($thing, 'field')) {
					//If a field name is not set, clear any conditions
					$conditionMet = true;
				
				} else {
					//Otherwise, attempt to look up the value of that field
					$value = null;
					$attributes = arrayKey($this->fieldInfo, $section, $thing['field']);
					$this->cmsPluginApiLinkFrameworkField($mergeFieldsRow, false, ifNull($attributes, array('name' => $thing['field'])), $value, false);
					
					if (!isset($thing['value'])) {
						//If no specific value is mentioned, only check if the value looks set
						$conditionMet = engToBoolean($value);
					} else {
						//Otherwise, check that the value is exactly equal
						$conditionMet = $value == $thing['value'];
					}
				}
			}
			
			if (!$displayThings || !$conditionMet) {
				continue;
			}
			
			//In admin mode, keep track of whether there has been any output
			if (checkPriv() && !$this->frameworkOutputted) {
				if ($type == CMS_FRAMEWORK_MERGE) {
					$this->frameworkOutputted = (bool) trim(arrayKey($mergeFieldsRow, $thing));
				} elseif ($type == CMS_FRAMEWORK_INCLUDE) {
					$this->frameworkOutputted = !empty($thing);
				} else {
					$this->frameworkOutputted = (bool) trim($thing);
				}
			}
			
			//Display html as-is
			if ($type == CMS_FRAMEWORK_HTML) {
				echo $thing;
			
			//Use a call to getVLPPhrase to display phrases
			} elseif ($type == CMS_FRAMEWORK_PHRASE) {
				echo $this->phrase($thing);
			
			//If we run into a named mergeField, check the $mergeFields array for the HTML we need to display it
			} elseif ($type == CMS_FRAMEWORK_MERGE) {
				if ($thing == 'ODD_OR_EVEN') {
					//Special case - return 'odd' then 'even' alternately for a mergeField named ODD_OR_EVEN
					echo $this->cmsPluginApiLinkOddOrEven();
				
				} elseif ($thing == 'LAST_ODD_OR_EVEN') {
					//Special case - return the last value of "LAST_ODD_OR_EVEN" used
					echo $this->cmsPluginApiLinkOddOrEven(false);
				
				} elseif ($thing == 'FIRST') {
					//Special case - return the last value of "LAST_ODD_OR_EVEN" used
					echo $this->cmsPluginApiLinkFirst();
				
				} elseif ($thing == 'PATH') {
					//Special case - treat a mergeField named PATH as the path to the framework
					echo $this->frameworkPath;
				
				} else {
					//Otherwise display the string from the mergeFields array.
					//(Note that this may be null or unset, but I'm not treating that as an error)
					echo arrayKey($mergeFieldsRow, $thing);
				}
			
			//Display some text from another Plugin
			} elseif ($type == CMS_FRAMEWORK_INCLUDE) {
				if (inc($thing['plugin'])) {
					if (engToBoolean(arrayKey($thing, 'static'))) {
						echo call_user_func(array($thing['plugin'], $thing['method']), $mergeFieldsRow);
					} else {
						$plugin = new $thing['plugin'];
						$plugin->setInstanceVariables(array(
							$this->cID, $this->cType, $this->cVersion, $this->skinId, $this->slotName,
							$this->instanceName, $this->instanceId,
							$this->pluginName, $this->pluginClassName,
							$this->pluginClassNameForPhrases,
							$this->pluginId,
							$this->defaultFramework, $this->framework,
							$this->defaultTheme, $this->themeStyle,
							$this->slotLevel),
							$this->eggId, $this->tabId);
						echo $plugin->$thing['method']($mergeFieldsRow);
					}
				}
			
			//Display an input field
			} elseif ($type == CMS_FRAMEWORK_INPUT) {
				//Get the attriubtes
				$attributes = $this->fieldInfo[$section][$thing];
				$value = null;
				$type = arrayKey($attributes, 'type');
				$readOnly = engToBooleanArray($attributes, 'read_only');
				
					
				//Check for a list of values, if one has been set
				$lov = false;
				$this->cmsPluginApiLinkFrameworkLOV($type, $attributes, $lov);
				
				//Draw the field
				if (($type == 'checkbox' && !is_array($lov))
				 || $type == 'date'
				 || $type == 'file'
				 || $type == 'hidden'
				 || $type == 'password'
				 || ($type == 'select' && !$readOnly)
				 || $type == 'submit'
				 || $type == 'text'
				 || $type == 'textarea'
				 || $type == 'toggle') {
					$this->cmsPluginApiLinkFrameworkField($mergeFieldsRow, $type, $attributes, $value, $readOnly);
				}
				
				if (!$readOnly && $type == 'select' && issetArrayKey($attributes, 'null')) {
					echo '
						<option value="">',
							htmlspecialchars($this->phrase($attributes['null'])),
						'</option>'; 
				}
				
				//Draw each value from the LOV
				if (is_array($lov)) {
					$i = 0;
					foreach ($lov as $saveVal => &$dispVal) {
						if ($type == 'select' && !$readOnly) {
							echo '
								<option value="', htmlspecialchars($saveVal), '"', $saveVal == $value? ' selected="selected"' : '', '>',
									htmlspecialchars($dispVal),
								'</option>'; 
						} else {
							$this->cmsPluginApiLinkFrameworkField($mergeFieldsRow, $type, $attributes, $value, $readOnly, ++$i, $saveVal, $dispVal);
						}
					}
					
					//Toggles will need a hidden field to could how many there were
					if ($lov && ($type == 'toggle')) {
						echo '
							<input type="hidden" name="', htmlspecialchars($attributes['name']), '__n" value="', $i, '">'; 
					}
				}
				
				//Close the select tag
				if ($type == 'select') {
					echo '
						</select>';
				}
				
				unset($attributes);
				unset($value);
				unset($lov);
			}
		}
	}
}

return false;
?>