/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */


//Create an encapsulated object for the Admin Floating Box functions
window.tribiqSK = function() {};

tribiqSK.pageSize = 50;
tribiqSK.paginationMaxLength = 70;
tribiqSK.periodicRefreshTime = 120000;

tribiqSK.itemDoubleClickTime = 500;
tribiqSK.searchDelayTime = 700;





//Go to the content panel by default
tribiqSK.defaultPath = 'tribiq__content';
tribiqSK.defaultPathInSelectMode = 'tribiq__overview/blank_location';

//Start up the JavaScript engine for the Storekeeper as soon as the page has loaded
tribiqSK.init = function() {
	
	tribiqSK.pageTitle = document.title;
	
	if (tribiqA.siteSettings.storekeeper_page_size
	 && tribiqA.siteSettings.storekeeper_page_size == 1*tribiqA.siteSettings.storekeeper_page_size) {
		tribiqSK.pageSize = 1*tribiqA.siteSettings.storekeeper_page_size;
	}
	
	if (tribiqA.siteSettings.storekeeper_periodic_refresh_mins
	 && tribiqA.siteSettings.storekeeper_periodic_refresh_mins == 1*tribiqA.siteSettings.storekeeper_periodic_refresh_mins) {
		tribiqSK.periodicRefreshTime = 60000 * tribiqA.siteSettings.storekeeper_periodic_refresh_mins;
	}
	
	
	if (tribiqSK.selectMode) {
		tribiqSK.showLeftColumn = false;
	} else {
		tribiqSK.showLeftColumn = true;
	}
	
	//Set the search box to the default search Phrase
	tribiqSK.setSearch(undefined);
	
	//Load the map
	try {
		//If this is a select mode window launched from storekeeper, try to copy the map off of the launching storekeeper
		if (	   tribiqSK.selectMode
				&& self.parent
				&& self.parent.tribiqSK
				&& self.parent.tribiqSK.map
				&& self.parent.tribiqSK.pluginList === tribiqSK.pluginList
				&& self.parent.tribiqCSSJSVersionNumber === window.tribiqCSSJSVersionNumber
		) {
			tribiqSK.map = self.parent.tribiqSK.map;
			tribiqSK.init2();
			return;
		
		//Check if this window had an opener, which already had Storekeeper loaded
		} else if (window.opener
				&& window.opener.tribiqSK
				&& window.opener.tribiqSK.map
				&& window.opener.tribiqSK.pluginList === tribiqSK.pluginList
				&& window.opener.tribiqCSSJSVersionNumber === window.tribiqCSSJSVersionNumber
		) {
			tribiqSK.map = window.opener.tribiqSK.map;
			tribiqSK.init2();
			return;
		
		//Check if this window had an opener, which already had Storekeeper preloaded
		} else if (window.opener
				&& window.opener.get
				&& window.opener.get('tribiq_sk_iframe')
				&& window.opener.get('tribiq_sk_iframe').contentWindow
				&& window.opener.get('tribiq_sk_iframe').contentWindow.tribiqSK
				&& window.opener.get('tribiq_sk_iframe').contentWindow.tribiqSK.map
				&& window.opener.get('tribiq_sk_iframe').contentWindow.tribiqSK.pluginList === tribiqSK.pluginList
				&& window.opener.get('tribiq_sk_iframe').contentWindow.tribiqCSSJSVersionNumber === window.tribiqCSSJSVersionNumber
		) {
			tribiqSK.map = window.opener.get('tribiq_sk_iframe').contentWindow.tribiqSK.map;
			tribiqSK.init2();
			return;
		}
	} catch (e) {
		//Run the code below if something went wrong
	}
	
	//Otherwise launch an AJAX request to get the map
	$.get(URLBasePath + 'admin/ajax.php?_json=1', function(data) {
		try {
			data = JSON.parse(data);
			
			if (typeof data != 'object') {
				throw 0;
			}
		} catch (e) {
			tribiqA.showMessage(data);
			return;
		}
		
		tribiqSK.map = data;
		tribiqSK.init2();
	}, 'text');
}

tribiqSK.init2 = function() {
	tribiqSK.lookForBranches(tribiqSK.map);
	
	tribiq.tooltips();
	
	
	tribiqSK.createSortArrayTopLevel('sortedTopLevelItems');
	tribiqSK.setTopLevelItems();
	
	
	if (tribiqSK.checkQueue()) {
		return;
	
	} else if (!tribiqSK.selectMode) {
		tribiqSK.go();
	}
}

$(document).ready(tribiqSK.init);


tribiqSK.savePrefs = function() {
	$.post(
		URLBasePath + 'admin/ajax.php',
		{ prefs: JSON.stringify(tribiqSK.prefs) },
		function(message) {
			if (message) {
				tribiqA.showMessage(message);
			}
		}, 'text');
}


tribiqSK.shortenPath = function(path) {
	if (path) {
		path = ('/' + path + '/').replace(
			/\/panel\//g, '/').replace(
			/\/second_level_items\//g, '/').replace(
			/\/collection_buttons\//g, '/').replace(
			/\/item_buttons\//g, '/').replace(
			/\/waypoints\//g, '/').replace(
			/\/items\//g, '/');
		
		return path.substr(1, path.length-2);
	} else {
		return path;
	}
}


tribiqSK.implodeKeys = function(input) {
	var output = '';
	
	for (i in input) {
		output += (output? ',' : '') + i;
	}
	
	return output;
}




//Go to a series of locations in turn
tribiqSK.checkQueue = function() {
	var len;
	if (tribiqSK.checkQueueNotEmpty()) {
		var ids = '';
		var queued = window.tribiqSKQueue.shift();
		
		if (queued.selectedItems) {
			ids = tribiqSK.implodeKeys(queued.selectedItems);
			
			tribiqSK.selectedItems = queued.selectedItems;
			tribiqSK.saveSelection(true);
		}
			
		if (queued.refinerName) {
			queued.refiner = {name: queued.refinerName, id: ids};
		}
		
		tribiqSK.go(queued.path, queued.branch, queued.refiner, true, !--len);
		
		return true;
	} else {
		return false;
	}
}

tribiqSK.checkQueueNotEmpty = function() {
	return window.tribiqSKQueue && (len = window.tribiqSKQueue.length);
}


//Go to a location
tribiqSK.goNum = 0;
tribiqSK.lastSuccessfulGoNum = 0;
tribiqSK.go = function(path, branch, refiner, queued, lastInQueue, backwards) {
	if (tribiqSK.uploading) {
		return false;
	}
	
	//If the map has not yet loaded, wait until it has!
	if (!tribiqSK.map) {
		if (!tribiqSK.selectMode) {
			window.tribiqSKQueue = [{path: path, branch: -1}];
		}
		return;
	}
	
	if (!queued) {
		window.tribiqSKQueue = false;
	}
	
	var closeUpViewItem, inCloseUpView, specificItemRequested, specificItemsRequested, selectItem;
	
	if (path) {
		var pathAndItem = path.split('//');
		var len = pathAndItem.length;
		
		//Is there any deeplinking in the path?
		if (len > 2) {
			var lastLink;
			for (var p = 0; p < len-1; p += 2) {
				
				//Navigate to the start of the branch
				var link;
				if (p == 0) {
					link = tribiqSK.getFromToFromLink(pathAndItem[0]);
					window.tribiqSKQueue = [{path: link.from, branch: -1}];
				
				} else {
					var combinedPath;
					
					if (pathAndItem[p].substr(0, 1) == '/') {
						combinedPath = pathAndItem[p].substr(1);
					} else {
						combinedPath = lastLink.to + '/' + pathAndItem[p];
					}
					
					link = tribiqSK.getFromToFromLink(combinedPath);
					
					if (lastLink.to != link.from) {
						window.tribiqSKQueue.push({path: link.from});
					}
				}
				
				//Select any items 
				var selectedItems = {};
				if (pathAndItem[p+1]) {
					var items = pathAndItem[p+1].split(',');
					for (i in items) {
						selectedItems[items[i]] = true;
					}
				}
				
				//Look for any refiners
				var refinerName = false;
				if (link.refiner) {
					refinerName = link.refiner;
				}
				
				//Select an item on the last panel, if asked for
				var linkTo = link.to;
				if (p+2 == len-1) {
					linkTo += '//' + pathAndItem[p+2];
				}
				
				//Do the branch
				window.tribiqSKQueue.push({path: linkTo, branch: true, selectedItems: selectedItems, refinerName: refinerName});
				
				lastLink = link;
			}
			
			tribiqSK.checkQueue();
			return;
		
		//Is an item requested in the path?
		} else if (len == 2) {
			path = pathAndItem[0];
			
			if (pathAndItem[1].indexOf(',') != -1) {
				specificItemsRequested = pathAndItem[1];
				inCloseUpView = false;
				selectItem = false;
				
			} else if (closeUpViewItem = pathAndItem[1]) {
				var len = closeUpViewItem.length - 1;
				if (closeUpViewItem.substr(len) == '/') {
					closeUpViewItem = closeUpViewItem.substr(0, len);
					selectItem = false;
				} else {
					selectItem = true;
				}
				
				specificItemRequested = true;
				inCloseUpView = true;
			} else {
				inCloseUpView = false;
				selectItem = false;
			}
		} else {
			inCloseUpView = false;
			selectItem = false;
		}
		
		path = tribiqSK.shortenPath(path);
	}
	
	//Check if a path has been given, and that this path exists, and don't go there if not!
	if (!path || !tribiqSK.followPathOnMap(path)) {
		//If a path has not been set, go to the initial panel
		path = tribiqSK.shortenPath(tribiqSK.selectMode? tribiqSK.defaultPathInSelectMode : tribiqSK.defaultPath);
		inCloseUpView = false;
		selectItem = false;
	}
	
	//Number each request that is made, so we can tell which ones are outdated
	var goNum = ++tribiqSK.goNum;
	
	
	var filtersSet = false;
	var filters = $.extend(true, {}, tribiqSK.loadFromBranches(path, branch, 'filters'));
	var searchTerm = tribiqSK.loadFromBranches(path, branch, 'searches');
	
	if (refiner === undefined) {
		refiner = tribiqSK.loadRefiner(path, branch);
	}
	
	//If this panel requires a refiner but none has been set, don't attempt to show it and go to the panel above instead.
	if (!refiner && engToBoolean(tribiqSK.followPathOnMap(path, 'refiner_required'))) {
		tribiqSK.go(tribiqSK.followPathOnMap(path, 'back_link'));
		return;
	}
	
	
	//Attempt to find the refiner used on the panel before this one, if there was one
	var lastRefiners = {};
	var branches = tribiqSK.branches.length - 1;
	
	//If we're resetting all branches, there will be no previous refiner
	if (branch !== -1) {
		
		//Get the refiners on each of the branches that we've previously followed
		for (b = 0; b < branches; ++b) {
			var rememberRefiner = tribiqSK.branches[b].refiners[tribiqSK.branches[b+1].from];
			if (rememberRefiner) {
				lastRefiners['refiner__' + rememberRefiner.name] = rememberRefiner.id;
			}
		}
		
		//If we're branching with this link, get the refiner on the current panel
		if (branch) {
			var rememberRefiner = tribiqSK.branches[branches].refiners[tribiqSK.path];
			if (rememberRefiner) {
				lastRefiners['refiner__' + rememberRefiner.name] = rememberRefiner.id;
			}
		}
		
		//Add the current refiner, if there is one
		if (refiner) {
			lastRefiners['refiner__' + refiner.name] = refiner.id;
		}
	}
	
	
	var labelTag = tribiqSK.followPathOnMap(path, 'label_tag');
	if (!labelTag) {
		labelTag = 'name';
	}
	
	//Create a list of which columns are to be shown.
	var columns;
	var firstColumn = false;
	var columnsShown = false;
	var shownColumns = {};
	
	if (tribiqSK.prefs[path] && tribiqSK.prefs[path].shownColumns) {
		shownColumns = tribiqSK.prefs[path].shownColumns;
	}
	
	if (columns = tribiqSK.followPathOnMap(path, 'columns')) {
		for (c in columns) {
			if (!firstColumn) {
				firstColumn = c;
			}
			
			if (engToBoolean(columns[c].always_show)) {
				columnsShown = shownColumns[c] = true;
			} else if (shownColumns[c] !== undefined) {
				if (shownColumns[c]) {
					columnsShown = true;
				}
			} else {
				if (engToBoolean(columns[c].show_by_default)) {
					columnsShown = shownColumns[c] = true;
				}
			}
		}
	}
	//Show the first column by default, if all others are turned off
	if (!columnsShown) {
		shownColumns[ifNull(firstColumn, labelTag)] = true;
	}
	
	//Check if we need to check with the server for updates/changes to this panel using php
	var php = engToBoolean(tribiqSK.followPathOnMap(path, 'php'));
	var db_items = tribiqSK.followPathOnMap(path, 'db_items');
	var server_side = db_items && !engToBoolean(db_items.client_side);
	var thisPageSize = tribiqSK.pageSize;
	
	if (php || db_items) {
		var requests = {};
		var url = URLBasePath + 'admin/ajax.php?_json=1&path=' + path;
		
		if (tribiqSK.selectMode) {
			url += '&_select_mode=1';
		}
		
		if (queued && !lastInQueue) {
			requests._queued = 1;
		}
		
		if (refiner !== undefined) {
			requests.refinerId = refiner.id;
			requests.refinerName = refiner.name;
		}
		
		for (f in lastRefiners) {
			requests[f] = lastRefiners[f];
		}
		
		if (reorder && reorder.column) {
			thisPageSize = 1000;
		}
		
		if (server_side || tribiqSK.CSVExport) {
			requests._limit = thisPageSize;
			
			if (tribiqSK.refreshToPage) {
				requests._start = (tribiqSK.refreshToPage-1) * thisPageSize;
			} else {
				requests._start = 0;
				var requestItem;
				
				if (specificItemsRequested) {
					requests._item = specificItemsRequested;
				
				} else if (specificItemRequested) {
					requests._item = closeUpViewItem;
				
				} else if (backwards) {
					if (requestItem = tribiqSK.getSelectedItemFromLastPanel(path)) {
						requests._item = requestItem;
					}
				}
			}
			
			//Send the values of any filters
			if (filters !== undefined) {
				//Don't apply hidden filters, or any empty filters
				for (var c in filters) {
					var thisHasFilters = false;
					
					for (var filter in tribiqSK.possibleFilters) {
						if (filters[c][filter]) {
							thisHasFilters = true;
						}
					}
					
					if (filters[c].shown && thisHasFilters) {
						filtersSet = true;
					} else {
						delete filters[c];
					}
				}
				
				requests._filters = JSON.stringify(filters);
			}
			
			if (searchTerm !== undefined) {
				requests._search = searchTerm;
			}
			
			//Work out which column to sort on
			var reorder = tribiqSK.followPathOnMap(path, 'reorder');
			if (reorder && reorder.column) {
				requests._sort_col = reorder.column;
			} else if (tribiqSK.prefs[path] && tribiqSK.prefs[path].sortBy) {
				requests._sort_col = tribiqSK.prefs[path].sortBy;
				requests._sort_desc = tribiqSK.prefs[path].sortDesc? 1 : 0;
			} else {
				requests._sort_col = labelTag;
			}
			
			//CSV export ability:
			//Rather than send data to the server and display it in the client,
			//send data to the server and download it as a CSV file.
			if (tribiqSK.CSVExport) {
				tribiqSK.CSVExport = undefined;
				tribiqSK.refreshToPage = undefined;
				
				delete requests._start;
				delete requests._limit;
				
				var shownColumnsInCSV = shownColumns;
				if (tribiqSK.prefs[tribiqSK.path].shownColumnsInCSV) {
					for (var c in tribiqSK.prefs[tribiqSK.path].shownColumnsInCSV) {
						shownColumnsInCSV[c] = tribiqSK.prefs[tribiqSK.path].shownColumnsInCSV[c];
					}
				}
				
				requests._shownColumnsInCSV = JSON.stringify(shownColumnsInCSV);
				requests._sortedColumns = JSON.stringify(ifNull(tribiqSK.prefs[tribiqSK.path].sortedColumns, tribiqSK.sortedColumns));
				
				document.location.href = url + tribiq.urlRequest(requests) + '&_csv=1';
				return;
			}
		}
		
		//Disable the current panel/page from refreshing
		tribiqSK.stopRefreshing();
		
		//Get an update on this panel from the server, and don't continue until we recieve a response
		get('preloader_circle').style.visibility = 'visible';
		$.get(url, requests, function(merge) {
			try {
				merge = JSON.parse(merge);
				
				if (typeof merge != 'object') {
					throw 0;
				}
			} catch (e) {
				tribiqA.showMessage(merge);
				return;
			}
			
			tribiqSK.go2(path, branch, goNum, labelTag, thisPageSize, specificItemRequested, inCloseUpView, closeUpViewItem, selectItem, searchTerm, filtersSet, refiner, lastRefiners, shownColumns, server_side, backwards, merge);
		}, 'text');
	} else {
		//Otherwise just display the panel using the client's map
		tribiqSK.go2(path, branch, goNum, labelTag, thisPageSize, specificItemRequested, inCloseUpView, closeUpViewItem, selectItem, searchTerm, filtersSet, refiner, lastRefiners, shownColumns, server_side, backwards);
	}
}

//Tidy these long params up into an object..?

//Part 2 of the go function, run when we've got the data back from the server
	//(Or run straight away if the server was never polled)
tribiqSK.go2 = function(path, branch, goNum, labelTag, thisPageSize, specificItemRequested, inCloseUpView, closeUpViewItem, selectItem, searchTerm, filtersSet, refiner, lastRefiners, shownColumns, server_side, backwards, merge) {
	
	//Check that this isn't an out-of-date request that has come in syncronously via AJAX
	if (goNum != tribiqSK.goNum) {
		return;
	} else {
		tribiqSK.lastSuccessfulGoNum = tribiqSK.goNum;
	}
	
	tribiqSK.stopRefreshing();
	
	//If this is a different panel, scroll back up to the top
	if (tribiqSK.path != path || branch === true || branch === -1) {
		tribiqSK.scrollToPageTop = true;
	}
	
	var lastTitle;
	var lastPath = tribiqSK.path;
	if (tribiqSK.focus) {
		lastTitle = tribiqSK.focus.title;
	}
	
	tribiqSK.path = path;
	tribiqSK.labelTag = labelTag;
	tribiqSK.thisPageSize = thisPageSize;
	tribiqSK.filtersSet = filtersSet;
	tribiqSK.server_side = server_side;
	tribiqSK.shownColumns = shownColumns;
	tribiqSK.inCloseUpView = inCloseUpView;
	tribiqSK.closeUpViewItem = closeUpViewItem;
	tribiqSK.selectItem = selectItem;
	tribiqSK.specificItemRequested = specificItemRequested;
	
	if (!tribiqSK.prefs[tribiqSK.path]) {
		tribiqSK.prefs[tribiqSK.path] = {};
	}
	
	
	//Add functionality for multiple "branches"
	if (branch === -1) {
		//branch = -1 turns off all previously collected branches
		tribiqSK.resetBranches();
	} else if (branch) {
		//branch = true turns this link into a new branch
		tribiqSK.branch(path, lastPath, lastTitle);
	}
		//Otherwise branch = undefined continues in the current branch
	
	//Redraw the top level items, unless we've branched and the current location does not contain the true section
	if (!tribiqSK.focus || branch == -1 || tribiqSK.branches.length <= 1) {
		tribiqSK.setTopLevelItems();
	}
	
	tribiqSK.saveRefiner(refiner);
	tribiqSK.lastRefiners = lastRefiners;
	
	
	delete tribiqSK.focus;
	
	//Focus on this part of the map
	if (merge) {
		tribiqSK.focus = merge;
	} else {
		var map = tribiqSK.followPathOnMap(tribiqSK.path);
		tribiqSK.focus = $.extend(true, {}, map);
	}
	
	
	//Set the format for labels
	if (!(tribiqSK.labelFormat = tribiqSK.focus.label_format)) {
		tribiqSK.labelFormat = '[[' + tribiqSK.labelTag + ']]';
	}
	
	//Check if we have a column description, and set a default one if not
	if (!tribiqSK.focus.columns) {
		tribiqSK.focus.columns = {'name': {'title': 'Name'}};
	}
	
	
	//Handle preferences
	if (!tribiqSK.prefs[path]) {
		tribiqSK.prefs[path] = {};
	}
	
	//Sort by the reorder column if we're reordering
	var reorder = tribiqSK.followPathOnMap(path, 'reorder');
	if (reorder && reorder.column) {
		tribiqSK.sortBy = reorder.column;
		tribiqSK.sortDesc = false;
	
	//Look up the user's choice of search column
	} else if (tribiqSK.prefs[path].sortBy && tribiqSK.focus.columns[tribiqSK.prefs[path].sortBy]) {
		tribiqSK.sortBy = tribiqSK.prefs[path].sortBy;
		tribiqSK.sortDesc = tribiqSK.prefs[path].sortDesc;
	
	//Otherwise sort by name by default
	} else {
		tribiqSK.sortBy = tribiqSK.labelTag;
		tribiqSK.sortDesc = false;
	}
	
	//Look for a user's preference for view mode, otherwise look for a default view mode, otherwise default to grid view
	if (tribiqSK.prefs[path].view_mode) {
		tribiqSK.view_mode = tribiqSK.prefs[tribiqSK.path].view_mode;
	} else if (tribiqSK.focus.view_mode) {
		tribiqSK.view_mode = tribiqSK.focus.view_mode;
	} else {
		tribiqSK.view_mode = 'grid';
	}
	
	//Check that view_mode is a valid choice
	if (tribiqSK.view_mode !== 'grid' && tribiqSK.view_mode !== 'summary' && tribiqSK.view_mode !== 'list') {
		tribiqSK.view_mode = 'grid';
	}
	
	tribiqSK.createSortArray('sortedColumns', 'columns', 'ord');
	tribiqSK.createSortArray('sortedItemButtons', 'item_buttons', 'ord');
	tribiqSK.createSortArray('sortedCollectionButtons', 'collection_buttons', 'ord');
	
	//Check if the Admin has a customised sort order on this Panel
	if (tribiqSK.prefs[tribiqSK.path].sortedColumns) {
		//Get an array-flip of their sorted columns, so we can look up each column's order
		existingSortedColumns = {};
		for (var colNo in tribiqSK.prefs[tribiqSK.path].sortedColumns) {
			var c = tribiqSK.prefs[tribiqSK.path].sortedColumns[colNo];
			existingSortedColumns[c] = colNo;
		}
		
		//Loop through the columns in the current view...
		var lastColName = false;
		for (var colNo in tribiqSK.sortedColumns) {
			var c = tribiqSK.sortedColumns[colNo];
			
			if (tribiqSK.isShowableColumn(c, false, true)) {
				//...checking to see if there are any newly added columns
				if (existingSortedColumns[c] === undefined) {
					//If so, attempt to find a new place for this column by looking at where the column before it was
					var colPos = existingSortedColumns[lastColName];
					
					if (colPos === undefined) {
						colPos = 0;
					} else {
						colPos = 1*colPos + 1;
					}
					
					//Add the column in
					tribiqSK.prefs[tribiqSK.path].sortedColumns.splice(colPos, 0, c);
					
					//Update the array of placements
					for (var d in existingSortedColumns) {
						if (existingSortedColumns[d] >= colPos) {
							++existingSortedColumns[d]
						}
					}
					
					existingSortedColumns[c] = colPos;
				}
			}
			
			lastColName = c;
		}
		
		tribiqSK.sortedColumns = tribiqSK.prefs[tribiqSK.path].sortedColumns;
	}
	
	
	
	if (tribiqSK.selectMode) {
		tribiqSK.multipleSelectEnabled = window.tribiqSKMultipleSelect;
	} else {
		//Check to see if any item toolbar buttons allow for multiple selections, and enable multiple select if so
		tribiqSK.multipleSelectEnabled = false;
		if (tribiqSK.focus.item_buttons) {
			for (i in tribiqSK.focus.item_buttons) {
				if (!tribiqSK.isInfoTag(i)) {
					if (tribiqSK.focus.item_buttons[i].multiple_select) {
						tribiqSK.multipleSelectEnabled = true;
						break;
					}
				}
			}
		}
	}
	
	if (tribiqSK.focus.items && tribiqSK.focus.items.count == 1 && engToBoolean(tribiqSK.focus.allow_bypass)) {
		tribiqSK.branches[tribiqSK.branches.length-1].bypasses[tribiqSK.path] = true;
	} else {
		delete tribiqSK.branches[tribiqSK.branches.length-1].bypasses[tribiqSK.path];
	}
	
	
	//Check the queue to see if there are any more navigation commands queued after this destination. If so, run them, and stop processing this one
	if (tribiqSK.checkQueue()) {
		return;
	}
	
	//If there is only one item and "allow_bypass" is enabled, skip right past this panel
	if (tribiqSK.focus.items && tribiqSK.focus.items.count == 1 && engToBoolean(tribiqSK.focus.allow_bypass)) {
		if (backwards) {
			if (tribiqSK.getBackButtonTitle() !== false) {
				tribiqSK.back();
				return;
			}
		} else {
			var link;
			if (tribiqSK.focus.items) {
				for (id in tribiqSK.focus.items) {
					if (!tribiqSK.isInfoTag(id)) {
						tribiqSK.selectedItems = {};
						tribiqSK.selectedItems[id] = true;
						tribiqSK.saveSelection();
						
						if (tribiqSK.itemDoubleClickAction(id)) {
							return;
						} else {
							break;
						}
					}
				}
			}
		}
	}
	
	
	//Look to see if there are any item_link type columns
	var menuIds = '';
	var cTypes = {};
	for (c in tribiqSK.focus.columns) {
		if (tribiqSK.isShowableColumn(c)) {
			if (tribiqSK.focus.columns[c].item_link == 'menu_item') {
				
				for (i in tribiqSK.focus.items) {
					if (!tribiqSK.isInfoTag(i) && tribiqSK.focus.items[i][c]) {
						menuIds += (menuIds? ',' : '') + tribiqSK.focus.items[i][c];
					}
				}
				
			} else if (tribiqSK.focus.columns[c].item_link == 'content_item' || tribiqSK.focus.columns[c].item_link == 'content_item_or_url') {
				
				for (i in tribiqSK.focus.items) {
					if (!tribiqSK.isInfoTag(i) && tribiqSK.focus.items[i][c]) {
						var cTypeAndId = tribiqSK.focus.items[i][c].split('_');
						if (cTypeAndId[0]
						 && cTypeAndId[0] != 'null'
						 && cTypeAndId[0] === cTypeAndId[0].replace(/\W/, '')
						 && cTypeAndId[1]
						 && cTypeAndId[1] == 1*cTypeAndId[1]) {
							if (!cTypes[cTypeAndId[0]]) {
								cTypes[cTypeAndId[0]] = cTypeAndId[1];
							} else {
								cTypes[cTypeAndId[0]] += ',' + cTypeAndId[1];
							}
						}
					}
				}
			}
		}
	}
				
	tribiqSK.contentItems = {};
	for (t in cTypes) {
		tribiqSK.contentItems[t] = tribiq.nonAsyncAJAX(
			URLBasePath + 'admin/ajax.php?_json=1&path=tribiq__content/content&refinerName=content_type&refinerId=' + t
								+ '&refiner__content_type=' + t
								+ '&refiner__trash=1'
								+ '&_get_item_links=' + cTypes[t],
			false, true);
	}
	
	tribiqSK.menuItems = {};
	if (menuIds) {
		tribiqSK.menuItems = tribiq.nonAsyncAJAX(
			URLBasePath + 'admin/ajax.php?_json=1&path=tribiq__menu/menu_items&_get_item_links=' + menuIds,
			false, true);
	}
	
	
	get('preloader_circle').style.visibility = 'hidden';
	
	//Display a pop-up message, if this is not a refresh and one has been requested
	if (tribiqSK.refreshToPage === undefined && !backwards && tribiqSK.focus.popout_message) {
		tribiqA.showMessage(tribiqSK.focus.popout_message, false);
	}
	
	//Show/hide the search box, and set the search term
		//(Don't do this if this was a page refresh, not a page jump)
	if (tribiqSK.refreshToPage === undefined) {
		tribiqSK.setSearch(searchTerm);
	}
	
	tribiqSK.sortItems();
	
	if ((engToBoolean(tribiqSK.focus.php) || tribiqSK.focus.db_items) && !engToBoolean(tribiqSK.focus.disable_periodic_refresh)) {
		tribiqSK.periodicRefresh();
	}
	
	//Highlight the view options button if filters have been applied
	get('listViewOptionsDropdown').className = filtersSet? 'hot' : '';
}

tribiqSK.scrollToPageTop = false;
tribiqSK.nextPage = function(keepCloseUpView) {
	
	if (tribiqSK.lockPageClicks) {
		return false;
	}
	
	if (tribiqSK.page < tribiqSK.pageCount) {
		tribiqSK.scrollToPageTop = true;
		tribiqSK.goToPage(tribiqSK.page + 1, keepCloseUpView);
	}
}

tribiqSK.scrollToPageBottom = false;
tribiqSK.prevPage = function(keepCloseUpView) {
	
	if (tribiqSK.lockPageClicks) {
		return false;
	}
	
	if (tribiqSK.page > 1) {
		tribiqSK.scrollToPageBottom = true;
		tribiqSK.goToPage(tribiqSK.page - 1, keepCloseUpView);
	}
}

tribiqSK.goToPage = function(page, keepCloseUpView) {
	if (tribiqSK.uploading) {
		return false;
	}
	
	tribiqSK.lockPageClicks = true;
	
	if (!keepCloseUpView) {
		tribiqSK.inCloseUpView = false;
	}
	
	if (tribiqSK.server_side) {
		tribiqSK.page = page;
		tribiqSK.refresh();
	} else {
		tribiqSK.showPage(page);
	}
}

tribiqSK.showPage = function(page) {
	tribiqSK.page = page;
	tribiqSK.drawPanel();
}

tribiqSK.sortItems = function() {
	if (!tribiqSK.server_side) {
		tribiqSK.createSortArray('sortedItems', 'items', tribiqSK.sortBy, tribiqSK.sortDesc);
	}
	tribiqSK.searchItems();
}

tribiqSK.searchItems = function() {
	var i;
	
	if (tribiqSK.server_side) {
		
		tribiqSK.searchedItems = tribiqSK.focus.__item_sort_order__;
		tribiqSK.searchMatches = tribiqSK.focus.items.count;
		
		if (tribiqSK.specificItemRequested) {
			//If we've loaded straight into close-up view, work out which page we're on
			tribiqSK.refreshToPage = undefined;
			for (var itemNo in tribiqSK.searchedItems) {
				if (tribiqSK.searchedItems[itemNo] == tribiqSK.closeUpViewItem) {
					tribiqSK.refreshToPage = 1 + Math.floor(itemNo / tribiqSK.thisPageSize);
					break;
				}
			}
		}
		
	} else if (tribiqSK.searchTerm === undefined || get('search').style.visibility == 'hidden') {
		tribiqSK.searchedItems = tribiqSK.sortedItems;
		tribiqSK.searchMatches = tribiqSK.focus.items? tribiqSK.focus.items.count : undefined;
		
	} else {
		tribiqSK.searchedItems = {};
		tribiqSK.searchMatches = 0;
		
		var matches;
		for (itemNo in tribiqSK.sortedItems) {
			i = tribiqSK.sortedItems[itemNo];
			
			matches = false;
			for (c in tribiqSK.focus.columns) {
				if (!tribiqSK.isInfoTag(c) && engToBoolean(tribiqSK.focus.columns[c].searchable)) {
					value = tribiqSK.columnValue(i, c, true);
					numeric = 1*value;
					
					if (value == numeric) {
						matches = numeric == tribiqSK.searchTerm;
					} else if (value) {
						matches = value.toLowerCase().indexOf(tribiqSK.searchTerm) !== -1;
					}
					
					if (matches) break;
				}
			}
			
			if (matches) {
				tribiqSK.searchedItems[tribiqSK.searchMatches++] = tribiqSK.sortedItems[itemNo];
			}
		}
	}
	
	tribiqSK.itemsOrder = {};
	for (itemNo in tribiqSK.searchedItems) {
		if (!tribiqSK.isInfoTag(itemNo)) {
			i = tribiqSK.searchedItems[itemNo];
			tribiqSK.itemsOrder[i] = 1*itemNo;
		}
	}
	
	if (tribiqSK.refreshToPage !== undefined) {
		tribiqSK.showPage(tribiqSK.refreshToPage);
		tribiqSK.refreshToPage = undefined;
	} else {
		var ord, page = 1;
		
		//Work out the current page number for client-side sorting
		if (!tribiqSK.server_side) {
			//If we've not got a specific target, check if any items were selected here and use the first we find as a target
			if (!tribiqSK.closeUpViewItem) {
				tribiqSK.loadSelection();
				for (i in tribiqSK.selectedItems) {
					tribiqSK.closeUpViewItem = i;
					break;
				}
			}
			
			//If the item we want is in the results, make sure we show that page
			if (ord = tribiqSK.itemsOrder[tribiqSK.closeUpViewItem]) {
				page = Math.floor(ord / tribiqSK.thisPageSize) + 1;
			}
		
		} else if (tribiqSK.focus.__page__) {
			page = tribiqSK.focus.__page__;
		}
		
		tribiqSK.showPage(page);
	}
}

tribiqSK.drawPanel = function() {
	get('viewModeSelected').className = tribiqSK.view_mode + '_view';
	
	//Stop reordering from running, if it was running
	if (tribiqSK.reordering) {
		$(".sortable").sortable('destroy');
		tribiqSK.reordering = undefined;
	}
	
	tribiqSK.loadSelection();
	
	if (tribiqSK.inCloseUpView) {
		if (tribiqSK.itemsOrder[tribiqSK.closeUpViewItem] !== undefined) {
			tribiqSK.selectedItems = {};
			tribiqSK.selectedItems[tribiqSK.closeUpViewItem] = true;
			
			//Implement linking to a panel with an item selected using the same logic as linking to close-up view
			if (tribiqSK.selectItem) {
				tribiqSK.inCloseUpView = false;
			}
		} else {
			tribiqSK.selectedItems = {};
			tribiqSK.inCloseUpView = false;
		}
	
	}
	
	if (tribiqSK.focus.reorder && tribiqSK.focus.reorder.column
	 && !tribiqSK.inCloseUpView && tribiqSK.view_mode == 'grid' && !tribiqSK.selectMode) {
		tribiqSK.reordering = true;
	}
	
	var scrollTo = tribiqSK.closeUpViewItem;
	tribiqSK.selectItem = false;
	tribiqSK.lastClicked = false;
	tribiqSK.CSVExportEnabled = false;
	tribiqSK.setBanner();
	
	if (tribiqSK.inCloseUpView) {
		get('search_term').readOnly = true;
		tribiqSK.hidePanelTitle();
		tribiqSK.setBackButtonInCloseUpView();
		tribiqSK.hideCollectionButtons();
		tribiqSK.setCloseUpView();
		tribiqSK.hideItemCount();
	} else {
		tribiqSK.setPanelTitle();
		tribiqSK.setBackButton();
		tribiqSK.setCollectionButtons();
		tribiqSK.setPanelItems();
		tribiqSK.setItemCount();
		get('search_term').readOnly = false;
	}
	
	tribiqSK.setTrash();
	
	tribiqSK.saveSelection();
	
	tribiqSK.setPages();
	tribiqSK.setItemButtons();
	
	//Start reordering running, if it is enabled on this panel
	if (tribiqSK.reordering) {
		$(".sortable").sortable({
			opacity: 0.8,
			stop: function (sorted) {
				//Handle the results of a reorder
				
				//Get a list of item ids in the new order
				var sortSerial = $(".sortable").sortable('serialize');
				var newOrder = sortSerial.replace(/&?r?item\[\]=/g, ',').substr(1).split(',');
				var oldOrder = [];
				var values = [];
				var items = {};
				var saves = '';
				
				//Create an array of which items we can see
				for (itemNo in newOrder) {
					var i = newOrder[itemNo];
					items[i] = true;
				}
				
				//Look through the searchedItems array and get their old order, and their value from the items array
				for (itemNo in tribiqSK.searchedItems) {
					var i = tribiqSK.searchedItems[itemNo];
					if (items[i]) {
						oldOrder.push(i);
						values.push(tribiqSK.focus.items[i][tribiqSK.focus.reorder.column]);
					}
				}
				
				//Look through newOrder and oldOrder for any changes, and update those column
				var actionRequests = tribiqSK.getKey();
				for (itemNo in newOrder) {
					var i = newOrder[itemNo];
					if (i != oldOrder[itemNo]) {
						saves += (saves? ',' : '') + i;
						tribiqSK.focus.items[i][tribiqSK.focus.reorder.column] = actionRequests['item__' + i] = values[itemNo];
					}
				}
				actionRequests.reorder = true;
				actionRequests.id = saves;
				
				//Send these results via AJAX
				var actionTarget =
					'plugins.php?' +
						'__pluginClassName__=' + tribiqSK.focus.reorder.class_name +
						'&__path__=' + tribiqSK.path +
						'&method_call=storekeeperAJAX';
				
				$.post(
					URLBasePath + actionTarget,
					actionRequests,
					//Refresh the panel to show the new order
					tribiqSK.focus.db_items && engToBoolean(tribiqSK.focus.db_items.client_side)?
						tribiqSK.sortItems
					:	tribiqSK.refresh
				);
			}
		});
		
		//Bugfix for a glitch in IE 8 that hides the labels of items
		if (tribiq.browserIsIE()) {
			$('#rightColumnContent .sortable .item').each(function(i, el) {el.className = el.className});
		}
	}
	
	//If there's an item specified to highlight, attempt to scroll to it
	if (!tribiqSK.refreshIsPeriodic) {
		if (!scrollTo) {
			scrollTo = tribiqSK.getKeyId(true);
		}
		
		var offset;
		if (scrollTo && !tribiqSK.inCloseUpView && (offset = $(get('item_' + scrollTo)).position())) {
			offset = offset.top + $('#rightColumnContent').scrollTop();
			$('#rightColumnContent').animate({ scrollTop: Math.max(0, offset - 200) }, 700);
		
		//If the next/previous buttons were clicked, attempt to scroll to the top/bottom of the page
		} else if (tribiqSK.scrollToPageTop && !tribiqSK.inCloseUpView) {
			$('#rightColumnContent').animate({ scrollTop: 0 }, 700);
		
		} else if (tribiqSK.scrollToPageBottom && !tribiqSK.inCloseUpView) {
			$('#rightColumnContent').animate({ scrollTop: 99999 }, 700);
		}
	}
	
	tribiqSK.lockPageClicks = false;
	tribiqSK.scrollToPageTop = false;
	tribiqSK.scrollToPageBottom = false;
	tribiqSK.refreshIsPeriodic = false;
}

//Go backwards, if possible
tribiqSK.back = function() {
	if (tribiqSK.uploading) {
		return false;
	}
	
	if (tribiqSK.inCloseUpView) {
		//When in close-up view, the back button should take the user out of it
		tribiqSK.inCloseUpView = false;
		tribiqSK.showPage(tribiqSK.page);
		
	} else {
		//Deslect all items before leaving
		tribiqSK.deselectAllItems();
		tribiqSK.clearRefiner();
		tribiqSK.clearSearch();
		
		//Check to see if we've reached the start of our current branch
		if (tribiqSK.branches.length > 1 && tribiqSK.path == tribiqSK.branches[tribiqSK.branches.length-1].to) {
			//Close the branch and take the user to the previous branch
			tribiqSK.go(tribiqSK.branches.pop().from, undefined, undefined, undefined, undefined, true);
		
		//Otherise check to see if there is a panel naturally above the current panel
		} else if (tribiqSK.focus.back_link) {
			//Go there if so
			tribiqSK.go(tribiqSK.focus.back_link, undefined, undefined, undefined, undefined, true);
		}
	}
}




//Scan the map, registering any link that's a branch
tribiqSK.knownBranches = {};
tribiqSK.lookForBranches = function(map, path, panelPath) {
	
	if (map.link && map.link.path) {
		panelPath = map.link.path;
	}
	
	for (i in map) {
		if (map[i] && typeof map[i] == 'object') {
			p = path? path + '/' + i : i;
			
			if (map[i].link && typeof map[i].link == 'object') {
				if (map[i].link.branch) {
					
					if (!tribiqSK.knownBranches[panelPath]) {
						tribiqSK.knownBranches[panelPath] = {};
					}
					if (!tribiqSK.knownBranches[panelPath][map[i].link.path]) {
						tribiqSK.knownBranches[panelPath][map[i].link.path] = {};
					}
					
					tribiqSK.knownBranches[panelPath][map[i].link.path][ifNull(map[i].link.refiner, 1)] = tribiqSK.shortenPath(p);
					continue;
				}
			}
			
			tribiqSK.lookForBranches(map[i], p, panelPath);
		}
	}
}

//Remove every branch taken so far from memory
tribiqSK.resetBranches = function() {
	tribiqSK.branches = [{'bypasses': {}, 'filters': {}, 'refiners': {}, 'searches': {}, 'selected_items': {}}];
}
tribiqSK.resetBranches();

//Add a new branch
tribiqSK.branch = function(path, lastPath, lastTitle) {
	
	path = tribiqSK.shortenPath(path);
	
	tribiqSK.branches[tribiqSK.branches.length] = {
		'from': lastPath,
		'to': path,
		'title': lastTitle,
		'bypasses': {},
		'filters': {},
		'refiners': {},
		'searches': {},
		'selected_items': {}
	};
}

//Set the current location in the URL
tribiqSK.setHash = function(path, selectedItem) {
	
	//Don't bother setting a hash in select mode, where you can't see the path anyway
	if (tribiqSK.selectMode) {
		return;
	}
	
	var hash = '';
	var b, kb, lastTo, failed = false;
	
	//Attempt to include the current branches taken in the URL by creating a deep link
	for (i in tribiqSK.branches) {
		if (i > 0) {
			b = tribiqSK.branches[i];
			
			//Is this branch in the map?
			if (b.to && tribiqSK.knownBranches[b.from] && tribiqSK.knownBranches[b.from][b.to]
			 && (kb = tribiqSK.knownBranches[b.from][b.to][b.refiners[b.to]? b.refiners[b.to].name : 1])) {
				
				kb = tribiqSK.shortenPath(kb);
				
				if (lastTo) {
					//Use a relative path on the other end of the branch, to save space in the URL
					if (kb.substr(0, lastTo.length) == lastTo) {
						kb = tribiqSK.shortenPath(kb.substr(lastTo.length + 1));
					} else {
						//If we couldn't get a relative path then there was something odd with the branch
						//and we cannot create a deep link to it
						failed = true;
						break;
					}
					hash += '//';
				}
				
				//Add any selected items to the deep link
				hash += kb + '//' + tribiqSK.implodeKeys(tribiqSK.branches[i-1].selected_items[b.from]);
				
				lastTo = b.to;
			} else {
				//If the branch wasn't registered then we can't create a deep link to it
				//Note that this should never happen, but if it does break I don't want Storekeeper breaking too
				failed = true;
				break;
			}
		}
	}
	
	//If we successfully built a deep link, combine it with the current path
		//Note that this involves running some of the above logic again
	if (!failed && lastTo) {
		//Use a relative path on the other end of the branch, to save space in the URL
		if (path.substr(0, lastTo.length) == lastTo) {
			path = hash + '//' + tribiqSK.shortenPath(path.substr(lastTo.length + 1));
		} else {
			//If we couldn't get a relative path then there was something odd with the branch
			//and we cannot create a deep link to it
		}
	}
	
	//Add the selected item to the URL
	if (selectedItem) {
		//No matter what the generated path was, there should always be two slashes between the selected item and the path
		if (tribiq.rightHandedSubStr(path, 2) == '//') {
			path += selectedItem;
		} else if (tribiq.rightHandedSubStr(path, 1) == '/') {
			path += '/' + selectedItem;
		} else {
			path += '//' + selectedItem;
		}
	}
	
	//Add the path to the hash
	document.location.hash = path;
	tribiq.currentHash = document.location.hash;
}



tribiqSK.searchOnClick = function(el) {
}

//Update the search term
tribiqSK.searchOnKeyUp = function(el) {
	if (tribiqSK.doSearchAfterDelay) {
		clearTimeout(tribiqSK.doSearchAfterDelay);
	}
	
	tribiqSK.doSearchAfterDelay = tribiq.setTimeout('tribiqSK.doSearch(get("search_term"))', tribiqSK.searchDelayTime);
}

tribiqSK.searchOnChange = function(el) {
	if (tribiqSK.server_side) {
		if (tribiqSK.doSearchAfterDelay) {
			clearTimeout(tribiqSK.doSearchAfterDelay);
			tribiqSK.doSearch(el);
		}
	}
}

tribiqSK.doSearch = function(el) {
	var searchTerm;
	if (el.value === '') {
		searchTerm = undefined;
	} else {
		searchTerm = el.value;
		
		//Ensure that the searct term is numeric if possible
		if (searchTerm == 1*searchTerm) {
			searchTerm = 1*searchTerm;
		} else {
			searchTerm = searchTerm.toLowerCase();
		}
	}
	
	//Don't launch a search if the search term has not changed
	if (searchTerm === tribiqSK.searchTerm) {
		return;
	}
	
	tribiqSK.searchTerm = searchTerm;
	tribiqSK.saveSearch();
	
	if (tribiqSK.server_side) {
		tribiqSK.goToPage(1);
	} else {
		tribiqSK.searchItems();
	}
}

tribiqSK.itemClicked = false;
tribiqSK.lastClicked = false;
tribiqSK.selectedItems = new Object();
tribiqSK.itemClick = function(el, e) {
	
	tribiqSK.itemClicked = true;
	var id = el.id.substr(5);
	
	if (tribiqSK.lastClicked == id && tribiq.browserIsMobile()) {
		return tribiqSK.itemDoubleClick(el);
	}
	
	//If the admin is holding down the shift key...
	if (tribiqSK.multipleSelectEnabled && (e || event).shiftKey && tribiqSK.lastClicked) {
		//...select everything between the current item and the last item that they clicked on
		tribiqSK.selectItemRange(id, tribiqSK.lastClicked);
	
	//Else if the admin is holding down the Ctrl/Apple key...
	} else if (tribiqSK.multipleSelectEnabled && ((e || event).ctrlKey || (e || event).metaKey)) {
		//...toogle the item that they've clicked on
		if (tribiqSK.selectedItems[id]) {
			delete tribiqSK.selectedItems[id];
			$(el).removeClass('selected');
		} else {
			tribiqSK.selectedItems[id] = true;
			$(el).addClass('selected');
		}
		tribiqSK.lastClicked = id;
	
	} else {
		//Otherwise select the item that they've just clicked on, and nothing else
		tribiqSK.deselectAllItems();
		tribiqSK.selectedItems[id] = true;
		$(el).addClass('selected');
		tribiqSK.lastClicked = id;
	}
	
	
	tribiqSK.setItemButtons();
	tribiqSK.saveSelection();
	
	return false;
}

//If the admin clicks on the background, deselect every item
tribiqSK.itemBGClick = function() {
	if (tribiqSK.inCloseUpView) {
		return;
	} else if (tribiqSK.itemClicked) {
		tribiqSK.itemClicked = false;
	} else {
		tribiqSK.deselectAllItems();
		tribiqSK.setItemButtons();
		tribiqSK.saveSelection();
	}
}

//If the admin doubleclicks on the background, select every item
tribiqSK.itemBGDoubleClick = function() {
	if (tribiqSK.inCloseUpView) {
		return;
	} else if (tribiqSK.itemDoubleClicked) {
		tribiqSK.itemDoubleClicked = false;
	} else if (tribiqSK.view_mode == 'grid' && tribiqSK.multipleSelectEnabled) {
		tribiqSK.selectAll();
	}
}

tribiqSK.selectAll = function() {
	tribiqSK.selectItemRange();
	tribiqSK.setItemButtons();
	tribiqSK.saveSelection();
}

tribiqSK.selectItemRange = function(start, stop) {
	
	var select = start === undefined;
	
	for (itemNo in tribiqSK.searchedItems) {
		if (!tribiqSK.isInfoTag(itemNo)) {
			var i = tribiqSK.searchedItems[itemNo];
			
			if (tribiqSK.shownItems && tribiqSK.shownItems[i]) {
				if (select || i === start || i === stop) {
					tribiqSK.selectedItems[i] = true;
					$(get('item_' + i)).addClass('selected');
				} else {
					delete tribiqSK.selectedItems[i];
					$(get('item_' + i)).removeClass('selected');
				}
				
				if ((i === start && i === stop)) {
					select = false;
				} else if (i === start || i === stop) {
					select = !select;
				}
			}
		}
	}
}

tribiqSK.deselectAllItems = function() {
	tribiqSK.selectItemRange(-1);
}

//Remember which items an admin had selected in the current panel
tribiqSK.saveSelection = function(dontSetHash) {
	tribiqSK.branches[tribiqSK.branches.length-1].selected_items[tribiqSK.path] = tribiqSK.selectedItems;
	
	//Check if there was only one item selected
	var oneItemSelected = false;
	for (i in tribiqSK.selectedItems) {
		if (tribiqSK.selectedItems[i]) {
			if (oneItemSelected === false) {
				oneItemSelected = i;
			} else {
				oneItemSelected = false;
				break;
			}
		}
	}
	
	//Set the hash in the URL
	if (!dontSetHash && !tribiqSK.checkQueueNotEmpty()) {
		if (!oneItemSelected) {
			tribiqSK.setHash(tribiqSK.path, '');
		} else if (!tribiqSK.inCloseUpView) {
			tribiqSK.setHash(tribiqSK.path, oneItemSelected);
		} else {
			tribiqSK.setHash(tribiqSK.path, oneItemSelected + '/');
		}
	}
}

//Check to see if any items were previously selected in this panel, and select them again if so
tribiqSK.loadSelection = function() {
	if (tribiqSK.branches[tribiqSK.branches.length-1].selected_items[tribiqSK.path]) {
		tribiqSK.selectedItems = tribiqSK.branches[tribiqSK.branches.length-1].selected_items[tribiqSK.path];
		
		for (i in tribiqSK.selectedItems) {
			//Remove any items that have disappeared since this panel was last shown
			if (!tribiqSK.focus.items[i]) {
				delete tribiqSK.selectedItems[i];
			
			//Set the "last clicked" to the first item that we find as a work-around for not actually knowing what it should be
			} else if (!tribiqSK.lastClicked) {
				tribiqSK.lastClicked = i;
			}
		}
	} else {
		tribiqSK.selectedItems = new Object();
	}
}

//Remember a refiner on the current panel
tribiqSK.saveRefiner = function(refiner) {
	tribiqSK.branches[tribiqSK.branches.length-1].refiners[tribiqSK.path] = tribiqSK.refiner = refiner;
}

//Check to see if there was a refiner on a panel
tribiqSK.loadRefiner = function(path, branch) {
	//New branch? No refiner if so.
	if (branch === -1 || branch) {
		return undefined;
	}
	
	if (tribiqSK.branches[tribiqSK.branches.length-1].refiners[path]) {
		return tribiqSK.branches[tribiqSK.branches.length-1].refiners[path];
	}
	
	return undefined;
}

tribiqSK.clearRefiner = function() {
	tribiqSK.refiner = undefined;
	tribiqSK.saveRefiner();
}

//Remember an admin's search
tribiqSK.saveSearch = function() {
	tribiqSK.branches[tribiqSK.branches.length-1].searches[tribiqSK.path] = tribiqSK.searchTerm;
}

//Check to see if there was a Search on a panel
tribiqSK.loadFromBranches = function(path, branch, type) {
	//New branch? Clear the search if so.
	if (branch === -1 || branch) {
		return undefined;
	}
	
	if (tribiqSK.branches[tribiqSK.branches.length-1]
	 && tribiqSK.branches[tribiqSK.branches.length-1][type]
	 && tribiqSK.branches[tribiqSK.branches.length-1][type][path]) {
		return tribiqSK.branches[tribiqSK.branches.length-1][type][path];
	}
	
	return undefined;
}

tribiqSK.setSearch = function(searchTerm) {
	
	var visibility = 'hidden';
	tribiqSK.searchTerm = searchTerm;
	
	if (tribiqSK.focus && tribiqSK.focus.columns) {
		for (c in tribiqSK.focus.columns) {
			if (!tribiqSK.isInfoTag(c) && engToBoolean(tribiqSK.focus.columns[c].searchable)) {
				visibility = 'visible';
			}
		}
	}
	
	get('search').style.visibility = visibility;
	
	if (tribiqSK.searchTerm === undefined) {
		get('search_term').value = '';
	} else {
		get('search_term').value = tribiqSK.searchTerm;
	}
}

tribiqSK.clearSearch = function() {
	get('search_term').value = '';
	tribiqSK.searchTerm = undefined;
	tribiqSK.saveSearch();
}

tribiqSK.itemDoubleClicked = false;
tribiqSK.itemJustDoubleClicked = false;
tribiqSK.itemDoubleClick = function(el) {
	if (tribiqSK.uploading) {
		return false;
	}
	
	//Stop multiple clicks
	if (tribiqSK.itemJustDoubleClicked) {
		return false;
	} else {
		tribiqSK.itemJustDoubleClicked = true;
		tribiq.setTimeout('tribiqSK.itemJustDoubleClicked = false;', tribiqSK.itemDoubleClickTime);
	}
	
	tribiqSK.itemClicked = true;
	tribiqSK.itemDoubleClicked = true;
	
	var id = el.id.substr(5);
	
	if (!tribiqSK.itemDoubleClickAction(id) && engToBoolean(tribiqSK.focus.enable_close_up_view)) {
		tribiqSK.inCloseUpView = true;
		tribiqSK.closeUpViewItem = id;
		tribiqSK.drawPanel();
	}
	
	return false;
}

tribiqSK.itemDoubleClickAction = function(id) {
	var link = tribiqSK.focus.items[id].link;
	
	if (!link && tribiqSK.focus.item) {
		link = tribiqSK.focus.item.link;
	}
	
	if (link) {
		if (!engToBoolean(link.unselect_items)) {
			tribiqSK.selectedItems = {};
			tribiqSK.selectedItems[id] = true;
			tribiqSK.saveSelection();
		}

		tribiqSK.action(tribiqSK.focus.items[id], true, undefined, link);
		return true;
	} else {
		return false;
	}
}

tribiqSK.topLevelClick = function(id, j) {
	if (tribiqSK.uploading) {
		return false;
	}
	
	tribiqSK.clearRefiner();
	tribiqSK.clearSearch();
	tribiqSK.resetBranches();
	
	if (j !== undefined) {
		tribiqSK.action(tribiqSK.map[id].second_level_items[j], false, -1);
	} else {
		tribiqSK.action(tribiqSK.map[id], false, -1);
	}
}

tribiqSK.viewTrash = function () {
	if (!tribiqSK.selectMode && !tribiqSK.inCloseUpView && tribiqSK.focus && tribiqSK.focus.trash && !engToBoolean(tribiqSK.focus.trash.empty)) {
		tribiqSK.action(tribiqSK.focus.trash, false);
	}
	
	return false;
}

tribiqSK.collectionButtonClick = function(id) {
	if (tribiqSK.focus && tribiqSK.focus.collection_buttons && tribiqSK.focus.collection_buttons[id]) {
		tribiqSK.action(tribiqSK.focus.collection_buttons[id], false);
	}
}

tribiqSK.itemButtonClick = function(id) {
	if (tribiqSK.focus && tribiqSK.focus.item_buttons && tribiqSK.focus.item_buttons[id]) {
		tribiqSK.action(tribiqSK.focus.item_buttons[id], true);
	}
}

//Handle what happens when an admin clicks on something that will cause an action
tribiqSK.action = function(object, itemLevel, branch, link, extraRequests) {
	if (tribiqSK.uploading) {
		return false;
	}
	
	tribiqA.closeBox('AdminMoreItemActions');
	tribiqA.closeBox('AdminMoreCollActions');
	
	if (!link && object.link) {
		link = object.link;
	}
	
	//In select mode, don't let an admin navigate past a certain point using double-click actions on items
	if (link && tribiqSK.selectMode && window.tribiqSKMaxPath !== false && tribiqSK.path == window.tribiqSKMaxPath && link.path != tribiqSK.path) {
		return;
	}
	
	
	//For backwards compatability for browsers without flash, attempt to convert file upload tags into ajax->confirm->form tags
	if (object.upload && !object.ajax) {
		object.ajax = {
			class_name: object.upload.class_name,
			confirm: {
				message: '<input type="file" name="Filedata"/>',
				html: true,
				button_message: tribiqA.phrase.upload,
				cancel_button_message: tribiqA.phrase.cancel,
				form: true
			},
			request: object.upload.request
		};
	}
	
	
	tribiqSK.pickItemsItemLevel = itemLevel;
	tribiqSK.postPickItemsObject = false;
	
	if (link) {
		//If a link is set, go to the link
		if (engToBoolean(link.unselect_items)) {
			tribiqSK.selectedItems = {};
			tribiqSK.saveSelection();
		}
		
		tribiqSK.go(
			link.path,
			branch !== undefined? branch : engToBoolean(link.branch),
			link.refiner? {'id': itemLevel? tribiqSK.getKeyId() : '', 'name': link.refiner} : undefined);
	
	} else if (object.frontend_link) {
		var id, item;
		var frontend_link = object.frontend_link;
		
		if (itemLevel && (id = tribiqSK.getKeyId(true)) && (item = tribiqSK.focus.items[id]) && item.frontend_link) {
			frontend_link = item.frontend_link;
		}
		
		if (frontend_link.substr(0, 17) == 'admin/welcome.php') {
			tribiq.goToURL(tribiq.addBasePath(frontend_link));
		} else if (frontend_link.indexOf('://') !== -1) {
			window.open(frontend_link);
		} else if (window.opener && !window.opener.tribiqSK) {
			window.opener.location = tribiq.addBasePath(frontend_link);
		} else if (window.storekeeperChildWindow && !window.storekeeperChildWindow.closed) {
			window.storekeeperChildWindow.location = tribiq.addBasePath(frontend_link);
		} else {
			window.storekeeperChildWindow = window.open(tribiq.addBasePath(frontend_link));
		}
	
	} else if (object.admin_box) {
		var key = ifNull(tribiqSK.getKey(itemLevel), {});
		
		if (object.admin_box.key) {
			for (var r in object.admin_box.key) {
				key[r] = object.admin_box.key[r];
			}
		}
		
		tribiqAB.open(object.admin_box.path, key, object.admin_box.tab);
	
	} else if (object.floating_box) {
		tribiqAFB.open(
			object.floating_box.class_name,
			object.floating_box.name,
			tribiq.urlRequest(object.floating_box.request) +
			tribiq.urlRequest(tribiqSK.getKey(itemLevel))
		);
	
	} else if (object.popout) {
		var id, item, title;
		var popout = $.extend(true, {}, object.popout);
		
		if (itemLevel && (id = tribiqSK.getKeyId(true)) && (item = tribiqSK.focus.items[id])) {
			if (item.popout) {
				popout = $.extend(popout, item.popout);
			}
			
			if (!popout.title) {
				popout.title = tribiqSK.applyMergeFields(tribiqSK.labelFormat, false, id);
			}
		}
			
		if (item && popout.href) {
			popout.href += popout.href.indexOf('?') === -1? '?' : '&';
			
			if (item.checksum) {
				popout.href += 'c=' + encodeURIComponent(item.checksum);
			} else if (id == 1*id) {
				popout.href += 'id=' + id;
			} else {
				popout.href += 'c=' + encodeURIComponent(id);
			}
			
			if (item.filename) {
				popout.href += '&filename=' + encodeURIComponent(item.filename);
			}
		}
		
		if (popout.href) {
			popout.href = tribiq.addBasePath(popout.href);
		}
		
		$.fancybox(popout);
	
	} else if (object.pick_items && !itemLevel) {
		
		//if (tribiqSK.selectMode) {
			//return;
		//}
		
		if (object.pick_items.ajax) {
			tribiqSK.postPickItemsObject = object.pick_items;
		}
		
		//Use storekeeper in select mode to combine two items
		tribiqSK.actionTarget =
			'plugins.php?' +
				'__pluginClassName__=' + object.pick_items.class_name +
				'&__path__=' + tribiqSK.path +
				'&method_call=storekeeperAJAX';
		tribiqSK.actionRequests = tribiqSK.getKey(itemLevel);
		
		if (object.pick_items.request) {
			$.extend(tribiqSK.actionRequests, object.pick_items.request);
		}
		
		tribiqA.SK(
			'tribiqSK', 'pickItems',
			engToBoolean(object.pick_items.multiple_select),
			object.pick_items.path,
			object.pick_items.target_path,
			object.pick_items.min_path,
			object.pick_items.max_path,
			object.pick_items.disallow_identical_min_path,
			true,
			object.pick_items.one_to_one_choose_phrase,
			object.pick_items.one_to_many_choose_phrase);
	
	} else if (object.combine_items && itemLevel) {
		
		//if (tribiqSK.selectMode) {
			//return;
		//}
		
		if (object.combine_items.ajax) {
			tribiqSK.postPickItemsObject = object.combine_items;
		}
		
		//Use storekeeper in select mode to combine two items
		tribiqSK.actionTarget =
			'plugins.php?' +
				'__pluginClassName__=' + object.combine_items.class_name +
				'&__path__=' + tribiqSK.path +
				'&method_call=storekeeperAJAX';
		tribiqSK.actionRequests = tribiqSK.getKey(itemLevel);
		
		if (object.combine_items.request) {
			$.extend(tribiqSK.actionRequests, object.combine_items.request);
		}
		
		tribiqA.SK(
			'tribiqSK', 'pickItems',
			engToBoolean(object.combine_items.multiple_select),
			object.combine_items.path,
			object.combine_items.target_path,
			object.combine_items.min_path,
			object.combine_items.max_path,
			object.combine_items.disallow_identical_min_path,
			true,
			tribiqSK.itemsSelected > 1 && object.combine_items.many_to_one_choose_phrase?
				object.combine_items.many_to_one_choose_phrase
			  : object.combine_items.one_to_one_choose_phrase,
			tribiqSK.itemsSelected > 1 && object.combine_items.many_to_many_choose_phrase?
				object.combine_items.many_to_many_choose_phrase
			  : object.combine_items.one_to_many_choose_phrase);
	
	} else if (object.ajax) {
		
		//Run an AJAX function
		tribiqSK.actionTarget =
			'plugins.php?' +
				'__pluginClassName__=' + object.ajax.class_name +
				'&__path__=' + tribiqSK.path +
				'&method_call=storekeeperAJAX';
		tribiqSK.actionRequests = tribiqSK.getKey(itemLevel);
		
		if (object.ajax.request) {
			$.extend(tribiqSK.actionRequests, object.ajax.request);
		}
		
		if (extraRequests) {
			$.extend(tribiqSK.actionRequests, extraRequests);
		}
		
		
		//If a confirm is set, set up a pop-up floating box box to ask the admin before carrying out the action
		if (object.ajax.confirm) {
			
			var isForm = engToBoolean(object.ajax.confirm.form);
			var isHTML = engToBoolean(object.ajax.confirm.html);
			var isDownload = engToBoolean(object.ajax.confirm.download);
			
			//Get the message/html for this box
			var message;
			
			//Use the multiple select message if more than one item is selected
			if (itemLevel && tribiqSK.itemsSelected > 1) {
				message = ifNull(object.ajax.confirm.multiple_select_message, object.ajax.confirm.message);
			
			//Apply mergefields to populate the message with information from the selected row
			//Note that if they're not going to be escaped below then they need to be here.
			} else if (object.ajax.confirm.message) {
				message = tribiqSK.applyMergeFields(object.ajax.confirm.message, isHTML);
			}
			
			//If no message is set, try and get one using an AJAX call
			if (!message) {
				message = tribiq.nonAsyncAJAX(URLBasePath + tribiqSK.actionTarget + tribiq.urlRequest(tribiqSK.actionRequests), false)
			}
			
			//If this is a download, add the current search/sorting information in,
			//just in case the download should differ depending on the current view
			if (isDownload) {
				tribiqSK.actionRequests._download = 1;
				
				if (tribiqSK.searchTerm !== undefined) {
					tribiqSK.actionRequests._search = tribiqSK.searchTerm;
				}
				
				if (tribiqSK.prefs[tribiqSK.path] && tribiqSK.prefs[tribiqSK.path].sortBy) {
					tribiqSK.actionRequests._sort_col = tribiqSK.prefs[tribiqSK.path].sortBy;
					tribiqSK.actionRequests._sort_desc = tribiqSK.prefs[tribiqSK.path].sortDesc? 1 : 0;
				} else {
					tribiqSK.actionRequests._sort_col = tribiqSK.labelTag;
				}
			}
			
			//Start generating the box.
			//If there is a form, the message should be surrounded by <form></form> tags.
			var html = '';
			var buttonsHTML = '';
			if (isForm || isDownload || object.upload) {
				html += 
					'<form id="jqmodal_form" action="' + htmlspecialchars(URLBasePath + tribiqSK.actionTarget + '&_sk_form_submission=1') + '"' +
						' onsubmit="get(\'preloader_circle\').style.visibility = \'visible\';"' +
						' target="tribiq_iframe" method="post"' + (object.upload? ' enctype="multipart/form-data"' : '') + '>';
				
				for (r in tribiqSK.actionRequests) {
					html += '<input type="hidden" value="' + htmlspecialchars(tribiqSK.actionRequests[r]) + '" name="' + htmlspecialchars(r) + '"/>';
				}
			}
			
			//HTML escape the message if needed
			if (isHTML) {
				html += message;
			} else {
				html += $.trim(htmlspecialchars(message)).replace(/\n/g, '<br/>');
			}
			
			//If there is a form, the confirm button should submit the form...
			if (isForm || isDownload) {
				html += '</form>';
				
				buttonsHTML =
					'<input type="button" class="submit_selected" value="' + htmlspecialchars(object.ajax.confirm.button_message) + '" onclick="get(\'jqmodal_form\').submit();"/>';
			//...otherwise it should launch a syncronous AJAX request.
			} else {
				buttonsHTML =
					'<input type="button" class="submit_selected" value="' + htmlspecialchars(object.ajax.confirm.button_message) + '" onclick="tribiqSK.action2();"/>';
			}
			
			buttonsHTML +=
				'<input type="button" class="submit" value="' + htmlspecialchars(object.ajax.confirm.cancel_button_message) + '"/>';
			
			
			tribiqA.floatingBox(html, buttonsHTML, object.ajax.confirm.message_type);
		
		//If there is no confirmation then do the action straight away
		} else {
			tribiqSK.action2();
		}
		
	
	} else if (engToBoolean(object.do_csv_export)) {
		tribiqSK.doCSVExport();
	}
	

}

//Apply merge fields to a message, using the currently selected item's columns as fields
tribiqSK.applyMergeFields = function(string, escapeHTML, i, keepNewLines) {
	var string2 = string;
	
	if (i === undefined) {
		i = tribiqSK.getKeyId();
	}
	
	for (c in tribiqSK.focus.items[i]) {
		var columnValue = tribiqSK.columnValue(i, c, !escapeHTML);
		
		if (!keepNewLines && columnValue !== undefined) {
			columnValue = columnValue.replace(/\n/g, ' ');
		}
		
		while (string != (string2 = string.replace('[[' + c + ']]', columnValue))) {
			string = string2;
		}
	}
	return string;
}

tribiqSK.pickItems = function(path, keyIn, row) {
	
	//Remove any "parent__" paramaters from the key, which are going to be just dupicated here
	var key = {};
	for (var i in keyIn) {
		if (i == 'id') {
			key[i] = keyIn[i];
		} else if (i.substr(0, 8) != 'parent__') {
			key['child__' + i] = keyIn[i];
		}
	}
	
	if (tribiqSK.postPickItemsObject) {
		if (tribiqSK.pickItemsItemLevel) {
			key.id2 = key.id;
			delete key.id;
		}
		tribiqSK.action(tribiqSK.postPickItemsObject, true, undefined, undefined, key);
	
	} else if (tribiqSK.actionTarget) {
		if (tribiqSK.pickItemsItemLevel) {
			tribiqSK.actionRequests.id2 = key.id;
			for (k in key) {
				if (k != 'id') {
					tribiqSK.actionRequests[k] = key[k];
				}
			}
		} else {
			for (k in key) {
				tribiqSK.actionRequests[k] = key[k];
			}
		}
		
		tribiqSK.action2();
	}
	
	tribiqSK.postPickItemsObject = false;
}

tribiqSK.action2 = function() {
	if (tribiqSK.actionTarget) {
		//Number each request that is made, so we can tell which ones are outdated
		var goNum = ++tribiqSK.goNum;
		get('preloader_circle').style.visibility = 'visible';
		
		$.post(URLBasePath + tribiqSK.actionTarget, tribiqSK.actionRequests, function(message) {
			//Check that this isn't an out-of-date request that has come in syncronously via AJAX
			if (goNum != tribiqSK.goNum) {
				return;
			}

			if (message) {
				if (tribiqA.showMessage(message) === false) {
					return;
				}
			}
			
			tribiqSK.selectCreatedIds();
		}, 'text');
	}
	tribiqSK.actionTarget = false;
	delete tribiqSK.actionRequests;
}

tribiqSK.selectCreatedIds = function() {
	var newIds;
	if (newIds = tribiq.nonAsyncAJAX(URLBasePath + 'plugins.php?__method_call__=getNewId', true, true)) {
		tribiqSK.deselectAllItems();
		tribiqSK.selectedItems = newIds;
		tribiqSK.saveSelection();
	}
	
	get('preloader_circle').style.visibility = 'hidden';
	
	
	var items = '';
	for (var i in tribiqSK.selectedItems) {
		items += (items? ',' : '') + i;
	}
	
	if (newIds && items && !tribiqSK.inCloseUpView) {
		//Make sure show the specific items if possible
		tribiqSK.go(tribiqSK.path + '//' + items);
	} else {
		//Otherwise use a normal refresh.
		tribiqSK.refresh();
	}
}


//Refresh the current view
tribiqSK.refreshIsPeriodic = false;
tribiqSK.refresh = function(periodic) {
	if (tribiqSK.uploading) {
		return false;
	}
	
	tribiqSK.stopRefreshing();
	
	tribiqSK.refreshToPage = tribiqSK.page;
	tribiqSK.refreshIsPeriodic = periodic;
	
	if (tribiqSK.inCloseUpView) {
		tribiqSK.go(tribiqSK.path + '//' + tribiqSK.closeUpViewItem + '/');
	} else {
		tribiqSK.go(tribiqSK.path);
	}
}

//Reload the storekeeper, making sure to go via the admin login page in case a login/db_update is needed
tribiqSK.reload = function(hash, dontAutoDetectMode) {
	if (tribiqSK.uploading) {
		return false;
	}

	var task = '';
	
	if (hash === undefined) {
		hash = tribiq.currentHash.replace('#', '');
	}
	
	if (!dontAutoDetectMode) {
		if (hash == 'tribiq__administration//site_reset') {
			task = 'site_reset';
		
		} else if (hash.substr(0, 30) == 'tribiq__administration/backups') {
			task = 'restore';
		
		} else {
			task = 'reload_sk';
		}
	}

	window.location.href = URLBasePath + 'admin/welcome.php?task=' + task + '&sk=' + encodeURIComponent(hash);
}

tribiqSK.encodeItemIdForStorekeeper = function(id) {
	return '~' + encodeURIComponent(id).replace(/%/g, '~');
}

tribiqSK.refreshToShowItem = function(itemId) {
	if (tribiqSK.uploading) {
		return false;
	}
	
	//If this is the admin panel, do a SK reload instead
	if (tribiqSK.path == 'tribiq__administration') {
		tribiqSK.reload();
		return;
	}
	
	//Escape the item id if needed
	if (tribiqSK.focus.db_items && tribiqSK.focus.db_items.encode_id_column) {
		itemId = tribiqSK.encodeItemIdForStorekeeper(itemId);
	}
	
	var itemIds = (itemId + '').split(',');
	tribiqSK.selectedItems = {};
	
	for (var i in itemIds) {
		tribiqSK.selectedItems[itemIds[i]] = true;
	}
	
	tribiqSK.saveSelection();
	
	tribiqSK.stopRefreshing();
	if (tribiqSK.inCloseUpView) {
		tribiqSK.go(tribiqSK.path + '//' + itemId + '/');
	} else {
		tribiqSK.go(tribiqSK.path + '//' + itemId);
	}
}

//Keep refreshing panels that can change
tribiqSK.refreshing = false;
tribiqSK.periodicRefresh = function() {
	if (tribiqSK.uploading) {
		return false;
	}
	
	tribiqSK.stopRefreshing();
	tribiqSK.refreshing = tribiq.setTimeout('tribiqSK.refresh(true)', tribiqSK.periodicRefreshTime);
}

tribiqSK.stopRefreshing = function() {
	if (tribiqSK.refreshing) {
		clearTimeout(tribiqSK.refreshing);
	}
}




//WiP CSV export ability
tribiqSK.doCSVExport = function() {
	tribiqSK.CSVExport = true;
	tribiqSK.refresh();
}


//Get the current collection key, as well as the current item id(s) if this is something on the item level
tribiqSK.getKey = function(itemLevel) {
	
	key = {};
	if (tribiqSK.focus.key) {
		for (i in tribiqSK.focus.key) {
			if (!tribiqSK.isInfoTag(i)) {
				key[i] = tribiqSK.focus.key[i];
			}
		}
	}
	
	if (tribiqSK.refiner) {
		key.refinerId = tribiqSK.refiner.id;
		key.refinerName = tribiqSK.refiner.name;
	}
	
	if (tribiqSK.lastRefiners) {
		for (f in tribiqSK.lastRefiners) {
			key[f] = tribiqSK.lastRefiners[f];
		}
	}
	
	if (itemLevel && tribiqSK.selectedItems) {
		key.id = tribiqSK.getKeyId();
	}
	
	if (tribiqSK.selectMode && window.parent && window.parent.tribiq) {
		if (window.parent.tribiqSK) {
			var parentKey = window.parent.tribiqSK.getKey(true);
			for (i in parentKey) {
				key['parent__' + i] = parentKey[i];
			}
		} else if (window.parent.tribiq.cID) {
			key.parent__cID = window.parent.tribiq.cID;
			key.parent__cType = window.parent.tribiq.cType;
			key.parent__cVersion = window.parent.tribiq.cVersion;
		}
	}
	
	return key;
}

tribiqSK.getKeyId = function(limitOfOne) {
	
	var id = '';
	if (tribiqSK.selectedItems) {
		comma = '';
		id = '';
		for (var i in tribiqSK.selectedItems) {
			if (!tribiqSK.isInfoTag(i)) {
				if (limitOfOne) {
					return i;
				} else {
					id += comma + i;
					comma = ',';
				}
			}
		}
	}
	
	return id;
}


//From a given path, get part of the map object
tribiqSK.followPathOnMap = function(path, attribute, getLocation) {
	
	if (!path) {
		path = tribiqSK.defaultPath;
	}
	
	var focus = tribiqSK.map;
	var split = path.split('/');
	var from = false;
	var to = false;
	
	for (i in split) {
		var tag = split[i];
		
		if (tag === '') {
			continue;
		}
		
		if (i == 0) {
			from = to = tag;
		}
		
		if (focus[tag]) {
			focus = focus[tag];
		
		} else if (focus.panel && focus.panel[tag]) {
			focus = focus.panel[tag];
		
		} else if (focus.second_level_items && focus.second_level_items[tag]) {
			focus = focus.second_level_items[tag];
		
		} else if (focus.collection_buttons && focus.collection_buttons[tag]) {
			focus = focus.collection_buttons[tag];
		
		} else if (focus.item_buttons && focus.item_buttons[tag]) {
			focus = focus.item_buttons[tag];
		
		} else if (focus.waypoints && focus.waypoints[tag]) {
			focus = focus.waypoints[tag];
		
		} else if (focus.items && focus.items[tag]) {
			focus = focus.items[tag];
		
		} else {
			if (focus.panel) {
				focus = focus.panel;
				
				if (focus.second_level_items && focus.second_level_items[tag]) {
					focus = focus.second_level_items[tag];
				
				} else if (focus.collection_buttons && focus.collection_buttons[tag]) {
					focus = focus.collection_buttons[tag];
				
				} else if (focus.item_buttons && focus.item_buttons[tag]) {
					focus = focus.item_buttons[tag];
				
				} else if (focus.waypoints && focus.waypoints[tag]) {
					focus = focus.waypoints[tag];
				
				} else if (focus.items && focus.items[tag]) {
					focus = focus.items[tag];
				
				} else {
					return false;
				}
			} else {
				return false;
			}
		}
		
		if (focus.link && focus.link.path) {
			from = to;
			to = focus.link.path;
		}
	}
	
	if (focus.panel) {
		focus = focus.panel;
	}
	
	if (getLocation) {
		if (!focus.link) {
			return false;
		} else {
			return {from: from, to: to, branch: focus.link.branch, refiner: focus.link.refiner};
		}
	} else if (attribute) {
		return focus[attribute];
	} else {
		return focus;
	}
}

//Given a <link> tag, get where it came from and where it goes to
tribiqSK.getFromToFromLink = function(path) {
	return tribiqSK.followPathOnMap(path, false, true);
}




//  View Modes  //

tribiqSK.changeView = function(mode) {
	if (tribiqSK.uploading) {
		return false;
	}
	
	tribiqSK.prefs[tribiqSK.path].view_mode = mode;
	tribiqSK.view_mode = mode;
	
	tribiqSK.inCloseUpView = false;
	tribiqSK.scrollToPageTop = true;
	tribiqSK.drawPanel();
	
	tribiqSK.savePrefs();
	return false;
}

tribiqSK.showViewOptions = function(e) {
	if (tribiqSK.uploading) {
		return false;
	}
	
	if (!tribiqSK.focus.columns) {
		return false;
	}
	
	var width = 300;
	var left = 1;
	var top = 1;

	tribiqA.openBox(false, '', 'AdminViewModeOptions', e, width, left, top, false, 10, 'black');
	tribiqSK.setViewOptions();
	return false;
}

tribiqSK.setViewOptions = function(toggledColumn) {
	
	if (!tribiqSK.CSVExportEnabled) {
		tribiqSK.showCSVInViewOptions = false;
	}
	
	var html =
		'<div class="colsortheader">' + 
			'<a onclick="tribiqA.closeBox(\'AdminViewModeOptions\');"></a>' +
			tribiqA.phrase.colDisplaySort + 
		'</div>' +
		'<div class="colsortfields" id="colsortfields">';
	
	tribiqAB.shownTab = 'cp';
	tribiqAB.focus = {
		tab: 'cp',
		tabs: {
			cp: {
				edit_mode: {
					on: true,
					always_on: true
				},
				fields: {}
			}
		}
	};
	
	tribiqAB.focus.tabs.cp.fields.showcol__title_ = {
		ord: -3,
		full_width: true,
		pre_field_html: '<div class="colPickerTitle_show" style="">',
		snippet: {html: tribiqA.phrase.show},
		post_field_html: '</div>'
	};
	
	if (tribiqSK.showCSVInViewOptions) {
		tribiqAB.focus.tabs.cp.fields.showcsv__title_ = {
			ord: -2,
			same_row: true,
			pre_field_html: '<div class="colPickerTitle_csv">',
			snippet: {html: tribiqA.phrase.csv},
			post_field_html: '</div>'
		};
	}
	
	tribiqAB.focus.tabs.cp.fields.sortcol__title_ = {
		ord: -1,
		same_row: true,
		pre_field_html: '<div class="colPickerTitle_sort">',
		snippet: {html: tribiqA.phrase.sort},
		post_field_html: '</div>'
	};
	
	var colNo = 0, lastCol = false, lastColName = false;
	for (colNo in tribiqSK.sortedColumns) {
		var c = tribiqSK.sortedColumns[colNo];
		if (tribiqSK.isShowableColumn(c, false, tribiqSK.showCSVInViewOptions)) {
			
			var cansortcol =
				engToBoolean(tribiqSK.focus.columns[c].disallow_sorting) || (tribiqSK.server_side && !tribiqSK.focus.columns[c].db_column)?
					'' : ' cansortcol';
			
			tribiqAB.focus.tabs.cp.fields['showcol_' + c] = {
				ord: 10 * colNo + 3,
				same_row: true,
				type: 'checkbox',
				onclick: "tribiqSK.showHideColumn(this, '" + htmlspecialchars(c) + "');",
				value: tribiqSK.shownColumns[c],
				title: tribiqA.phrase.showCol,
				'class': tribiqSK.view_mode == 'grid' ||
						 engToBoolean(tribiqSK.focus.columns[c].always_show) ||
						 (tribiqSK.view_mode == 'summary' && c == tribiqSK.labelTag)?
						 	'notused' : ''
			};
			
			if (tribiqSK.showCSVInViewOptions) {
				delete tribiqAB.focus.tabs.cp.fields['showcol_' + c].onclick;
				
				if (engToBoolean(engToBoolean(tribiqSK.focus.columns[c].server_side_only))) {
					tribiqAB.focus.tabs.cp.fields['showcol_' + c].style = 'visibility: hidden;';
				} else {
					tribiqAB.focus.tabs.cp.fields['showcol_' + c].disabled = 'disabled';
				}
				
				var value = tribiqSK.shownColumns[c];
				
				if (tribiqSK.prefs[tribiqSK.path].shownColumnsInCSV !== undefined
				 && tribiqSK.prefs[tribiqSK.path].shownColumnsInCSV[c] !== undefined) {
					value = tribiqSK.prefs[tribiqSK.path].shownColumnsInCSV[c];
				}
				
				tribiqAB.focus.tabs.cp.fields['showcsv_' + c] = {
					ord: 10 * colNo + 4,
					same_row: true,
					type: 'checkbox',
					onclick: "tribiqSK.showHideColumnInCSV(this, '" + htmlspecialchars(c) + "');",
					value: value,
					'class': tribiqSK.view_mode == 'grid' ||
							 engToBoolean(tribiqSK.focus.columns[c].always_show) ||
							 (tribiqSK.view_mode == 'summary' && c == tribiqSK.labelTag)?
								'notused' : ''
				};
			}
			
			tribiqAB.focus.tabs.cp.fields['sortcol_' + c] = {
				ord: 10 * colNo + 5,
				same_row: true,
				type: 'toggle',
				onclick: cansortcol? "tribiqSK.changeSortOrder(this, '" + htmlspecialchars(c) + "');" : '',
				value: tribiqSK.sortBy == c? (tribiqSK.sortDesc? '\u25BC' : '\u25B2') : (cansortcol? '\u25B3' : ' '),
				pre_field_html: '<span class="sort_col_wrap" title="' + tribiqA.phrase.sortByCol + '">',
				post_field_html: '<label style="cursor: pointer;" for="sortcol_' + c + '">' + htmlspecialchars(tribiqSK.focus.columns[c].title) + '</label></span>'
			};
			
			if (tribiqSK.server_side && tribiqSK.focus.columns[c].db_column && !engToBoolean(tribiqSK.focus.columns[c].disallow_filtering)) {
				
				var filterType = false;
				var hidden = !tribiqSK.getFilterValue('shown', c);
				var hiddenPreviously = toggledColumn == c? !hidden : hidden;
				var invertLink =
					'<a class="' + (tribiqSK.getFilterValue('not', c)? 'not' : '') + '"' +
						' title="' + tribiqA.phrase.invertFilter + '"' +
						' onclick="tribiqSK.invertFilter(this, \'' + htmlspecialchars(c) + '\', \'' + htmlspecialchars(tribiqSK.focus.columns[c].format) + '\');">';
				
				if (engToBoolean(engToBoolean(tribiqSK.focus.columns[c].server_side_only))) {
					//Don't show any filters for columns you cannot usually see!
				
				} else if (tribiqSK.focus.columns[c].format == 'date' || tribiqSK.focus.columns[c].format == 'datetime') {
					filterType = 'date';
					tribiqAB.focus.tabs.cp.fields['date_after_col_' + c] = {
						ord: 10 * colNo + 6,
						label: tribiqA.phrase.after,
						type: 'date',
						_change_filter_on_change: true,
						value: tribiqSK.getFilterValue('date_after_col_', c),
						hidden: hidden,
						_h: hiddenPreviously
					};
					tribiqAB.focus.tabs.cp.fields['date_before_col_' + c] = {
						ord: 10 * colNo + 7,
						label: tribiqA.phrase.before,
						type: 'date',
						_change_filter_on_change: true,
						value: tribiqSK.getFilterValue('date_before_col_', c),
						hidden: hidden,
						_h: hiddenPreviously
					};
				
				} else if (tribiqSK.focus.columns[c].format == 'yes_or_no') {
					filterType = 'yes_or_no';
					
					//Attempt to add a colon to the title of the column
					var label = ('' + tribiqSK.focus.columns[c].title);
					if (label.indexOf(':') == -1) {
						label += ':';
						label = label.replace(/\s*:/, ':');
					}
					
					tribiqAB.focus.tabs.cp.fields['yes_or_no_' + c] = {
						ord: 10 * colNo + 6,
						row_class: 'yes_or_no',
						label: invertLink + htmlspecialchars(label) + '</a>',
						snippet: {
							html: invertLink + (tribiqSK.getFilterValue('not', c)? tribiqA.phrase.no : tribiqA.phrase.yes) + '</a>'
						},
						hidden: hidden,
						_h: hiddenPreviously
					};
				
				} else if (tribiqSK.focus.columns[c].format == 'enum' && tribiqSK.focus.columns[c].values) {
					filterType = 'enum';
					
					//Get the list of values for this column, adding on an empty value if one is not present
					var values = tribiqSK.focus.columns[c].values;
					if (!values[''] && !values[0]) {
						values = {};
						values[''] = tribiqA.phrase.selectListSelect;
						
						for (var v in tribiqSK.focus.columns[c].values) {
							values[v] = tribiqSK.focus.columns[c].values[v];
						}
					}
					
					tribiqAB.focus.tabs.cp.fields['enum_' + c] = {
						ord: 10 * colNo + 6,
						row_class: 'enum',
						label: invertLink + (tribiqSK.getFilterValue('not', c)? tribiqA.phrase.isnt : tribiqA.phrase.is) + '</a>',
						type: 'select',
						_change_filter_on_change: true,
						values: values,
						value: tribiqSK.getFilterValue('enum_', c),
						hidden: hidden,
						_h: hiddenPreviously
					};
				
				} else if (engToBoolean(tribiqSK.focus.columns[c].searchable)) {
					filterType = 'search';
					tribiqAB.focus.tabs.cp.fields['searchcol_' + c] = {
						ord: 10 * colNo + 6,
						label: invertLink + tribiqA.phrase.contains + '</a>',
						type: 'text',
						quick_validate_onkeyup: tribiqSK.searchDelayTime,
						value: tribiqSK.getFilterValue('searchcol_', c),
						hidden: hidden,
						_h: hiddenPreviously
					};
				}
				
				if (filterType) {
					tribiqAB.focus.tabs.cp.fields['togglefilter_' + c] = {
						ord: 10 * colNo + 2,
						same_row: true,
						pre_field_html: '<div class="togglefilter">',
						type: 'toggle',
						onclick: "tribiqSK.toggleFilter(this, '" + htmlspecialchars(c) + "', '" + filterType + "');",
						'class': hidden? 'togglefilter_closed' : 'togglefilter_open',
						value: hidden? ' ' : 'x',
						title: hidden? tribiqA.phrase.filterByCol : tribiqA.phrase.filterByColStop,
						post_field_html: '</div>'
					};
				}
			}
			
			tribiqAB.focus.tabs.cp.fields['up__' + c] = {
				ord: 10 * colNo,
				full_width: true,
				pre_field_html: '<div class="mov_col">',
				type: 'toggle',
				value: '\u25B3',
				style: 'visibility: hidden;',
				title: tribiqA.phrase.moveColForward
			};
			
			tribiqAB.focus.tabs.cp.fields['down__' + c] = {
				ord: 10 * colNo + 1,
				same_row: true,
				type: 'toggle',
				style: 'width: 11px; height: 9px; float: right; font-size: 7px;',
				value: '\u25BD',
				style: 'visibility: hidden;',
				title: tribiqA.phrase.moveColBack,
				post_field_html: '</div>'
			};
			
			if (lastCol !== false) {
				delete tribiqAB.focus.tabs.cp.fields['up__' + c].style;
				tribiqAB.focus.tabs.cp.fields['up__' + c].onclick = 'tribiqSK.switchColumnOrder(' + colNo + ', ' + lastCol + ')';
				delete tribiqAB.focus.tabs.cp.fields['down__' + lastColName].style;
				tribiqAB.focus.tabs.cp.fields['down__' + lastColName].onclick = 'tribiqSK.switchColumnOrder(' + colNo + ', ' + lastCol + ')';
			}
			lastCol = colNo;
			lastColName = c;
		}
	}
	
	
	var showResetButton = false;
	if (tribiqSK.prefs[tribiqSK.path]) {
		for (var p in tribiqSK.prefs[tribiqSK.path]) {
			if (p) {
				showResetButton = true;
				break;
			}
		}
	}
	
	if (showResetButton) {
		tribiqAB.focus.tabs.cp.fields.reset = {
			ord: 10 * colNo + 11,
			full_width: true,
			type: 'toggle',
			onclick: "tribiqSK.resetPrefs();",
			value: tribiqA.phrase.reset
		};
	}
	
	if (tribiqSK.CSVExportEnabled) {
		tribiqAB.focus.tabs.cp.fields.toggle_csv = {
			ord: 10 * colNo + 12,
			full_width: true,
			same_row: showResetButton,
			type: 'toggle',
			style: 'float: right; width: auto;',
			onclick: "tribiqSK.showCSVInViewOptions = !tribiqSK.showCSVInViewOptions; tribiqSK.setViewOptions();",
			'class': hidden? 'togglefilter_closed' : 'togglefilter_open',
			value: tribiqSK.showCSVInViewOptions? tribiqA.phrase.hideExport : tribiqA.phrase.showExport
		};
	}
	
	tribiqAB.sortTabs();
	html += tribiqAB.draw2(true);
	
	html += '</div>';
	
	get('tribiq_fbAdminViewModeOptions').innerHTML = html;
	tribiq.addJQueryElements('#tribiq_fbAdminViewModeOptions ');
	tribiqSK.size(true);
}

tribiqSK.showHideColumn = function(el, c) {
	//Ignore requests to hide columns that are forced on
	if (engToBoolean(tribiqSK.focus.columns[c].always_show)) {
		el.checked = true;
		return;
	}
	
	//Ensure someone can never hide the last column!
	var count = 0;
	for (i in tribiqSK.focus.columns) {
		if (!tribiqSK.isInfoTag(i)) {
			if (tribiqSK.shownColumns[i]) {
				++count;
			}
		}
	}
	
	if (count < 2 && !el.checked) {
		el.checked = true;
	}
	
	//Otherwise toggle this column's visibility
	if (el.checked) {
		tribiqSK.shownColumns[c] = true;
	} else {
		tribiqSK.shownColumns[c] = false;
	}
	
	if (!tribiqSK.inCloseUpView) {
		tribiqSK.drawPanel();
	}
	
	tribiqSK.prefs[tribiqSK.path].shownColumns = tribiqSK.shownColumns;
	tribiqSK.savePrefs();
}

tribiqSK.showHideColumnInCSV = function(el, c) {
	if (!tribiqSK.prefs[tribiqSK.path].shownColumnsInCSV) {
		tribiqSK.prefs[tribiqSK.path].shownColumnsInCSV = {};
	}
	
	if (el.checked) {
		tribiqSK.prefs[tribiqSK.path].shownColumnsInCSV[c] = true;
	} else {
		tribiqSK.prefs[tribiqSK.path].shownColumnsInCSV[c] = false;
	}
	
	tribiqSK.savePrefs();
}

tribiqSK.switchColumnOrder = function(i, j) {
	var tmp = tribiqSK.sortedColumns[i];
	tribiqSK.sortedColumns[i] = tribiqSK.sortedColumns[j];
	tribiqSK.sortedColumns[j] = tmp;
	
	tribiqSK.prefs[tribiqSK.path].sortedColumns = tribiqSK.sortedColumns;
	
	tribiqSK.drawPanel();
	tribiqSK.setViewOptions();
	
	tribiqSK.savePrefs();
}

tribiqSK.resetPrefs = function() {
	tribiqSK.prefs[tribiqSK.path] = {};
	tribiqSK.savePrefs();
	
	tribiqA.closeBox('AdminViewModeOptions');
	
	tribiqSK.refresh();
}



tribiqSK.checkIfColumnPickerChangesAreAllowed = function(c) {
	//Don't allow the sort order to be changed in close up view
	if (tribiqSK.inCloseUpView) {
		return false;
	}
	
	//Don't allow the sort order to be changed when reordering
	if (tribiqSK.focus.reorder && tribiqSK.focus.reorder.column) {
		return false;
	}
	
	return true;
}

tribiqSK.changeSortOrder = function(el, c) {
	if (!tribiqSK.checkIfColumnPickerChangesAreAllowed(c)) {
		return;
	}
	
	if (tribiqSK.sortBy == c) {
		tribiqSK.sortDesc = !tribiqSK.sortDesc;
	} else {
		tribiqSK.sortBy = c;
		tribiqSK.sortDesc = false;
	}
	
	tribiqSK.prefs[tribiqSK.path].sortBy = tribiqSK.sortBy;
	tribiqSK.prefs[tribiqSK.path].sortDesc = tribiqSK.sortDesc;
	
	if (tribiqSK.server_side) {
		tribiqSK.goToPage(1);
	} else {
		tribiqSK.sortItems();
	}
	
	tribiqSK.setViewOptions();
	tribiqSK.savePrefs();
}

tribiqSK.possibleFilters = {searchcol_: true, enum_: true, date_after_col_: true, date_before_col_: true, yes_or_no_: true};

tribiqSK.toggleFilter = function(el, c, type) {
	if (!tribiqSK.checkIfColumnPickerChangesAreAllowed(c)) {
		return;
	}
	
	//Toogle whether this filter is hidden or shown
	var shown = !tribiqSK.getFilterValue('shown', c);
	tribiqSK.setFilterValue('shown', c, shown);
	
	//Unset values of hidden filters
	var refreshNeeded = false;
	if (!shown) {
		for (var filter in tribiqSK.possibleFilters) {
			if (tribiqSK.getFilterValue(filter, c)) {
				tribiqSK.setFilterValue(filter, c, '')
				refreshNeeded = true;
			}
		}
		tribiqSK.setFilterValue('not', c, false);
	}
	
	//yes_or_no type filters should turn on by default if activated
	if (type == 'yes_or_no') {
		if (shown) {
			refreshNeeded = true;
			tribiqSK.setFilterValue('yes_or_no_', c, 1);
		}
	}
	
	
	tribiqSK.setViewOptions(c);
	
	//Animate in/out the field that was just shown/hidden
	tribiqAB.hideShowFields(function() {
		tribiqSK.size(true);
		//Focus a text field straight away if we can
		if (get('searchcol_' + c) && $(get('searchcol_' + c)).is(':visible')) {
			get('searchcol_' + c).focus();
		}
	});
	
	
	//If a filter with a value set was hidden, we will need to refresh to update the panel with the changes
	if (refreshNeeded) {
		tribiqSK.refresh();
	}
}

tribiqSK.invertFilter = function(el, c, format) {
	if (!tribiqSK.checkIfColumnPickerChangesAreAllowed(c)) {
		return;
	}
	
	//Toogle whether this filter is inverted or not
	var not = !tribiqSK.getFilterValue('not', c);
	tribiqSK.setFilterValue('not', c, not);
	
	//Toggle the class on the label
	
	if (format == 'yes_or_no' || format == 'enum') {
		//enum / yes_or_no type fields will need the html redrawn to do this
		tribiqSK.setViewOptions();
	} else {
		el.className = not? 'not' : '';
	}
	
	//If a filter with a value set was inverted, we will need to refresh to update the panel with the changes
	tribiqSK.refreshIfFilterSet(c);
}

tribiqSK.refreshIfFilterSet = function(c) {
	for (var filter in tribiqSK.possibleFilters) {
		if (tribiqSK.getFilterValue(filter, c)) {
			tribiqSK.refresh();
		}
	}
}

tribiqSK.getFilterValue = function(filter, c) {
	if (tribiqSK.branches[tribiqSK.branches.length-1].filters[tribiqSK.path]
	 && tribiqSK.branches[tribiqSK.branches.length-1].filters[tribiqSK.path][c]
	 && tribiqSK.branches[tribiqSK.branches.length-1].filters[tribiqSK.path][c][filter]) {
		return tribiqSK.branches[tribiqSK.branches.length-1].filters[tribiqSK.path][c][filter];
	} else {
		return '';
	}
}

tribiqSK.setFilterValue = function(filter, c, value) {
	if (!tribiqSK.branches[tribiqSK.branches.length-1].filters[tribiqSK.path]) {
		tribiqSK.branches[tribiqSK.branches.length-1].filters[tribiqSK.path] = {};
	}
	if (!tribiqSK.branches[tribiqSK.branches.length-1].filters[tribiqSK.path][c]) {
		tribiqSK.branches[tribiqSK.branches.length-1].filters[tribiqSK.path][c] = {};
	}
	tribiqSK.branches[tribiqSK.branches.length-1].filters[tribiqSK.path][c][filter] = value;
}

tribiqSK.changeFilters = function() {
	
	//Don't allow the filters to be changed in close up view
	if (tribiqSK.inCloseUpView) {
		tribiqSK.setViewOptions();
		return;
	}
	
	//Don't allow the filters to be touched when reordering
	if (tribiqSK.focus.reorder && tribiqSK.focus.reorder.column) {
		tribiqSK.setViewOptions();
		return;
	}
	
	
	var value;
	for (var c in tribiqSK.focus.columns) {
		if (tribiqSK.isShowableColumn(c)) {
			for (var filter in tribiqSK.possibleFilters) {
				if (get(filter + c)) {
					if (value = tribiqAB.readField(filter + c)) {
						tribiqSK.setFilterValue(filter, c, value);
					} else {
						tribiqSK.setFilterValue(filter, c, '');
					}
				}
			}
		}
	}
	
	
	tribiqSK.refresh();
}

tribiqSK.hideViewOptions = function(e) {
	tribiqA.closeBox('AdminViewModeOptions');
	return false;
}




//refine tags added by the CMS for location/tracking purposes
tribiqSK.isInfoTag = function(i) {
	return i == 'back_link'
		|| i == 'class_name'
		|| i == 'count'
		|| i == 'ord';
}

tribiqSK.isShowableColumn = function(c, shown, allowServerSide) {
	if (shown) {
		return tribiqSK.shownColumns[c] && tribiqSK.isShowableColumn(c);
	} else {
		return tribiqSK.focus.columns[c] && !tribiqSK.isInfoTag(c) && tribiqSK.focus.columns[c].title && (allowServerSide || !engToBoolean(tribiqSK.focus.columns[c].server_side_only));
	}
}


//  Sorting Functions  //

tribiqSK.createSortArray = function(name, toSort, column, desc) {
	//Build an array to sort, containing:
		//0: The item's actual index
		//1: The value to sort by
		//2: Whether this value is numeric
	var value;
	var numeric;
	tribiqSK[name] = [];
	if (tribiqSK.focus[toSort]) {
		for (i in tribiqSK.focus[toSort]) {
			if (!tribiqSK.isInfoTag(i) && tribiqSK.focus[toSort][i]) {
				//Check if the value is a number, and if so make sure that it is numeric so it is sorted numericaly
				value = tribiqSK.focus[toSort][i][column];
				numeric = 1*value;
				
				if (value == numeric) {
					tribiqSK[name].push([i, numeric, true]);
				} else if (value) {
					tribiqSK[name].push([i, value.toLowerCase(), false]);
				} else {
					tribiqSK[name].push([i, 0, true]);
				}
			}
		}
	}
	
	//Sort this array
	if (desc) {
		tribiqSK[name].sort(tribiqSK.sortArrayDesc);
	} else {
		tribiqSK[name].sort(tribiqSK.sortArray);
	}
	
	//Remove fields that were just there to help sort
	for (i in tribiqSK[name]) {
		tribiqSK[name][i] = tribiqSK[name][i][0];
	}
}

//A copy and paste of the above function, for the top level menu items
tribiqSK.createSortArrayTopLevel = function(name) {
	var value;
	var numeric;
	tribiqSK[name] = [];
	for (i in tribiqSK.map) {
		if (!tribiqSK.isInfoTag(i)) {
			//Check if the value is a number, and if so make sure that it is numeric so it is sorted numericaly
			value = tribiqSK.map[i].ord;
			numeric = 1*value;
			
			if (value == numeric) {
				tribiqSK[name].push([i, numeric, true]);
			} else if (value) {
				tribiqSK[name].push([i, value.toLowerCase(), false]);
			} else {
				tribiqSK[name].push([i, 0, true]);
			}
		}
	}
	
	//Sort this array
	tribiqSK[name].sort(tribiqSK.sortArray);
	
	//Remove fields that were just there to help sort
	for (i in tribiqSK[name]) {
		tribiqSK[name][i] = tribiqSK[name][i][0];
	}
}

//A copy and paste of the above function, for the second level menu items
tribiqSK.createSortArraySecondLevel = function(name, topLevelItem) {
	var value;
	var numeric;
	
	tribiqSK[name] = [];
	if (tribiqSK.map[topLevelItem].second_level_items) {
		for (i in tribiqSK.map[topLevelItem].second_level_items) {
			if (!tribiqSK.isInfoTag(i)) {
				//Check if the value is a number, and if so make sure that it is numeric so it is sorted numericaly
				value = tribiqSK.map[topLevelItem].second_level_items[i].ord;
				numeric = 1*value;
				
				if (value == numeric) {
					tribiqSK[name].push([i, numeric, true]);
				} else if (value) {
					tribiqSK[name].push([i, value.toLowerCase(), false]);
				} else {
					tribiqSK[name].push([i, 0, true]);
				}
			}
		}
	}
	
	//Sort this array
	tribiqSK[name].sort(tribiqSK.sortArray);
	
	//Remove fields that were just there to help sort
	for (i in tribiqSK[name]) {
		tribiqSK[name][i] = tribiqSK[name][i][0];
	}
}

//Given two elements from the above function, say which order they should be in
tribiqSK.sortArray = function(a, b) {
	if (a[1] === b[1]) {
		//If their values are the same type and identical, say that they're identical
		return 0;
	
	} else if (a[2]? b[2] : !b[2]) {
		//If they're the same type, use a < to work out which is smallest
		return a[1] < b[1]? -1 : 1;
	
	} else {
		//Otherwise order by numeric data first, then strings
		return a[2]? -1 : 1;
	}
}

tribiqSK.sortArrayDesc = function(a, b) {
	return tribiqSK.sortArray(b, a);
}



tribiqSK.dateDays = ['Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday'];
tribiqSK.dateMonths = ['January','Feburary','March','April','May','June','July','August','September','October','November','December'];

tribiqSK.columnValue = function(i, c, dontHTMLEscape) {
	
	if (tribiqSK.focus.columns[c] === undefined) {
		return '';
	}
	
	var value = tribiqSK.focus.items[i][c];
	if (value === undefined || value === false || value === null) {
		value = '';
	} else {
		value = '' + value;
	}
	
	
	//Is this item an item link..?
	var item_link = tribiqSK.focus.columns[c].item_link;
	if (item_link) {
		var item = false;
		if (item_link == 'menu_item') {
			if (value && (item = tribiqSK.menuItems.items) && (item = item[value]));
			
		} else if (item_link == 'content_item' || item_link == 'content_item_or_url') {
			var cTypeAndId = value.split('_');
			if (cTypeAndId[0]
			 && cTypeAndId[0] != 'null'
			 && cTypeAndId[1]
			 && (item = tribiqSK.contentItems[cTypeAndId[0]])
			) {
				if ((item = item.items) && (item = item[cTypeAndId[1]]));
				
			} else if (item_link == 'content_item_or_url'
					&& value
					&& value.substr(0, 1) != '_'
					&& value.substr(1, 2) != '_'
			) {
				item = {name: value, frontend_link: value}
			
			} else {
				return '';
			}
		}
		
		if (item) {
			if (dontHTMLEscape) {
				return item.name;
			} else {
				var html = '<table class="no_border item_link"><tr>';
				
				if (item.css_class) {
					html += '<td><div class="listicon ' + item.css_class + '"></div></td>';
				}
				
				html += '<td class="item_link_text" width="95%">' + htmlspecialchars(item.name) + '</td>';
				
				if (item.deep_link || item.frontend_link) {
					html +=
						' <td class="item_link">';
					
					if (item.frontend_link) {
						html +=
								'<a href="' + htmlspecialchars(tribiq.addBasePath(item.frontend_link)) + '" target="_blank">' +
									'<img src="images/store_keeper/show_in_front_end_btn.gif"/>' +
								'</a>';
					}
					
					if (item.deep_link) {
						html +=
								'<a href="storekeeper.php#' + htmlspecialchars(item.deep_link) + '" target="_blank">' +
									'<img src="images/store_keeper/open_in_new_sk_window_btn.gif"/>' +
								'</a>';
					}
					
					html +=
						'</td>';
				}
				
				return html + '</tr></table>';
			}
		}
	}
	
	
	//Otherwise check to see if this is a formated column
	var format = tribiqSK.focus.columns[c].format;
	
	//Most formats allow additional text seperated by a space
	var extra = '';
	
	if (format) {
		if (value && (format != 'date' && format != 'datetime')) {
			var pos = value.indexOf(' ');
			if (pos != -1) {
				extra = value.substr(pos);
				value = value.substr(0, pos);
			}
		}
		
		var numeric = 1*tribiqSK.focus.items[i][c] == tribiqSK.focus.items[i][c];
		var length = ('' + tribiqSK.focus.items[i][c]).length;
		
		if (format == 'true_or_false') {
			value = engToBoolean(value)? tribiqA.phrase.tru : tribiqA.phrase.fal;
			
		} else if (format == 'yes_or_no') {
			value = engToBoolean(value)? tribiqA.phrase.yes : tribiqA.phrase.no;
			
		} else if (format == 'enum' && tribiqSK.focus.columns[c].values && tribiqSK.focus.columns[c].values[value]) {
			value = tribiqSK.focus.columns[c].values[value];
			
		} else if (format == 'plugin_name') {
			if (!value) {
				value = tribiqA.phrase.core;
			} else if (tribiqSK.pluginNames[value]) {
				value = tribiqSK.pluginNames[value];
			} else {
				value = '';
			}
			
		} else if (format == 'filesize' && value == 1*value) {
			value = tribiqA.formatFilesizeNicely(value, 1);
			
		} else if (format == 'language_local_name' && tribiqSK.languageNames[value]) {
			value = tribiqSK.languageNames[value];
			
		} else if (format == 'language_local_name_with_id' && tribiqSK.languageNames[value]) {
			value = tribiqSK.languageNames[value] + ' (' + value + ')';
			
		} else if (format == 'date' || format == 'datetime') {
			var date = false, unix = false;
			
			if (value && 1*value == 1*value) {
				//Format a date as a UNIX time (seconds since 1st Jan 1970)
				date = new Date(1000 * value);
				unix = true;
			
			} else if (length == 10 || length == 19) {
				//Format a date (in raw MySQL format) using the MySQL's syntax
				var dates = value.replace(/[:-]/g, ' ').split(' ');
				
				if (format == 'datetime' && length == 19) {
					date = new Date(1*dates[0], 1*dates[1]-1, 1*dates[2], 1*dates[3], 1*dates[4], 1*dates[5]);
				} else {
					date = new Date(1*dates[0], 1*dates[1]-1, 1*dates[2]);
				}
			}
			
			if (date && date.getDate()) {
				value = tribiq.formatDate(date);
				
				if (format == 'datetime') {
					value += ' ' + date.getHours() + ':' + tribiq.rightHandedSubStr('0' + date.getMinutes(), 2);
					
					if (unix && date.getTimezoneOffset() != (new Date()).getTimezoneOffset()) {
						if (date.getTimezoneOffset() > 0) {
							value += ' (UCT -' + (date.getTimezoneOffset() / 60) + ')';
						} else {
							value += ' (UCT +' + (date.getTimezoneOffset() / -60) + ')';
						}
					}
				}
			
			} else {
				value = '';
			}
		}
		
		value = value + extra;
	}
	
	
	if (dontHTMLEscape) {
		return value;
	
	} else if (tribiqSK.focus.columns[c].length_limit && value.length > tribiqSK.focus.columns[c].length_limit) {
		return '<span class="tooltip" title="' + htmlspecialchars(value) + '">' +
					htmlspecialchars(value.substr(0, tribiqSK.focus.columns[c].length_limit)) +
				'...</span>';
	
	} else {
		return htmlspecialchars(value);
	}
}

tribiqSK.rowCssClass = function(i) {
	if (tribiqSK.focus.items[i] && tribiqSK.focus.items[i].row_css_class) {
		return ' ' + tribiqSK.focus.items[i].row_css_class;
	} else {
		return '';
	}
}

tribiqSK.columnCssClass = function(i, c) {
	if (tribiqSK.focus.items[i] && tribiqSK.focus.items[i][c + '__css_class']) {
		return ' ' + tribiqSK.focus.items[i][c + '__css_class'];
	} else {
		return '';
	}
}




//	Display Functions  //

tribiqSK.setBackButton = function() {
	var title = tribiqSK.getBackButtonTitle();
	
	if (title === false) {
		get('backButton').style.visibility = 'hidden';
	} else {
		get('backButton').style.visibility = 'visible';
		get('backButtonText').innerHTML = htmlspecialchars(title);
		
		get('backButtonText').style.fontSize = Math.min(12, Math.round(1200 / ('' + title).length)) + 'px';

	}
	
}

tribiqSK.getBackButtonTitle = function() {
	return tribiqSK.getFromLastPanel(tribiqSK.branches.length-1, tribiqSK.path, 'title');
}

tribiqSK.getSelectedItemFromLastPanel = function(path) {
	var i, selectedItems, selectedItem = false;
	if (selectedItems = tribiqSK.getFromLastPanel(tribiqSK.branches.length-1, tribiqSK.path, 'selected_items', true)) {
		if (selectedItems = selectedItems[path]) {
			for (i in selectedItems) {
				selectedItem = i;
				break;
			}
		}
	}
	
	return selectedItem;
}

tribiqSK.getFromLastPanel = function(branchLevel, path, thing, branchBelow) {

	//Is this the start of the branch?
	if (branchLevel > 0 && path == tribiqSK.branches[branchLevel].to) {
		//Set the title of the back button to the last panel in the previous branch
		if (branchLevel > 1 && tribiqSK.branches[branchLevel-1].bypasses[tribiqSK.branches[branchLevel].from]) {
			//However if the previous panel was a bypass, call tribiqSK.getFromLastPanel() again
			//to get the title of the panel before that
			return tribiqSK.getFromLastPanel(branchLevel-1, tribiqSK.branches[branchLevel].from, thing, branchBelow);
		} else {
			//Don't allow the admin to go above the min-path in select mode
			if (tribiqSK.selectMode && path == window.tribiqSKMinPath && (window.tribiqSKDisallowIdenticalMinPath || tribiqSK.branches[branchLevel].from != path)) {
				return false;
			
			//A match is found.
			//Return eiher the current branch or the branch below if branchBelow is set
			} else if (branchBelow) {
				return tribiqSK.branches[branchLevel-1][thing];
			} else {
				return tribiqSK.branches[branchLevel][thing];
			}
		}
	
	//If the previous panel was a bypass, call tribiqSK.getFromLastPanel() again
	//to get the title of the panel before that
	} else if (tribiqSK.branches[branchLevel].bypasses[backLink]) {
		return tribiqSK.getFromLastPanel(branchLevel, backLink, thing, branchBelow);
	
	} else if (tribiqSK.branches[branchLevel][thing]) {
		return tribiqSK.branches[branchLevel][thing];
	
	} else {
		//Otherwise check to see if there is a panel naturally above the current panel and use the title from there
		var backLink = tribiqSK.followPathOnMap(path, 'back_link');
		
		if (!backLink) {
			return false;
		
		//Don't allow the admin to go above the min-path in select mode
		} else if (tribiqSK.selectMode && path == window.tribiqSKMinPath && (window.tribiqSKDisallowIdenticalMinPath || backLink != path)) {
			return false;
		
		//Otherwise just attempt to read the title off from the map
		} else {
			return tribiqSK.followPathOnMap(backLink, thing);
		}
	}
}



tribiqSK.setBackButtonInCloseUpView = function() {
	get('backButton').style.visibility = 'visible';
	get('backButtonText').innerHTML = htmlspecialchars(tribiqSK.focus.title);
	
	get('backButtonText').style.fontSize = Math.min(12, Math.round(1300 / ('' + tribiqSK.focus.title).length)) + 'px';

}

tribiqSK.setTopLevelItems = function() {
	var html = '';
	
	
	
	//Work out which top level item was selected, either by looking at the first branch, or if there are no
	//branches, the current path.
	var path = tribiqSK.path;
	if (tribiqSK.branches.length > 1) {
		path = tribiqSK.branches[1].from;
	}
	
	for (itemNo in tribiqSK.sortedTopLevelItems) {
		var i = tribiqSK.sortedTopLevelItems[itemNo];
			
		var selected = (path + '/').substr(0, (i + '').length + 1) == i + '/';
		
		html += 
			'<div class="item" onclick="tribiqSK.topLevelClick(\'' + i + '\');">' +
				'<div class="legend ' + (selected? 'legend_on' : '') + '">' +
					'<a class="' + htmlspecialchars(tribiqSK.map[i].css_class) + '"' +
						' title="' + htmlspecialchars(tribiqSK.map[i].tooltip) + '">' +
							'<span>' +
								htmlspecialchars(tribiqSK.map[i].name) +
							'</span>' +
					'</a>' +
				'</div>' +
			'</div>';
		
		if (selected) {
			tribiqSK.createSortArraySecondLevel('sortedSecondLevelItems', i);
			
			for (jtemNo in tribiqSK.sortedSecondLevelItems) {
				var j = tribiqSK.sortedSecondLevelItems[jtemNo];
				
				var selected = (path + '/').substr(0, (i + '/' + j).length + 1) == i + '/' + j + '/';
				
				html += 
					'<div class="item second_level_item" onclick="tribiqSK.topLevelClick(\'' + i + '\', \'' + j + '\');">' +
						'<div class="legend ' + (selected? 'legend_on' : '') + '">' +
							'<a title="' + htmlspecialchars(tribiqSK.map[i].second_level_items[j].tooltip) + '">' +
								(tribiq.browserIsIE()? '&#9658; ' : '&#9656; ') +
								'<span>' +
									htmlspecialchars(tribiqSK.map[i].second_level_items[j].name) +
								'</span>' +
							'</a>' +
						'</div>' +
					'</div>';
			}
		}
	}
	
	get('leftColumn').innerHTML = html;
	tribiq.tooltips('#leftColumn a[title]', {defaultPosition: 'left'});
}

tribiqSK.setBanner = function() {
	if (tribiqSK.focus.banner) {
		get('storeHeaderGraphic').style.display = 'block';
		get('storeHeaderGraphic').style.backgroundImage = 'url(\'' + htmlspecialchars(tribiqSK.focus.banner) + '\')';
	} else {
		get('storeHeaderGraphic').style.display = 'none';
	}
	
	tribiqSK.size(true);
}

tribiqSK.setCollectionButtons = function() {
	var barHTML = tribiqSK.moreCollActions = '';
	
	for (itemNo in tribiqSK.sortedCollectionButtons) {
		var i = tribiqSK.sortedCollectionButtons[itemNo];
		var button = tribiqSK.focus.collection_buttons[i];
		
		//Check if this button should be hidden in select mode (or hidden if not in select mode)
		//Also, automatically hide a button in select mode if it calls a floating box or a front-end link
		if ((tribiqSK.selectMode && button.hide_in_select_mode)
	//	||  (tribiqSK.selectMode && button.pick_items)
	//	||  (tribiqSK.selectMode && button.floating_box)
		||  (tribiqSK.selectMode && button.frontend_link)
		|| (!tribiqSK.selectMode && button.select_mode_only)) {
			continue;
		}
			
		//Check if this button should only be shown in a certain refiner
		if (button.only_show_on_refiner && (
			!tribiqSK.refiner || tribiqSK.refiner.name != button.only_show_on_refiner
		)) {
			continue;
		}
		
		//In select mode, don't show a link to a panel that we can't reach due to a max-path restriction
		if (button.link && tribiqSK.selectMode && window.tribiqSKMaxPath !== false
		 && tribiqSK.path == window.tribiqSKMaxPath && button.link.path != tribiqSK.path) {
			continue;
		}
		
		//If this is an export csv button, remember that CSV exports are available
		if (engToBoolean(button.do_csv_export)) {
			tribiqSK.CSVExportEnabled = true;
		}
		
		//Hide the export to csv button if there are no rows
		if (!tribiqSK.searchMatches && (engToBoolean(button.do_csv_export) || engToBoolean(button.hide_when_no_search_results))) {
			continue;
		}
		
		var html = '<div class="' + (button.css_class? 'button ' + htmlspecialchars(button.css_class) : 'more_action') + '"' +
				' title="' + htmlspecialchars(button.tooltip) + '"';
		
		//Add a flash upload button, if Flash is enabled
		if (button.upload && tribiq.flashMajorVersion() >= 9) {
			button.upload.itemLevel = false;
			html += ' style="width: 29px; height: 28px;">'
					+ '<input type="file" name="uploadify" id="uploadify_' + tribiqSK.setUploadNum + '_' + tribiqSK.uploads.length + '" style="height: 28px; width: 28px;"/>';
			tribiqSK.uploads[tribiqSK.uploads.length] = button.upload;
		} else {
			html += ' onclick="tribiqSK.collectionButtonClick(\'' + i + '\');">';
		}
		
		if (!button.css_class && !button.upload && button.name) {
			html += htmlspecialchars(button.name);
		}
		
		html += '</div>';
		
		if (!button.css_class && !button.upload && button.name) {
			//Show the button as a row in a "more actions" dropdown
			tribiqSK.moreCollActions += html;
		
		} else {
			//Show the button as a button in the toolbar
			if (barHTML) {
				barHTML += '<div class="button_separator"></div>';
			}
			barHTML += html;
		}
	}
	
	//Add the select all button if multiple select is enabled an any items exist
	if (tribiqSK.multipleSelectEnabled) {
		for (itemNo in tribiqSK.searchedItems) {
			if (barHTML) {
				barHTML += '<div class="button_separator"></div>';
			}
			barHTML += '<div class="button select_all" onclick="tribiqSK.selectAll();" title="' + tribiqA.phrase.selectAll + '"></div>';
			break;
		}
	}
	
	if (tribiqSK.moreCollActions) {
		if (barHTML) {
			barHTML += '<div class="button_separator"></div>';
		}
		barHTML += '<div class="button more_actions" onclick="tribiqSK.openMoreCollActions(event);" title="' + tribiqA.phrase.moreActionsTooltip + '"></div>';
	}
	
	if (barHTML) {
		get('collectionToolbar').style.visibility = 'visible';
		get('collectionToolbarButtons').innerHTML = barHTML;
		tribiq.tooltips('#collectionToolbarButtons div.button[title]');
		tribiqSK.setUploads();
	} else {
		tribiqSK.hideCollectionButtons();
	}
}

tribiqSK.hideCollectionButtons = function() {
	get('collectionToolbar').style.visibility = 'hidden';
	get('collectionToolbarButtons').innerHTML = '';
}

tribiqSK.setItemButtons = function() {
	var barHTML = tribiqSK.moreItemActions = '';
	tribiqSK.itemsSelected = 0;
	
	for (id in tribiqSK.selectedItems) {
		++tribiqSK.itemsSelected;
	}
	
	if (tribiqSK.itemsSelected > 0) {
		for (itemNo in tribiqSK.sortedItemButtons) {
			var i = tribiqSK.sortedItemButtons[itemNo];
			var button = tribiqSK.focus.item_buttons[i];
			
			//Check if this button should be hidden in select mode (or hidden if not in select mode)
			//Also, automatically hide a button in select mode if it calls a floating box or a front-end link
			if ((tribiqSK.selectMode && button.hide_in_select_mode)
		//	||  (tribiqSK.selectMode && button.combine_items)
		//	||  (tribiqSK.selectMode && button.floating_box)
			||  (tribiqSK.selectMode && button.frontend_link)
			|| (!tribiqSK.selectMode && button.select_mode_only)) {
				continue;
			}
			
			//Check if this button should only be shown in a certain refiner
			if (button.only_show_on_refiner && (
				!tribiqSK.refiner || tribiqSK.refiner.name != button.only_show_on_refiner
			)) {
				continue;
			}
			
			//Check if this button only works with single items, and then don't show it if multiple items are selected
			if (tribiqSK.itemsSelected > 1) {
				if (!engToBoolean(button.multiple_select)) {
					continue;
				}
			} else {
				if (engToBoolean(button.multiple_select_only)) {
					continue;
				}
			}
			
			//In select mode, don't show a link to a panel that we can't reach due to a max-path restriction
			if (button.link && tribiqSK.selectMode && window.tribiqSKMaxPath !== false
			 && tribiqSK.path == window.tribiqSKMaxPath && button.link.path != tribiqSK.path) {
				continue;
			}
			
			//If this button requires traits, then check that they are there for the selected item or items
			var c = false;
			if (button.traits) {
				for (trait in button.traits) {
					if (!tribiqSK.isInfoTag(trait) && engToBoolean(button.traits[trait])) {
						for (id in tribiqSK.selectedItems) {
							if (!tribiqSK.focus.items[id].traits || !engToBoolean(tribiqSK.focus.items[id].traits[trait])) {
								//If any are missing, don't show this button as an option
								c = true; continue;
							}
						}
						if (c) continue;
					}
				}
				if (c) continue;
			}
			
			var tooltip;
			if (tribiqSK.itemsSelected > 1 && button.multiple_select_tooltip) {
				tooltip = button.multiple_select_tooltip;
			} else {
				tooltip = button.tooltip;
			}
			
			
			var html = '<div class="' + (button.css_class? 'button ' + htmlspecialchars(button.css_class) : 'more_action') + '"' +
						' title="' + htmlspecialchars(tooltip) + '"';
			
			//Add a flash upload button, if Flash is enabled
			if (button.upload && tribiq.flashMajorVersion() >= 9) {
				button.upload.itemLevel = true;
				html += ' style="width: 29px; height: 28px;">'
						+ '<input type="file" name="uploadify" id="uploadify_' + tribiqSK.setUploadNum + '_' + tribiqSK.uploads.length + '" style="height: 28px; width: 28px;"/>';
				tribiqSK.uploads[tribiqSK.uploads.length] = button.upload;
			} else {
				html += ' onclick="tribiqSK.itemButtonClick(\'' + i + '\');"">';
			}
			
			if (!button.css_class && !button.upload && button.name) {
				html += htmlspecialchars(button.name);
			}
			
			html += '</div>';
		
			if (!button.css_class && !button.upload && button.name) {
				//Show the button as a row in a "more actions" dropdown
				tribiqSK.moreItemActions += html;
			
			} else {
				//Show the button as a button in the toolbar
				if (barHTML) {
					barHTML += '<div class="button_separator"></div>';
				}
				barHTML += html;
			}
		}
	}
	
	if (tribiqSK.moreItemActions) {
		if (barHTML) {
			barHTML += '<div class="button_separator"></div>';
		}
		barHTML += '<div class="button more_actions" onclick="tribiqSK.openMoreItemActions(event);" title="' + tribiqA.phrase.moreActionsTooltip + '"></div>';
	}
	
	
	if (barHTML) {
		get('itemToolbar').style.visibility = 'visible';
		get('itemToolbarButtons').innerHTML = barHTML;
		tribiq.tooltips('#itemToolbarButtons div.button[title]');
		tribiqSK.setUploads();
	} else {
		get('itemToolbar').style.visibility = 'hidden';
		get('itemToolbarButtons').innerHTML = '';
	}
	
	get('itemToolbarTitle').innerHTML =
		(tribiqSK.itemsSelected > 1?
			'<span class="n_selected_items">' +
				tribiqSK.itemsSelected +
			'</span> '
		 :  '') + 
		tribiqA.phrase.selected;
	
	//Enable or disable the "Choose" button
	if (tribiqSK.selectMode) {
		var choosePhrase = ifNull(window.tribiqSKChoosePhrase, tribiqA.phrase.choose);
		
		if ((tribiqSK.path == window.tribiqSKTargetPath || window.tribiqSKTargetPath === false) && tribiqSK.itemsSelected) {
			get('choose_button').disabled = false;
			get('choose_button').className = window.tribiqSKChooseButtonActiveClass? 'submit_selected' : 'submit';
			
			if (tribiqSK.itemsSelected > 1 && tribiqSKChooseMultiplePhrase) {
				choosePhrase = tribiqSKChooseMultiplePhrase;
			}
		
		} else {
			get('choose_button').disabled = true;
			get('choose_button').className = 'submit_disabled';
		}
		
		get('choose_button').value = htmlspecialchars(choosePhrase);
	}	
}

tribiqSK.openMoreItemActions = function(e) {
	if (tribiqSK.moreItemActions) {
		var width = 200;
		var left = 1;
		var top = 1;
	
		tribiqA.openBox(
			'<div class="colsortheader">' +
				'<a onclick="tribiqA.closeBox(\'AdminMoreItemActions\');"></a>' + 
				htmlspecialchars(tribiqA.phrase.moreActions) +
			'</div>' +
			tribiqSK.moreItemActions,
			'', 'AdminMoreItemActions',
			e, width, left, top,
			false, 10, 'black');
		tribiq.tooltips('#tribiq_fbAdminMoreItemActions div.more_action[title]', {defaultPosition: 'right'});
	}
}

tribiqSK.openMoreCollActions = function(e) {
	if (tribiqSK.moreCollActions) {
		var width = 200;
		var left = 1;
		var top = 1;
	
		tribiqA.openBox(
			'<div class="colsortheader">' +
				'<a onclick="tribiqA.closeBox(\'AdminMoreCollActions\');"></a>' + 
				htmlspecialchars(tribiqA.phrase.moreActions) +
			'</div>' +
			tribiqSK.moreCollActions,
			'', 'AdminMoreCollActions',
			e, width, left, top,
			false, 10, 'black');
		tribiq.tooltips('#tribiq_fbAdminMoreCollActions div.more_action[title]', {defaultPosition: 'right'});
	}
}

tribiqSK.uploads = [];
tribiqSK.setUploadNum = 0;
tribiqSK.uploading = false;
tribiqSK.setUploads = function() {
	var i;
	for (i in tribiqSK.uploads) {
		var upload = tribiqSK.uploads[i];
		
		if (!upload.request) {
			upload.request = tribiqSK.getKey(upload.itemLevel);
		} else {
			$.extend(upload.request, tribiqSK.getKey(upload.itemLevel));
		}
		
		upload.request.__pluginClassName__ = upload.class_name;
		upload.request.__path__ = tribiqSK.path;
		upload.request.method_call = 'storekeeperAJAX';
		upload.request.session_key = tribiqSK.sessionKey;
		upload.request.session_id = tribiqSK.sessionId;
		
		$('#uploadify_' + tribiqSK.setUploadNum + '_' + i).uploadify({
			uploader: URLBasePath + 'libraries/mit/uploadify/uploadify.swf',
			script: URLBasePath + 'plugins.php',
			
			scriptData: upload.request,
			queueID: 'fileQueue',
			auto: true,
			multi: engToBoolean(upload.multi),
			
			buttonImg: null,
			hideButton: true,
			wmode: 'transparent',
			cancelImg: URLBasePath + 'libraries/mit/uploadify/cancel.png',
			width: 28,
			height: 28,
			
			fileDesc: upload.fileDesc,
			fileExt: upload.fileExt,
			
			onSelect: function(event, queueID, fileObj) {
				tribiqSK.uploading = true;
				tribiqSK.stopRefreshing();
				
				get('fileQueue').style.display = '';
				get('fileQueue').style.left = Math.floor(($(window).width() - 374) / 2) + 'px';
			},
			
			onComplete: function(event, queueID, fileObj, response, data) {
				if (response && response != 1) {
					tribiqA.showMessage(response);
				}
				
				if (data.fileCount == 0) {
					tribiqSK.uploading = false;
					get('fileQueue').style.display = 'none';
					
					if (!upload.itemLevel) {
						tribiqSK.selectCreatedIds();
					} else {
						tribiqSK.refresh();
					}
				}
				
				return true;
			},
			
			onCancel: function(event, queueID, fileObj, data) {
				if (data.fileCount == 0) {
					tribiqSK.uploading = false;
					get('fileQueue').style.display = 'none';
					
					if (!upload.itemLevel) {
						tribiqSK.selectCreatedIds();
					} else {
						tribiqSK.refresh();
					}
				}
				
				return true;
			}
		});
	}
	
	tribiqSK.uploads = [];
	++tribiqSK.setUploadNum;
}

tribiqSK.setPages = function() {
	if (tribiqSK.searchMatches !== undefined) {
		tribiqSK.pageCount = Math.ceil(tribiqSK.searchMatches / tribiqSK.thisPageSize);
	} else {
		tribiqSK.pageCount = 1;
	}
	
	get('pagePrev').style.display =
	get('pageNext').style.display =
	get('pagePrevOff').style.display =
	get('pageNextOff').style.display = 'none';
	
	if (tribiqSK.pageCount == 1) {
		get('pages').innerHTML = '';
	} else {
		if (tribiqSK.page > 1) {
			get('pagePrev').style.display = '';
		} else {
			get('pagePrevOff').style.display = '';
		}
		
		if (tribiqSK.page < tribiqSK.pageCount) {
			get('pageNext').style.display = '';
		} else {
			get('pageNextOff').style.display = '';
		}
		
		
		var positions1 = [
				-999999,
				-100000, -70000, -40000, -20000,
				-10000, -7000, -4000, -2000,
				-1000, -700, -400, -200,
				-100, -70, -40, -20,
				-10, -7, -4, -3, -2,
				-1, 0,
				1, 2, 3, 4, 7,
				10, 20, 40, 70,
				100, 200, 400, 700,
				1000, 2000, 4000, 7000,
				10000, 20000, 40000, 70000,
				100000,
				999999
			];
		var positions2 = {};
		
		//Check if each is there, and include it if so
		for (var i in positions1) {
			var rel = positions1[i];
			
			//Check if the set position is out of range, and replace it with the first/last page in range if needed
			var pos = tribiqSK.page - 1 + rel;
			if (pos < 0) {
				pos = 0;
			} else if (pos >= tribiqSK.pageCount) {
				pos = tribiqSK.pageCount-1;
			} else {
				//Otherwise if the numbers are in range then round numbers, depending on how far away they are from the current page
				for (var round = 100000; round > 1; round /= 10) {
					if (rel < -round || round < rel) {
						pos = pos - ((tribiqSK.page - 1) % round) - 1;
						break;
					}
				}
				
				if (pos < 0) {
					pos = 0;
				} else if (pos >= tribiqSK.pageCount) {
					pos = tribiqSK.pageCount-1;
				}
			}
			
			positions2[1*pos + 1] = true;
		}
		
		//Count how long the string is
		var len = '';
		for (var p in positions2) {
			len += (len? ' ' : '') + p;
		}
		len = len.length;
		
		//If it's too long, start cutting page numbers to fit it in
		var max = tribiqSK.paginationMaxLength;
		if (len > max) {
			len -= (' ' + tribiqSK.page).length;
			len -= (' ' + tribiqSK.pageCount).length;
			max -= (' ' + tribiqSK.page).length;
			max -= (' ' + tribiqSK.pageCount).length;
			var quota = max;
			for (var p in positions2) {
				if (p != 1 && p != tribiqSK.page && p != tribiqSK.pageCount) {
					if (quota > max) {
						delete positions2[p];
					}
					quota = (quota + max) % len;
				}
			}
		}
		
		//Draw all of the chosen page numbers
		var html = '';
		for (var p in positions2) {
			html += '<a class="' + (p == tribiqSK.page? 'currentPage' : 'page') + '" onclick="tribiqSK.goToPage(' + p + ');">' + p + '</a> ';
		}
		
		get('pages').innerHTML = html;
	}
}


tribiqSK.getPanelTitle = function() {
	var title = tribiqSK.focus.title;
	
	if (tribiqSK.selectMode && (tribiqSK.path == window.tribiqSKTargetPath || window.tribiqSKTargetPath === false)) {
		if (window.tribiqSKMultipleSelect && tribiqSK.focus.multiple_select_mode_title) {
			title = tribiqSK.focus.multiple_select_mode_title;
		} else if (tribiqSK.focus.select_mode_title) {
			title = tribiqSK.focus.select_mode_title;
		}
	}
	
	return title;
}

tribiqSK.setPanelTitle = function() {
	get('rightColumnTitle').style.display = '';
	
	var title = tribiqSK.getPanelTitle();
	
	if (tribiqSK.focus.item && tribiqSK.focus.item.css_class) {
		get('panelTitle').innerHTML =
			'<table cellpadding="0" cellspacing="0"><tr><td><div class="listicon ' + tribiqSK.focus.item.css_class + '"></div></td><td>' + htmlspecialchars(title) + '</td></tr></table>';
	} else {
		get('panelTitle').innerHTML = htmlspecialchars(title);
	}
	
	get('panelTitle').style.fontSize = Math.min(18, Math.round(1300 / ('' + title).length)) + 'px';
	
	document.title = tribiqSK.pageTitle + tribiqA.phrase.colon + title;
}

tribiqSK.hidePanelTitle = function() {
	get('rightColumnTitle').style.display = 'none';
}


tribiqSK.setTrash = function () {
	if (!tribiqSK.selectMode && !tribiqSK.inCloseUpView && tribiqSK.focus && tribiqSK.focus.trash) {
		get('trash_button').style.display = '';
		get('trash_button').className =
			engToBoolean(tribiqSK.focus.trash.empty)? 'trash_button_empty' : 'trash_button_full';
	
	} else {
		get('trash_button').style.display = 'none';
	}
}

tribiqSK.setPanelItems = function() {
	
	var html = '';
	var itemsExist = false;
	
	var reordering = '';
	if (tribiqSK.reordering) {
		reordering = ' class="sortable"';
	}
	
	//Add a table header for list view
	if (tribiqSK.view_mode == 'grid') {
		html += '<span' + reordering + ' class="grid_view">';
		
	} else if (tribiqSK.view_mode == 'list') {
		html += '<table class="list_view"><thead><tr>' +
					'<th class="action_column">&nbsp;</th>';
		
		var lastCol = false;
		for (var colNo in tribiqSK.sortedColumns) {
			var c = tribiqSK.sortedColumns[colNo];
			if (tribiqSK.isShowableColumn(c, true)) {
				html += '<th>' + htmlspecialchars(tribiqSK.focus.columns[c].title) + '</th>';
				lastCol = c;
			}
		}
		
		html += '</tr></thead><tbody' + reordering + '>';
	
	} else if (tribiqSK.view_mode == 'summary') {
		html += '<table class="summary_view"><tbody' + reordering + '>';
	}
	
	tribiqSK.shownItems = {};
	if (tribiqSK.focus.items) {
		//Work out which items to display for this page
		var pageStop = tribiqSK.page * tribiqSK.thisPageSize;
		var pageStart = pageStop - tribiqSK.thisPageSize;
		
		pageStop = Math.min(pageStop, tribiqSK.searchMatches);
		
		get('pageStart').innerHTML = pageStart + 1;
		get('pageStop').innerHTML = pageStop;
		
		var firstRow = 'firstrow ';
		for (itemNo in tribiqSK.searchedItems) {
			if (!tribiqSK.isInfoTag(itemNo)) {
				itemsExist = true;
				var i = tribiqSK.searchedItems[itemNo];
				
				if (itemNo >= pageStop) {
					break;
				} else if (itemNo < pageStart) {
					continue;
				}
				
				if (!tribiqSK.focus.items[i]) {
					continue;
				}
				
				
				var att =
					' id="item_' + htmlspecialchars(i) + '"' +
					' class="' + (tribiqSK.view_mode == 'list'? '' : 'item') + (tribiqSK.selectedItems[i]? ' selected' : '') + tribiqSK.rowCssClass(i) + '"' +
					' onclick="tribiqSK.itemClick(this, event);"' +
					' ondblclick="tribiqSK.itemDoubleClick(this);"';
				
				var image = '';
				if (tribiqSK.focus.items[i].image) {
					image = 'style="background-image: url(\'' + tribiq.addURLBasePathIfNeeded(tribiqSK.focus.items[i].image) + '\'); background-position: center center; background-repeat: no-repeat;"';
				}
				
				var tooltip = tribiqSK.focus.items[i].tooltip;
				if (!tooltip && tribiqSK.focus.item && tribiqSK.focus.item.tooltip) {
					tooltip = tribiqSK.applyMergeFields(tribiqSK.focus.item.tooltip, true, i);
				}
				
				var title = tooltip?
					' title="' + htmlspecialchars(tooltip) + '"'
				  : '';
				
				if (tribiqSK.view_mode == 'list') {
					html += 
						'<tr' + att + '>' +
							'<td class="' + firstRow + 'firstcol listicon">' +
								'<div ' + (
									//In list view don't show a custom image unless an icon is not set and the image is
									tribiqSK.getItemCSSClass(i)?
										'class="' + htmlspecialchars(tribiqSK.getItemCSSClass(i)) + '"'
									  : image
									) + title + '>' +
								'</div>' +
							'</td>';
					
					var firstCell = 'firstcell ';
					for (var colNo in tribiqSK.sortedColumns) {
						var c = tribiqSK.sortedColumns[colNo];
						if (tribiqSK.isShowableColumn(c, true)) {
							html +=
								'<td class="' + firstRow + firstCell + (lastCol == c? 'lastcell' : '') +
									(engToBoolean(tribiqSK.focus.columns[c].align_right)? ' right' : '') +
									tribiqSK.columnCssClass(i, c) +
								'">' +
									tribiqSK.columnValue(i, c) +
								'</td>';
							firstCell = '';
						}
					}
					
					html += '</tr>';
				
				} else if (tribiqSK.view_mode == 'summary') {
					
					//Check if this item has a link, and if it can be clicked
					var link = tribiqSK.focus.items[i].link;
					if (!link && tribiqSK.focus.item) {
						link = tribiqSK.focus.item.link;
					}
					
					if (link && tribiqSK.selectMode && window.tribiqSKMaxPath !== false &&
						tribiqSK.path == window.tribiqSKMaxPath && link.path != tribiqSK.path) {
							link = false;
					}
					
					var summary = '';
					if (tribiqSK.focus.items[i].summary) {
						summary = '<p>' + tribiqSK.focus.items[i].summary + '</p>';
					
					} else if (tribiqSK.focus.items[i].tooltip) {
						summary = '<p>' + tribiqSK.focus.items[i].tooltip + '</p>';
						title = '';
					}
					
					
					html += 
						'<tr id="ritem_' + htmlspecialchars(i) + '" class="' + firstRow + tribiqSK.rowCssClass(i) + '">' +
							'<td class="summary_icon">' +
								'<div' + att + title + '>' +
									'<div class="image_container">' +
										'<div class="image ' + htmlspecialchars(tribiqSK.getItemCSSClass(i)) + '" ' + image + '></div>' +
									'</div>' +
								'</div>' +
							'</td><td class="summary_cell">' + 
								'<h2><a' + (
									link?
										' class="link" href="#" onclick="return tribiqSK.itemDoubleClick(get(\'item_' + htmlspecialchars(i) + '\'));"'
									  : '') + '>' +
									  	tribiqSK.applyMergeFields(tribiqSK.labelFormat, true, i) + 
								'</a></h2>' +
								summary;
					
					var odd = true;
					for (var colNo in tribiqSK.sortedColumns) {
						var c = tribiqSK.sortedColumns[colNo];
						if (c != tribiqSK.labelTag && tribiqSK.isShowableColumn(c, true)) {
							html +=
								'<div class="' + (odd? 'left' : 'right') + '">' +
									'<table class="no_border"><tr><td class="col_label">' + htmlspecialchars(tribiqSK.focus.columns[c].title) + ':&nbsp;</td>' +
									'<td width="95%" class="' + tribiqSK.columnCssClass(i, c) + '">' + tribiqSK.columnValue(i, c) + '</td></tr></table>' +
								'</div>';
							odd = !odd;
						}
					}
					
					html += 
							'</td>' + 
						'</tr>';
				
				} else {
					html += 
						'<div' + att + '>' +
							'<div class="image_container"' + title + '>' +
								'<div class="image ' + htmlspecialchars(tribiqSK.getItemCSSClass(i)) + '" ' + image + '></div>' +
							'</div>' +
							'<div class="legend">' +
								$.trim(tribiqSK.applyMergeFields(tribiqSK.labelFormat, true, i, true)).replace(/\n/g, '<br/>') +
							'</div>' +
						'</div>';
				}
				firstRow = '';
				
				tribiqSK.shownItems[i] = true;
			}
		}
	}
	
	for (i in tribiqSK.selectedItems) {
		//Remove any items that have disappeared from view
		if (!tribiqSK.shownItems[i]) {
			delete tribiqSK.selectedItems[i];
		}
	}
	
	//Add a table footer for list view
	if (tribiqSK.view_mode == 'grid') {
		html += '</span>';
	} else {
		html += '</tbody></table>';
	}
	
	
	//Display a message if there were no items to display
	if (!itemsExist) {
		html = '<div class="no_items">';
		if (tribiqSK.searchTerm === undefined && !tribiqSK.filtersSet) {
			html += htmlspecialchars(tribiqSK.focus.no_items_message? tribiqSK.focus.no_items_message : tribiqA.phrase.noItems);
		} else {
			html += htmlspecialchars(tribiqSK.focus.no_items_in_search_message? tribiqSK.focus.no_items_in_search_message : tribiqA.phrase.noItemsInSearch);
		}
		html += '</div>';
	}
	
	
	get('rightColumnContent').innerHTML = html;
	
	if (tribiqSK.view_mode == 'list') {
		tribiq.tooltips('#rightColumnContent .listicon div[title]');
	} else if (tribiqSK.view_mode == 'summary') {
		tribiq.tooltips('#rightColumnContent div.item[title]', {defaultPosition: 'top'});
	} else {
		tribiq.tooltips('#rightColumnContent div.item .image_container[title]', {defaultPosition: 'top'});
	}
	tribiq.tooltips('#rightColumnContent span.tooltip[title]');
}

tribiqSK.getItemCSSClass = function (i) {
	if (tribiqSK.focus.items[i].css_class) {
		return tribiqSK.focus.items[i].css_class;
	} else if (tribiqSK.focus.item) {
		return tribiqSK.focus.item.css_class;
	} else {
		return undefined;
	}
}

tribiqSK.closeUpViewNext = function() {
	
	if (tribiqSK.lockPageClicks) {
		return false;
	}
	
	if (tribiqSK.itemNo < tribiqSK.searchMatches -1) {
		tribiqSK.closeUpViewItem = tribiqSK.searchedItems[tribiqSK.itemNo + 1];
		
		if (tribiqSK.itemNo >= tribiqSK.page * tribiqSK.thisPageSize - 1) {
			tribiqSK.nextPage(true);
		} else {
			tribiqSK.drawPanel();
		}
	}
	
	return false;
}

tribiqSK.closeUpViewPrev = function() {
	
	if (tribiqSK.lockPageClicks) {
		return false;
	}
	
	if (tribiqSK.itemNo > 0) {
		tribiqSK.closeUpViewItem = tribiqSK.searchedItems[tribiqSK.itemNo - 1];
		
		if (tribiqSK.itemNo <= (tribiqSK.page - 1) * tribiqSK.thisPageSize) {
			tribiqSK.prevPage(true);
		} else {
			tribiqSK.drawPanel();
		}
	}
	
	return false;
}

tribiqSK.setCloseUpView = function(imageNo) {
	
	if (!imageNo) {
		if (!tribiqSK.setCloseUpViewImageNo) {
			tribiqSK.setCloseUpViewImageNo = 1;
		}
		
		imageNo = tribiqSK.setCloseUpViewImageNo;
	} else {
		tribiqSK.setCloseUpViewImageNo = imageNo;
	}
	
	var html = '';
	var i = tribiqSK.closeUpViewItem;
	tribiqSK.itemNo = tribiqSK.itemsOrder[i];
	
	var images = 0;
	var pagination = '';
	if (tribiqSK.focus.items[i].images) {
		for (img in tribiqSK.focus.items[i].images) {
			if (!tribiqSK.isInfoTag(img)) {
				++images;
				if (images == imageNo) {
					firstImage = tribiq.addURLBasePathIfNeeded(tribiqSK.focus.items[i].images[img]);
				}
			}
		}
	}
	
	if (!images && tribiqSK.focus.items[i].image) {
		images = 1;
		firstImage = tribiq.addURLBasePathIfNeeded(tribiqSK.focus.items[i].image);
	}

				
	html +=
	'<div class="itemDetail close_up_view">' + 
		'<div id="previous_next">' + 
			'<a id="next' + (tribiqSK.itemNo < tribiqSK.searchMatches -1 ? '' : '_disabled') + '" href="#" onclick="return tribiqSK.closeUpViewNext();"></a>' + 
			'<a id="previous' + (tribiqSK.itemNo > 0? '' : '_disabled') + '" href="#" onclick="return tribiqSK.closeUpViewPrev();"></a>' + 
		'</div>' + 
		'<div class="content' + tribiqSK.rowCssClass(i) + '">' +
			'<div class="images">';
	
	if (images > 0) {
		html +=
				'<div class="images_image" style="background-image: url(\'' + firstImage + '\');">' + 
				'</div>';
		
		if (images > 1) {
			html +=
				'<div class="pagination">' + 
					'<ul>';
			
			for (var img=1; img <= images; ++img) {
				html +=
						'<li>' + 
							'<a' + (img == imageNo? ' class="selected"' : '') + ' href="#" onclick="return tribiqSK.setCloseUpView(' + img + ');">' + img + '</a>' + 
						'</li>';
			}
			
			html +=
					'</ul>' + 
				'</div>';
		}
	
	} else {
		html +=
				'<div class="images_image">' +
					'<div class="item">' +
						'<div class="image_container">' +
							'<div class="image ' + htmlspecialchars(tribiqSK.getItemCSSClass(i)) + '"></div>' +
						'</div>' +
					'</div>' +
				'</div>';
	}
	
	html +=
			'</div>';
	
	
	document.title = tribiqSK.pageTitle + tribiqA.phrase.colon + tribiqSK.getPanelTitle() + tribiqA.phrase.colon + tribiqSK.applyMergeFields(tribiqSK.labelFormat, false, i);
	
	html += 
			'<div class="inner_content">' +
				'<h2>' + tribiqSK.applyMergeFields(tribiqSK.labelFormat, true, i) + '</h2>';
					
	if (tribiqSK.focus.items[i].close_up_view) {
		html += 
						tribiqSK.focus.items[i].close_up_view;
	
	} else if (tribiqSK.focus.items[i].summary) {
		html += 
				'<p>' + tribiqSK.focus.items[i].summary + '</p>';
	
	} else if (tribiqSK.focus.items[i].tooltip) {
		html += 
				'<p>' + tribiqSK.focus.items[i].tooltip + '</p>';
	}
	
	html += 	'<table class="no_border">';
	for (var colNo in tribiqSK.sortedColumns) {
		var c = tribiqSK.sortedColumns[colNo];
		if (c != tribiqSK.labelTag && tribiqSK.isShowableColumn(c)) {
			html +=
					'<tr>' +
						'<td class="col_label">' + htmlspecialchars(tribiqSK.focus.columns[c].title) + ':&nbsp;</td>' +
						'<td width="95%" class="' + tribiqSK.columnCssClass(i, c) + '">' + tribiqSK.columnValue(i, c) + '</td>' +
					'</tr>';
		}
	}
	html += 	'</table>';
					
	if (tribiqSK.focus.items[i].close_up_view_bottom) {
		html += 
						tribiqSK.focus.items[i].close_up_view_bottom;
	}
	
	html +=
			'</div>' + 
		'</div>' + 
	'</div>';
	
	
	get('rightColumnContent').innerHTML = html;
	tribiq.tooltips('#rightColumnContent span.tooltip[title]');
	
	return false;
}

tribiqSK.setItemCount = function() {
	var count = 0;
	
	if (tribiqSK.searchMatches !== undefined) {
		count = tribiqSK.searchMatches;
	}
	
	if (count) {
		get('itemCount').innerHTML = htmlspecialchars(count);
		get('itemCountWrap').style.visibility = 'visible';
	} else {
		tribiqSK.hideItemCount();
	}
}

tribiqSK.hideItemCount = function() {
	get('itemCount').innerHTML = '';
	get('itemCountWrap').style.visibility = 'hidden';
}

tribiqSK.closeSelectMode = function() {
	window.parent.tribiqA.closeBox('AdminStorekeeper', true);
	tribiqSK.go(tribiqSK.defaultPathInSelectMode);
	return false;
}

tribiqSK.choose = function() {
	//Stop admins making choices on outdated panels
	if (tribiqSK.lastSuccessfulGoNum != tribiqSK.goNum) {
		return;
	}
	
	if (window.parent
	 && tribiqSK.selectMode
	 && (tribiqSK.path == window.tribiqSKTargetPath || window.tribiqSKTargetPath === false)
	 && tribiqSK.itemsSelected
	 && (tribiqSK.itemsSelected == 1 || window.tribiqSKMultipleSelect)) {
		
		//Disable the current panel/page from refreshing
		tribiqSK.stopRefreshing();
		
		var key = tribiqSK.getKey(true);
		var path = tribiqSK.path;
		
		var row;
		key._items = {};
		for (var i in tribiqSK.selectedItems) {
			if (!tribiqSK.isInfoTag(i)) {
				if (window.tribiqSKLabelColumn) {
					key._items[i] = tribiqSK.focus.items[key.id][window.tribiqSKLabelColumn];
				} else {
					key._items[i] = ('' + tribiqSK.applyMergeFields(tribiqSK.labelFormat, false, i)).replace(/\s+/g, ' ');
				}
			}
			if (!window.tribiqSKMultipleSelect) {
				break;
			}
		}
		if (!window.tribiqSKMultipleSelect) {
			row = tribiqSK.focus.items[key.id];
			row.__label_tag__ = tribiqSK.labelTag;
		}
		
		tribiqSK.closeSelectMode();
		
		if (window.tribiqSKMultipleSelect) {
			window.parent[window.tribiqSKCallbackObject][window.tribiqSKCallbackFunction](path, key);
		} else {
			window.parent[window.tribiqSKCallbackObject][window.tribiqSKCallbackFunction](path, key, row);
		}
	}
}



//Automatically set the box to the correct height for the users screen, or the maximum height requested, whichever is smaller
tribiqSK.sizing = false;
tribiqSK.lastSize = false;
tribiqSK.size = function(refresh) {
	if (tribiqSK.sizing) {
		clearTimeout(tribiqSK.sizing);
	}
	
	var height = Math.floor($(window).height());
	
	if (height) {
		if (refresh || tribiqSK.lastSize != height) {
			
			var outerGap = 164;
			var colGap = 39;
			var contentGap = 29;
			var graphicGap = 66;
			
			if (tribiqSK.selectMode) {
				--colGap;
			}
		
			get('leftColumn').style.height = (1*height - outerGap - colGap) + 'px';
			get('rightColumn').style.height = (1*height - outerGap - colGap + 3) + 'px';
			get('storekeeperOuter').style.height = (1*height - outerGap) + 'px';
			
			if (get('colsortfields')) {
				if (tribiq.browserIsIE6()) {
					get('colsortfields').style.height = (1*height - outerGap) + 'px';
				} else {
					get('colsortfields').style.maxHeight = (1*height - outerGap) + 'px';
				}
			}
			
			if (tribiqSK.focus && tribiqSK.focus.banner) {
				get('rightColumnContent').style.height = (1*height - outerGap - colGap - contentGap - graphicGap) + 'px';
			//} else if (tribiq.browserIsIE()) {
				//get('rightColumnContent').style.height = (1*height - outerGap - colGap - contentGap - 24) + 'px';
				//$("#rightColumnContent").css("overflow-x","hidden");
			} else {
				get('rightColumnContent').style.height = (1*height - outerGap - colGap - contentGap) + 'px';
			}
			
			
			if (tribiqSK.showLeftColumn) {
				get('leftColumn').style.display = get('leftColumnOuter').style.display = '';
				get('rightColumnContent').style.width = '710px';
				get('rightColumn').style.width = '712px';
			} else {
				get('leftColumn').style.display = get('leftColumnOuter').style.display = 'none';
				get('rightColumnContent').style.width = '914px';
				get('rightColumn').style.width = '916px';
			}
			
			if (tribiqSK.selectMode) {
				get('choose_button').style.display = '';
			} else {
				get('choose_button').style.display = 'none';
			}
			
			tribiqSK.lastSize = height;
		}
	}
	
	tribiqSK.sizing = tribiq.setTimeout('tribiqSK.size()', 500);
}

$(document).ready(tribiqSK.size);;