<?php
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

header('Content-Type: text/javascript; charset=UTF-8');
require '../cmsincludes/cache.inc.php';

//Get a list of plugin ids from the url
$plugins = explode(',', $_GET['ids']);
$pluginDetails = array();

//Ensure that the site name and subdirectory are part of the ETag, as Plugins can have different ids on different servers
$ETag = 'tribiq-plugin-js-'. CSS_JS_VERSION. '--'. $_SERVER["HTTP_HOST"]. '-';

//Add the id of each running Plugin to the ETag
foreach ($plugins as $pluginId) {
	if ($pluginId) {
		$ETag .= '-'. (int) $pluginId;
	}
}

if ($_GET['admin']) {
	$ETag .= '--admin';
}

//Cache this combination of running Plugin JavaScript
useCache($ETag);
useGZIP($_GET['gz']);


require 'visitorheader.inc.php';
header('Content-Type: text/javascript; charset=UTF-8');

function incJS($path, $file) {
	$file = $path. $file;
	
	if (file_exists($file. '.js.php')) {
		chdir($path);
		require $file. '.js.php';
		chdir(CMS_ROOT);
	
	} elseif (file_exists($file. '.pack.js')) {
		require $file. '.pack.js';
	} elseif (file_exists($file. '.min.js')) {
		require $file. '.min.js';
	} elseif (file_exists($file. '.js')) {
		require $file. '.js';
	}
	
	echo "\n/**/\n";
}


//Get a list of each Plugin
foreach ($plugins as $pluginId) {
	if ($pluginId) {
		$pluginDetails[$pluginId] = getPluginDetails($pluginId);
	}
}

//Add Plugin inheritances as well
$checkedPlugins = array();
$limit = 9;
do {
	--$limit;
	$changes = false;
	foreach (array_keys($pluginDetails) as $pluginId) {
		if (!issetArrayKey($checkedPlugins, $pluginId)) {
			$checkedPlugins[$pluginId] = true;
			foreach (getPluginDependencies($pluginDetails[$pluginId]['plugin_name'], "'inheritance'") as $inheritance) {
				$changes = true;
				unset($pluginDetails[$inheritance['plugin_id']]);
				$pluginDetails[$inheritance['plugin_id']] = $inheritance;
			}
		}
	}
} while ($limit && $changes);

//Try to put the Plugins in dependency order (technically this isn't needed, but it looks clearer to read)
$pluginDetails = array_reverse($pluginDetails, true);


//Add JavaScript support elements for each Plugin on the page
foreach ($pluginDetails as $plugin) {
	//Create a namespace for each Plugin used on this page
	echo '
		tribiq.enc(', $plugin['plugin_id'], ', "', $plugin['plugin_name'], '", "', $plugin['class_name'], '", "', $plugin['vlp_class'], '");';
	
	//Add the Plugin's library include if it exists
	incJS(CMS_ROOT. 'plugins/'. $plugin['plugin_name']. '/js/', 'plugin');
	
	//For Admins, add the Plugin's admin js if it exists
	if ($_GET['admin']) {
		incJS(CMS_ROOT. 'plugins/'. $plugin['plugin_name']. '/js/', 'admin');
	}
}