/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */


//Create an encapsulated object for all CMS functions
window.tribiq = function() {};


tribiq.AFBOpen = false;
tribiq.getEl = false;

//Shortcut to document.getElementById()
function get(el) {
	//Is there an admin floating box open?
		//To avoid clashes with ids on the page and in the box, demand that the element returned be inside the box
		//(Unless it's already prefixed with the tribiq name, in which case it should be safe)
	if (tribiq.AFBOpen && el.substr(0, 6) != 'tribiq') {
		tribiq.getEl = false;
		
		$('#tribiq_fbAdminFloatingBox #' + el).each(function(i, el) {  if (i == 0) tribiq.getEl = el;  })
		
		var getEl = tribiq.getEl;
		tribiq.getEl = false;
		
		if (getEl) {
			return getEl;
		}
	}
	
	//If there wasn't an admin floating box open (or there was but the element we wanted wasn't inside)
	//then return document.getElementById() as normal
	return document.getElementById(el);
}

//Given a string, this function makes it safe to use in the URL after a hash (i.e. a safe id for Storekeeper)
function encodeItemIdForStorekeeper(id) {
	return '~' + encodeURIComponent('' + id).replace(/%/g, '~');
}

//Reverses encodeItemIdForStorekeeper()
function decodeItemIdForStorekeeper(id) {
	if (('' + id).substr(0, 1) == '~') {
		return decodeURIComponent(('' + id).substr(1).replace(/~/g, '%'));
	} else {
		return id;
	}
}

function engToBoolean(text) {
	return text && text != '0' && (text + '').toLowerCase() != 'no' && (text + '').toLowerCase() != 'false'? 1:0;
}

function htmlspecialchars(text) {
	if (text === undefined || text === null || text === false) {
		return '';
	} else {
		return ('' + text).replace(/&/g, '&amp;').replace(/"/g, '&quot;').replace(/\</g, '&lt;').replace(/>/g, '&gt;');
	}
}

function ifNull(a, b, c) {
	return a? a : (b? b : c);
}



//Allow us to monitor our useage of the setTimeout function, for debugging purposes
tribiq.setTimeoutMonitor = {};
tribiq.setTimeout = function(a, b) {
	if (tribiq.setTimeoutMonitor[a]) {
		++tribiq.setTimeoutMonitor[a];
	} else {
		tribiq.setTimeoutMonitor[a] = 1;
	}
	
	return setTimeout(a, b);
}


//Redirect the user to a URL using JavaScript
tribiq.goToURL = function(URL) {
	document.location.href = URL;
	return false;
}

tribiq.addURLBasePathIfNeeded = function(url) {
	if (url.indexOf('//') == -1) {
		return URLBasePath + url;
	} else {
		return url;
	}
}

//Take a request string, and check it's formatted correctly
tribiq.addAmp = function(request) {
	
	//For backwards compatability purposes, we'll accept a string with a URL already set, and strip the requests out
	var pos = request.indexOf('?');
	if (pos != -1) {
		request = request.substr(pos+1);
	}
	
	//Add an & to the beginning if needed
	if (request != '' && request.substr(0, 1) != '&') {
		return '&' + request;
	} else {
		return request;
	}
}

//Convert an array into a string for a URL if needed
tribiq.urlRequest = function(arr) {
	
	//Have a catch that stops this function being called twice on itself
	if (typeof arr == 'string') {
		return tribiq.addAmp(arr);
	}
	
	request = '';
	
	if (arr) {
		for (i in arr) {
			if (typeof arr[i] != 'object') {
				request += '&' + encodeURIComponent(i) + '=';
				
				if (arr[i] !== undefined && arr[i] !== false && arr[i] !== null) {
					request += encodeURIComponent(arr[i]);
				}
			}
		}
	}
	
	return request;
}

//Make a non-asyncornous AJAX call.
tribiq.nonAsyncAJAX = function(url, post, json) {
	var xmlHttp = new Object();
	
	if (window.ActiveXObject) {
		xmlHttp = new ActiveXObject('Microsoft.XMLHTTP');
	} else if (window.XMLHttpRequest) {
		xmlHttp = new XMLHttpRequest();
	}
	
	//Use GET or POST, as requested.
	if (post == undefined || !post) {
		//If you're using GET then any variables need to be set in the URL
		xmlHttp.open('GET', url, false);
		xmlHttp.send(null);
	} else {
		//If you're using POST then variables need to be set in the POST.
		// (This uses the same format as GET, however without the initial ?)
		
		//Check to see if the caller took the time to seperate the two different inputs out,
		//or if they have dumped them all into the url
		if (post === true) {
			//If post has just been set to true, try to check the url for the actual inputs!
			qMark = url.indexOf('?');
			
			if (qMark == -1) {
				//Case where POST must be used, but there are not actually any requests
				post = '';
			} else {
				//Get variables from the URL and put them in the POST
				post = url.substr(qMark+1);
				url = url.substr(0, qMark);
			}
		}
		
		xmlHttp.open('POST', url, false);
		xmlHttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
		xmlHttp.send(post);
	}
	
	//Either return the response as-is, of if JSON was set, do JSON.parse on it first.
	if (json == undefined || !json) {
		return xmlHttp.responseText;
	} else {
		try {
			return JSON.parse(xmlHttp.responseText);
		} catch (e) {
			if (xmlHttp.responseText) {
				if (window.tribiqA) {
					tribiqA.floatingBox(xmlHttp.responseText, true, 'error');
				} else {
					alert(xmlHttp.responseText);
				}
			}
		}
	}
	
	delete xmlHttp;
}

//Look up a Plugin's VLP Phrase
tribiq.getPluginPhrase = function(vlpClass, codeIn, request) {
	
	if (!request) {
		request = '';
	}
	
	if (typeof codeIn != 'string') {
		code = '';
		for (var i in codeIn) {
			code += (code? ',' : '') + codeIn[i];
		}
	} else {
		code = codeIn;
	}
	
	var phrases =
		tribiq.nonAsyncAJAX(URLBasePath + 'plugins.php'
			+ '?__class__=' + encodeURIComponent(vlpClass)
			+ '&__code__=' + encodeURIComponent(code)
			+ '&__method_call__=getPhrase' + tribiq.urlRequest(request),
		false, true);
	
	if (code.indexOf(',') == -1) {
		return phrases[code];
	} else {
		return phrases;
	}
}

//A version of nonAsyncAJAX for Plugins
tribiq.pluginAJAX = function(pluginName, requests, post, json) {
	return tribiq.nonAsyncAJAX(URLBasePath + 'plugins.php?__pluginName__=' + pluginName + '&method_call=handleAJAX' + tribiq.urlRequest(requests), post, json);
}

//As above, but fetch by class name
tribiq.pluginClassAJAX = function(pluginClassName, requests, post, json) {
	return tribiq.nonAsyncAJAX(URLBasePath + 'plugins.php?__pluginClassName__=' + pluginClassName + '&method_call=handleAJAX' + tribiq.urlRequest(requests), post, json);
}

tribiq.addBasePath = function(url) {
	if (url.indexOf('://') == -1) {
		return URLBasePath + url;
	} else {
		return url;
	}
}


//Link to a content item
tribiq.linkToItem = function(cID, cType, request, adminlogin) {
	
	if (!cType) {
		cType = 'html';
	}
	
	if (!request) {
		request = '';
	}
	
	var basePath = URLBasePath;
	if (adminlogin) {
		basePath += 'admin/welcome.php';
	} else {
		basePath += tribiq.indexDotPHP;
	}
	
	if (typeof tribiqA == 'undefined' && !adminlogin && tribiq.suffix !== undefined && (cID !== 1*cID) && cType == 'html' && request == '') {
		return basePath + cID + tribiq.suffix;
	
	} else if (cType == 'html') {
		return basePath + '?cID=' + cID + tribiq.urlRequest(request);
	
	} else {
		return basePath + '?cID=' + cID + '&cType=' + cType + tribiq.urlRequest(request);
	}
}




//Functions for managing plugin slots

//dummy plugin encapsulation funciton, will get overwritten if plugin encapsulation is being used and cmsincludes/plugin_api/javascript.js is on the page
tribiq.enc = function(id, name, className, pluginClassNameForPhrases) {
	
}

//Attempt to get the name of a slot from an element within the slot
tribiq.getSlotnameFromEl = function(el, getContainerId) {
	if (typeof el == 'string') {
		return el;
	
	} else if (typeof el == 'object') {
		while(el.parentNode) {
			el = el.parentNode;
			if (el.id && el.id.substr(0, 7) == 'plgslt_') {
				
				var hyphen = el.id.indexOf('-'),
					slotName;
				
				//Extract the slot name out from the container id
				if (hyphen == -1) {
					slotName = el.id.substr(7);
				} else {
					slotName = el.id.substr(7, hyphen - 7);
				}
				
				//Check if this is a name of a slot that exists!
				if (!tribiq.slots[slotName]) {
					continue;
				}
				
				if (getContainerId) {
					return el.id;
				} else {
					return slotName;
				}
			}
		}
	}
	return false;
}

tribiq.getContainerIdFromEl = function(el) {
	return tribiq.getSlotnameFromEl(el, true);
}

//Scroll to the top of a slot if needed
tribiq.scrollToSlotTop = function(slotName, neverScrollDown) {
	//Check that the top of the slot is actually visible
	var scrollTop = tribiq.scrollTop();
	var slotTop = $(get('plgslt_' + slotName)).offset().top;
	slotTop = Math.max(0, slotTop - 80);
	
	//Have an option to only scroll up, and never down
	if (neverScrollDown && scrollTop < slotTop) {
		return;
	}
	
	//Scroll to the correct place
	$(tribiq.browserIsSafari()? 'body' : 'html').animate({ scrollTop: slotTop }, 700);
}

//Refresh a plugin in a slot
tribiq.refreshPluginSlot = function(slotName, instanceId, additionalRequests, recordInURL, scrollToTopOfSlot) {
	
	if (!slotName) {
		return;
	}
	
	//Remove the Nested Plugin id from the slotname if needed
	slotName = slotName.split('-')[0];
	
	if (!additionalRequests) {
		additionalRequests = '';
	} else {
		additionalRequests = tribiq.urlRequest(additionalRequests);
	}
	
	if (tribiq.slots[slotName] && tribiq.slots[slotName].tabId
	 && additionalRequests.indexOf('&tab=') == -1
	 && additionalRequests.indexOf('&tab_no=') == -1) {
		additionalRequests += '&tab=' + tribiq.slots[slotName].tabId;
	}
	
	//Allow a slot to be refreshed by name only, in which case we'll check its current instance id
	if (instanceId == 'lookup') {
		instanceId = tribiq.slots[slotName].instanceId;
	}
	
	if (scrollToTopOfSlot === undefined) {
		scrollToTopOfSlot = true;
	}
	
	if (scrollToTopOfSlot) {
		//Scroll to the top of a slot if needed
		tribiq.scrollToSlotTop(slotName, true);
	}
	
	//Fade the slot out to give a graphical hint that something is happening
	$('#plgslt_' + slotName).animate({opacity: .5}, 150);
	
	//Run an AJAX request to reload the contents
	//(I'm using jQuery so that this is done asyncronously)
	$.ajax({
		dataType: 'text',
		url: URLBasePath
			+ 'plugins.php?method_call=refreshPlugin'
								+ '&cID=' + tribiq.cID + '&cType=' + tribiq.cType + '&cVersion=' + tribiq.cVersion
								+ '&instanceId=' + instanceId
								+ '&slotName=' + slotName
								+ additionalRequests,
		success: function(html){
			tribiq.replacePluginSlotContents(slotName, instanceId, html, additionalRequests, recordInURL);
		}
	});
}

//Listen out for changes to the hash on the URL, to add support for browser back buttons and AJAX reloading
tribiq.currentHash = '';
tribiq.currentHashSlot = false;
tribiq.watchingForHashChanges = false;
tribiq.checkForHashChanges = function(timed) {
	if (tribiq.watchingForHashChanges) {
		clearTimeout(tribiq.watchingForHashChanges);
	}
	
	//Don't do anything if the hash has not changed since the last time
	if (!timed || tribiq.currentHash !== document.location.hash) {
		//Extract the instance id and the request from the hash
		var hash = document.location.hash.substr(1);
		
		//If this is a storekeeper window, go to that location
		if (typeof tribiqSK != 'undefined' && typeof tribiqSK.go == 'function') {
			//Check if there is an editor open
			var message = tribiqA.onbeforeunload();
			//If there was, give the Admin a chance to stop leaving the page
			if (message === undefined || confirm(message)) {
				tribiqAFB.close();
				tribiqSK.go(hash, -1);
			} else {
				//If the admin doesn't want to leave, put the hash back to how it was
				tribiqSK.saveSelection();
			}
		
		//Give an Admin the ability to cancel navigation if they were editing something.
		} else if (typeof tribiqA != 'undefined'
				&& tribiq.checkSlotsBeingEdited()
				&& !confirm(tribiqA.phrase.leavePageWarning)
		) {
			document.location.hash = tribiq.currentHash;
		
		//Otherwise is this is the front-end
		} else {
			if ((pos = hash.indexOf('|')) !== -1) {
				var instanceId = hash.substr(0, pos);
				
				//Check if the instance is actually on the page!
				if (tribiq.instances[instanceId]) {
					tribiq.currentHashSlot = tribiq.instances[instanceId].slotName;
					//Change the Plugin to use that request
					tribiq.refreshPluginSlot(
						tribiq.currentHashSlot,
						instanceId,
						hash.substr(pos+1));
				}
			} else {
				//If there was an empty hash or no hash, and we just changed a Plugin, reset it.
				tribiq.refreshPluginSlot(
					tribiq.currentHashSlot,
					'lookup',
					'');
			}
		}
	}
	
	//Remember the hash that we've just used
	tribiq.currentHash = document.location.hash;
	
	//Keep watching for hash changes
	tribiq.watchingForHashChanges = tribiq.setTimeout('tribiq.checkForHashChanges(true)', 500);
}

//Watch for the recorded URLs on page load
$(tribiq.checkForHashChanges);



//Attempt to submit a form via AJAX using jQuery serialize
tribiq.formSubmit = function(el) {
	var slotName = tribiq.getSlotnameFromEl(el);
	
	if (tribiq.slotFormSubmissions[slotName]) {
		return false;
	} else {
		tribiq.slotFormSubmissions[slotName] = true;
		
		//Check to see if there is a file upload in this slot
		if ($('#plgslt_' + slotName + ' :file').length > 0) {
			//If so, don't attempt to use jQuery serialize(), and do a normal form submission
			return true;
		}
		
		var url = URLBasePath + 'plugins.php?method_call=refreshPlugin&inIframe=true'; 
		var post = $(el).serialize();
		var instanceId = tribiq.slots[slotName].instanceId;
		
		if (tribiq.lastButtonClickedName) {
			post += '&' + encodeURIComponent(tribiq.lastButtonClickedName) + '=' + encodeURIComponent(tribiq.lastButtonClickedValue);
		}
		
		//Fade the slot out to give a graphical hint that something is happening
		$('#plgslt_' + slotName).animate({opacity: .5}, 200);
		
		$.ajax({
			type: 'POST',
			dataType: 'text',
			url: url,
			data: post,
			complete: function(XMLHttpRequest, textStatus) {
				delete tribiq.slotFormSubmissions[slotName];
			},
			success: function(html) {
				tribiq.replacePluginSlotContents(slotName, instanceId, html);
			}
		});
		
		return false;
	}
}


tribiq.uneschyp = function(string) {
	return string.replace(/`r/g, "\r").replace(/`n/g, "\n").replace(/`h/g, "-").replace(/`t/g, "`");
}


tribiq.slots = new Object();
tribiq.plugins = new Object();
tribiq.instances = new Object();

//Create encapculated objects for slots/instances
tribiq.slot = function(pluginInstances) {
	for (var i in pluginInstances) {
		
		var p = new Object();
		
		p.slotName = pluginInstances[i][0];
		p.instanceId = pluginInstances[i][1];
		p.pluginId = pluginInstances[i][2];
		
		if (pluginInstances[i][3]) {
			p.tabId = pluginInstances[i][3];
		} else {
			p.tabId = 0;
		}
		
		//If we are replacing an existing instance in admin mode, delete that first
		var old;
		if (old = tribiq.slots[p.slotName]) {
			if (old.instanceId) {
				delete tribiq.instances[old.instanceId];
				delete window[old.pluginClassName].slots[old.slotName];
			}
		}
		
		//If this slot is non-empty, get info on the Plugin inside
		if (p.instanceId) {
			//Check if the Plugin is running properly..
			if (!p.pluginId || !tribiq.plugins[p.pluginId]) {
				//...and remove the instance if not.
				p.instanceId = 0;
				p.pluginId = 0;
			
			//Otherwise get info on the Plugin inside
			} else {
				p.pluginName = tribiq.plugins[p.pluginId].pluginName;
				p.pluginClassName = tribiq.plugins[p.pluginId].pluginClassName;
				p.pluginClassNameForPhrases = tribiq.plugins[p.pluginId].pluginClassNameForPhrases;
				
				//Make info on this Plugin Instance availible to the core and to the Plugin
				tribiq.instances[p.instanceId] = p;
				window[p.pluginClassName].slots[p.slotName] = p;
			}
		}
		
		//Record which instance is in which slot
		tribiq.slots[p.slotName] = p;
		
		
		//Could do with expanding later, if/when we implement client events
	}
}



//Callback function for refreshPluginSlot()
tribiq.slotFormSubmissions = {};
tribiq.replacePluginSlotContents = function(slotName, instanceId, contents, additionalRequests, recordInURL) {
	
	delete tribiq.slotFormSubmissions[slotName];
	tribiq.currentHashSlot = slotName;

	//Look for the settings at the start
	var tabId = 0;
	var cutoff = contents.indexOf('<!--/INFO-->');
	var info = false;
	var scriptsToRun = new Array();
	var scriptsToRunBefore = new Array();
	var scrollToTopOfSlot = false;
	var showInFloatingBox = false;
	
	if (cutoff != -1) {
		//Get each tag from the info, then chop the tags off of the start of the content
		info = contents.substr(0, cutoff+4).split('--><!--');
		contents = contents.substr(cutoff+12);
		
		//Look through the info at the top of the AJAX return
		for (var i in info) {
			var details = info[i].split('--');
			
			//Allow plugins to reject the AJAX reload and request an entire page reload
			if (details[0] == 'FORCE_PAGE_RELOAD') {
				document.location.href = tribiq.uneschyp(details[1]);
				return;
			
			//Watch out for Tab Ids from nested Plugins
			} else if (details[0] == 'TAB_ID') {
				tabId = tribiq.uneschyp(details[1]);
			
			//Allow plugins to name JavaScript function(s) they wish to be run
			} else if (details[0] == 'SCRIPT') {
				scriptsToRun[scriptsToRun.length] = tribiq.uneschyp(details[1]);
			
			} else if (details[0] == 'SCRIPT_BEFORE') {
				scriptsToRunBefore[scriptsToRunBefore.length] = tribiq.uneschyp(details[1]);
			
			} else if (details[0] == 'SCROLL_TO_TOP') {
				scrollToTopOfSlot = true;
			
			//Allow plugins to open themselves in a floating box
			} else if (details[0] == 'SHOW_IN_FLOATING_BOX') {
				showInFloatingBox = true;
			}
		}
	}
	
	
	//Stop any animations currently on the slot
	$(get('plgslt_' + slotName)).stop(true, true);
	$(get('plgslt_' + slotName)).clearQueue();
	$('#plgslt_' + slotName).animate({opacity: 1}, 200);
	
	
	var oldHTML;
	if (!tribiq.fancyboxOpen && showInFloatingBox === true) {
		oldHTML = get('plgslt_' + slotName).innerHTML.replace(/id=["']\w*["']/gi, '');
	
	} else if (tribiq.fancyboxOpen && showInFloatingBox === false) {
		$.fancybox.close();
		tribiq.fancyboxOpen = false;
	}
	
	if (!window.tribiq_inIframe && !tribiq.fancyboxOpen && typeof tribiqA != 'undefined') {
		//Admins need some more tasks, other than just changing the innerHTML
		tribiqA.replacePluginSlot(slotName, instanceId, tabId, contents, info, scriptsToRunBefore);
	} else {
		for (script in scriptsToRunBefore) {
			if (tribiq.slots[slotName]) {
				tribiq.callScript(tribiq.slots[slotName].pluginClassName, scriptsToRunBefore[script]);
			}
		}
		
		//If we're not in admin mode, just refresh the slot's innerHTML
		tribiq.slot([[slotName, instanceId, tribiq.slots[slotName].pluginId, tabId]]);
		get('plgslt_' + slotName).innerHTML = contents;
	}
	
	//Allow plugins to call JavaScript function(s) after they have been refreshed
	for (script in scriptsToRun) {
		if (tribiq.slots[slotName]) {
			tribiq.callScript(tribiq.slots[slotName].pluginClassName, scriptsToRun[script]);
		}
	}
	
	tribiq.addJQueryElements('#plgslt_' + slotName + ' ');
	
	
	if (scrollToTopOfSlot) {
		//Scroll to the top of a slot if needed
		tribiq.scrollToSlotTop(slotName, true);
	}
	
	
	if (!tribiq.fancyboxOpen && showInFloatingBox === true) {
		tribiq.fancyboxOpen = true;
		tribiq.inlineTmpHTML = oldHTML;
		get('plgslt_' + slotName).className
		$.fancybox({
			href: '#plgslt_' + slotName,
			onClosed: function() {
				tribiq.fancyboxOpen = false;
				get('plgslt_' + slotName).innerHTML = tribiq.inlineTmpHTML;
				delete tribiq.inlineTmpHTML;
				
				//Try to fix a Fancybox bug, where Fancybox adds a wrapper <div> with extra inline styles.
				//I'm not trying to get rid of the wrapper <div>, but I am trying to kill the extra styles as they are causing bugs
				if (get('plgslt_' + slotName).parentNode.style.overflow == 'auto' && get('plgslt_' + slotName).parentNode.style.position == 'relative') {
					get('plgslt_' + slotName).parentNode.setAttribute('style', '');
				}
				
				tribiq.refreshPluginSlot(slotName, instanceId);
			}
		});
		tribiq.resizeFancyBox();
	
	} else if (tribiq.fancyboxOpen) {
		tribiq.resizeFancyBox();
	
	} else if (recordInURL) {
		document.location.hash = instanceId + '|' + additionalRequests.substr(1);
		tribiq.currentHash = document.location.hash;
	}
}

tribiq.callScript = function(className, script) {
	var functionName;
	
	if (typeof script == 'string') {
		script = JSON.parse(script);
	}
	
	if (script && script[0]) {
		if (window[script[0]] && typeof window[script[0]][script[1]] == 'function') {
			className = script.shift();
			functionName = script.shift();
		
		} else if (window[className] && typeof window[className][script[0]] == 'function') {
			functionName = script.shift();
		
		} else {
			return;
		}
		
		window[className][functionName].apply(null, script);
	}
}

tribiq.resizeFancyBox = function() {
	setTimeout('$.fancybox.resize();', 5);
}


//Add jQuery elements automatically by class name
tribiq.addJQueryElements = function(path) {
	if (!/android|iphone|ipod|series60|symbian|windows ce|blackberry/i.test(navigator.userAgent)) {
		jQuery(function($) {
			//Tooltips
			tribiq.tooltips(path + 'a[title]');
			tribiq.tooltips(path + 'img[title]');
			tribiq.tooltips(path + 'area[title]');
			tribiq.tooltips(path + 'input[title]');
			
			//Fancybox/Lightbox replacement
			$(path + "a[rel^='fancybox']").fancybox({});
			$(path + "a[rel^='lightbox']").fancybox({});
			
			//jQuery datepickers
			$(path + 'input.jquery_datepicker').each(function(i, el) {
				if (el.id && window.tribiqA && get('_value_for__' + el.id)) {
					//Admin mode version
					$(el).datepicker({
						dateFormat: tribiqA.siteSettings.storekeeper_date_format,
						altField: '#_value_for__' + el.id,
						altFormat: 'yy-mm-dd',
						showOn: 'focus',
						onSelect: function(dateText, inst) { tribiqAB.fieldChange(this, this.name); }
					});
				
				} else if (el.id && get(el.id + '__0') && get(el.id + '__1') && get(el.id + '__2') && get(el.id + '__3')) {
					el.value = $.datepicker.formatDate(tribiq.dpf, $.datepicker.parseDate('yy-mm-dd', get(el.id + '__0').value));
					
					$('#' + el.id).datepicker({
						dateFormat: tribiq.dpf,
						altField: '#' + el.id + '__0',
						altFormat: 'yy-mm-dd',
						showOn: 'focus'
					});
					
					$('#' + el.id + '__1').remove();
					$('#' + el.id + '__2').remove();
					$('#' + el.id + '__3').remove();
				
				} else {
					$(el).datepicker({
						dateFormat: tribiq.dpf,
						showOn: 'button',
						buttonImage: URLBasePath + 'admin/images/icons/calendar.gif',
						buttonImageOnly: true
					});
				}
			});
			
			if (typeof tribiqA != 'undefined') {
				tribiq.tooltips(path + 'div[title]');
				tribiq.tooltips(path + 'span[title]');
				tribiqA.addJQueryElements(path);
			}
		});
	}
	
	$(path + ':submit').click(function() {tribiq.buttonClick(this)});
	$(path + ' submit').click(function() {tribiq.buttonClick(this)});
}
tribiq.addJQueryElements('');


//The User presses a key on date field
tribiq.dateFieldKeyUp = function(el, event, adminBoxId) {
	//Check to see if it was a space, delete or backspace key
	if (event.keyCode == 8 || event.keyCode == 32 || event.keyCode == 46) {
		//If so, clear the date that has been set
		if (adminBoxId) {
			tribiqAB.blankField(adminBoxId);
			tribiqAB.fieldChange(el, adminBoxId);
		}
		
		tribiq.clearDateField(el);
	}
}


tribiq.clearDateField = function(el) {
	if (typeof el === 'string') {
		el = get(el);
	}
	
	if (el) {
		$(el).datepicker('hide');
		
		el.value = '';
		if (el.id && get(el.id + '__0')) {
			get(el.id + '__0').value = '';
		}
		
		el.blur();
	}
}


tribiq.lastButtonClickedName = false;
tribiq.lastButtonClickedValue = false;
tribiq.buttonClick = function(el) {
	if (el.name) {
		tribiq.lastButtonClickedName = el.name;
		tribiq.lastButtonClickedValue = el.value;
	} else {
		tribiq.lastButtonClickedName = false;
		tribiq.lastButtonClickedValue = false;
	}
}


tribiq.formatDate = function(date) {
	if (!date) {
		return '';
	}
	
	if (typeof date == 'string') {
		var dates = date.replace(/[:-]/g, ' ').split(' ');
		date = new Date(1*dates[0], 1*dates[1]-1, 1*dates[2]);
	}
	
	if (!window.tribiqA || !tribiqA.siteSettings || !tribiqA.siteSettings.storekeeper_date_format) {
		return $.datepicker.formatDate('d M yy', date);
	} else {
		return $.datepicker.formatDate(tribiqA.siteSettings.storekeeper_date_format, date);
	}
}


//Wrapper function for jQuery tooltips
tribiq.tooltips = function(target, options) {
	tribiq.closeTooltip();
	if (target === undefined) {
		//Add tooltips to an entire page after it has been loaded
		tribiq.tooltips('a[title]');
		tribiq.tooltips('img[title]');
		tribiq.tooltips('area[title]');
		tribiq.tooltips('input[title]');
	} else {
		$(target).each(function(i, el) {
			
			if (!options) {
				options = {};
			}
			
			var rel;
			if (el.rel && (rel = (el.rel + '').substr(0, 8)) && (rel == 'fancybox' || rel == 'lightbox')) {
				return;
			
			//Convert old admin-style tooltips
			} else if (typeof tribiqA == 'function') {
				var pos = el.title.indexOf('|');
				if (pos != -1) {
					el.setAttribute('title', '<h3>' + el.title.substr(0, pos) + '</h3><p>' + el.title.substr(pos+1) + '</p>');
				}
			}
			
			if (!options.maxWidth) {
				options.maxWidth = '300px';
			}
			
			$(el).tipTip(options);
		});
	}
}

tribiq.closeTooltip = function() {
	if (get('tiptip_holder')) {
		get('tiptip_holder').style.display = 'none';
	}
}



//Add a new plugin stylesheet to the page dynamically
//This is so that if we add a new plugin or switch themes, the new JS can be added
//without needing a page reload
tribiq.javaScriptOnPage = new Object();
tribiq.addPluginJavaScript = function(pluginName, file, path, alwaysAdd) {
	
	if (!path) {
		path = 'js';
	}
	
	//Work out the path from the plugin name and the theme name
	filePath = 'plugins/' + pluginName + '/' + path + '/' + file;
	
	//Make sure that a script file is only included once, unless the "alwaysAdd" override is used
	if (!alwaysAdd && tribiq.javaScriptOnPage[filePath]) {
		return;
	}
	
	eval(tribiq.nonAsyncAJAX(URLBasePath + filePath));
	
	tribiq.javaScriptOnPage[filePath] = true;
}



//Add a new plugin stylesheet to the page dynamically
//This is so that if we add a new plugin or switch themes, the new CSS can be added
//without needing a page reload

tribiq.pluginStyleSheetsOnPage = new Object();
tribiq.addPluginStyleSheet = function(pluginName, theme) {
	
	//Make sure that the stylesheet is not already on the page!
	if (!tribiq.pluginStyleSheetsOnPage[pluginName]) {
		tribiq.pluginStyleSheetsOnPage[pluginName] = new Array();
	}
	
	if (tribiq.pluginStyleSheetsOnPage[pluginName][theme]) {
		return false;
	} else {
		tribiq.pluginStyleSheetsOnPage[pluginName][theme] = true;
	}
	
	//Work out the path from the plugin name and the theme name
	var path = 'cmsstyles/plugin.css.php?t=' + encodeURIComponent(pluginName) + ',' + encodeURIComponent(theme) +
				'&gz=' + tribiq.useGZ + '&v=' + window.tribiqCSSJSVersionNumber;
	
	//Add the new style sheet
	var newStyleSheet = document.createElement('link');
	
	newStyleSheet.type = 'text/css';
	newStyleSheet.rel = 'stylesheet';
	newStyleSheet.href = URLBasePath + path;
	
	document.getElementsByTagName('head')[0].appendChild(newStyleSheet);
	
	return true;
}


tribiq.captcha = function(publicKey, divId, hideAudio) {
	if (hideAudio) {
		RecaptchaOptions.callback = function() {tribiq.captchaHideAudio()}
	}
	
	if (typeof Recaptcha == 'undefined') {
		$.getScript(
			'http://www.google.com/recaptcha/api/js/recaptcha_ajax.js',
			function() {
				Recaptcha.create(publicKey, divId, RecaptchaOptions);
			}
		);
	} else {
		Recaptcha.create(publicKey, divId, RecaptchaOptions);
	}
}

tribiq.captchaHideAudio = function() {
	$('.recaptcha_only_if_image').attr('href', '#').attr('title', '').css('cursor', 'default');
	$('.recaptcha_only_if_image img').attr('alt', '').attr('tooltip', '').attr('src', $('.recaptcha_only_if_audio img').attr('src'));
}


tribiq.getMouseX = function(e) {
	if (e.pageX != undefined) {
		return e.pageX;
	} else {
		return e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
	}
}

tribiq.getMouseY = function(e) {
	if (e.pageY != undefined) {
		return e.pageY;
	} else {
		return e.clientY + document.body.scrollTop + document.documentElement.scrollTop;
	}
}

tribiq.scrollTop = function() {
	return $(tribiq.browserIsSafari()? 'body' : 'html').scrollTop();
}

tribiq.scrollLeft = function() {
	return $(tribiq.browserIsSafari()? 'body' : 'html').scrollLeft();
}

tribiq.browserIsIE6 = function() {
	var ver = /MSIE ([0-6]{1,}[\.0-9]{0,})/.exec(navigator.userAgent);
	return ver && ver[1] && 1*ver[1] <= 6 && !(/opera|OPERA/.test(navigator.userAgent))? true : false;
}

tribiq.browserIsIE = function() {
	return /msie|MSIE/.test(navigator.userAgent) && !(/opera|OPERA/.test(navigator.userAgent));
}

tribiq.browserIsChrome = function() {
	return /Chrome/.test(navigator.userAgent);
}

tribiq.browserIsSafari = function() {
	return /Safari/.test(navigator.userAgent);
}

tribiq.browserIsWebKit = function() {
	return /WebKit/.test(navigator.userAgent);
}

tribiq.browserIsiPad = function() {
	return /iPad/.test(navigator.userAgent);
}

tribiq.browserIsiPhone = function() {
	return /iPhone/.test(navigator.userAgent);
}

tribiq.browserIsMobile = function() {
	return tribiq.browserIsiPad() || tribiq.browserIsiPhone();
}

//Get the major version number
tribiq.flashMajorVersion = function() {
	if (!$.fn.flash.hasFlash()) {
		return 0;
	} else {
		return 1*$.fn.flash.hasFlash.playerVersion().split(',')[0];
	}
}

tribiq.rightHandedSubStr = function(string, ammount) {
	if (tribiq.browserIsIE()) {
		var len = string.length;
		return string.substr(len - ammount, ammount);
	} else {
		return string.substr(-ammount);
	}
}

tribiq.opacity = function(el, opacity) {
	
	el.style.opacity = opacity;
	
	if (tribiq.browserIsIE()) {
		el.style.filter = 'alpha(opacity=' + Math.floor(100*opacity) + ')';
	}
}

	try {
		document.execCommand("BackgroundImageCache", false, true);
	} catch(err) {}