<?php
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

/**
 * This third section contains variables that are set automatically by the CMS for use by Plugin Developers
 */

class plugin_environment_variables {

 /**
  * $cID holds the content id of the page currently being displayed.
  */
	protected $cID;
	
 /**
  * $containerId holds the HTML Document-Object-Model id of the Plugin Slot.
  * Should Plugin Developers need more DOM elements with ids, we suggest using the $containerId
  * as a base, to ensure that they are unique on the page.
  */
	protected $containerId;
	
 /**
  * $cType holds the content type of the page currently being displayed.
  */
	protected $cType;
	
 /**
  * $cVersion holds the version of the page currently being displayed.
  * The version will always be the published version for Visitors, but could
  * be a historic version for Admins.
  */
	protected $cVersion;
	
 /**
  * Attribute information on any inputs defined in the framework.
  * An array of array or arrays, going Section->Field->Attributes
  */
	protected $fieldInfo;
 
 /**
  * $getVariables holds a copy of the $_GET global variable.
  * Rather than accessing $_GET or $_REQUEST directly, we recommend that Plugin Developers use
  * $getVariables.
  */
	protected $getVariables;
	
 /**
  * $instanceId holds the id of the current instance that is running, as it is in the
  * plugin_instances table
  */
	protected $instanceId;
	
 /**
  * $instanceName holds the name of the current instance that is running, as it is in the
  * plugin_instances table
  */
	protected $instanceName;
	
 /**
  * $languageId holds the id of the language of the current content item
  */
	protected $languageId;
	
 /**
  * If this Plugin is part of a Plugin Nest, $eggId will store the Plugin's id in the nested_plugins table
  * If this Plugin is not part of a Nest, $eggId will be 0.
  */
	protected $eggId;
	
 /**
  * $pluginId holds the id of the current plugin that is running, as it is in the plugins table
  */
	protected $pluginId;
	
 /**
  * $pluginClassName holds the classname of the current plugin that is running.
  * 
  * Plugin Developers should not count on this matching up to the current classname, as another
  * Plugin Developer may be extending your class - in which case this will match up with the 
  * new class name.
  */
	protected $pluginClassName;
	
	protected $pluginClassNameForPhrases;
	
 /**
  * $pluginName holds the directory of the current plugin that is running.
  * 
  * Plugin Developers should not count on this matching up to the current directory, as another
  * Plugin Developer may be extending your class - in which case this will match up with the 
  * new plugin's directory.
  */
	protected $pluginName;

 /**
  * $postVariables holds a copy of the $_POST global variable.
  * Rather than accessing $_POST or $_REQUEST directly, we recommend that Plugin Developers use
  * $postVariables.
  */
	protected $postVariables;
	
 /**
  * $skinId holds the id of the Skin in use on this page
  */
	protected $skinId;

 /**
  * $slotName holds the name of the Slot that is currently being displayed
  */
	protected $slotName;

 /**
  * $slotName holds the level of the Slot that is currently being displayed
  * 1 is item level, 2 is Template, and 3 is Template Family.
  */
	protected $slotLevel;
	
 /**
  * If this Plugin is part of a tabbed Plugin Nest, $tabId will store the Tab's id in the nested_plugins table
  * If this Plugin is not part of a Nest, or the Plugin is part of a Nest but no tabs have been set up, $tabId will be 0.
  */
	protected $tabId;

}