<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

/**
 * This section contains methods that can be called to alter how the CMS displays a plugin.
 * They must be called from within the init() method above in order to take effect.
 */

class plugin_initialization_functions extends plugin_environment_variables {

 /**
  * The callScript() method allows Plugin Developers to request a JavaScript function 
  * be run as soon as a Plugin has loaded.
  * 
  * If called more than once then multiple scripts can be requested, but due to the way some browsers work
  * there can be potential race-conditions, so multiple calls should not be used for related functions.
  * 
  * Be warned that this is work in progress, and likely to change
  * 
  * Note: You can't use hyphens or new lines in your function names or parameters
  */
	protected final function callScript(/*[*/$className, /*]*/ $scriptName /*[, $arg1 [, $arg2 [, ... ]]]*/) {
		$args = func_get_args();
		$this->cmsPluginApiLinkCallScriptWhenLoaded(false, $args);
	}
	
	protected final function callScriptBeforeAJAXReload($className, $scriptName /*[, $arg1 [, $arg2 [, ... ]]]*/) {
		$args = func_get_args();
		$this->cmsPluginApiLinkCallScriptWhenLoaded(true, $args);
	}
	
 /**
  * More advanced version of callScript() with more options
  * Note: $beforeLoad will only have an effect when refreshing via AJAX, and is ignored when first loading the page
  */
	protected final function callScriptAdvanced($beforeAJAXReload, $className, $scriptName /*[, $arg1 [, $arg2 [, ... ]]]*/) {
		$args = func_get_args();
		array_splice($args, 0, 1);
		$this->cmsPluginApiLinkCallScriptWhenLoaded($beforeAJAXReload, $args);
	}
	
	protected final function captcha() {
		
		$publickey = setting('google_recaptcha_public_key');
		require_once 'libraries/mit/recaptcha/recaptchalib.php';
		
		if (arrayKey($_REQUEST, 'method_call') == 'refreshPlugin') {
			$html = '<div id="tribiq_captcha_'. $this->slotName. '"></div>';
			$this->callScript(
				'tribiq', 'captcha',
				$publickey,
				'tribiq_captcha_'. $this->slotName,
				!setting('google_recaptcha_show_audio_challenge'));
		} else {
			$html = recaptcha_get_html($publickey, null, substr($_SERVER['SCRIPT_URI'], 0, 5) == 'https');
			
			if (!setting('google_recaptcha_show_audio_challenge')) {
				$this->callScript('tribiq', 'captchaHideAudio');
			}
		}
		
		return $html;
	}
	
	protected final function checkCaptcha() {
		
		$privatekey = setting('google_recaptcha_private_key');
		require_once 'libraries/mit/recaptcha/recaptchalib.php';
        
        $resp = recaptcha_check_answer ($privatekey,
                                        $_SERVER['REMOTE_ADDR'],
                                        post('recaptcha_challenge_field'),
                                        post('recaptcha_response_field'));

        return (bool) $resp->is_valid;
	}

 /**
  * Plugins have the ability to reload via AJAX.
  * This can either be at the CMS's request, or by using a few of the helper methods, and the
  * Plugin Developer's request.
  * 
  * However sometimes technical restrictions may mean that a Plugin cannot reload via AJAX.
  * If the forcePageReload() method is used, the CMS will initiate a full page reload instead.
  */
	protected final function forcePageReload($reload = true) {
		$this->cmsPluginApiLinkForcePageReload($reload);
	}

 /**
  * This takes the Visitor to a different url, either using an actual header redirect, or by using
  * JavaScript if your Plugin is reloading via AJAX.
  */
	protected final function headerRedirect($link) {
		$this->cmsPluginApiLinkHeaderRedirect($link);
	}

 /**
  * Allow a Plugin to place itself in/remove itself from a floating box
  * 
  * Note: will not work for Plugins in Nests
  */
	protected final function showInFloatingBox($showInFloatingBox = true) {
		$this->cmsPluginApiShowInFloatingBox($showInFloatingBox);
	}

 /**
  * Scroll the Visitor's view to the top of the Plugin's Slot when reloading a Plugin via AJAX
  * 
  * Note: will not work for Plugins in Nests
  */
	protected final function scrollToTopOfSlot($scrollToTop = true) {
		$this->cmsPluginApiScrollToTopOfSlot($scrollToTop);
	}



 /**
  * In Admin mode, Plugin Slots can have one of two types of border.
  * The first type marks the boundaries of the slot
  * The second type indicates that the Admin has chosen to edit the slot.
  * 
  * By default the CMS will display the normal boundary border.
  * The markSlotAsBeingEdited() method will switch this to the editing border.
  */
	protected final function markSlotAsBeingEdited($beingEdited = true, $canBeEdited = true) {
		$this->cmsPluginApiLinkMarkSlotAsBeingEdited($beingEdited, $canBeEdited);
	}


 /**
  * Register a $_GET request that this Plugin uses
  * Note: these request names should be suitably prefixed to avoid clashes
  */
	protected final function registerGetRequest($request, $defaultValue = null) {
		cms_core::$importantGetRequests[$request] = $defaultValue;
	}


 /**
  * Override the title of the current page
  */
	protected final function setPageTitle($title) {
		cms_core::$pageTitle = $title;
	}


 /**
  * Override the title of the current menu item
  */
	protected final function setMenuTitle($title) {
		cms_core::$menuTitle = $title;
	}
}