<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

/**
  * This section contains methods designed to help perform common tasks
  * 
  * While using these methods is optional, every Plugin Developer should be using them as this ensures that
  * all Plugins perform common tasks consistently.
  * In addition, they cover a lot of common functionality that Plugins need, so we would highly recommend that
  * Plugin Developers use them whenever they can
 */

class plugin_base_class extends cms_plugin_api_link {
	
	
	protected final function absCMSDir() {
		return CMS_ROOT;
	}
	
	protected final function absCMSDirURL() {
		return httpOrHttps(). httpHost(). SUBDIRECTORY;
	}
	
	protected final function absPluginAJAXURL($requests = '') {
		return httpOrHttps(). httpHost(). SUBDIRECTORY. 'plugins.php?pluginClassName='. $this->pluginClassName. '&method_call=handleAJAX'. urlRequest($requests);
	}
	
	protected final function absPluginDir() {
		return CMS_ROOT. 'plugins/'. $this->pluginName. '/';
	}
	
	protected final function absPluginDirURL() {
		return httpOrHttps(). httpHost(). SUBDIRECTORY. 'plugins/'. $this->pluginName. '/';
	}


 /**
  * The cache() method allows Plugin Developers to work with cached values
  * 
  * Cached values could be things such as XML feeds from another website, which do not need
  * to be updated every time a user views the page
  * 
  * Plugin Developers must write then provide the name of a method within their Plugin that will
  * return the value of the cache when called, and an expiry time for the cache (defaults to 10 minutes).
  * 
  * Be warned that this is work in progress, and likely to change
  */
	protected final function cache($methodName, $expiryTimeInSeconds = 600, $request = '') {
		
		//Look in the database for an up to date cache
		$sql = "
			SELECT cache
			FROM ". DB_NAME_PREFIX. "plugin_instance_cache
			WHERE last_updated > NOW() - INTERVAL ". (int) $expiryTimeInSeconds. " SECOND
			  AND method_name = '". mysql_real_escape_string($methodName). "'
			  AND request = '". mysql_real_escape_string($request). "'
			  AND instance_id = ". (int) $this->instanceId;
		$result = my_mysql_query($sql);
		
		//If we found one, then return it and stop.
		if ($row = mysql_fetch_assoc($result)) {
			return $row['cache'];
		}
		
		//Otherwise, get a new value and update the cache
		$cache = $this->$methodName($request);
		
		$sql = "
			REPLACE INTO ". DB_NAME_PREFIX. "plugin_instance_cache SET
				cache = '". mysql_real_escape_string($cache). "',
				last_updated = NOW(),
				method_name = '". mysql_real_escape_string($methodName). "',
				request = '". mysql_real_escape_string($request). "',
				instance_id = ". (int) $this->instanceId;
		my_mysql_query($sql);
		
		return $cache;
	}


 /**
  * checkFrameworkSectionExists() checks the current theme to see if a section in it exists.
  * 
  * If a Plugin Developer wishes, this function could enable Theme Creators to switch sections of the
  * Plugin on and off as they see fit, without worrying about the framework() function throwing
  * "Section not found in the theme" errors.
  * 
  * Alternately, Plugin Developers could check to see if a section is in a Theme, and then fall back
  * to a different section if it was not present.
  *
  * Themes should not infulence the execution of code too heavily, so Plugin Developers should not go
  * much further that that.
  */
	protected final function checkFrameworkSectionExists($section = 'Outer') {
		return $this->cmsPluginApiLinkCheckFrameworkSectionExists($section);
	}
	
	
	/** 
	 * Check whether another Plugin is included on the current page
	 */
	protected final function checkPluginOnPage($className) {
		global $slotContents;
		
		foreach ($slotContents as &$instance) {
			if ($instance['class_name'] && $instance['class_name'] == $className) {
				return true;
			}
		}
		
		return false;
	}


 /**
  * Clear the cache
  */
	protected final function clearCache($methodName, $request = '', $useLike = false) {
		
		//Look in the database for an up to date cache
		$sql = "
			DELETE
			FROM ". DB_NAME_PREFIX. "plugin_instance_cache
			WHERE last_updated > NOW() - INTERVAL ". (int) $expiryTimeInSeconds. " SECOND
			  AND method_name = '". mysql_real_escape_string($methodName). "'
			  AND instance_id = ". (int) $this->instanceId;
		
		if ($useLike) {
			$sql .= "
			  AND request LIKE '". mysql_real_escape_string($request). "%'";
		} else {
			$sql .= "
			  AND request = '". mysql_real_escape_string($request). "'";
		}
		
		my_mysql_query($sql);
	}
	
	
 /**
  * The clearRequest() method removes an entry from the $postVariables and the $getVariables
  */
	protected final function clearRequest($name) {
		if (isset($this->postVariables[$name])) {
			unset($this->postVariables[$name]);
		}
		if (isset($this->getVariables[$name])) {
			unset($this->getVariables[$name]);
		}
	}
	
	
 /**
  * closeForm() sloses a form opened by the openForm() function
  */
	protected final function closeForm() {
		return '
				</form>';
	}
	
	
	protected final function CMSDir() {
		return SUBDIRECTORY;
	}
	
	//Deprecated as a utility function. Please use the core function instead
	protected final function curl($URL) {
		return curl($URL);
	}
	
	//Deprecated as a utility function. Please use the core function instead
	protected final function displayHTMLAsPlainText(&$text, $excerptLength = 200, $stripTitles = false, $highlightTerm = false, $excerptCutOffPhrase = false) {
		
		if ($excerptCutOffPhrase === false) {
			$excerptCutOffPhrase = $this->phrase('_EXCERPT_CUTOFF');
		}
		
		displayHTMLAsPlainText($text, $excerptLength, $stripTitles, $highlightTerm, $excerptCutOffPhrase);
	}
	
	
 /**
  * Included with the Plugin common functionality is a Templating System.
  * 
  * It should be noted that this isn't a "smart" system which allows for program code within a template.
  * Instead it behaves more like the XSLT system, in that it returns a series of HTML with Merge Fields
  * to put values in.
  * 
  * The framework() method is used to display a section from the Theme's Template.
  * 
  * The framework() method takes the following inputs:
  * 		$section
  * 			The Section within the theme to display. Defaults to 'Outer', which is a special name for
  *				the whole section.
  * 			Sections must have unique names, so a name is enough to determine which section a Plugin
  * 			Developer is asking for no matter where it actually is in the Theme's Template. 
  * 		$mergeFields
  * 			An array of Merge Fields to display.
  * 			This needs to be an associative array, with the name of the Merge Field as the index,
  *				leading to the string that you wish to put in it as the value.
  * 			For every Merge Field framework() comes across, it looks in the $mergeFields array
  * 			for that index and then displays the string that is there.
  * 
  * 			If Plugin Developers wish to display a template more than once (e.g. for multiple rows)
  * 			then $mergeFields can be passed an array of arrays. The framework() will then display
  *				the template multiple times; once for each sub-array.
  * 
  * Sections can just be defined one after the other in a theme, but it is also possible to define a
  * section within another section.
  * By default framework() will only display the section that the Plugin Developer has asked for,
  * and will skip any sections it sees inside it.
  * This means that Sections can be laid out in a structured manner within a Theme Template for added
  * readability, without worrying about any effect on the rendering.
  * 
  * However two different approaches are provided for including these sub-sections. The first one is by
  * allowing sub-Sections by name:
  * 		$allowedSubSections
  * 			This takes an array of Section names.
  * 			The framework() function will then display any sections whose names are indexes in
  * 			this array.
  * 			If an array is stored, then that array will be used as the merge fields for that section.
  * 		$subSectionDepthLimit
  * 			This sets a finite limit on the recursion depth. The default is 5.
  * 
  * The second approach is for a Plugin Developer to manually call each Section by name, in the order in
  * which they should be displayed. As Plugin Developers may need to display a Section within another
  * Section, the following controls are availible:
  * 		$half
  * 			If a Sections contains one sub-Section, then this sub-Section logically divides the outer
  * 			Section into a Header and a Footer.
  *				If $half is set to 1, then the framework() method will only display the Header.
  * 			If $half is set to 2, then the framework() method will only display the Footer.
  * 		$halfwayPoint
  * 			If a Section contains more than one sub-Section, then by default the framework()
  * 			method will pick the first it seems to use as the divider. However this may not be the
  * 			desired behaviour, so Plugin Developers should specifically name the section they wish
  *				to be the divider.
  * Note that these two different methods are not mutually exclusive and can be combined, if a Plugin
  * Developer wishes
  */
	protected final function framework(
								$section = 'Outer', $mergeFields = array(),
								$allowedSubSections = array(), $subSectionDepthLimit = 5,
								$half = false, $halfwayPoint = true
							 ) {
		
		$this->cmsPluginApiLinkFramework(
				$section, $mergeFields, $allowedSubSections, $subSectionDepthLimit, $half, $halfwayPoint);
	}


	/** 
	 * Get the Plugin Framework's directory
	 */
	protected final function frameworkDir() {
		return $this->cmsPluginApiLinkGetFrameworkDir();
	}


	/** 
	 * Get the Plugin Framework's name
	 */
	protected final function frameworkName() {
		return $this->cmsPluginApiLinkGetFrameworkName();
	}
	
	
 /**
  * The frameworkHead() method is a shortcut function to the Header functionality of the
  * framework() method.
  * If Plugin Developers may wish to call this method rather than call framework() with the
  * $half input set, as it makes their code more readable.
  */
	protected final function frameworkHead(
								$section = 'Outer',
								$halfwayPoint = true,
								$mergeFields = array(),
								$allowedSubSections = array(), $subSectionDepthLimit = 5) {
		$this->cmsPluginApiLinkFramework(
				$section, $mergeFields, $allowedSubSections, $subSectionDepthLimit, 1, $halfwayPoint);
	}
	
	
 /**
  * Gets a list of input fields in a framework section.
  * Works recursively, so includes any subsections as well.
  */
	protected final function frameworkFields($section = 'Outer', $allowedSubSections = array(), $subSectionDepthLimit = 5) {
		$fields = array();
		$this->cmsPluginApiLinkListFrameworkFields($section, $fields, $allowedSubSections, $subSectionDepthLimit);
		return $fields;
	}
	
	
 /**
  * The frameworkFoot() method is a shortcut function to the Footer functionality of the
  * framework() method.
  * If Plugin Developers may wish to call this method rather than call framework() with the
  * $half input set, as it makes their code a bit more readable.
  */
	protected final function frameworkFoot(
								$section = 'Outer',
								$halfwayPoint = true,
								$mergeFields = array(),
								$allowedSubSections = array(), $subSectionDepthLimit = 5) {
		$this->cmsPluginApiLinkFramework(
				$section, $mergeFields, $allowedSubSections, $subSectionDepthLimit, 2, $halfwayPoint);
	}
	
	
 /**
  * The get() method allows Plugin Developers to access the $getVariables array() from a read-only context
  * This is good practice, as it would be confusing to actually change something in the $getVariables array().
  */
	protected final function get($name) {
		return arrayKey($this->getVariables, $name);
	}
	
	
 /**
  * The getCIDAndCTypeFromSetting() method extracts a content id and a content type from a &quot;content&quot;
  * type setting
  * Given that the content item might not be in the user's current language, if AVLS is installed we will
  * check to see if there is an equivalent content item in their language
  */
	protected final function getCIDAndCTypeFromSetting(&$cID, &$cType, $setting, $getLanguageEquivalent = true) {
		
		$cID = false;
		$cType = false;
		
		if ($cIDAndCType = $this->setting($setting)) {
			$cIDAndCTypeArray = explode('_', $cIDAndCType);
			if ($cIDAndCTypeArray[0] && $cIDAndCTypeArray[1]) {
				$cID = $cIDAndCTypeArray[0];
				$cType = $cIDAndCTypeArray[1];
				
				if ($getLanguageEquivalent) {
					$this->langEquivalentItem($cID, $cType);
				}
				
				return (checkPriv() || getPublishedVersion($cID, $cType));
			}
		}
		
		return false;
	}
	
	
 /**
  * The getMediaIdTypeAndURLFromSetting() method extracts a media id and a media type from an &quot;image&quot;,
  * a &quot;media&quot; or a a &quot;movie&quot; type setting
  */
	protected final function getMediaIdTypeAndURLFromSetting(&$mediaId, &$mediaType, &$mediaURL, $setting) {
		
		$mediaId= false;
		$mediaType = false;
		$mediaURL = false;
		
		if ($mediaIdAndType = $this->setting($setting)) {
			$mediaIdAndTypeArray = explode('_', $mediaIdAndType);
			if ($mediaIdAndTypeArray[0] && $mediaIdAndTypeArray[1]) {
				$mediaType = $mediaIdAndTypeArray[0];
				$mediaChecksum = $mediaIdAndTypeArray[1];
				
				$mediaURL = 'cmsincludes/'. $mediaType. '.php?c='. $mediaChecksum;
				return true;
			}
		}
		
		return false;
	}
	
	//Deprecated as a utility function. Please use the core function instead
	protected final function inc($pluginClass) {
		return inc($pluginClass);
	}
	
	//Deprecated as a utility function. Please use the core function instead
	protected final function itemStickyImage($cID, $cType, $cVersion = false) {
		return itemStickyImage($cID, $cType, $cVersion);
	}
	
	//Deprecated as a utility function. Please use the core function instead
	protected final function langEquivalentItem(&$cID, &$cType) {
		langEquivalentItem($cID, $cType);
	}
	
	
 /**
  * The linkToItem() method allows Plugin Developers to create a hyperlink to another content item.
  * If the content type is html, then an alias may be passed instead of a content id.
  */
	protected final function linkToItem($cID, $cType = 'html', $fullPath = false, $request = '') {
		return linkToItem($cID, $cType, false, $fullPath, $request, true, false);
	}
	
	
 /**
  * The linkToItemAnchor() method allows Plugin Developers to create a hyperlink to another content item.
  * If the content type is html, then an alias may be passed instead of a content id.
  */
	protected final function linkToItemAnchor($cID, $cType = 'html', $fullPath = false, $request = '') {
		return ' href="'. htmlspecialchars(linkToItem($cID, $cType, false, $fullPath, $request, true, false)). '"';
	}
	
 /**
  * Shortcut to calling both linkToItemAnchor() and linkToItemJS().
  */
	protected final function linkToItemAnchorAndJS($cID, $cType = 'html', $fullPath = false, $request = '') {
		return array($this->linkToItemAnchor($cID, $cType, $fullPath, $request), $this->linkToItemJS($cID, $cType, $request));
	}
	
 /**
  * JavaScript version of linkToItem(), in the same form as the other JS functions
  */
	protected final function linkToItemJS($cID, $cType = 'html', $request = '') {
		return $this->pluginClassName. '.goToItem(\''. addslashes($cID). '\', \''. addslashes($cType). '\', \''. addslashes(htmlspecialchars($request)). '\');';
	}
	
 /**
  * Loads a Plugin's framework from the framework.html file.
  * This is done automatically by the CMS when you make a call to the framework() method, so you normally don't need to call it
  */
	
	protected final function loadFramework() {
		$this->cmsPluginApiLinkLoadFramework();
	}
	
	
 /**
  * The openForm() method provides Plugin Developers with the functionality to open an HTML form within
  * the Plugin Slot.
  * On submission, the form will automatically reload the plugin via AJAX, with the values of the form's
  * fields availible in the $postVariables variable.
  * 
  * As the openForm() method takes care of the fiddly job of submitting the form and reloading the page,
  * we recommend that Plugin Developers use it, unless they are trying to make a form that submits to
  * another location.
  * 
  * Due to the way the forcePageReload() method works, if a Plugin Developer makes a call to the
  * openForm() method and includes a form in their Plugin, any code to handle the submission should
  * be included in the init() method rather than the showSlot method.
  * 
  * Plugin Developers should remember to close the form with the closeForm() method.
  */
	protected final function openForm($onSubmit = '', $extraAttributes = '', $action = false) {
		return '
				<form method="post"'. $extraAttributes. '
				  onsubmit="'. htmlspecialchars($onSubmit). ' return tribiq.formSubmit(this);"
				  action="'. htmlspecialchars(ifNull($action, linkToItem($this->cID, $this->cType))). '">
					'. $this->remember('cID', $this->cID). '
					'. $this->remember('tab', $this->tabId). '
					'. $this->remember('cType', $this->cType). '
					'. $this->remember('slotName', $this->slotName). '
					'. $this->remember('instanceId', $this->instanceId);
	}
	
	
	protected final function pagination($paginationStyleSettingName, $currentPage, $pages, &$html) {
		
		//Attempt to check if the named class exists, and fall back to 'pagSimpleNumbers' if not
		list($class, $method) = explode('::', $this->setting($paginationStyleSettingName), 2);
		
		if ($class && $method && inc($class) && method_exists($class, $method)) {
		} else {
			$class = 'cms_core';
			$method = 'pagCloseWithNPIfNeeded';
		}
		
		$class = new $class;
		$class->setInstanceVariables(array(
			$this->cID, $this->cType, $this->cVersion, $this->skinId, $this->slotName,
			$this->instanceName, $this->instanceId,
			$this->pluginName, $this->pluginClassName,
			$this->pluginClassNameForPhrases,
			$this->pluginId,
			$this->defaultFramework, $this->framework,
			$this->defaultTheme, $this->themeStyle,
			$this->slotLevel),
			$this->eggId, $this->tabId);
		$class->$method($currentPage, $pages, $html);
	}
	
	
	/** 
	 * Tribiq CMS is a multilingual CMS; Plugin Developers cannot just write English sentances directly into
	 * their Plugins! Instead Plugin Developers should use the getPhrase() method to display text.
	 */
	protected final function phrase($code, $replace = array()) {
		return getVLPPhrase($code, $replace, $this->languageId, $returnFalseOnFailure = false, $this->pluginClassNameForPhrases);
	}
	
	
	protected final function pluginAJAXURL($requests = '') {
		return 'plugins.php?pluginClassName='. $this->pluginClassName. '&method_call=handleAJAX'. urlRequest($requests);
	}
	
	
	protected final function pluginDir() {
		return 'plugins/'. $this->pluginName. '/';
	}
	
	
 /**
  * The post() method allows Plugin Developers to access the $postVariables array() from a read-only context
  * This is good practice, as it would be confusing to actually change something in the $postVariables array().
  */
	protected final function post($name) {
		return arrayKey($this->postVariables, $name);
	}
	
	
 /**
  * The refreshPluginSlotAnchor() method allows Plugin Developers to create a hyperlink to their own Plugin.
  * 
  * When the user clicks on the link, the CMS will reload the Plugin via AJAX, with any addition requests
  * availible in the $getVariables.
  * 
  * Note that the CMS will always provide Plugins with the cID, cType, cVersion, slotName, instanceId,
  * pluginName and pluginId, so these need not be passed as additional requests.
  * 
  * Also note that any additional requests will be wiped clean if the forcePageReload() method is called.
  */
	protected final function refreshPluginSlotAnchor($requests = '', $scrollToTopOfSlot = true) {
		return
			$this->linkToItemAnchor($this->cID, $this->cType, $fullPath = false, '&slotName='. $this->slotName. ($this->tabId? '&tab='. $this->tabId : ''). urlRequest($requests)).
			' onclick="'.
				$this->refreshPluginSlotJS($requests, $scrollToTopOfSlot).
				' return false;"';
	}
	
	
 /**
  * Shortcut for calling refreshPluginSlotAnchor() and refreshPluginSlotJS().
  */
	protected final function refreshPluginSlotAnchorAndJS($requests = '', $scrollToTopOfSlot = true) {
		return array($this->refreshPluginSlotAnchor($requests, $scrollToTopOfSlot), $this->refreshPluginSlotJS($requests, $scrollToTopOfSlot));
	}
	
	
 /**
  * The refreshPluginSlotJS() method provides a line of JavaScript code, to allow Plugin Developers to
  * reload their Plugin via AJAX.
  * 
  * Note that anything passed into the $requests variable may need to be html-escaped,
  * depending on where a Plugin Developer uses this.
  */
	protected final function refreshPluginSlotJS($requests = '', $scrollToTopOfSlot = true) {
		return 
			$this->pluginClassName.'.refreshPluginSlot('.
				'\''. $this->slotName. '\', '.
				'\''. addslashes(htmlspecialchars(urlRequest($requests))). '\', '.
				($scrollToTopOfSlot? 1 : 0). ');';
	}
	
	
 /**
  * The remember() method is a shortcut function for writing a hidden field in a form.
  * Plugin Developers must provide the name of the field, and optionally a value for it.
  * 
  * If a value is not specified then the request() method will be used to look one up.
  */
	protected final function remember($name, $value = false, $htmlId = false) {
		
		if ($value === false) {
			$value = $this->request($name);
		}
		
		if ($htmlId === true) {
			$htmlId = $name;
		}
		
		if ($htmlId) {
			$htmlId = ' id="'. htmlspecialchars($htmlId). '"';
		}
		
		return '<input type="hidden"'. $htmlId. ' name="'. htmlspecialchars($name). '" value="'. htmlspecialchars($value). '" />';
	}
	
	//Given a string with phrase codes in the format [[_PHRASE_CODE]], this function will replace all of the phrases with their values.
	protected final function replacePhraseCodesInString(&$string /*[, $arg1 [, $arg2 [, ... ]]]*/) {
		$content = preg_split('/\[\[([^\[\]]+)\]\]/', $string, -1,  PREG_SPLIT_DELIM_CAPTURE);
		$string = '';
		foreach ($content as $i => &$str) {
			if ($i%2) {
				//Every odd element will be a phrase code
				$string .= $this->phrase($str);
			} else {
				//Every even element will be plain text
				$string .= $str;
			}
		}
	}
	
	
	
 /**
  * The request() method is a shortcut to checking the getVariables and the postVariables array.
  * Plugin developers can use it if they are prepared to accept requests by GET or by POST.
  * 
  * Note that unlike PHP's $_REQUEST, the request() method doesn't provide any cookie information
  */
	protected final function request($name) {
		return is_array($this->postVariables) && isset($this->postVariables[$name])? $this->postVariables[$name] : arrayKey($this->getVariables, $name);
	}
	
	
	//Reset the ODD_OR_EVEN theme merge field
	protected final function resetOddOrEven() {
		$this->cmsPluginApiLinkResetOddOrEven();
	}
	
	
 /**
  * The setting() method retrieves the value of a setting for this plugin/instance
  */
	protected final function setting($name) {
		return $this->cmsPluginApiLinkGetSetting($name);
	}
	
	
 /**
  * The showFileLink() method generates an RSS link
  * 
  * For use with the showLink() admin placeholder method
  */
	protected final function showFileLink($requests = '') {
		return
			httpOrHttps(). httpHost(). SUBDIRECTORY.
			'plugins.php?pluginClassName='. $this->pluginClassName. '&method_call=showFile'.
			urlRequest($requests);
	}
	
	
 /**
  * The showFileLink() method generates a URL that you can use to direct a visitor to your showFloatingBox() method
  * 
  * For use with the showFloatingBox() admin placeholder method
  */
	protected final function showFloatingBoxLink($requests = '') {
		return
			httpOrHttps(). httpHost(). SUBDIRECTORY.
			'plugins.php?pluginClassName='. $this->pluginClassName. '&method_call=showFloatingBox'.
			'&cID='. $this->cID.
			'&cType='. $this->cType.
		  (checkPriv()?
			'&cVersion='. $this->cVersion
		   : '').
			'&instanceId='. $this->instanceId.
			'&slotName='. $this->slotName.
			'&eggId='. $this->eggId.
			urlRequest($requests);
	}
	
	
 /**
  * The showIframeLink() method generates a URL that you can use to direct a visitor to your showIframe() method
  * 
  * For use with the showIframe() admin placeholder method
  */
	protected final function showIframeLink($requests = '') {
		return
			httpOrHttps(). httpHost(). SUBDIRECTORY.
			'plugins.php?pluginClassName='. $this->pluginClassName. '&method_call=showIframe'.
			'&cID='. $this->cID.
			'&cType='. $this->cType.
		  (checkPriv()?
			'&cVersion='. $this->cVersion
		   : '').
			'&instanceId='. $this->instanceId.
			'&slotName='. $this->slotName.
			'&eggId='. $this->eggId.
			urlRequest($requests);
	}
	
	
 /**
  * The showImageLink() method generates a link to an image
  * 
  * For use with the showLink() admin placeholder method
  */
	protected final function showImageLink($requests) {
		return
			httpOrHttps(). httpHost(). SUBDIRECTORY.
			'plugins.php?pluginClassName='. $this->pluginClassName. '&method_call=showImage'.
			urlRequest($requests);
	}
	
	
 /**
  * The showRSSLink() method generates an RSS link
  * 
  * For use with the showRSS() admin placeholder method
  */
	protected final function showRSSLink() {
		return
			httpOrHttps(). httpHost(). SUBDIRECTORY.
			'plugins.php?'.
				'cID='. $this->cID.
				'&cType='. $this->cType.
			  (checkPriv()?
				'&cVersion='. $this->cVersion
			   : '').
				'&slotName='. $this->slotName.
				'&instanceId='. $this->instanceId.
				'&eggId='. $this->eggId.
				'&method_call=showRSS';
	}

	//Deprecated as a utility function. Please use the core function instead
	protected final function sqlToSearchContentTable($cType, $hidePrivateItems = true, $onlyShow = false, $specificId = false) {
		return sqlToSearchContentTable($cType, $hidePrivateItems, $onlyShow, $specificId);
	}
	
	
 /**
  * Get the Plugin Theme Style's prefix, or false if there is no CSS file
  */
	protected final function themePrefix() {
		return $this->cmsPluginApiLinkGetThemePrefix();
	}
}