<?php 
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

//Allow a plugin to show on a page on its own.


require_once 'documentation_for_developers/helper_functions.inc.php';

//Workaround for a bug where uploadify won't work with sessions properly
if (array_key_exists('session_id', $_REQUEST) && array_key_exists('session_key', $_REQUEST)) {
	session_id(request('session_id'));
	session_start();
	if (session('session_key') !== request('session_key')) {
		exit;
	}
} else {
	session_start();
}


//Check which method call is being requested
//Some method calls are associated with instances and content items, and some are not
if (request('method_call') == 'refreshPlugin'
 || request('method_call') == 'showFloatingBox'
 || request('method_call') == 'showIframe'
 || request('method_call') == 'showRSS'
 || request('method_call') == 'showSlot'
 || (request('method_call') == 'showFile' && request('cID') && request('cType') && request('instanceId'))) {

	//showRSS and showFloatingBox method calls relate to content items
	require 'visitorheader.inc.php';
	useGZIP($siteConfig['compress_web_pages']);
	
	//If an admin is logged in, include admin functions for refreshing Plugins
	if (checkPriv()) {
		require_once CMS_ROOT. 'cmsincludes/admin.inc.php';
	}
	
	
	//Check the content item that this is being linked from, and whether the current user has permissions to access it
	$error = false;
	$instance = false;
	if (($cID = is_numeric(request('cID'))? request('cID') : getIdFromAlias(request('cID')))
	 && ($cType = request('cType'))
	 && (includeContentType($cType))
	 && ($cVersion = getShowableVersion($cID, $cType, request('cVersion'), session('admin_username'), session('extranetUserID')))) {
	 	
		list ($use_template_family, $use_template_filename, $use_template_id, $custom_bodytag) = getTemplateFamily($cID, $cType, $cVersion);
		$use_template_filename = ensureTemplateExists($use_template_id, $use_template_family, $siteConfig, $cType);
		$skin = getSkin($cID, $cType, $cVersion, $use_template_id, true);
		
		if (!defined('TEMPLATE_PATH')) {
			define('TEMPLATE_PATH', SITENAME. '/'. $use_template_family);
		}
		if (!defined('SKIN_PATH')) {
			define('SKIN_PATH', 'templates/'. TEMPLATE_PATH. '/skins/'. $skin['name']. '/');
		}
		if (!defined('SKIN_PATH_URL')) {
			define('SKIN_PATH_URL', 'templates/'. TEMPLATE_PATH. '/skins/'. rawurlencode($skin['name']). '/');
		}
		
		if ($instanceId = request('instanceId')) {
			
			getRunningPlugins();
			getSlotContents(
				$slotContents, $cID, $cType, $cVersion,
				$use_template_id, $use_template_filename, $use_template_family, $skin['id'],
				$instanceId, HTMLId(request('slotName')), $ajaxReload = true);
			
			foreach ($slotContents as $s => &$i) {
				
				if ($i['instance_id'] == $instanceId) {
					$instance = $i;
					$slotName = $s;
					break;
				} elseif (!$error) {
					$error = $i['error'];
				}
			}
		}
	}
	
	if (!$instance) {
		if (request('method_call') == 'refreshPlugin' && checkPriv()) {
			if ($error) {
				echo $error;
			} elseif (checkRowExists('plugin_inst_temp_link',
						array(
							'family_name' => $use_template_family,
							'template_id' => $use_template_id,
							'slot_name' => HTMLId(request('slotName')),
							'instance_id' => 0))
				   || checkRowExists('plugin_inst_item_link',
						array(
							'content_id' => $cID,
							'content_type' => $cType,
							'content_version' => $cVersion,
							'slot_name' => HTMLId(request('slotName')),
							'instance_id' => 0))
			) {
				echo getPhrase('[Slot set to <em>Show Nothing</em>]');
			} else {
				echo '<span class="tribiq_empty_slot_message">', getPhrase('[Empty Slot]'), '</span>';
			}
		}
		exit;
	}
	

} elseif (request('method_call') == 'handleAJAX'
		|| request('method_call') == 'storekeeperAJAX'
		|| request('method_call') == 'quickValidateAdminBox'
		|| request('method_call') == 'showFile'
		|| request('method_call') == 'showImage') {
	
	//Allow handleAJAX, showFile and showImage to be for visitors or admins as needed
	if (session('admin_logged_in') || request('method_call') == 'storekeeperAJAX' || request('method_call') == 'quickValidateAdminBox') {
		require 'adminheader.inc.php';
	} else {
		require 'visitorheader.inc.php';
	}
	
	//Use compression where possible, except in a couple of cases where we're using ob_start()
	if (request('method_call') != 'storekeeperAJAX') {
		useGZIP($siteConfig['compress_web_pages']);
	}
	
	if (request('__pluginClassName__')) {
		if (!($plugin = activatePluginClass(request('__pluginClassName__')))) {
			exit;
		}
	} elseif (request('pluginClassName')) {
		if (!($plugin = activatePluginClass(request('pluginClassName')))) {
			exit;
		}
	} elseif (request('__pluginName__')) {
		if (!($plugin = activatePlugin(request('__pluginName__')))) {
			exit;
		}
	} else {
		if (!($plugin = activatePlugin(request('pluginName')))) {
			exit;
		}
	}
	
	$plugin->setGETVariables($_GET);
	$plugin->setPOSTVariables($_POST);
	

} elseif (get('__method_call__') == 'getPhrase') {
	
	//Look up a visitor phrase
	require 'visitorheader.inc.php';
	
	$mrg = array();
	foreach($_GET as $i => $v) {
		if ($i != '__class__' && $i != '__code__' && $i != '__method_call__') {
			$mrg[$i] = htmlspecialchars($v);
		}
	}
	
	$phrases = array_flip(explode(',', $_GET['__code__']));
	
	foreach ($phrases as $code => &$phrase) {
		$phrase = getVLPPhrase($code, $mrg, $langOverride = false, $returnFalseOnFailure = false, $_GET['__class__']);
	}
	
	header('Content-Type: text/javascript; charset=UTF-8');
	echo json_encode($phrases);
	exit;
	

} elseif (post('__method_call__') == 'saveTheme') {
	
	//Save an admin's choice of theme
	require 'adminheader.inc.php';
	exitIfNotCheckPriv('_PRIV_MANAGE_PLUGIN_THEME');
	
	setTheme($_POST);
	
	exit;
	

} elseif (request('__method_call__') == 'getNewId') {
	//Get at the id of the most recently uploaded item to Storekeeper
		//(A work-around for not being able to return this when uploading things)
	require 'adminheader.inc.php';
	if (is_array(session('sk_new_ids'))) {
		echo json_encode($_SESSION['sk_new_ids']);
		unset($_SESSION['sk_new_ids']);
	} else {
		echo json_encode(false);
	}
	exit;
	

//Admin floating boxes and the handleAJAX method do not relate to content items
} else {
	
	//Check to see if the CMS' library of functions has been included, and include it if not
	if (!function_exists('checkPriv')) {
		require 'adminheader.inc.php';
		useGZIP($siteConfig['compress_web_pages']);
	}
	
	//Is this an Admin Thickbox Tab? If so, check by tab id
	if (request('adminFloatingBoxTabId')) {
		
		if (!$tab = getPluginAdminFloatingBoxTabDetails(request('adminFloatingBoxTabId'))) {
			exit;
		} elseif ($tab['priv'] && !checkPriv($tab['priv'])) {
			exit;
		} elseif (!($plugin = activatePlugin($tab['plugin_name']))) {
			exit;
		}
	
	//Otherwise look for a Plugin Name
	} else {
		if (request('pluginClassName')) {
			if (!($plugin = activatePluginClass(request('pluginClassName')))) {
				exit;
			}
		} elseif (request('__pluginName__')) {
			if (!($plugin = activatePlugin(request('__pluginName__')))) {
				exit;
			}
		} else {
			if (!($plugin = activatePlugin(request('pluginName')))) {
				exit;
			}
		}
	}
	
	//Look up the primary keys that this thickbox uses
	$primaryKeys = array();
	if (request('method_call') == 'showAdminFloatingBox') {
		$plugin->setAdminFloatingBoxPrimaryKey(request('tbName'), $primaryKeys);
	} elseif (request('primaryKeyParams')) {
		$primaryKeys = explode(',', request('primaryKeyParams'));
	} elseif (request('adminFloatingBoxTabId')) {
		$plugin->setAdminFloatingBoxPrimaryKey($tab['thickbox_name'], $primaryKeys);
	}
	
	//Look up the primary key from the request
	$firstKey = false;
	$primaryKey = array_flip($primaryKeys);
	foreach($primaryKey as $key => &$value) {
		if ($firstKey === false) {
			$firstKey = $key;
		}
		$value = $_REQUEST[$key];
	}
	
	//Workaround for the new storekeeper, where everything works off of a column called id
	if ($firstKey && !isset($primaryKey[$firstKey]) && issetArrayKey($_REQUEST, 'id')) {
		$primaryKey[$firstKey] = request('id');
	}
	
	$plugin->setGETVariables($_GET);
	$plugin->setPOSTVariables($_POST);
}


//Check which method call is being requested and then launch that method

//Output a file
if (request('method_call') == 'showFile') {
	
	if (request('cID') && request('cType') && request('instanceId')) {
		$instance['class']->showFile();
	} else {
		$plugin->showFile();
	}
	
	
//Show an image
} elseif (request('method_call') == 'showImage') {
	
	$plugin->showImage();
	

//Show a RSS feed
} elseif (request('method_call') == 'showRSS') {
	
	header('Content-Type: application/xml; charset=UTF-8');
	$instance['class']->showRSS();
	

//Show the Plugin on its own
} elseif (request('method_call') == 'showSlot' || request('method_call') == 'showIframe') {
	
	$item_meta = getItemMeta($cID, $cVersion);
	require CMS_ROOT. 'templatewrap/templatehead.php';
	require CMS_ROOT. 'templatewrap/templatebodystart.php';
	
	echo '
		<script type="text/javascript">
			window.tribiq_inIframe = true;
		</script>';
	
	if (issetArrayKey($instance, 'class')) {
		$instance['class']->show(false, request('method_call'));
	}
	
	require CMS_ROOT. 'templatewrap/templatefoot.php';


//Show a thickbox
} elseif (request('method_call') == 'showFloatingBox') {
	
	$instance['class']->show(false, 'showFloatingBox');


//Handle an AJAX request
} elseif (request('method_call') == 'handleAJAX') {
	
	$plugin->handleAJAX();


//Handle a file download from Storekeeper
} elseif (request('method_call') == 'storekeeperAJAX' && $_POST['_download']) {
	$plugin->storekeeperDownload(request('__path__'), request('id'), request('refinerName'), request('refinerId'));
	exit;


//Handle an AJAX request from Storekeeper
} elseif (request('method_call') == 'storekeeperAJAX') {
	
	$newIds = false;
	$message = false;

	if (request('_sk_form_submission')) {
		ob_start();
	} else {
		useGZIP($siteConfig['compress_web_pages']);
	}
	
	
	$newIds = $plugin->storekeeperAJAX(request('__path__'), request('id'), request('id2'), request('refinerName'), request('refinerId'));
	
	if ($newIds && !is_array($newIds)) {
		$newIds = explode(',', $newIds);
	}
	
	if (request('_sk_form_submission')) {
		$message = trim(ob_get_contents());
		ob_end_clean();
		
		//If the admin's browser does not have the Flash Uploader, strip out it's success signal
		if ($_FILES['Filedata'] && $message == '1') {
			$message = '';
		}
		
		//Send the results to Storekeeper in the parent frame
		useGZIP($siteConfig['compress_web_pages']);
		echo '
			<html>
				<body>
					<script type="text/javascript">';
		
		
		if ($message) {
			$messageType = 'error';
			
			if		  (substr($message, 0, 24) == '<!--Message_Type:None-->') {
				$message = substr($message, 24);
				$messageType = false;
			} else if (substr($message, 0, 25) == '<!--Message_Type:Error-->') {
				$message = substr($message, 25);
				$messageType = 'error';
			} else if (substr($message, 0, 27) == '<!--Message_Type:Success-->') {
				$message = substr($message, 27);
				$messageType = 'success';
			} else if (substr($message, 0, 27) == '<!--Message_Type:Warning-->') {
				$message = substr($message, 27);
				$messageType = 'warning';
			} else if (substr($message, 0, 28) == '<!--Message_Type:Question-->') {
				$message = substr($message, 28);
				$messageType = 'question';
				
				//Undocumented trick to refresh the Storekeeper!
			} else if (substr($message, 0, 25) == '<!--Reload_Storekeeper-->') {
					echo '
									self.parent.tribiqSK.reload();
								</script>
							</body>
						</html>';
					exit;
			}
		
			echo '
						self.parent.tribiqA.floatingBox(\''. jsOnClickEscape($message). '\', true, \''. $messageType. '\');';
		}
		
		if ($newIds) {
			echo '
						self.parent.tribiqSK.deselectAllItems();';
			
			foreach ($newIds as $id) {
				echo '
						self.parent.tribiqSK.selectedItems[\'', addslashes($id), '\'] = true;';
			}
			
			echo '
						self.parent.tribiqSK.saveSelection();';
		}
		
		echo '
						self.parent.tribiqSK.refresh();
					</script>
				</body>
			</html>';
	
	} else {
		if ($newIds) {
			if (!is_array(session('sk_new_ids'))) {
				$_SESSION['sk_new_ids'] = array();
			}
			foreach ($newIds as $id) {
				$_SESSION['sk_new_ids'][$id] = true;
			}
		}
	}


//Handle a quick-validate call for an Admin Box
} elseif (request('method_call') == 'quickValidateAdminBox') {
	$values = json_decode($_POST['values'], true);
	$rowClasses = json_decode($_POST['row_classes'], true);
	$snippets = array();
	
	$plugin->quickValidateAdminBox(post('path'), post('tab'), $values, $rowClasses, $snippets);
	
	echo json_encode(array('row_classes' => $rowClasses, 'snippets' => $snippets));
	exit;


//Show an Admin Thickbox
} elseif (request('method_call') == 'showAdminFloatingBox') {
	
	require_once CMS_ROOT. 'cmsincludes/thickbox.inc.php';
	$plugin->setAdminFloatingBoxTitleAndValidatePrimaryKey(request('tbName'), $primaryKey, $title);
	
	thickboxHeader($title, array(), request('tbName'), $primaryKeys, false, false, false, false, false, $plugin->cmsPluginApiLinkGetPluginClassName(), $primaryKey);
	thickboxFooter();


//Show an Admin Thickbox Tab
} elseif (request('method_call') == 'showAdminFloatingBoxTab') {
	require_once CMS_ROOT. 'cmsincludes/thickbox.inc.php';
	$showEditButton = (bool) $primaryKey[$primaryKeys[0]];
	$editMode = !$primaryKey[$primaryKeys[0]] || $_POST['formMode'] == "edit";
	
	echo thickboxFormHead($editMode);
	$plugin->showAdminFloatingBoxTab($tab['target_class_name'], $tab['thickbox_name'], $tab['tab_label'], $primaryKey, $editMode, $showEditButton);
	echo thickboxFormFoot($editMode, $showEditButton);


//Validate an Admin Thickbox Tab
} elseif (request('method_call') == 'validateAdminFloatingBoxTab') {
	
	$plugin->validateAdminFloatingBoxTab($tab['target_class_name'], $tab['thickbox_name'], $tab['tab_label'], $primaryKey);


//Save an Admin Thickbox Tab
} elseif ($_POST['method_call'] == 'saveAdminFloatingBoxTab') {
	
	$plugin->saveAdminFloatingBoxTab($tab['target_class_name'], $tab['thickbox_name'], $tab['tab_label'], $primaryKey);


//Refresh a Plugin in a slot
} elseif (request('method_call') == 'refreshPlugin') {
			
	function eschyp($text) {
		$searches = array('`', '-', "\n", "\r");
		$replaces = array('`t', '`h', '`n', '`r');
		return str_replace($searches, $replaces, $text);
	}
	
	//Display an info section at the top of the result, to help the CMS pick up on a few things
	if ($url = $instance['class']->checkHeaderRedirectLocation()) {
		echo '<!--INFO--><!--FORCE_PAGE_RELOAD--', eschyp($url), '--><!--/INFO-->';
	
	} elseif ($instance['class']->checkForcePageReloadVar()) {
		echo '<!--INFO--><!--FORCE_PAGE_RELOAD--', eschyp(linkToItem($cID, $cType, false, true)), '--><!--/INFO-->';
	
	} else {
		
		echo '<!--INFO-->';
		
		if ($instance['class']->checkScrollToTopVar() === true) {
			echo
				'<!--SCROLL_TO_TOP-->';
		}
		
		//Lets a Plugin will be placed in a floating box when it reloads
		if (($showInFloatingBox = $instance['class']->checkShowInFloatingBoxVar()) === true) {
			echo
				'<!--SHOW_IN_FLOATING_BOX-->';
		}
		
		if ($tabId = (int) $instance['class']->cmsPluginApiLinkGetTabId()) {
			echo
				'<!--TAB_ID--', $tabId, '-->';
		}
		
		
		//Check if the Plugin wants any JavaScript run
		$scripts = array();
		$scriptsBefore = array();
		$instance['class']->cmsPluginApiLinkCheckRequestedScripts($scripts, $scriptsBefore);
		
		if (count($scripts)) {
			foreach ($scripts as &$script) {
				echo '<!--SCRIPT--', eschyp(json_encode($script)), '-->';
			}
		}
		
		if (count($scriptsBefore)) {
			foreach ($scriptsBefore as &$script) {
				echo '<!--SCRIPT_BEFORE--', eschyp(json_encode($script)), '-->';
			}
		}

			
		if (checkPriv()) {
			echo
				'<!--PLUGIN_ID--',
					eschyp($instance['plugin_id']),
				'-->';
			
			echo
				'<!--PARENT_ID--',
					eschyp((bool) getInstanceParent($instance['instance_id'])),
				'-->';
			
			echo
				'<!--NAMESPACE--',
					eschyp($instance['class_name']),
				'-->';
			
			echo
				'<!--INSTANCE_SETTINGS_TEXT--',
					getPhrase('Settings for the Plugin Instance: | &quot;[[instance_name]]&quot;<br/>Click to access its Settings', array('instance_name' => eschyp($instance['instance_name']))),
				'-->';

			echo 
				'<!--INSTANCE_IN_SLOT_TEXT--',
					getPhrase('[[slot_name]]|Plugin: [[plugin_name]]<br/>Instance: [[instance_name]]<br/>Scope: [[level]]<br/>Click to change the scope of this Plugin Instance, to remove it or to replace it.',
						array(
							'level' => eschyp($instance['class']->cmsPluginApiLinkGetSlotLevel()),
							'slot_name' => eschyp(str_replace('_', ' ', $slotName)),
							'plugin_name' => eschyp(getPluginDisplayName($instance['plugin_id'])),
							'instance_name' => eschyp($instance['instance_name']))),

				'-->';
			
			if ($instance['class']->cmsPluginApiLinkGetThemeParams($frameworkPluginName, $frameworkPluginClassName, $themePluginName)) {
				
				echo
					'<!--THEME_CSS--',
						eschyp($instance['class']->cmsPluginApiLinkGetThemePrefix()),
					'--',
						eschyp($themePluginName),
					'--',
						eschyp($instance['theme']),
					'--',
						is_file(CMS_ROOT. $path = $instance['class']->cmsPluginApiLinkGetThemeDir(). 'stylesheet.css')? 1:null,
					'--',
						eschyp($frameworkPluginName),
					'--',
						eschyp($instance['framework']),
					'--',
						is_file(CMS_ROOT. $path = $instance['class']->cmsPluginApiLinkGetFrameworkDir(). 'javascript.js')? 1:null,
					'--',
						eschyp(json_encode(listPluginThemeCSS($instance['plugin_name'], $instance['class_name'], $instance['framework'], $frameworkPluginClassName, $skin['id']))),
					'-->';
			} else {
				echo
					'<!--THEME_CSS-------------->';
			}
		
			//Workaround for setting edit mode: Add a comment saying whether or not this is the case, for
			//the javascript function to pick up on, and change the CSS class if needed
			if ($instance['class']->beingEdited()) {
				echo '<!--IN_EDIT_MODE-->';
			} elseif ($instance['class']->beingEdited($canBe = true)) {
				echo '<!--CAN_BE_EDITED-->';
			}
			
			if (!$instance['init']) {
				echo
					'<!--NOT_SHOWN_IN_VISITOR_MODE-->';
			}
		}
		
		echo '<!--/INFO-->';
		
		$instance['class']->showSlot();
	}
}





?>