<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */



//This Plugin displays an image; it is intended to be used at the top of a page to display a Masthead
class tribiq_banner extends plugin_base_class {
	
	var $mergeFields = array();
	var $subSections = array();
	
	function allowPluginToBeShownOnErrorPage() {
		return true;
	}
	
	//The init method is called by the CMS lets Plugin Developers run code before the Plugin and the page it is on are displayed.
	//In visitor mode, the Plugin is only displayed if this method returns true.
	function init() {
		//Check to see if an item is set in the hyperlink_target setting 
		if ($this->setting('link_type') == '_CONTENT_ITEM' && ($linkExists = $this->getCIDAndCTypeFromSetting($cID, $cType, 'hyperlink_target'))) {
			if ($this->inc('tribiq_avls')) {
				tribiq_avls::equivalentItem($cID, $cType);
			}
			
			$request = '';
			if ($cType == 'document' && $this->setting('download_now')) {
				$request = 'download_now=true';
			}
			
			//Use the Theme Section for a Masthead with a link and set the link
			$this->mergeFields['Link_Href'] = 'href="'. htmlspecialchars($this->linkToItem($cID, $cType, $fullPath = false, $request)). '"';
			
			if ($this->setting('target_blank')) {
				$this->mergeFields['Target_Blank'] = " target=\"_blank\"";
			}
			
			//Check the Privacy settings on this banner
			if (!checkPriv()) {
				switch ($this->setting('hide_private_item')) {
					case '_LOGGED_IN':
						if (!session('extranetUserID')) {
							return false;
						}
						break;
					
					case '_LOGGED_OUT':
						if (session('extranetUserID')) {
							return false;
						}
						break;
					
					case '_PRIVATE':
						if (!checkPerm($cID, $cType)) {
							return false;
						}
				}
			}
		
		} elseif ($this->setting('link_type') == '_CONTENT_ITEM' && !$linkExists) {
			return false;
			
		} elseif ($this->setting('link_type') == '_EXTERNAL_URL' && $this->setting('url')) {
			$this->mergeFields['Link_Href'] = 'href="'. htmlspecialchars($this->setting('url')). '"';
			
			if ($this->setting('target_blank')) {
				$this->mergeFields['Target_Blank'] = " target=\"_blank\"";
			}
		}
		
		$this->mergeFields['Image_Style'] = '';
		
		//Attempt to find a masthead image to display
		//Check to see if an overwrite has been set, and use it if so
		$mediaId = $mediaType = $mediaURL = $mediaId2 = $mediaType2 = $mediaURL2 = false;
		if ($this->setting('image_source') == '_CUSTOM_IMAGE'
		 && ($this->getMediaIdTypeAndURLFromSetting($mediaId, $mediaType, $mediaURL, 'overwrite_masthead_image'))
		 && ($mediaType == 'image')) {
			$this->subSections['Image'] = true;
			$this->mergeFields['Image_Src'] = $mediaURL;
			
			if ($this->setting('use_rollover')
			 && ($this->getMediaIdTypeAndURLFromSetting($mediaId2, $mediaType2, $mediaURL2, 'rollover_image'))
			 && ($mediaType2 == 'image')) {
				$this->mergeFields['Image_Style'] .= 'id="'. $this->containerId. '_img" ';
				
				$this->mergeFields['Rollover_Images'] =
					'<img id="'. $this->containerId. '_rollout" style="width: 1px; height: 1px; visibility: hidden;" src="'. htmlspecialchars($mediaURL). '"/>'.
					'<img id="'. $this->containerId. '_rollover" style="width: 1px; height: 1px; visibility: hidden;" src="'. htmlspecialchars($mediaURL2). '"/>';
				
				$this->mergeFields['Wrap'] =
					'onmouseout="get(\''. $this->containerId. '_img\').src = get(\''. $this->containerId. '_rollout\').src;" '.
					'onmouseover="get(\''. $this->containerId. '_img\').src = get(\''. $this->containerId. '_rollover\').src;" ';
			}
		
		} elseif ($this->setting('image_source') == '_STICKY_IMAGE'
		 && $cID && ($stickImage = $this->itemStickyImage($cID, $cType))) {
			$this->subSections['Image'] = true;
			$this->mergeFields['Image_Src'] = $stickImage;
		
		} elseif ($this->setting('image_source') == '_THEME_IMAGE') {
			$this->subSections['Default_Image'] = true;
		}
		
		if (issetArrayKey($this->subSections, 'Image')) {
			$setWidth = $this->setting('set_width') && (int) $this->setting('width');
			$setHeight = $this->setting('set_height') && (int) $this->setting('height');
			
			if ($setWidth || $setHeight) {
				$this->mergeFields['Image_Style'] .= 'style="';
				if ($setWidth) {
					$this->mergeFields['Image_Style'] .= 'width: '. (int) $this->setting('width'). 'px;';
				}
				if ($setHeight) {
					$this->mergeFields['Image_Style'] .= ' height: '. (int) $this->setting('height'). 'px;';
				}
				$this->mergeFields['Image_Style'] .= '"';
			}
		}
		
		$this->subSections['Text'] = (bool) $this->setting('text');
		$this->subSections['Title'] = (bool) $this->setting('title');
		
		return issetArrayKey($this->subSections, 'Default_Image') || issetArrayKey($this->subSections, 'Image')
			|| issetArrayKey($this->subSections, 'Text') || issetArrayKey($this->subSections, 'Title');
	}
	
	
	//The showSlot method is called by the CMS, and displays the Plugin on the page
	function showSlot() {
		if (issetArrayKey($this->subSections, 'Default_Image') || issetArrayKey($this->subSections, 'Image')
		 || issetArrayKey($this->subSections, 'Text') || issetArrayKey($this->subSections, 'Title')) {
			
			if ($this->subSections['Text']) {
				$this->mergeFields['Text'] = $this->setting('text');
			}
			
			if ($this->subSections['Title']) {
				$this->mergeFields['Title'] = htmlspecialchars($this->setting('title'));
			}
			
			$this->mergeFields['Image_Alt'] = htmlspecialchars($this->setting('alt_tag'));
			
			//Display the Plugin
			$this->framework('Outer', $this->mergeFields, $this->subSections);
		}
	}
	
}
?>
