<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */


class tribiq_contact_form extends plugin_base_class {
	
	var $errors = array();
	var $message = false;
	var $mode = 'Contact_Form';
	var $objects = array();

	var $mergeFields = array();
	var $sections = array();
	
	function init()	{
	
		if ($this->post('contact_form_submit')) {
			$this->scrollToTopOfSlot();
			if ($this->submitContactForm()) {
				$this->mode = 'Contact_Form_Sent';
				$this->message = $this->phrase('_CONTACT_FORM_CONFIRMATION_MSG');
			}
		}
		
		if ($this->mode == 'Contact_Form') {
			if ($this->setting('use_captcha') && !issetArrayKey($_SESSION, 'captcha_passed__'. $this->instanceId)) {
				if (!issetArrayKey($_SESSION, 'extranetUserID') || $this->setting('extranet_users_use_captcha')) {
					$this->mergeFields['Captcha'] = $this->captcha();
					$this->sections['Captcha'] = true;
				}
			}
		}
	
		return true;
	}
	
	function prepareSectionForDisplay($varName, $phraseName = false) {
		if (!$phraseName) {
			$phraseName = $this->setting($varName. '_name');
		}
		
		if ($this->setting($varName) == 'optional' || $this->setting($varName) == 'mandatory') {
			$this->mergeFields[$varName . '_label'] = $this->phrase($phraseName. ':');
			$this->sections[$varName] = true;
		}
	}
	
	
	function showSlot() {
		
		$this->mergeFields['Contact_from_header'] = $this->phrase($this->setting('Heading'));
		$this->mergeFields['Clear_Link'] = $this->refreshPluginSlotAnchor();
		$this->mergeFields['Clear_JS'] = $this->refreshPluginSlotJS();
		
		//A message if given
		if ($this->message) {
			$this->sections['Message_Display']=true;
			$this->mergeFields['Message']=$this->message;
		}
		
		//Display any errors we encountered
		if (count($this->errors)){
			$this->sections['Error_Display']=$this->errors;
		}
		
		if ($this->mode == 'Contact_Form') {
			//Display a form that lets the user enter the email address that they used to register,
			//and then have their password emailed to them
			$this->sections['Contact_Form'] = true;
			$this->sections['Contact_Form_Buttons'] = true;

			$this->mergeFields['Open_Form']=$this->openForm();
				$this->prepareSectionForDisplay('Name_field', '_NAME');
				$this->prepareSectionForDisplay('Email_field', '_EMAIL');
				$this->prepareSectionForDisplay('Phone_field', '_PHONE');
				$this->prepareSectionForDisplay('Additional_field_1');
				$this->prepareSectionForDisplay('Additional_field_2');
				$this->prepareSectionForDisplay('Additional_field_3');
				$this->prepareSectionForDisplay('Additional_field_4');
				$this->prepareSectionForDisplay('Additional_field_5');
				$this->prepareSectionForDisplay('Additional_field_6');
				$this->prepareSectionForDisplay('Textarea_field');
			$this->mergeFields['Close_Form'] = $this->closeForm();
		}
		$this->framework('Outer', $this->mergeFields, $this->sections);
	}
	
	function processVar($varName, $phraseName = false) {
		if (!$phraseName) {
			$phraseName = $this->setting($varName. '_name');
		}
		
		$this->objects[$varName . '_label'] = $this->phrase($phraseName. ':');
		
		if ($this->post($varName)) {
			$this->objects[$varName]= $this->post($varName);
		} elseif ($this->setting($varName) == 'mandatory') {
			$this->errors[] = array('Error' => $this->phrase($phraseName. '_INCOMPLETE'));
		}
	}
	
	function processEmailVar($varName, $phraseName = false) {
		if (!$phraseName) {
			$phraseName = $this->setting($varName. '_name');
		}
		
		if ($this->setting($varName) == 'optional' || $this->setting($varName) == 'mandatory') {
			$this->objects[$varName . '_label'] = $this->phrase($phraseName. ':');
			
			if ($this->post($varName)) {
				if (validateEmailAddress($this->post($varName))) {
					$this->objects[$varName]= $this->post($varName);
				
				} else {
					$this->errors[] = array('Error' => $this->phrase($phraseName. '_INVALID'));
				}
			
			} elseif ($this->setting($varName) == 'mandatory') {
				$this->errors[] = array('Error' => $this->phrase($phraseName. '_INCOMPLETE'));
			}
		}
	}

	function submitContactForm(){
		$this->objects['CMS_URL'] = $this->absCMSDirURL();
		$this->objects['IP_Address'] = $_SERVER['REMOTE_ADDR'];
		
		$this->processVar('Name_field', '_NAME');
		$this->processEmailVar('Email_field', '_EMAIL');
		$this->processVar('Phone_field', '_PHONE');
		$this->processVar('Additional_field_1');
		$this->processVar('Additional_field_2');
		$this->processVar('Additional_field_3');
		$this->processVar('Additional_field_4');
		$this->processVar('Additional_field_5');
		$this->processVar('Additional_field_6');
		$this->processVar('Textarea_field');
		
		if ($this->setting('use_captcha') && !issetArrayKey($_SESSION, 'captcha_passed__'. $this->instanceId)) {
			if (!issetArrayKey($_SESSION, 'extranetUserID') || $this->setting('extranet_users_use_captcha')) {
				if ($this->checkCaptcha()) {
					$_SESSION['captcha_passed__'. $this->instanceId] = true;
				} else {
					$this->errors[] = array('Error' => $this->phrase('_CAPTCHA_INVALID'));
				}
			}
		}
		

		if (!count($this->errors)){
			if ($this->setting('email_address') && $this->setting('email_template') && $this->inc('tribiq_email_template_manager')) {
				unset($_SESSION['captcha_passed__'. $this->instanceId]);
				
				if (tribiq_email_template_manager::sendEmailsUsingTemplate($this->setting('email_address'), $this->setting('email_template'), $this->objects, array())) {
					return true;
				
				} else {
					$this->errors[] = array('Error' => $this->phrase('_EMAIL_MESSAGE_NOT_SENT'));
					return false;
				}
			} else {
				$this->errors[] = array('Error' => $this->phrase('_EMAIL_MESSAGE_NOT_SENT'));
				return false;
			}
		}
		return false;
	}
}
?>
