<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

class tribiq_menu extends plugin_base_class {

	var $currentMenuId = false;
	var $sectionId;
	var $startFrom;
	var $numLevels;
	var $maxLevel1MenuItems;
	var $language;
	var $showAdminAddMenuItem;
	var $onlyFollowOnLinks;
	var $onlyIncludeOnLinks;
	var $showInvisibleMenuItems;
	
	function allowPluginToBeShownOnErrorPage() {
		return true;
	}
	
	//Load settings from the instance of this plugin
	function init() {
		$this->sectionId				= 'Main';
		$this->startFrom				= $this->setting('menu_start_from');
		$this->numLevels				= $this->setting('menu_number_of_levels');
		$this->maxLevel1MenuItems		= $this->setting('menu_max_level_one_items');
		$this->language					= false;
		$this->showAdminAddMenuItem		= $this->setting('menu_show_admin_add_button');
		$this->onlyFollowOnLinks		= !$this->setting('menu_show_all_branches');
		$this->onlyIncludeOnLinks		= false;
		$this->showInvisibleMenuItems	= false;
		
		if ($this->showAdminAddMenuItem) {
			$this->markSlotAsBeingEdited(false);
		}
		
		//Get the menu item for this content item
		$this->currentMenuId = getSectionMenuItemFromContent($this->cID, $this->cType, $this->sectionId);
		
		return $this->currentMenuId || !$this->setting('hide_if_current_item_not_in_menu');
	}
	
	//Main Display function for the slot
	function showSlot() {
		
		if (!$this->currentMenuId && $this->setting('hide_if_current_item_not_in_menu')) {
			return;
		}
		
		//Load Settings
		//Work out where to start from
		if ($this->startFrom == 'Menu Level 1' || $this->startFrom == '_MENU_LEVEL_1') {
			$parentMenuId = 0;
			
		} elseif ($this->startFrom == 'Current Menu Level' || $this->startFrom == '_MENU_CURRENT_LEVEL') {
			$parentMenuId = $this->currentMenuId;
			
		} elseif ($this->startFrom == 'Menu Level Above' || $this->startFrom == '_MENU_LEVEL_ABOVE') {
			$parentMenuId = getMenuParent($this->currentMenuId);
			
		} elseif ($this->startFrom == '_MENU_LEVEL_TWO_ABOVE') {
			$parentMenuId = getMenuParent(getMenuParent($this->currentMenuId));
			
		} elseif ($this->startFrom == 'Menu Level 2' || $this->startFrom == '_MENU_LEVEL_2') {
			$this->getMenuAncestor($this->currentMenuId, $ancestor, $ancestorButOne);
			$parentMenuId = $ancestor;
			
		} elseif ($this->startFrom == 'Menu Level 3' || $this->startFrom == '_MENU_LEVEL_3') {
			$this->getMenuAncestor($this->currentMenuId, $ancestor, $ancestorButOne);
			$parentMenuId = $ancestorButOne;
		}
		
		
		$headerObjects = array();
		$subSections = array('Title' => true);
		
		//Set default values for this->maxLevel1MenuItems and the this->language
		if (!$this->maxLevel1MenuItems) {
			$this->maxLevel1MenuItems = 9;
		}
		
		if ($this->language === false) {
			$this->language = $_SESSION['user_lang']? $_SESSION['user_lang'] : DEFAULT_LANGUAGE;
		}
		
		//Display a title for sidemenus
		if ($this->checkFrameworkSectionExists('Title')) {
			if ($parentMenuId && $parentMenuDetails = getMenuItemFromMenuID($parentMenuId)) {
				$headerObjects['Parent_Name'] = htmlspecialchars($parentMenuDetails['name']);
				if ($parentMenuDetails['content_id']) {
					$headerObjects['Parent_Link'] = htmlspecialchars(
						$this->linkToItem(ifNull($parentMenuDetails['html_content_alias'], $parentMenuDetails['content_id']), $parentMenuDetails['content_type'])
					);
				}
			} else {
				$homepage = $this->getHomepage($_SESSION['user_lang']);
				$headerObjects['Parent_Name'] = htmlspecialchars($homepage['name']);
				$headerObjects['Parent_Link'] = htmlspecialchars($this->linkToItem($homepage['alias']));
			}
		}
		
		//Get the menu structure from the database.
		$menuArray =
			cms_core::getMenuStructure($this->sectionId, $this->currentMenuId, $parentMenuId,
							 $this->numLevels, $this->maxLevel1MenuItems, $this->language,
							 $this->onlyFollowOnLinks, $this->onlyIncludeOnLinks, 
							 $this->showAdminAddMenuItem, $this->showInvisibleMenuItems,
							 0, false);
		
		//Draw the menu items we found
		$this->drawMenu($menuArray, 0, $headerObjects, $subSections);
	}
	
	
	//Recursive function to draw menu items from the database
	function drawMenu(&$menuArray, $recurseCount = 0, $headerObjects = array(), $subSections = array()) {
		
		if (!$this->checkFrameworkSectionExists($theme = 'Level_'. ++$recurseCount)) {
			$theme = 'Level_X';
		}
		
		if (checkPriv() && is_array($menuArray) && count($menuArray) == 1) {
			foreach ($menuArray as $i => &$row) {
				if (checkPriv() && issetArrayKey($row, 'is_admin_add_menu_item')) {
					$headerObjects['Admin_Button_Class'] = ' class="pluginAdminButton"';
				}
			}
		}
		
		$this->frameworkHead($theme, true, $headerObjects, $subSections);
			$maxI = count($menuArray);
			if (is_array($menuArray)) {
				foreach ($menuArray as $i => &$row) {
					$this->drawMenuLink($i+1, $row, $recurseCount, $maxI);
				}
			}
		$this->frameworkFoot($theme, true, $headerObjects, $subSections);
	
		if ($recurseCount>1000) {
			echo "Function aborted due to infinite recursion loop";
			exit;
		}
	}
	
	
	//Draw the wrapping around a menu item, then draw the menu item,
	//then call drawMenu() on its children
	function drawMenuLink($i, &$row, $recurseCount, $maxI) {
		
		if ((!$this->checkFrameworkSectionExists($theme = 'Level_'. $recurseCount. '_Link_'. $i))
		 && (!$this->checkFrameworkSectionExists($theme = 'Level_'. $recurseCount. '_Link'))) {
				$theme = 'Level_X_Link';
		}
		
		$on = issetArrayKey($row, 'on')? "_on" :null;
		
		$objects = array(
			'Class' => "level". $recurseCount. $on. " level". $recurseCount. "_". $i. $on,
			'Hyperlink' => $this->drawMenuItem($row),
			'id' => arrayKey($row, 'id'),
			'Access_Key' => arrayKey($row, 'accesskey')
		);
	
		if (issetArrayKey($row, 'children')) {
			$objects['Class'] .= " has_child";
		}
		
		if (arrayKey($row, 'content_id') == $this->cID && arrayKey($row, 'content_type') == $this->cType) {
			$objects['Class'] .= " current";
		}
		
		if (checkPriv() && issetArrayKey($row, 'is_admin_add_menu_item')) {
			$objects['Class'] .= " pluginAdminButton";
		}
		
		//A function that can be overwritten to allow extra functionality
		if ($this->addExtraMergeFields($row, $objects, $recurseCount, $i, $maxI)) {
		
			$this->frameworkHead($theme, true, $objects);
			
			if (issetArrayKey($row, 'children') && is_array($row['children'])) {
				$this->drawMenu($row['children'], $recurseCount);
			}
			
			$this->frameworkFoot($theme, true, $objects);
		}
	}
	
	//A function that can be overwritten to allow extra functionality
	function addExtraMergeFields(&$menuItem, &$mergeFields, $recurseCount, $i, $maxI) {
		return true;
	}
	
	
	//Draw a menu item
	function drawMenuItem(&$row) {
		$menuItem = '';
		if (issetArrayKey($row, 'name')) {
			$menuItem = htmlentities($row['name'],ENT_COMPAT,"UTF-8");
		} elseif (issetArrayKey($row, 'img')) {
			$menuItem = $row['img'];
		}
		
		if ($row['url']) {
			$menuItem = '<a href="'. $row['url']. '"'.
				(issetArrayKey($row, 'onclick')?
				  ' onclick="'. $row['onclick']. '"' :null).
				(issetArrayKey($row, 'rel_tag')?
				  ' rel="'. $row['rel_tag']. '"' :null).
				(issetArrayKey($row, 'accesskey')?
				  ' accesskey="'. $row['accesskey']. '"' :null).
				(issetArrayKey($row, 'target')?
				  ' target="'. $row['target']. '"' :null).
				(issetArrayKey($row, 'title')?
				  ' title="'. $row['title']. '"' :null). '>'. $menuItem. '</a>';
		
		} else {
			$menuItem = '<a class="unlinked_menu_item">'. $menuItem. '</a>';
		}

		if (!issetArrayKey($row, 'active')) {
			$menuItem = "<em>". $menuItem. "</em>";
		}
		
		return $menuItem;
	}
	
	
	function getHomepage( $langId ) {
	
		$sql = "
			SELECT h.title, h.alias, h.id,  m.name
			FROM ". DB_NAME_PREFIX. "languages AS l
			INNER JOIN ". DB_NAME_PREFIX. "content_html AS h
			   ON l.homepage_id = h.id
			LEFT OUTER JOIN ". DB_NAME_PREFIX. "menuitems AS m
			   ON m.content_id = h.id
			  AND m.content_type = 'html'
			WHERE l.id = '". mysql_real_escape_string($langId). "'
			  ". (checkPriv()? null: "AND h.`status` = 'published'"). "
			ORDER BY h.version DESC, m.redundancy = 'primary' DESC";
		
		$result = my_mysql_query($sql);
		$homepage = mysql_fetch_array($result);
		
		if (!$homepage['name']) {
			$homepage['name'] = $homepage['title'];
		}
		
		return $homepage;
	}
	
	
	function getMenuAncestor($mID, &$ancestor, &$ancestorButOne) {
		$ancestor = $ancestorButOne = $mID;
		$recurseLimit = 100;
		while ((--$recurseLimit) && ($newMID = getMenuParent($ancestor))) 	{
			$ancestorButOne = $ancestor;
			$ancestor = $newMID;
		}
	}
	
	
	
}


?>