<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

class tribiq_plugin_nest extends plugin_base_class {
	
	var $firstTab = false;
	var $lastTab = false;
	var $tabNum = false;
	var $mergeFields = array();
	var $sections = array();
	var $tabs = array();
	var $plugins = array();
	var $show = false;
	
	public function init() {
		
		if ($this->specificEgg()) {
			$this->tabNum = ifNull(getRow('nested_plugins', 'tab', $this->specificEgg()), 1);
			$this->tabId = getRow('nested_plugins', 'id', array('is_tab' => 1, 'tab' => $this->tabNum));
			$this->loadTabs();
		
		} elseif ($this->loadTabs()) {
			$this->sections['Tabs'] = true;
			$this->sections['Tab'] = array();
			
			$i = -1;
			foreach ($this->tabs as $tab) {
				if (!$this->firstTab) {
					$this->firstTab = $tab['id'];
					$this->tabNum = $tab['tab'];
					$this->tabId = $tab['id'];
				}
				$this->lastTab = $tab['id'];
				
				if ($this->request('tab') == $tab['id']) {
					$this->tabNum = $tab['tab'];
					$this->tabId = $tab['id'];
				
				} elseif (!$this->request('tab') && $this->request('tab_no') == $tab['tab']) {
					$this->tabNum = $tab['tab'];
					$this->tabId = $tab['id'];
				}
				
				$tabIds[$tab['tab']] = $tab['id'];
				
				if ($this->checkFrameworkSectionExists('Tab')) {
					$this->sections['Tab'][$tab['tab']] = array(
						'Class' => 'tab_'. ++$i. ' tab',
						'Tab_Link' => $this->refreshPluginSlotTabAnchor('tab='. $tab['id']),
						'Tab_Name' => htmlspecialchars($this->phrase($tab['tab_title'])));
				}
			}
			
			if ($this->checkFrameworkSectionExists('Tab')) {
				$this->sections['Tab'][$this->tabNum]['Class'] .= '_on';
			}
			
			if ($this->checkFrameworkSectionExists('Next')) {
				$this->sections['Next'] = true;
				
				$nextTabId = false;
				if ($this->lastTab == $this->tabId) {
					if (!$this->setting('next_prev_buttons_loop')) {
						$this->mergeFields['Next_Disabled'] = '_disabled';
					} else {
						$nextTabId = $this->firstTab;
					}
				} else {
					foreach ($this->tabs as $tabNum => &$tab) {
						if ($tabNum > $this->tabNum) {
							$nextTabId = $tab['id'];
							break;
						}
					}
				}
				
				if ($nextTabId) {
					$this->mergeFields['Next_Link'] = $this->refreshPluginSlotTabAnchor('tab='. $nextTabId);
				}
			}
			
			if ($this->checkFrameworkSectionExists('Prev')) {
				$this->sections['Prev'] = true;
				
				$prevTabId = false;
				if ($this->firstTab == $this->tabId) {
					if (!$this->setting('next_prev_buttons_loop')) {
						$this->mergeFields['Prev_Disabled'] = '_disabled';
					} else {
						$prevTabId = $this->lastTab;
					}
				} else {
					foreach ($this->tabs as $tabNum => &$tab) {
						if ($tabNum >= $this->tabNum) {
							break;
						} else {
							$prevTabId = $tab['id'];
						}
					}
				}
				
				if ($prevTabId) {
					$this->mergeFields['Prev_Link'] = $this->refreshPluginSlotTabAnchor('tab='. $prevTabId);
				}
			}
			
			$this->registerGetRequest('tab', $this->firstTab);
		}
		
		
		//If all tabs were hidden, don't show anything
		if ($this->tabNum !== false && $this->loadTab($this->tabNum)) {
			$this->show = true;
		
		//...except if no tabs exist, don't hide anything
		} elseif (!checkRowExists('nested_plugins', array('instance_id' => $this->instanceId, 'is_tab' => 1)) && $this->loadTab($this->tabNum = 1)) {
			$this->show = true;
		}
		
		return $this->show;
	}

	
	public function showSlot() {
		
		if ($this->checkShowInFloatingBoxVar()) {
			if ($this->show) {
				global $slotContents;
				foreach ($this->plugins[$this->tabNum] as $id => $slotNameNestId) {
					if (issetArrayKey($slotContents, $slotNameNestId, 'class')) {
						if ($slotContents[$slotNameNestId]['class']->checkShowInFloatingBoxVar()) {
							$this->showPlugin($id, $slotNameNestId);
						}
					}
				}
			}
		
		} else {
			$this->addAdminControls();
			
			$this->frameworkHead(
				'Outer',
				'Plugins',
				$this->mergeFields,
				$this->sections);
				
				$this->frameworkHead(
					'Plugins',
					'Plugin',
					$this->mergeFields,
					$this->sections);
						
						if ($this->show) {
							foreach ($this->plugins[$this->tabNum] as $id => $slotNameNestId) {
								$this->showPlugin($id, $slotNameNestId);
							}
						}
				
				$this->frameworkFoot(
					'Plugins',
					'Plugin',
					$this->mergeFields,
					$this->sections);
			
			$this->frameworkFoot(
				'Outer',
				'Plugins',
				$this->mergeFields,
				$this->sections);
		}
	}
	
	
	protected function loadTabs() {
		
		if ($this->sections['Show_Title'] = (bool) $this->setting('show_title')) {
			$this->mergeFields['Title'] = htmlspecialchars($this->phrase($this->setting('title')));
		}
		
		$sql = "
			SELECT id, tab, tab_title
			FROM ". DB_NAME_PREFIX. "nested_plugins
			WHERE instance_id = ". (int) $this->instanceId. "
			  AND is_tab = 1
			ORDER BY tab";
		
		$result = my_mysql_query($sql);
		
		if (mysql_num_rows($result)) {
			while ($row = mysql_fetch_assoc($result)) {
				$this->tabs[$row['tab']] = $row;
			}
			
			
			$this->mergeFields['Nest'] = '';
			if ($this->setting('set_max_height')) {
				$this->mergeFields['Nest'] =
					'style="height: '. htmlspecialchars($this->setting('max_height')). '; max-height: '. htmlspecialchars($this->setting('max_height')). ';"';
			}
			
			return true;
		} else {
			return false;
		}
	}
	
	
	protected function loadTab($tabNum) {
		
		$sql = "
			SELECT np.id, np.tab, np.plugin_id, np.framework, st.theme
			FROM ". DB_NAME_PREFIX. "nested_plugins AS np
			LEFT JOIN ". DB_NAME_PREFIX. "plugin_slot_theme_link AS st
			   ON st.skin_id = ". (int) $this->skinId. "
			  AND st.slot_name = '". mysql_real_escape_string($this->slotName). "'
			  AND st.nest = np.id
			  AND st.instance_id = ". (int) $this->instanceId. "
			WHERE np.instance_id = ". (int) $this->instanceId. "
			  AND np.is_tab = 0
			  AND np.tab = ". (int) $tabNum;
		
		if ($this->specificEgg()) {
			$sql .= "
			  AND np.id = ". (int) $this->specificEgg();
		}
		
		$sql .= "
			ORDER BY np.ord";
		
		$show = false;
		global $slotContents;
		$this->plugins[$tabNum] = array();
		
		$result = my_mysql_query($sql);
		while ($row = mysql_fetch_assoc($result)) {
			$missingPlugin = false;
			if (($details = getPluginDetails($row['plugin_id']))
			 && (includePluginAndDependencies($details['plugin_name'], $missingPlugin))
			 && (method_exists($details['class_name'], 'showSlot'))) {
				
				$this->plugins[$tabNum][$row['id']] = $slotNameNestId = $this->slotName. '-'. $row['id'];
				
				$slotContents[$slotNameNestId] = $details;
				$slotContents[$slotNameNestId]['instance_id'] = $this->instanceId;
				$slotContents[$slotNameNestId]['framework'] = ifNull($row['framework'], $details['default_framework']);
				$slotContents[$slotNameNestId]['theme'] = ifNull($row['theme'], $details['default_theme']);
			}
		}
		
		$beingEdited = false;
		$canBeEdited = false;
		foreach ($this->plugins[$tabNum] as $id => $slotNameNestId) {
			$slotContents[$slotNameNestId]['instance_id'] = $this->instanceId;
			setInstance($slotContents[$slotNameNestId], $this->cID, $this->cType, $this->cVersion, $this->skinId, $this->slotName, true, $id, $this->tabId);
			
			if (issetArrayKey($slotContents[$slotNameNestId], 'init')) {
				$show = true;
				
				//Check for the forcePageReload and headerRedirect options in Plugins
				if ($reload = $slotContents[$slotNameNestId]['class']->checkForcePageReloadVar()) {
					$this->forcePageReload($reload);
				}
				if ($url = $slotContents[$slotNameNestId]['class']->checkHeaderRedirectLocation()) {
					$this->headerRedirect($url);
				}
				
				//Ensure that the JavaScript libraries is there for Plugins on reloads
				if ($this->needToAddCSSAndJS()) {
					$this->callScriptBeforeAJAXReload('tribiq_plugin_nest', 'addJavaScript', $slotContents[$slotNameNestId]['class_name'], $slotContents[$slotNameNestId]['plugin_id']);
					
					//Ensure that any Themes needed are there for Plugins on reloads
					if ($slotContents[$slotNameNestId]['theme'] && !is_numeric($slotContents[$slotNameNestId]['theme'])) {
						$this->callScriptBeforeAJAXReload('tribiq_plugin_nest', 'addThemeCSS', $slotContents[$slotNameNestId]['plugin_name'], $slotContents[$slotNameNestId]['theme']);
					}
				}
			}
			
			if (checkPriv() && issetArrayKey($slotContents[$slotNameNestId], 'class')) {
				$frameworkPluginName = $frameworkPluginClassName = $themePluginName = false;
				$slotContents[$slotNameNestId]['class']->cmsPluginApiLinkGetThemeParams($frameworkPluginName, $frameworkPluginClassName, $themePluginName);
				
				$this->callScript('tribiq_plugin_nest',
					'setPossibleThemes', $this->slotName, $id,
					json_encode(listPluginThemeCSS(
									$slotContents[$slotNameNestId]['plugin_name'],
									$slotContents[$slotNameNestId]['class_name'],
									$slotContents[$slotNameNestId]['framework'],
									$frameworkPluginClassName,
									$this->skinId)));
				
				if (!$beingEdited) {
					$beingEdited = $slotContents[$slotNameNestId]['class']->beingEdited($canBe = false);
				}
				if (!$canBeEdited) {
					$canBeEdited = $slotContents[$slotNameNestId]['class']->beingEdited($canBe = true);
				}
			}
		}
		
		//If we're adding Themes or JavaScript, add a short delay to the tab switching to cover for the browser loading things in
		$this->callScriptBeforeAJAXReload('tribiq_plugin_nest', 'sleep');
		
		//Add any Plugin JavaScript calls
		foreach ($this->plugins[$tabNum] as $id => $slotNameNestId) {
			if (issetArrayKey($slotContents[$slotNameNestId], 'class')) {
				//Add the script of a Nested Plugin to the Nest
				$scripts = array();
				$scriptsBefore = array();
				$slotContents[$slotNameNestId]['class']->cmsPluginApiLinkCheckRequestedScripts($scripts, $scriptsBefore);
				
				foreach ($scripts as &$script) {
					$this->scripts[] = $script;
				}
				foreach ($scriptsBefore as &$script) {
					$this->scriptsBefore[] = $script;
				}
				
				//Check to see if any Eggs wants to scroll to the top of the slot
				$scrollToTop = $slotContents[$slotNameNestId]['class']->checkScrollToTopVar();
				if ($scrollToTop !== null) {
					$this->scrollToTopOfSlot($scrollToTop);
				}
				
				//Check to see if any Eggs want to show themselves in a Floating Box, or stop showing themselves in a Floating Box
				if ($slotContents[$slotNameNestId]['class']->checkShowInFloatingBoxVar()) {
					$this->showInFloatingBox(true);
				}
			}
		}
		
		//If an Egg wanted to show themselves in a Floating Box, hide the ones that didn't want this.
		if ($this->checkShowInFloatingBoxVar()) {
			$unsets = array();
			foreach ($this->plugins[$tabNum] as $id => $slotNameNestId) {
				if (issetArrayKey($slotContents[$slotNameNestId], 'class')) {
					if (!$slotContents[$slotNameNestId]['class']->checkShowInFloatingBoxVar()) {
						unset($slotContents[$slotNameNestId]);
						$unsets[] = $id;
					}
				}
			}
			foreach ($unsets as $id) {
				unset($this->plugins[$tabNum][$id]);
			}
		}
		
		
		if (checkPriv()) {
			$this->markSlotAsBeingEdited($beingEdited, $canBeEdited);
		}
		
		return $show;
	}

	
	protected function addAdminControls() {
		if (checkPriv('_PRIV_MANAGE_PLUGIN_NEST') || checkPriv('_PRIV_REORDER_PLUGIN_NEST')) {
			echo '
				<div class="pluginSlotButton" style="width: ', getInstanceParent($this->instanceId)? 189 : 162, 'px; float: right; clear: both">';
			
			if (checkPriv('_PRIV_MANAGE_PLUGIN_NEST')) {
				echo '
					<a class="tribiq_adminButton tribiq_addEggButton" href="#" onclick="
						return tribiq_plugin_nest.pickNewPlugin(
									\'', $this->slotName, '\',
									', (int) $this->instanceId, ',
									', (int) $this->tabNum, ');
					" title="', getPhrase(
						'Copy or Move another Plugin here|Add to this Nest by copying or moving a Plugin Instance.'
					), '"></a>';
			} else {
				echo '
					<a class="tribiq_adminButton" style="visibility: hidden;"></a>';
			}
			
			if (checkPriv('_PRIV_REORDER_PLUGIN_NEST')) {
				echo '
					<a class="tribiq_adminButton tribiq_reorderButton" href="#" onclick="
								return tribiq_plugin_nest.showNestInSK(
											', (int) $this->instanceId, ',
											', (int) $this->pluginId, ',
											', (int) $id, ');
					" title="', getPhrase(
						'Arrange Nest|Manage and arrange this Nest in Storkeepeer.'
					), '"></a>';
			}
			
			echo '
				</div>';
		}
	}
	
	
	protected function showPlugin($id, &$slotNameNestId, $showAdminControls = true) {
		global $slotContents;
			
		if (issetArrayKey($slotContents[$slotNameNestId], 'init') || checkPriv()) {
		
			$this->frameworkHead(
				'Plugin',
				'Show_Slot',
				$this->mergeFields);
			
			if ($showAdminControls && (
				checkPriv('_PRIV_MANAGE_PLUGIN_NEST') || checkPriv('_PRIV_VIEW_NESTED_PLUGIN')
			 || checkPriv('_PRIV_MANAGE_NESTED_PLUGIN_THEME') || checkPriv('_PRIV_VIEW_VLP')
			)) {
				echo '
					<div class="pluginSlotButton" style="width: 103px; float: right; clear: both">
						<a class="tribiq_nestAdminButton';
				
				if (checkPriv('_PRIV_MANAGE_PLUGIN_NEST')) {
					echo '
						tribiq_nestDeleteButton" href="#" onclick="
							return tribiq_plugin_nest.removePluginConfirm(
										\'', $this->slotName, '\',
										', (int) $this->instanceId, ',
										', (int) $slotContents[$slotNameNestId]['plugin_id'], ',
										', (int) $id, ');
						" title="', getPhrase(
							'Remove Plugin|Remove the Nested Plugin &quot;[[name]]&quot;',
							array('name' => htmlspecialchars($slotContents[$slotNameNestId]['display_name']))
						);
				}
				
				echo
						'"></a>
						<a class="tribiq_nestAdminButton';
				
				if (checkPriv('_PRIV_VIEW_NESTED_PLUGIN')) {
					echo '
						 tribiq_nestSettingsButton" href="#" onclick="
							return tribiq_plugin_nest.settings(
										\'', $this->slotName, '\',
										', (int) $this->instanceId, ',
										', (int) $slotContents[$slotNameNestId]['plugin_id'], ',
										', (int) $id, ',
										', (int) $this->tabId, ');
						" title="', getPhrase(
							'Settings for the Nested Plugin:|&quot;[[name]]&quot;',
							array('name' => htmlspecialchars($slotContents[$slotNameNestId]['display_name']))
						);
				}
				
				echo
						'"></a>
						<a class="tribiq_nestAdminButton';
				
				if (checkPriv('_PRIV_MANAGE_NESTED_PLUGIN_THEME')) {
					echo '
						 tribiq_nestThemesButton" href="#" onclick="
							return tribiqA.pluginSlotChangeTheme(this, event, \'', $this->slotName, '\', ', (int) $id, ');
						" title="', getPhrase(
							'Change the appearance of this Plugin|Click to change the Theme for this Nested Plugin'
						);
				}
				
				echo
						'"></a>
						<a class="tribiq_nestAdminButton';
				
				if (checkPriv('_PRIV_VIEW_VLP') && $slotContents[$slotNameNestId]['vlp_class']) {
					echo '
						 tribiq_nestPhrasesButton" href="#" onclick="
							return tribiqA.showPhrasesByClassName(\'', $slotContents[$slotNameNestId]['vlp_class'], '\');
						" title="', getPhrase(
							'View/Manage Phrases|Click to view or manage Phrases for this Plugin.'
						);
				}
				
				echo
						'"></a>
					</div>
					<div class="clear" style="clear: both"></div>';
			}
			
			$slotContents[$slotNameNestId]['class']->show(false);
			
			$this->frameworkFoot(
				'Plugin',
				'Show_Slot',
				$this->mergeFields);
		}
	}
	
	
	//Allow one specific Egg to be shown for the showFloatingBox/showIframe/showRSS methods
	protected function specificEgg() {
		return
			request('method_call') == 'showFloatingBox' || request('method_call') == 'showIframe' || request('method_call') == 'showRSS'?
				(int) request('eggId')
			 :	false;
	}
	
	//Version of refreshPluginSlotAnchor, that doesn't automatically set the tab id
	protected function refreshPluginSlotTabAnchor($requests = '', $scrollToTopOfSlot = true) {
		return
			$this->linkToItemAnchor($this->cID, $this->cType, $fullPath = false, '&slotName='. $this->slotName. urlRequest($requests)).
			' onclick="'.
				$this->refreshPluginSlotJS($requests, $scrollToTopOfSlot).
				' return false;"';
	}

	
	public function showFloatingBox() {
		$this->showMethod('showFloatingBox');
	}
	public function showIframe() {
		$this->showMethod('showIframe');
	}
	public function showRSS() {
		$this->showMethod('showRSS');
	}
	
	protected function showMethod($method) {
		if ($this->show) {
			global $slotContents;
			
			foreach ($this->plugins[$this->tabNum] as $id => $slotNameNestId) {
				if ($id == $this->specificEgg()) {
					if (issetArrayKey($slotContents[$slotNameNestId], 'init') || checkPriv()) {
						$slotContents[$slotNameNestId]['class']->$method();
					}
				}
			}
		}
	}
	
	
	protected function needToAddCSSAndJS() {
		return arrayKey($_REQUEST, 'method_call') == 'refreshPlugin';
	}
	
	
	public function handleAJAX() {
		
		if ($this->get('sleep')) {
			//If we're adding Themes or JavaScript, add a short delay to the tab switching to cover for the browser loading things in
			
			//We could use usleep for a longer delay, but usually the usual delay for the AJAX request is enough to cover
			//usleep(10000);
			
			echo 1;
			exit;
		}
		
		if (checkPriv()) {
			if ($this->get('removePlugin') && checkPriv('_PRIV_MANAGE_PLUGIN_NEST')) {
				echo json_encode(array(
					'html' => $this->removePluginConfirm($this->get('pluginId'), $this->get('instanceId')),
					'buttonsHTML' =>
								'<input class="submit_selected" value="'. getPhrase('Remove'). '" type="button" onclick="
									tribiq_plugin_nest.removePlugin(
										\''. htmlspecialchars($this->get('slotName')). '\',
										'. (int) $this->get('instanceId'). ',
										'. (int) $this->get('id'). ');
								  "/>'.
								'<input value="'. getPhrase('Cancel'). '" type="button">',
					'warning' => 1
				));
			
			} elseif ($this->post('removePlugin') && checkPriv('_PRIV_MANAGE_PLUGIN_NEST')) {
				$this->removePlugin($this->post('id'), $this->post('instanceId'));
			
			} elseif ($this->get('addPluginInstance') && checkPriv('_PRIV_MANAGE_PLUGIN_NEST')) {
				echo json_encode(array(
					'html' => $this->addPluginConfirm($this->get('addPluginInstance'), $this->get('instanceId'), true),
					'buttonsHTML' =>
								'<input class="submit_selected" value="'. getPhrase('Add to Nest'). '" type="button" onclick="
									tribiq_plugin_nest.copyNewPlugin(
										\''. htmlspecialchars($this->get('slotName')). '\',
										'. (int) $this->get('instanceId'). ',
										'. (int) $this->get('addPluginInstance'). ',
										'. (int) $this->get('tabNum'). ');
								  "/>'.
								'<input value="'. getPhrase('Cancel'). '" type="button">',
					'warning' => 1
				));
			
			} elseif ($this->post('addPluginInstance') && checkPriv('_PRIV_MANAGE_PLUGIN_NEST')) {
				$this->addPlugin($this->post('addPluginInstance'), $this->post('instanceId'), $this->post('tabNum'));
			}
		}
	}

	
	public function fillStorekeeper($path, &$panel, $refinerName, $refinerId, $selectMode, $isXML) {
		switch ($path) {
			case 'tribiq__plugins/instances':
				$nestablePluginIds = array(getPluginIdByClassName('tribiq_plugin_nest') => true);
				
				$sql = "
					SELECT plugin_id
					FROM ". DB_NAME_PREFIX. "plugin_dependencies
					WHERE dependency_plugin_class_name = 'tribiq_plugin_nest'
					  AND `type` = 'dependency'";
				$result = my_mysql_query($sql);
				while($row = mysql_fetch_assoc($result)) {
					$nestablePluginIds[$row['plugin_id']] = true;
				}
				
				foreach ($panel['items'] as $id => &$item) {
					if (issetArrayKey($nestablePluginIds, $item['plugin_id'])) {
						$item['traits']['tribiq_plugin_nest__nest'] = true;
					}
				}
			
				break;
			
			
			case 'tribiq__plugins/tribiq_plugin_nest':
				
				$instance = getPluginInstanceDetails($this->get('refiner__nest'));
				$panel['title'] = getPhrase('Tabs and Plugins in the Nest "[[nest]]"', array('nest' => htmlspecialchars($instance['instance_name'])));
				
				foreach ($panel['items'] as $id => &$item) {
					$item['traits'] = array();
					if ($item['is_tab']) {
						$item['traits']['is_tab'] = true;
						$item['name'] = $item['tab_title'];
						$item['css_class'] = 'tab';
					} else {
						$item['traits']['is_not_tab'] = true;
						$item['name'] = $item['plugin_display_name'];
						
						if ($item['settings_media'] && substr($item['settings_media'], 0, 6) == 'image_') {
							$item['traits']['has_image'] = true;
							$item['checksum'] = substr($item['settings_media'], 6);
							
							$item['filename'] = 'dummy.gif';
							$item['image'] = 'cmsincludes/storekeeper_image.php?c='. $item['checksum'];
							$item['images'] = array('cmsincludes/storekeeper_closeup_image.php?c='. $item['checksum']);
						} else {
							$item['image'] = getPluginIconURL($item['plugin_name']);
						}
					}
				}
			
				break;
		}
	}
	
	public function storekeeperAJAX($path, $ids, $ids2, $refinerName, $refinerId) {
		switch ($path) {
			case 'tribiq__plugins/tribiq_plugin_nest':
				if ($this->get('add_plugin') && checkPriv('_PRIV_MANAGE_PLUGIN_NEST')) {
					echo $this->addPluginConfirm($this->get('id'), $this->get('refiner__nest'));
					
				} elseif ($this->post('add_plugin') && checkPriv('_PRIV_MANAGE_PLUGIN_NEST')) {
					$this->addPlugin($this->post('id'), $this->post('refiner__nest'));
				
				} elseif ($this->get('remove_plugin') && checkPriv('_PRIV_MANAGE_PLUGIN_NEST')) {
					$nestedItem = getNestDetails($this->get('id'));
					echo $this->removePluginConfirm($nestedItem['plugin_id'], $this->get('refiner__nest'));
					
				} elseif ($this->post('remove_plugin') && checkPriv('_PRIV_MANAGE_PLUGIN_NEST')) {
					$this->removePlugin($this->post('id'), $this->post('refiner__nest'));
				
				} elseif ($this->get('remove_tab') && checkPriv('_PRIV_REORDER_PLUGIN_NEST')) {
					echo $this->removeTabConfirm($this->get('id'), $this->get('refiner__nest'));
					
				} elseif ($this->post('remove_tab') && checkPriv('_PRIV_REORDER_PLUGIN_NEST')) {
					$this->removeTab($this->post('id'), $this->post('refiner__nest'));
					
				} elseif ($this->post('reorder') && checkPriv('_PRIV_REORDER_PLUGIN_NEST')) {
					foreach (explode(',', $ids) as $id) {
						if ($this->post('item__'. $id)) {
							$tabOrd = explode('.', $this->post('item__'. $id));
							$sql = "
								UPDATE ". DB_NAME_PREFIX. "nested_plugins SET
									tab = ". (int) arrayKey($tabOrd, 0). ",
									ord = ". (int) arrayKey($tabOrd, 1). "
								WHERE id = ". (int) $id. "
								  AND instance_id = ". (int) $this->post('refiner__nest');
							my_mysql_query($sql);
						}
					}
					$this->resyncNest($this->post('refiner__nest'));
				}
			
				break;
		}
	}
	
	
	

	public function fillAdminBox($path, &$box) {
		switch ($path) {
			case 'tribiq_plugin_nest__tab':
				if (!$this->get('refiner__nest')) {
					exit;
				}
				
				$box['key']['instanceId'] = $this->get('refiner__nest');
				$instance = getPluginInstanceDetails($box['key']['instanceId']);
				
				$details = array();
				if (issetArrayKey($box['key'], 'id')) {
					$details = getNestDetails($box['key']['id']);
					$box['title'] = getPhrase('Viewing/Editing the "[[tab]]" Tab in the Nest "[[nest]]"',
										array('tab' => htmlspecialchars($details['tab_title']), 'nest' => htmlspecialchars($instance['instance_name'])));
				} else {
					$box['tabs']['tab']['edit_mode']['always_on'] = true;
					$details['tab'] = $details['tab_title'] = 1 + (int) $this->maxTab($box['key']['instanceId']);
					$box['title'] = getPhrase('Adding a new Tab to the Nest "[[nest]]"',
										array('nest' => htmlspecialchars($instance['instance_name'])));
				}
				
				$box['tabs']['tab']['fields']['tab']['value'] = $details['tab'];
				$box['tabs']['tab']['fields']['tab_title']['value'] = $details['tab_title'];
				
				break;
		}
	}
	
	public function validateAdminBox($path, &$box, $definition, $values) {
		switch ($path) {
			case 'tribiq_plugin_nest__tab':
				if (!$values['tab']['tab_title']) {
					$box['tabs']['tab']['errors'][] = getPhrase('Please enter a Title for this Tab.');
				}
				
				break;
		}
	}
	
	public function saveAdminBox($path, &$box, $definition, $values) {
		switch ($path) {
			case 'tribiq_plugin_nest__tab':
				if (issetArrayKey($box['key'], 'id')) {
					$this->updateTab($values['tab']['tab_title'], $box['key']['id']);
				} else {
					$box['key']['id'] = $this->addTab($values['tab']['tab_title'], $box['key']['instanceId']);
				}
				
				break;
		}
	}
	
	
	
	
	
	protected function addPluginConfirm($addPluginInstance, $instanceId, $frontend = false) {
		//Display a confirmation box, asking the admin if they want to add the plugin
		$usage = checkInstancesUsage($instanceId, false);
		$usagePublished = checkInstancesUsage($instanceId, true);
		
		if ($frontend && $usage['count'] <= 1 && $usagePublished['count'] < 1) {
			echo json_encode(array());
			exit;
		}
		
		if (!checkRowExists('plugin_inst_temp_fam_link', array('instance_id' => $addPluginInstance))
		 && !checkRowExists('plugin_inst_temp_link', array('instance_id' => $addPluginInstance))
		 && !checkRowExists('plugin_inst_item_link', array('instance_id' => $addPluginInstance))
		 && !checkRowExists('plugin_instances', array('id' => $addPluginInstance, 'keep_in_album' => 1))) {
			$message =
				'<p>'. getPhrase(
					'Are you sure you wish to move the &quot;[[name]]&quot; Plugin Instance into the Nest &quot;[[nest]]&quot;?',
					array('name' => htmlspecialchars(getInstanceName($addPluginInstance)), 'nest' => htmlspecialchars(getInstanceName($instanceId)))
				). '</p>';
		} else {
			$message =
				'<p>'. getPhrase(
					'Are you sure you wish to copy the &quot;[[name]]&quot; Plugin Instance into the Nest &quot;[[nest]]&quot;?',
					array('name' => htmlspecialchars(getInstanceName($addPluginInstance)), 'nest' => htmlspecialchars(getInstanceName($instanceId)))
				). '</p>';
		}
		
		if ($usage['count'] > 0 || $usagePublished['count'] > 0) {
			$message .=
				'<p>'. getPhrase(
					'This will affect <span class="tribiq_x_published_items">[[published]] Published Content Item(s)</span> <span class="tribiq_y_items">(<a href="[[link]]" target="_blank">[[pages]] Content Item(s) in total</a>).</span>',
					array('pages' => (int) $usage['count'],
							'published' => (int) $usagePublished['count'],
							'link' => htmlspecialchars(getPluginInstanceUsageStorekeeperDeepLink($instanceId)))
				). '</p>';
		}
		
		return $message;
	}
	
	protected function removePluginConfirm($pluginId, $instanceId) {
		//Display a confirmation box, asking the admin if they want to delete the plugin
		$message =
			'<p>'. getPhrase(
				'Are you sure you wish to remove the &quot;[[name]]&quot; Plugin from the Nest &quot;[[nest]]&quot;? Any changes applied to settings while in this Nest will be lost.',
				array('name' => htmlspecialchars(getPluginDisplayName($pluginId)), 'nest' => htmlspecialchars(getInstanceName($instanceId)))
			). '</p>';
		
		$usage = checkInstancesUsage($instanceId, false);
		$usagePublished = checkInstancesUsage($instanceId, true);
		
		if ($usage['count'] > 1 || $usagePublished['count'] > 0) {
			$message .=
				'<p>'. getPhrase(
					'This will affect <span class="tribiq_x_published_items">[[published]] Published Content Item(s)</span> <span class="tribiq_y_items">(<a href="[[link]]" target="_blank">[[pages]] Content Item(s) in total</a>).</span>',
					array(
						'pages' => (int) $usage['count'],
						'published' => (int) $usagePublished['count'],
						'link' => htmlspecialchars(getPluginInstanceUsageStorekeeperDeepLink($instanceId)))
				). '</p>';
		}
		
		return $message;
	}
	
	protected function removeTabConfirm($nestedItemId, $instanceId) {
		//Display a confirmation box, asking the admin if they want to delete the tab
		$tab = getNestDetails($nestedItemId, $instanceId);
		$message =
			'<p>'. getPhrase(
				'Are you sure you wish to remove the &quot;[[tab]]&quot; Plugin from the Tab &quot;[[nest]]&quot;? Any Plugins it contains will be moved to the next tab and will not be removed.',
				array('tab' => htmlspecialchars($tab['tab_title']), 'nest' => htmlspecialchars(getInstanceName($instanceId)))
			). '</p>';
		
		$usage = checkInstancesUsage($instanceId, false);
		$usagePublished = checkInstancesUsage($instanceId, true);
		
		if ($usage['count'] > 1 || $usagePublished['count'] > 0) {
			$message .=
				'<p>'. getPhrase(
					'This will affect <span class="tribiq_x_published_items">[[published]] Published Content Item(s)</span> <span class="tribiq_y_items">(<a href="[[link]]" target="_blank">[[pages]] Content Item(s) in total</a>).</span>',
					array(
						'pages' => $usage['count'],
						'published' => $usagePublished['count'],
						'link' => htmlspecialchars(getPluginInstanceUsageStorekeeperDeepLink($instanceId)))
				). '</p>';
		}
		
		return $message;
	}
	
	
	protected function addPlugin($addPluginInstance, $instanceId, $tab = false) {
		//Add a new Plugin to the nest, placing it in the right-most tab
		if ($instance = getPluginInstanceDetails($addPluginInstance)) {
			
			if (!$tab) {
				$tab = ifNull($this->maxTab($instanceId), 1);
			}
			
			$ord = 1 + (int) $this->maxOrd($instanceId, $tab);
			
			$sql = "
				INSERT INTO ". DB_NAME_PREFIX. "nested_plugins SET
					instance_id = ". (int) $instanceId. ",
					tab = ". (int) $tab. ",
					ord = ". (int) $ord. ",
					plugin_id = ". (int) $instance['plugin_id']. ",
					framework = '". mysql_real_escape_string($instance['framework']). "'";
			my_mysql_query($sql);
			$nestedItemId = mysql_insert_id();
			
			$sql = "
				INSERT INTO ". DB_NAME_PREFIX. "plugin_settings (
					instance_id,
					name,
					nest,
					value
				) SELECT
					". (int) $instanceId. ",
					name,
					". (int) $nestedItemId. ",
					value
				FROM ". DB_NAME_PREFIX. "plugin_settings
				WHERE instance_id = ". (int) $addPluginInstance;
			my_mysql_query($sql);
			
			
			//Delete the instance that was just copied if it looks like it was created just to place in this nest
			if (!checkRowExists('plugin_inst_temp_fam_link', array('instance_id' => $addPluginInstance))
			 && !checkRowExists('plugin_inst_temp_link', array('instance_id' => $addPluginInstance))
			 && !checkRowExists('plugin_inst_item_link', array('instance_id' => $addPluginInstance))
			 && !checkRowExists('plugin_instances', array('id' => $addPluginInstance, 'keep_in_album' => 1))) {
				deletePluginInstance($addPluginInstance);
			}
			
			
			return $nestedItemId;
		} else {
			return false;
		}
	}
	
	protected function addTab($title, $instanceId) {
		//Create a new, empty tab at the end of the nest
		$tab = 1 + (int) $this->maxTab($instanceId);
		
		$sql = "
			INSERT INTO ". DB_NAME_PREFIX. "nested_plugins SET
				instance_id = ". (int) $instanceId. ",
				tab = ". (int) $tab. ",
				ord = 0,
				plugin_id = 0,
				is_tab = 1,
				tab_title = '". mysql_real_escape_string($title). "'";
		my_mysql_query($sql);
		$nestedItemId = mysql_insert_id();
		
		return $nestedItemId;
	}
	
	protected function updateTab($title, $nestedItemId) {
		$sql = "
			UPDATE ". DB_NAME_PREFIX. "nested_plugins SET
				tab_title = '". mysql_real_escape_string($title). "'
			WHERE id = ". (int) $nestedItemId;
		my_mysql_query($sql);
	}
	
	protected function removePlugin($nestedItemId, $instanceId) {
		//Remove a Plugin, being careful to keep the ordinals tidy
		if (($nestedItem = getNestDetails($nestedItemId, $instanceId)) && (!$nestedItem['is_tab'])) {
			$sql = "
				DELETE FROM ". DB_NAME_PREFIX. "plugin_settings
				WHERE instance_id = ". (int) $instanceId. "
				  AND nest != 0
				  AND nest IN (
					SELECT id
					FROM ". DB_NAME_PREFIX. "nested_plugins
					WHERE instance_id = ". (int) $instanceId. "
					  AND id = ". (int) $nestedItemId. "
				  )";
			my_mysql_query($sql);
			
			$sql = "
				DELETE FROM ". DB_NAME_PREFIX. "nested_plugins
				WHERE instance_id = ". (int) $instanceId. "
				  AND id = ". (int) $nestedItemId;
			my_mysql_query($sql);
			
			$this->resyncNest($instanceId);
		}
	}
	
	protected function removeTab($nestedItemId, $instanceId) {
		//Remove a tab, and merge Plugins in the tab with the next tab
		if (($nestedItem = getNestDetails($nestedItemId, $instanceId)) && ($nestedItem['is_tab'])) {
			//Merge with the previous tab, unless this is the first tab, in which case merge with the next tab
			$tab = $nestedItem['tab'] > 1? $nestedItem['tab'] - 1 : 1;
			
			//Get the highest ordinal in the tab to the left
			$maxOrd = $this->maxOrd($instanceId, $tab);
			
			//Delete the tab
			$sql = "
				DELETE FROM ". DB_NAME_PREFIX. "nested_plugins
				WHERE instance_id = ". (int) $instanceId. "
				  AND id = ". (int) $nestedItemId;
			my_mysql_query($sql);
			
			$this->resyncNest($instanceId);
		}
	}
	
	

	protected function resyncNest($instanceId) {
		//Look through a Plugin Nest, and ensure that all of the tab and ordinal numbers are valid by overwriting them
		$tab = 0;
		$ord = 0;
		
		$sql = "
			SELECT id, tab, ord, is_tab
			FROM ". DB_NAME_PREFIX. "nested_plugins
			WHERE instance_id = ". (int) $instanceId. "
			ORDER BY tab, ord";
		
		$result = my_mysql_query($sql);
		while ($row = mysql_fetch_assoc($result)) {
			if ($row['is_tab']) {
				//Catch the case where a Plugin was moved before the first tab
				if ($tab) {
					//If this is a new tab, reset the ordinal
					$ord = 0;
				}
				++$tab;
				$thisOrd = 0;
			} else {
				$thisOrd = ++$ord;
			}
			
			$sql = "
				UPDATE ". DB_NAME_PREFIX. "nested_plugins SET
					tab = ". (int) ifNull($tab, 1). ",
					ord = ". (int) $thisOrd. "
				WHERE instance_id = ". (int) $instanceId. "
				  AND id = ". $row['id'];
			my_mysql_query($sql);
		}
	}
	
	
	protected function maxTab($instanceId) {
		$sql = "
			SELECT MAX(tab) AS tab
			FROM ". DB_NAME_PREFIX. "nested_plugins
			WHERE is_tab = 1
			  AND instance_id = ". (int) $instanceId;
		$result = my_mysql_query($sql);
		$row = mysql_fetch_assoc($result);
		return arrayKey($row, 'tab');
	}
	
	protected function maxOrd($instanceId, $tab) {
		$sql = "
			SELECT MAX(ord) AS ord
			FROM ". DB_NAME_PREFIX. "nested_plugins
			WHERE tab = ". (int) $tab. "
			  AND is_tab = 0
			  AND instance_id = ". (int) $instanceId;
		$result = my_mysql_query($sql);
		$row = mysql_fetch_assoc($result);
		return arrayKey($row, 'ord');
	}
}
