<?php
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

require '../../adminheader.inc.php';
require CMS_ROOT. 'cmsincludes/thickbox.inc.php';

exitIfNotCheckPriv('_PRIV_VIEW_MENU_ITEM');
if ($_POST['mID']) {
	$hasEditPermission = $_POST['parentMenuID']? checkPriv('_PRIV_EDIT_MENU_ITEM') : checkPriv('_PRIV_EDIT_TOP_LEVEL_MENU_ITEM');
} else {
	$hasEditPermission = $_POST['parentMenuID']? checkPriv('_PRIV_ADD_MENU_ITEM') : checkPriv('_PRIV_ADD_TOP_LEVEL_MENU_ITEM');
}

$noSaving = false;
$noValidation = false;
$showEditButton = $hasEditPermission && $_POST['mID'];
$editMode = $_POST['formMode']=="edit" || !$_POST['mID'];
require 'includes/common_head.inc.php';


//Saving data
if ($_POST['save']=="true") {
	
	$menuId = $_POST['mID'];
	
	saveMenuDetails($_POST, $menuId);
	
	//Overwrite the menuId in the primary key if needed
	echo json_encode(array('mID' => $menuId));
	

//Check if a form is valid
} elseif ($_POST['validate']) {

	echo json_encode(validateMenuDetails($_POST));
	

//Call the display form function.
} else {
	
	echo getPhrase('Use this screen to set the Destination for this Menu Item. It may be an internal Content Item, an external URL, or nothing.');
	
	$dbDetails = array();
	$fields = $_POST;
	//Load menu details from the database if an id has been given
	if ($_POST['mID']) {
		$dbDetails = getMenuItemFromMenuID($_POST['mID']);
		
		foreach ($dbDetails as $field => $value) {
			if (!isset($fields[$field])) {
				$fields[$field] = $value;
			}
		}
		
	} elseif (!$fields['target_loc']) {
		$fields['target_loc'] = 'none';
	}
	
	
	thickboxForm($formHTMLHead, $formHTMLFoot, $editMode, $showEditButton);
	
	echo 
		$formHTMLHead, '
			
			<tr>
				<td>&nbsp;</td>
				<td>';
	
	if ($editMode) {
		echo '
					<table><tr>
						<td>
							<input type="radio" name="target_loc" id="target_loc_int" value="int" ',
								$fields['target_loc'] == 'int'? 'checked="checked"' :null, ' />
						</td><td>
							<label for="target_loc_int">', getPhrase('Link to a Content Item:'), '</label>
						</td>
					</tr><tr>
						<td>&nbsp;</td>
						<td>
							<input type="hidden" id="content_id" name="content_id" value="', htmlspecialchars($fields['content_id']), '" />
							<input type="hidden" id="content_type" name="content_type" value="', htmlspecialchars($fields['content_type']), '" />
							<input type="text" readonly="readonly" value="', htmlspecialchars(getTag($fields['content_id'], $fields['content_type'])), '" class="form_field" />
							<input type="button" class="launch_keeper" value="', getPhrase('_SELECT_ITEM'), '" onclick="
								tribiqA.getFromSK(false, \'content\', \''. jsOnClickEscape(ifNull($fields['content_type'], 'html')). '\', \''. jsOnClickEscape($fields['languageId']). '\');
							" />
						</td>
					</tr>';
		
		//For menu items linking to content items, there may be more than one menu item linking to content items.
		//Display some controls in edit mode to manage this
		if ($fields['target_loc'] == 'int' && $fields['content_id'] && $fields['content_type']) {
			echo '
					<tr>
						<td>&nbsp;</td>
						<td class="report_table_container thickbox" style="padding-left: 20px;">';
			
			//Get our current menu id. (If this is a new menu item and yet to get an id, we'll use the word 'this' instead)
			$currentMenuId = $_POST['mID']? (int) $_POST['mID'] : 'this';
			
			//Bogart the first place in the array for the current menu item. (Thereafter we'll order by redundancy.)
			$paths = array($currentMenuId => array('section_id' => $fields['section_id'], 'language_id' => $_POST['languageId']));
			
			//Loop through every menu item path pointing at this content item, ordering by redundancy. 
			$primary = 'checked="checked"';
			foreach(getMenuItemFromContent($fields['content_id'], $fields['content_type'], $fetchSecondaries = true) as $path) {
				
				//The first one that we come to should be marked as the primary, until the user makes a change
				if (!$fields['primary']) {
					$fields['primary'] = $path['id'];
				}
				
				$paths[$path['id']] = $path;
				$paths[$path['id']]['show_trail_from'] = $path['id'];
			}
			
			//If the Content Items had no Menu Item pointing to it, mark this new menu item to be the primary
			if (!$fields['primary']) {
				$fields['primary'] = $currentMenuId;
			}
			
			//Set/Modify the path to the current menu item.
			//I'm using the parent id rather than the menu id for this one case, so that the trail won't include the current name
			$paths[$currentMenuId]['show_trail_from'] = $fields['parent_id'];
			
			
			if (count($paths) > 1) {
				echo '
					<p>', getPhrase('_MULTIPLE_MENU_PATHS'), '</p>';
			}
			
			echo '
							<table style="width: 750px;">
							  <tr class="th_row">
								<th>', getPhrase('_PRIMARY'), '</th>
								<th>', getPhrase('Language'), '</th>
								<th>', getPhrase('_MENU_SECTION'), '</th>
								<th>', getPhrase('_MENU_PATH'), '</th>
							  </tr>';
			
			foreach ($paths as $pathId => $path) {
				
				//Get a breadcrumb trail of menu items leading through this path to the content item
				$menuArray =
					cms_core::getMenuStructure($path['section_id'], $path['show_trail_from'], 0,
									 					999, 999, false, true, true, false, true);
				
				echo '
							  <tr class="', $pathId == $currentMenuId? 'odd' : 'even', '">
								<td>
									<input type="radio" name="primary" id="primary_', $pathId, '" value="', $pathId, '"
										', $fields['primary'] == $pathId? 'checked="checked"' :null, ' />
								</td><td>
									', htmlspecialchars($path['language_id']), '
								</td><td>
									<label for="primary_', $pathId, '">
										',  htmlspecialchars($currentMenuId == $pathId? $_POST['sectionId'] : $path['section_id']), '
									</label>
								</td><td>
									<label for="primary_', $pathId, '">';
				
				//Display the trail
				while ($menuArray) {
					echo htmlspecialchars($menuArray[0]['name']);
					
					if ($menuArray = $menuArray[0]['children']) {
						echo ' &#187; ';
					}
				}
				
				//Special case for the current menu item: add the current name onto the end of the trail
				if ($currentMenuId == $pathId) {
					if ($fields['parent_id']) {
						echo ' &#187; ';
					}
					echo htmlspecialchars($fields['currentName']);
				}
				
				echo '
									</label>
								</td>
							  </tr>';
			}
			
			echo '
							</table>
							<div id="pagination"></div>
						</td>
					</tr>';
		}
		
		echo '
					<tr>
						<td>
							<input type="radio" name="target_loc" id="target_loc_ext" value="ext" ',
								$fields['target_loc'] == 'ext'? 'checked="checked"' :null, ' />
						</td><td>
							<label for="target_loc_ext">', getPhrase('Link to an external URL:'), '</label>
						</td>
					</tr><tr>
						<td>&nbsp;</td>
						<td>
							<input type="text" maxlength="255" class="form_field" id="ext_url" name="ext_url" value="', htmlspecialchars($fields['ext_url']), '" onclick="
								get(\'target_loc_ext\').checked = true;
							" onfocus="
								if(!this.value) this.value = \'http://\';
							"/>
							<input type="button" class="submit" value="', getPhrase('_TEST'), '" onclick="
								window.open(get(\'ext_url\').value);" />
						</td>
					</tr><tr>
						<td>
							<input type="radio" name="target_loc" id="target_loc_none" value="none" ',
								$fields['target_loc'] == 'none'? 'checked="checked"' :null, ' />
						</td><td>
							<label for="target_loc_none">', getPhrase("Don't link to anything"), '</label>
						</td>
					</tr></table>';
					
	} elseif ($fields['target_loc'] == 'int') {
		
		$latestVersion = getLatestVersion($fields['content_id'], $fields['content_type']);
		
		echo
			getContentStatusIcon($fields['content_id'], $fields['content_type'], $latestVersion),
			' ',
			htmlspecialchars(getTag($fields['content_id'], $fields['content_type']));
		
	} elseif ($fields['target_loc'] == 'ext' && $fields['ext_url']) {
		echo '<div class="menu_ext_url"></div> ', htmlspecialchars($fields['ext_url']);
	} else {
		echo getPhrase('_UNLINKED_MENU_ITEM_DESTINATION');
	}
	
	echo '
				</td>
			</tr>
		
		', $formHTMLFoot;
}
?>