<?php
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */


require '../../adminheader.inc.php';
require CMS_ROOT. 'cmsincludes/thickbox.inc.php';

if (issetArrayKey($_POST, 'nest')) {
	exitIfNotCheckPriv('_PRIV_VIEW_NESTED_PLUGIN');
	$hasEditPermission = checkPriv('_PRIV_MANAGE_NESTED_PLUGIN');
} else {
	exitIfNotCheckPriv('_PRIV_VIEW_PLUGIN_INSTANCE');
	$hasEditPermission = checkPriv('_PRIV_MANAGE_PLUGIN_INSTANCE');
}

$nest = arrayKey($_POST, 'nest');
$pluginId = arrayKey($_POST, 'pluginId');
$instanceId = arrayKey($_POST, 'instanceId');

$noSaving = false;
$noValidation = false;
$showEditButton = $instanceId && $hasEditPermission;
$editMode = !$instanceId || $_POST['formMode']=="edit";
require 'includes/common_head.inc.php';

getPluginSettingDesc($instanceId, $instanceName, $pluginName, $pluginId, $frameworks, $tab_programs, $sections, $valuesInDB, $nest, $editMode);

$newInstanceName = false;
//Create a new copy of this instance, with a new name, if requested
if ($_POST['save_action'] == 'create_new_instance') {
	$newInstanceName = substr(getPhrase('Tryout for "[[name]]" on [[place]]', array('name' => $instanceName, 'place' => getTag($_POST['cID'], $_POST['cType']))), 0, 255);
}


//Saving data
if ($_POST['save']=="true") {
	//Handle creating a new instance with a new name
	if ($newInstanceName) {
		renameInstance($instanceId, $nest, $newInstanceName, $error, true, false, (bool) $_POST['cID']);
		
		//If this has been called from a slot on a content item, update that slot with this new instance straight away
		if ($_POST['cID'] && $_POST['cType'] && $_POST['slotName']) {
			updatePluginInstanceInItemSlot($instanceId, $_POST['slotName'], $_POST['cID'], $_POST['cType']);
		}
	
	//Handle creating a new instance
	} elseif ($tab == 1 && !$instanceId) {
		$errors = array();
		createNewInstance($pluginId, $_POST['instance_name'], $instanceId, $errors);
	}
	
	//Loop through each field that would be in the thickbox
	foreach ($sections[$tab] as $field => &$details) {
		if ($details['Setting Name']) {
			if (isset($_POST[$field])) {
				$value = $_POST[$field];
			} elseif ($details['Type'] == 'tinyint(1)' || $details['Type'] == 'tinyint') {
				//Work around for checkboxes not appearing in the POST if not selected on the page
				$value = 0;
			}
			
			//Save the setting
			setPluginSetting($details['Setting Name'], $value, $instanceId, $nest);
		}
	}
	
	//Save the framework, if set
	if ($tab == 1) {
		if (isset($_POST['framework'])) {
			if ($nest || !checkPriv('_PRIV_MANAGE_CUSTOM_FRAMEWORK')) {
				saveInstance($instanceId, $_POST, $nest);
			} else {
				setInstanceCustomFramework($instanceId, $_POST, $_POST['use_custom_framework'], $_POST['custom_framework']);
			}
		}
	}
	
	//Clear anything that is cached for this instance
	$sql = "
		DELETE
		FROM ". DB_NAME_PREFIX. "plugin_instance_cache
		WHERE instance_id = ". (int) $instanceId;
	my_mysql_query($sql);
	
	echo json_encode(array('instanceId' => $instanceId, 'nest' => $nest, 'instance_new_name' => '', 'save_action' => ''));


} elseif ($_POST['validate']) {
	
	$errors = array();
	
	//Validate the new instance name, if chosing a new name
	if ($newInstanceName) {
		if (!renameInstance($instanceId, $nest, $newInstanceName, $error, true, true, (bool) $_POST['cID'])) {
			$errors[] = $error;
		}
	
	} elseif ($tab == 1 && !$instanceId) {
		$valid = createNewInstance($pluginId, $_POST['instance_name'], $instanceId, $errors, $onlyValidate = true);
	}
	
	echo json_encode(array('valid' => !count($errors), 'errArray' => $errors));
	

//Call the display form function.
} else {
	
	
	echo dynamicThickboxForm('',
		$editMode, $showEditButton, $sections[$tab], array(), $_POST, $valuesInDB);
}
?>