<?php
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

require '../../adminheader.inc.php';
require CMS_ROOT. 'cmsincludes/thickbox.inc.php';

exitIfNotCheckPriv('_PRIV_CREATE_FIRST_DRAFT');
$hasEditPermission = post('parent_id')? checkPriv('_PRIV_ADD_MENU_ITEM') : checkPriv('_PRIV_ADD_TOP_LEVEL_MENU_ITEM');

$noSaving = false;
$noValidation = false;
$showEditButton = $hasEditPermission && $_POST['mID'];
$editMode = $_POST['formMode']=="edit" || !$_POST['mID'];
require 'includes/common_head.inc.php';


//Saving data
if ($_POST['save']=="true") {
	
	if (includeContentType($_POST['content_type'])) {
		//Get the details that we'll need
		$submission = $_POST;
		$submission['target_loc'] = 'int';
		$submission['content_version'] = 1;
		$submission['name'] = $submission['title'];
		$submission['alias'] = tidyAlias($submission['alias']);
		
		//Call the setItemMeta, updateItemTemplate and saveMenuDetails save routines from elsewhere in the CMS
		$details = setItemMeta(0, 0, $submission['title'], $submission['language_id'], $description = '', $keywords = '', $publication_date = true);
		if ($submission['content_id'] = $details['cID']) {
			
			setAlias($submission['content_id'], $submission['content_version'], $submission['alias']);
			updateItemTemplate($submission['content_id'], $submission['content_type'], $submission['content_version'], $submission['template_id']);
			saveMenuDetails($submission, $menuId);
			
			if (inc('tribiq_cms_core_probusiness')) {
				tribiq_cms_core_probusiness::saveMenuVisibilityFB($menuId, post('invisible'));
			}
			
			if (checkPriv('_PRIV_MANAGE_ITEM_SLOT') && $submission['feature_instance_id'] && $instance = getPluginInstanceDetails($submission['feature_instance_id'])) {
			
				$featureSlot = 'Main_Slot_2';
				if ($template = getTemplateDetails($submission['template_id'])) {
					if (($skinId = $template['skin_id']) || (($family = getTemplateFamilyDetails($template['family_name'])) && ($skinId = $family['skin_id']))) {
						$skin = $desc = false;
						if (validateSkinFromXMLDescription($skinId, $skin, $desc)) {
							if ($desc->feature_slot) {
								$featureSlot = (string) $desc->feature_slot;
							}
						}
					}
				}
				
				updatePluginInstanceInItemSlot($submission['feature_instance_id'], $featureSlot, $submission['content_id'], $submission['content_type'], $submission['content_version']);
				
				sendSignal('eventQuickAddContentItem', array($submission['content_id'], $instance));
			}
			
			echo json_encode(array('cID' => $submission['content_id'], 'cType' => $submission['content_type']));
		}
	}
	

//Check if a form is valid
} elseif ($_POST['validate']) {
	
	$errors = array();
	
	if (!includeContentType($_POST['content_type'])) {
		$errors[] = getPhrase('The [[cType]] content type is not enabled!', array('cType' => htmlspecialchars($_POST['content_type'])));
	}
	
	if (!$_POST['title']) {
		$errors[] = getPhrase('Please enter a Title.');
	}
	
	if ($aliasErrors = validateAlias($_POST['alias'])) {
		$errors = array_merge($errors, $aliasErrors);
	}
	
	if (!$_POST['template_id']) {
		$errors[] = getPhrase('Please select a Template.');
	}

	echo json_encode(array('valid' => !count($errors), 'errArray' => $errors));
	

//Call the display form function.
} else {
	

	$dbDetails = array();
	$valuesInCache = $_POST;
	
	$requiredFields = array('title');
	$aboveTableHTML = '';
	
	$fields = array();
	
	$fields['title'] = 
		array(
			'Label' => 'Title:',
			'Type' => 'varchar(250)',
			'onkeyup' => "tribiqTab.autofillAlias(this); tribiqA.manField = this;"
		);
	
	$fields['alias'] = 
		array(
            'Label' => 'Alias:',
            'Type' => 'varchar(50)',
			'onkeyup' => "tribiqTab.aliasChange(this); tribiqTab.validateAlias();",
			'Post Field HTML' => htmlspecialchars(getSiteConfig('mod_rewrite_enabled')? getSiteConfig('mod_rewrite_suffix') : ''). ' <div id="aliasWarningDisplay"></div>'
        );
	
	$fields['template_id'] = 
		array(
            'Label' => getPhrase('Template:'),
            'Type' => 'template_id',
            'Template_cType' => $_POST['content_type'],
            'Template_Hide_Null_Option' => true
        );
	
	if (checkPriv('_PRIV_MANAGE_ITEM_SLOT')) {
		$fields['feature_instance_id'] = 
			array(
				'Label' => getPhrase('Plugin Instance in Feature Slot:'),
				'Type' => 'sk',
				'Nothing Selected Phrase' => getPhrase('Inherit default from Template or Template Family'),
				'Select Phrase' => getPhrase('Select Plugin Instance...'),
				'Path' => 'tribiq__plugins/feature_plugins_only////',
				'Target Path' => 'tribiq__plugins/instances',
				'Min Path' => 'tribiq__plugins',
				'Max Path' => 'tribiq__plugins/instances',
				'Disallow Identical Min Path' => true
			);
	}
	
	//Look for the default Template id, and preselect it
	foreach (getContentTypes($_POST['content_type']) as $contentType) {
		$dbDetails['template_id'] = $contentType['default_template_id'];
		break;	
	}
	
	
	thickboxForm($formHTMLHead, $formHTMLFoot, $editMode, $showEditButton, $aboveTableHTML);

	echo $formHTMLHead;
	
	echo '
		<tr>
			<td colspan="2">
				<input type="radio" name="quick" checked="checked" value="1" id="tribiq_quick" onchange="tribiqTab.quick();" />
				<label for="tribiq_quick">',  getPhrase('Quickly create a new Content Item (HTML Page):'). '</label>
			</td>
		</tr>';
	
	echo dynamicThickboxFormInner($editMode, $fields, $requiredFields, $valuesInCache, $dbDetails);
	
	if (inc('tribiq_cms_core_probusiness')) {
		tribiq_cms_core_probusiness::showMenuVisibilityFB(false, $editMode);
	}
	
	echo '
		<tr>
			<td colspan="2">
				<input type="radio" name="quick" value="" id="tribiq_not_quick" onchange="tribiqTab.quick();" />
				<label for="tribiq_not_quick">',  getPhrase('Link to an existing Content Item or to an external URL:'). '</label>
			</td>
		</tr>
		<tr>
			<td class="title"></td>
			<td><input type="button" id="tribiq_more_options" value="',  getPhrase('More Options...'). '" onclick="tribiqTab.moreOptions();"/></td>
		</tr>';
		
	echo $formHTMLFoot;
}
?>