<?php
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

require '../../adminheader.inc.php';
require CMS_ROOT. 'cmsincludes/thickbox.inc.php';

exitIfNotCheckPriv('_PRIV_CHANGE_USER_PASSWORD');
$hasEditPermission = checkPriv('_PRIV_CHANGE_USER_STATUS');

$noSaving = false;
$noValidation = false;
$showEditButton = $hasEditPermission && $_POST['userId'];
$editMode = $_POST['formMode']=="edit" || !$_POST['userId'];
require 'includes/common_head.inc.php';


//Saving data
if ($_POST['save']=="true") {
	
	$userId = $_POST['userId'];
	saveUser($_POST, $userId);
	
	echo json_encode(array());
	

//Check if a form is valid
} elseif ($_POST['validate']) {

	echo json_encode(validateUser($_POST));
	

//Call the display form function.
} else {
	
	//Load user details fro database if an id has been given
	$userDBDetails = getUserDetails($_POST['userId']);
	
	$fields = array();
	
	$fields['created_date'] = array(
		'Field' => 'created_date',
		'Type' => 'datetime',
		'Table' => 'users',
		'Numeric' => false,
		'Read Only' => true
	);
	
	$fields['modified_date'] = array(
		'Field' => 'modified_date',
		'Type' => 'datetime',
		'Table' => 'users',
		'Numeric' => false,
		'Read Only' => true
	);

	$fields['last_login'] = array(
		'Field' => 'last_login',
		'Type' => 'datetime',
		'Table' => 'users',
		'Numeric' => false,
		'Read Only' => true
	);
	
	
	//Right now the suspended_date column in user tables is not populated...
	/*
	if ($userDBDetails['status'] == 'suspended') {
		$fields['suspended_date'] = array(
			'Field' => 'suspended_date',
			'Type' => 'date',
			'Table' => 'users',
			'Numeric' => false,
			'Read Only' => true
		);
	}*/
	
	$fields['email_verified'] = array(
		'Field' => 'email_verified',
		'Label' => 'Email Verified:',
		'Type' => 'tinyint(1)',
		'Null' => 'NO',
		'Key' => '',
		'Default' => '',
		'Extra' => '',
		'Table' => 'users',
		'Numeric' => false
	);
	
	$fields['status'] = array(
		'Field' => 'status',
		'Type' => "enum('pending','active','suspended')",
		'Null' => 'NO',
		'Key' => '',
		'Default' => '',
		'Extra' => '',
		'Table' => 'users',
		'Numeric' => false,
		'Note Below' => getPhrase('Status must be Active in order for a User to be able to log in.')
	);
	
	if ($editMode && $userDBDetails['status'] != 'active') {
		if ($emailtemplate = getEmailTemplate('ENET Access Granted')) {
			$fields['send_user_email_notification'] = array(
				'Field' => 'email_user',
				'Type' => 'tinyint(1)',
				'Null' => 'NO',
				'Key' => '',
				'Default' => '',
				'Extra' => '',
				'Table' => '',
				'Numeric' => false
			);
			
			$fields['email_notification_text'] = array(
				'Field' => 'email_notification_text',
				'Read Only' => true,
				'Numeric' => false
			);
			
			$siteConfig = loadSiteConfig ();
	
			$searchArray = array( '[[first_name]]',
									'[[site_name]]',
									'[[url]]');
			$replaceArray = array( $userDBDetails['first_name'],
									$siteConfig['email_name_from'],
									$url = 'http://'. $_SERVER['HTTP_HOST']);
	
			$userDBDetails['email_notification_text'] = str_replace("\n\n", "\n", 
															$emailtemplate['subject'].
															"\n".
															strip_tags(str_replace($searchArray, $replaceArray, $emailtemplate['body'])));
		} else {
		
			$fields['email_notification_text'] = array(
				'Field' => 'email_notification_text',
				'Read Only' => true,
				'Numeric' => false
			);
			$userDBDetails['email_notification_text'] = getPhrase('_NOT_SET');
		}
	}
	
	//Don't let the admin make nonsensical state changes.
	//Narrow down their option for the state, depending on the current state
	//$fields['status']['Type'] = "enum('pending','active','suspended')";
	if ($userDBDetails['status'] == 'pending') {
		$fields['status']['Type'] = "enum('pending','active')";
	
	} elseif ($userDBDetails['status'] == 'active') {
		$fields['status']['Type'] = "enum('active','suspended')";
	
	} elseif ($userDBDetails['status'] == 'suspended') {
		$fields['status']['Type'] = "enum('active','suspended')";
	}
	
	$requiredFields = array();
	
	$columnsNotToShow = array();
	
	
	echo dynamicThickboxForm($aboveTableHTML, 
					$editMode, $showEditButton, $fields, $requiredFields, $_POST, $userDBDetails, $columnsNotToShow);
}

exit;
?>