<?php 
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

require '../../adminheader.inc.php';
require CMS_ROOT. 'cmsincludes/thickbox.inc.php';

exitIfNotCheckPriv('_PRIV_MANAGE_PLUGIN_INSTANCE');


//Warn the user if the plugin has been used in more than one place
$usage = checkInstancesUsage($_POST['instanceId'], false);
$instance = getPluginInstanceDetails($_POST['instanceId']);

//Don't use warnings/tryouts for nested Plugins that are only used in one place
if (issetArrayKey($_POST, 'nest') && $usage['count'] <= 1) {
	echo json_encode(array());
	exit;

//Or for newly created Plugin Instanced
} elseif (!issetArrayKey($_POST, 'instanceId') && issetArrayKey($_POST, 'pluginId')) {
	echo json_encode(array());
	exit;
}

$warningTitle = '';
$overwriteSelected = false;
if ($instance['keep_in_album']) {
	$warningTitle = '<p>'. getPhrase('You are changing the way this Album Plugin operates.'). '</p>';
	
} elseif ($_POST['cID'] && $usage['count'] > 1) {
	$warningTitle = '<p>'. getPhrase('You are changing the way this Plugin operates, and this Plugin is used elsewhere.'). '</p>';
	
} elseif (!$_POST['cID'] && $usage['count'] > 0) {
	$warningTitle = '<p>'. getPhrase('You are changing the way this Plugin operates, and this Plugin is in use.'). '</p>';
	
} else {
	$warningTitle = '<p>'. getPhrase('You are changing the way this Plugin operates.'). '</p>';
}

if ($_POST['cID'] && $usage['count'] <= 1) {
	if ($_POST['parent_id']) {
		echo json_encode(array());
		exit;
	}

	$changeInstanceLabel = getPhrase('(This Instance is only used on this Content Item.)');
	$overwriteSelected = true;
	
} elseif (!$_POST['cID'] && !$usage['count']) {
	echo json_encode(array());
	exit;

} else {
	$newInstanceSelected = true;
	$usagePublished = checkInstancesUsage($_POST['instanceId'], true);
	
	$changeInstanceLabel = getPhrase(
		'This will affect <span class="tribiq_x_published_items">[[published]] Published Content Item(s)</span> <span class="tribiq_y_items">(<a href="[[link]]" target="_blank">[[pages]] Content Item(s) in total</a>).</span>',
		array('pages' => $usage['count'],
				'published' => $usagePublished['count'],
				'link' => htmlspecialchars(getPluginInstanceUsageStorekeeperDeepLink($instance['instance_id'], $instance['plugin_id']))));
}

//If so, prepare HTML for a warning box
$popUpTB = $warningTitle;

if ($_POST['cID'] && !$_POST['parent_id'] && !issetArrayKey($_POST, 'doNotOfferTryout')) {
	$changeInstanceLabel = getPhrase('Apply these new settings to the Plugin Instance immediately.'). ' '. $changeInstanceLabel;
	
	$popUpTB .= '
	<p>&nbsp;<br/>'. getPhrase('Do you want to:'). '</p>
	<table>
	<tr><td>
		<input type="radio" name="instance_options_choice" id="create_new_instance"'. ($newInstanceSelected? ' checked="checked"' : ''). '/>
	</td><td>
		<label for="create_new_instance">'. getPhrase('Try out these new settings just here.'). '</label>
	</td></tr>
	<tr><td valign="top">
		<input type="radio" name="instance_options_choice" id="change_instance"'. ($overwriteSelected? ' checked="checked"' : ''). '/>
	</td><td>
		<label for="change_instance">'. $changeInstanceLabel. '</label>
	</td></tr></table>';

} else {
	$popUpTB .= '<p>'. $changeInstanceLabel. '</p>';
}

$buttons = '
	<input type="button" class="submit_selected" value="'. getPhrase('_SAVE'). '" onClick="tribiqTab.submitSettingsConfirmForm();" />
	<input type="button" class="submit" value="'. getPhrase('_CANCEL'). '" onClick="tribiqTab.cancelSettingsConfirmForm();" />';


echo json_encode(array('action' => 'confirm', 'html' => $popUpTB, 'buttons' => $buttons, 'warning' => 1));


?>